/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.actions;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.mapping.ResourceMapping;
import org.eclipse.core.resources.mapping.ResourceTraversal;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionDelegate;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public class BuildProjectActionDelegate extends ActionDelegate implements
        IObjectActionDelegate, IViewActionDelegate {

    protected IStructuredSelection selection;

    // The shell, required for the progress dialog
    protected Shell shell;

    private IWorkbenchPart targetPart;

    /**
     * Creates an array of the given class type containing all the objects in
     * the selection that adapt to the given class.
     * 
     * @param selection
     * @param c
     * @return
     */
    @SuppressWarnings("unchecked") //$NON-NLS-1$
    public static Object[] getSelectedAdaptables(ISelection selection, Class c) {
        List<Object> result = null;
        if (!selection.isEmpty()) {
            result = new ArrayList<Object>();
            Iterator elements = ((IStructuredSelection) selection).iterator();
            while (elements.hasNext()) {
                Object adapter = getAdapter(elements.next(), c);
                if (c.isInstance(adapter)) {
                    result.add(adapter);
                }
            }
        }
        if (result != null && !result.isEmpty()) {
            return result.toArray((Object[]) Array
                    .newInstance(c, result.size()));
        }
        return (Object[]) Array.newInstance(c, 0);
    }

    /**
     * Find the object associated with the given object when it is adapted to
     * the provided class. Null is returned if the given object does not adapt
     * to the given class
     * 
     * @param selection
     * @param c
     * @return Object
     */
    @SuppressWarnings("unchecked") //$NON-NLS-1$
    public static Object getAdapter(Object adaptable, Class c) {
        if (c.isInstance(adaptable)) {
            return adaptable;
        }
        if (adaptable instanceof IAdaptable) {
            IAdaptable a = (IAdaptable) adaptable;
            Object adapter = a.getAdapter(c);
            if (c.isInstance(adapter)) {
                return adapter;
            }
        }
        return null;
    }

    /**
     * Returns the selected projects.
     * 
     * @return the selected projects
     */
    protected IProject[] getSelectedProjects() {
        IResource[] selectedResources = getSelectedResources();
        if (selectedResources.length == 0)
            return new IProject[0];
        List<IResource> projects = new ArrayList<IResource>();
        for (int i = 0; i < selectedResources.length; i++) {
            IResource resource = selectedResources[i];
            if (resource.getType() == IResource.PROJECT) {
                projects.add(resource);
            }
        }
        return (IProject[]) projects.toArray(new IProject[projects.size()]);
    }

    /**
     * Get the selected projects for the selected resources
     * 
     * @return
     */
    protected List<IProject> getSelectedProjectsFromSelectedResources() {
        List<IProject> projects = new ArrayList<IProject>();
        IResource[] selectedResources = getSelectedResources();

        for (int i = 0; i < selectedResources.length; i++) {
            IResource resource = selectedResources[i];
            IProject project = resource.getProject();
            if (project != null) {
                projects.add(project);
            }
        }

        return projects;
    }

    /**
     * Returns an array of the given class type c that contains all instances of
     * c that are either contained in the selection or are adapted from objects
     * contained in the selection.
     * 
     * @param c
     * @return
     */
    @SuppressWarnings("unchecked") //$NON-NLS-1$
    protected Object[] getSelectedResources(Class c) {
        return getSelectedAdaptables(selection, c);
    }

    /**
     * Returns the selected resources.
     * 
     * @return the selected resources
     */
    protected IResource[] getSelectedResources() {
        List<IResource> resourceArray = new ArrayList<IResource>();
        IResource[] resources = (IResource[]) getSelectedResources(IResource.class);
        for (int i = 0; i < resources.length; i++)
            resourceArray.add(resources[i]);
        ResourceMapping[] resourceMappings = (ResourceMapping[]) getSelectedAdaptables(
                selection, ResourceMapping.class);
        for (int i = 0; i < resourceMappings.length; i++) {
            ResourceMapping resourceMapping = (ResourceMapping) resourceMappings[i];
            try {
                ResourceTraversal[] traversals = resourceMapping.getTraversals(
                        null, null);
                for (int j = 0; j < traversals.length; j++) {
                    IResource[] traversalResources = traversals[j]
                            .getResources();
                    for (int k = 0; k < traversalResources.length; k++) {
                        if (!resourceArray.contains(traversalResources[k]))
                            resourceArray.add(traversalResources[k]);
                    }
                }
            } catch (CoreException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        IResource[] selectedResources = new IResource[resourceArray.size()];
        resourceArray.toArray(selectedResources);
        return selectedResources;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.actions.ActionDelegate#run(org.eclipse.jface.action.IAction)
     */
    @Override
    public void run(IAction action) {
        List<IProject> projects = getSelectedProjectsFromSelectedResources();
        IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
        for (IProject project : projects) {
            MavenAction install = new MavenInstallAction(window
                    .getPartService().getActivePart().getSite().getShell());
            install.setProjectName(project.getName());
            install.run();
        }
    }

    /*
     * Method declared on IActionDelegate.
     */
    public void selectionChanged(IAction action, ISelection sel) {
        if (sel instanceof IStructuredSelection) {
            this.selection = (IStructuredSelection) sel;
            if (action != null) {
                setActionEnablement(action);
            }
        }
        if (sel instanceof ITextSelection) {
            IEditorPart part = getTargetPage().getActiveEditor();
            if (part != null) {
                IEditorInput input = part.getEditorInput();
                IResource r = (IResource) input.getAdapter(IResource.class);
                if (r != null) {
                    switch (r.getType()) {
                        case IResource.FILE:
                            this.selection = new StructuredSelection(r);
                            if (action != null) {
                                setActionEnablement(action);
                            }
                            break;
                    }
                } // set selection to current editor file;
            }
        }
    }

    /**
     * Method invoked from <code>selectionChanged(IAction, ISelection)</code>
     * to set the enablement status of the action. The instance variable
     * <code>selection</code> will contain the latest selection so the methods
     * <code>getSelectedResources()</code> and
     * <code>getSelectedProjects()</code> will provide the proper objects.
     * 
     * This method can be overridden by subclasses but should not be invoked by
     * them.
     */
    protected void setActionEnablement(IAction action) {
        try {
            action.setEnabled(isEnabled());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * Method declared on IObjectActionDelegate.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        if (targetPart != null) {
            this.shell = targetPart.getSite().getShell();
            this.targetPart = targetPart;
        }
    }

    /**
     * @return IWorkbenchPart
     */
    protected IWorkbenchPart getTargetPart() {
        return targetPart;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IViewActionDelegate#init(org.eclipse.ui.IViewPart)
     */
    public void init(IViewPart view) {
        targetPart = view;
    }

    private boolean isEnabled() {
        return true;
    }

    // PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    /**
     * Return the path that was active when the menu item was selected.
     * 
     * @return IWorkbenchPage
     */
    protected IWorkbenchPage getTargetPage() {
        if (getTargetPart() == null)
            return getActivePage();
        return getTargetPart().getSite().getPage();
    }

    /**
     * 
     * @return
     */
    public static IWorkbenchPage getActivePage() {
        IWorkbenchWindow window = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (window == null)
            return null;
        return window.getActivePage();
    }

}
