/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JbiEditorAction.java,v 1.7 2007/04/16 07:54:23 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.jbieditor.JbiEditor;

/**
 * Action to access the JBI Editor
 * 
 * @author LAM
 */
public class JbiEditorAction extends Action {

    private String projectName;

    private final IViewSite site;

    /**
     * Constructor
     * 
     * @param site
     *            The view site
     */
    public JbiEditorAction(final IViewSite site) {
        this.setText(Messages.JbiEditorAction_0);
        this.setToolTipText(Messages.JbiEditorAction_1);
        this.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
                Activator.PLUGIN_ID, "resources/icons/generic_xml_obj.gif")); //$NON-NLS-1$
        this.site = site;
    }

    /**
     * Gets the project name
     * 
     * @return the projectName
     */
    public String getProjectName() {
        return this.projectName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    @Override
    public void run() {
        final IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
        final IFile jbiFile = wsRoot.getProject(this.projectName).getFile(
                Constants.JBI_PATH);

        try {
            this.site.getPage().openEditor(new FileEditorInput(jbiFile),
                    "org.objectweb.petals.tools.plugin.development.jbieditor", //$NON-NLS-1$
                    true);
        } catch (final PartInitException e) {
            MessageDialog.openError(this.site.getShell(), JbiEditor.JBI_EDITOR,
                    Messages.JbiEditorAction_3);
        }
    }

    /**
     * Sets the project name
     * 
     * @param projectName
     *            the projectName to set
     */
    public void setProjectName(final String projectName) {
        this.projectName = projectName;
    }

}
