/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.actions;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.constants.JarPaths;
import org.objectweb.petals.tools.plugin.development.constants.ProjectType;
import org.objectweb.petals.tools.plugin.development.constants.WizardComposites;
import org.objectweb.petals.tools.plugin.development.constants.Wizards;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.InputStreamUtils;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.JbiUtils;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.maven.PomUtils;
import org.objectweb.petals.tools.plugin.development.wizards.BusinessCommon;
import org.objectweb.petals.tools.plugin.development.wizards.models.ProvidedSuBean;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * The JBI project creation {@link Job}
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class JobWizardProjectCreation extends Job {

    public static final int TOTAL_WORK_TICKS = 20;

    /**
     * NEXT_AVAILABLE_WORK_TICK = 5
     * 
     * Ticks 0 to 4 are reserved for this Job. When monitor is used in
     * IWizardCustomization.addSpecificRessourcesToProject() overriden method, 5
     * to 20 can be used.
     */
    public static final int NEXT_AVAILABLE_WORK_TICK = 5;

    private IProject newProjectHandle;

    private boolean useDefaultUri;

    private URI locationUri;

    private List<WizardCustomizationModel> specificTypesList = null;

    private int specificTypesIndex = -1;
    /** Determine if we should generate a component file and its packages or not. */
    private boolean createComponentFile = true;

    /**
     * The JBI project type
     */
    private ProjectType projectType;

    /**
     * The list of su dependencies FIXME : Could be dependencies instead of SUs
     */
    private List<ArtifactInformation> dependencies;

    private List<ProvidedSuBean> suList;

    private String mavenArtifactId;

    private String mavenGroupId;

    private String mavenVersion;

    private String mavenPackaging;

    private ArtifactInformation cdkArtifact;
    
    /**
     * TODO: use XML files for this part.
     * TODO: create a default XML package managing access to XML files. 
     * TODO: ensure that CDK versions have the same name in the different files. 
     */
    private HashMap<String, String> BCArtifacts = new HashMap<String, String>();
    private HashMap<String, String> SEArtifacts = new HashMap<String, String>();

    // private ArtifactInformation workspaceResource;

    /**
     * Creates a new instance of {@link JobWizardProjectCreation}
     * 
     */
    public JobWizardProjectCreation() {
        super(Messages.JobWizardProjectCreation_10);
        
        // TODO: see above about XML files...
        BCArtifacts.put( "2.0", "org.objectweb.petals.component.framework.bc.DefaultBindingComponent" ); //$NON-NLS-1$ //$NON-NLS-2$
        SEArtifacts.put( "2.0", "org.objectweb.petals.component.framework.se.DefaultServiceEngine" ); //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * Build the dependencies string
     * 
     * @return
     */
    private String buildDependenciesString() {
        final List<ArtifactInformation> dependencies = this
                .getAllDependencies();
        final StringBuffer sb = new StringBuffer(""); //$NON-NLS-1$

        for (final ArtifactInformation dependency : dependencies) {
            sb.append(dependency.toMaven());
        }

        return sb.toString();
    }

    /**
     * Create the component file with the good class name and package.
     * 
     * @param jbiFile
     * @param componentTemplate
     * @throws CoreException
     */
    private void createComponentFile() throws CoreException {
        String componentTemplate = null;
        if (this.isBindingComponent()) {
            componentTemplate = Constants.Templates.BC_TEMPLATE;
        } else {
            componentTemplate = Constants.Templates.SE_TEMPLATE;
        }

        final IPath path = this.getComponentMainPath().addFileExtension("java"); //$NON-NLS-1$
        final IFile file = this.newProjectHandle.getFile(path);
        InputStream is = this.getClass().getResourceAsStream(
                "/" + componentTemplate); //$NON-NLS-1$
        try {
            String content = InputStreamUtils.inputStreamAsString(is);
            content = content.replaceAll(Wizards.IN_FILE_JAVA_PACKAGE, this
                    .getPackageName());
            content = content.replaceAll(Wizards.IN_FILE_JAVA_CLASSNAME, this
                    .getComponentClassName());

            is = new ByteArrayInputStream(content.getBytes());
        } catch (final IOException e) {
        } finally {
            file.create(is, true, null);
        }
    }

    /**
     * Create the external listener file with the good class name and package.
     * 
     * @throws CoreException
     */
    private void createExternalListenerFile() throws CoreException {
        final Path path = this.getExternalListenerPath();
        final IFile file = this.newProjectHandle.getFile(path
                .addFileExtension("java")); //$NON-NLS-1$
        InputStream is = this.getClass().getResourceAsStream(
                "/" + Constants.Templates.EXTERNAL_LISTENER); //$NON-NLS-1$

        try {
            String content = InputStreamUtils.inputStreamAsString(is);
            content = content.replaceAll(Wizards.IN_FILE_JAVA_PACKAGE, this
                    .getListenerPackage());
            content = content.replaceAll(Wizards.IN_FILE_JAVA_CLASSNAME,
                    Constants.Templates.EXTERNAL_LISTENER);

            is = new ByteArrayInputStream(content.getBytes());
        } catch (final IOException e) {
        } finally {
            file.create(is, true, null);
        }
    }

    /**
     * Create the JBI descriptor file and fill data...
     * 
     * @throws CoreException
     */
    private void createJbiComponentDescriptorFile() throws CoreException {
        final IFile jbiFile = this.getJbiIFile();
        InputStream is = this.getClass().getResourceAsStream(
                this.getJbiResource());

        try {
            String content = InputStreamUtils.inputStreamAsString(is);

            // Component name
            content = content.replaceAll(Constants.Templates.NAME, this
                    .getComponentName());

            // Description
            content = content.replaceAll(Constants.Templates.DESCRIPTION, this
                    .getComponentDescription());

            // Component class name - beware, this part depends on if we generate a default component or not.
            String componentClassNameWithPackage;
            if( createComponentFile )
                componentClassNameWithPackage = this.getComponentClassNameWithPackage();
            else {
                // use version 2.0 of the CDK for the moment
                // TODO: centralize everything! Support other versions. 
                if( this.isBindingComponent())
                    componentClassNameWithPackage = BCArtifacts.get( "2.0" ); //$NON-NLS-1$
                else
                    componentClassNameWithPackage = SEArtifacts.get( "2.0" ); //$NON-NLS-1$
            }
            
            if( componentClassNameWithPackage == null )
                componentClassNameWithPackage = "TODO: add package and class name"; //$NON-NLS-1$
            /* update the jbi file */
            content = content.replaceAll( Constants.Templates.COMPONENT, componentClassNameWithPackage );                        

            // Bootstrap class name
            content = content.replaceAll(Constants.Templates.BOOTSTRAP, this
                    .getBootstrapClassNameWithPackage());

            // JBI listener
            content = content.replaceAll(Constants.Templates.JBI_LISTENER, this
                    .getJbiListenerNameWithPackage());

            // External listener
            content = content.replaceAll(Constants.Templates.EXTERNAL_LISTENER,
                    this.getExternalListenerNameWithPackage());

            is = new ByteArrayInputStream(content.getBytes());
        } catch (final IOException e) {
        } finally {
            jbiFile.create(is, true, null);
        }
    }

    /**
     * Create the JBI listener file with the good class name and package.
     * 
     * @throws CoreException
     */
    private void createJbiListenerFile() throws CoreException {
        final Path path = this.getJbiListenerPath();
        final IFile file = this.newProjectHandle.getFile(path
                .addFileExtension("java")); //$NON-NLS-1$
        InputStream is = this.getClass().getResourceAsStream(
                "/" + Constants.Templates.JBI_LISTENER); //$NON-NLS-1$

        try {
            String content = InputStreamUtils.inputStreamAsString(is);
            content = content.replaceAll(Wizards.IN_FILE_JAVA_PACKAGE, this
                    .getListenerPackage());
            content = content.replaceAll(Wizards.IN_FILE_JAVA_CLASSNAME,
                    Constants.Templates.JBI_LISTENER);

            is = new ByteArrayInputStream(content.getBytes());
        } catch (final IOException e) {
        } finally {
            file.create(is, true, null);
        }
    }

    /**
     * 
     * @throws CoreException
     */
    private void createJbiSUDescriptorFile() throws CoreException {
        final IFile jbiFile = this.getJbiIFile();
        InputStream is = this.getClass().getResourceAsStream(
                this.getJbiResource());

        try {
            String content = InputStreamUtils.inputStreamAsString(is);

            // generate Services string
            String services = ""; //$NON-NLS-1$
            // FIXME
            if (suList != null) {
                for (ProvidedSuBean service : suList) {
                    // TODO : build consumed/provided services from list
                    // services += ("EP" + service.getProvidedBean()
                    // .getEndpointName());
                }
            }
            content = content.replaceAll("IN_FILE_SERVICES", services); //$NON-NLS-1$
            
            // specify in the jbi file if the su is a bc - required in the jbi.xml file ?
            if( this.isBindingComponent())
                content = content.replaceAll( "IN_FILE_IS_BINDING_COMPONENT", "true" );
            else
                content = content.replaceAll( "IN_FILE_IS_BINDING_COMPONENT", "false" );

            // TODO !
            String xmlNS = ""; //$NON-NLS-1$
            content = content.replaceAll("IN_FILE_XMLNS", xmlNS); //$NON-NLS-1$

            // build the component dependency
            is = new ByteArrayInputStream(content.getBytes());
        } catch (final IOException e) {
        } finally {
            jbiFile.create(is, true, null);
        }
    }

    /**
     * Create the listeners package directory (src/main/java/GROUPID/listeners)
     */
    private void createListenerPackageDirectories() throws CoreException {
        final String packageAsDir = this.getListenerPackage().replace('.',
                File.separatorChar);
        final String directory = Wizards.JAVA_MAIN_PATH + File.separator
                + packageAsDir;
        BusinessCommon.createDirectories(this.newProjectHandle, directory);
    }

    /**
     * Creates a new project resource with the selected name.
     * <p>
     * In normal usage, this method is invoked after the user has pressed Finish
     * on the wizard; the enablement of the Finish button implies that all
     * controls on the pages currently contain valid values.
     * </p>
     * <p>
     * Note that this wizard caches the new project once it has been
     * successfully created; subsequent invocations of this method will answer
     * the same project resource without attempting to create it again.
     * </p>
     * 
     */
    private void createNewProject(final IProgressMonitor monitor)
            throws CoreException {

        monitor.subTask(Messages.JobWizardProjectCreation_18);

        // get a project descriptor
        URI location = null;
        if (!this.useDefaultUri) {
            location = this.locationUri;
        }

        final IWorkspace workspace = ResourcesPlugin.getWorkspace();
        final IProjectDescription description = workspace
                .newProjectDescription(this.newProjectHandle.getName());
        description.setLocationURI(location);

        // set project nature
        String[] natures = { getNatureId() };

        description.setNatureIds(natures);

        monitor.worked(1);
        this.createProject(description, monitor);
    }

    /**
     * Get the project nature id
     * 
     * @return
     */
    private String getNatureId() {
        return "org.objectweb.petals.tools.plugin.nature.jbi" //$NON-NLS-1$
                + this.projectType.toString().toLowerCase();
    }

    /**
     * Create the POM file
     * 
     * @throws CoreException
     */
    private void createPomFile() throws CoreException {
        final IFile pomFile = this.newProjectHandle.getFile(new Path(
                Wizards.POM_FILENAME));

        // get the POM model
        // TODO : Get the POM model depends on CDK version etc etc etc...
        InputStream pomStream = this.getClass().getResourceAsStream(
                this.getPomResource());

        // Get the maven.petals.plugin version value from Preferences
        final Preferences prefs = Activator.getDefault().getPluginPreferences();
        final String mavenPetalsPluginVersion = prefs
                .getString(ESBDevPrefConstants.MAVEN_PETALS_PLUGIN_VERSION_NAME);

        if ((this.mavenArtifactId != null) && (this.mavenGroupId != null)
                && (this.mavenVersion != null) && (this.mavenPackaging != null)
                && (mavenPetalsPluginVersion != null)) {

            String pomStr = ""; //$NON-NLS-1$
            try {
                pomStr = InputStreamUtils.inputStreamAsString(pomStream);

                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_ARTIFACT_ID,
                        this.mavenArtifactId);
                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_GROUP_ID,
                        this.mavenGroupId);
                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_VERSION,
                        this.mavenVersion);
                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_PACKAGING,
                        this.mavenPackaging);
                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_MPPV,
                        mavenPetalsPluginVersion);

                // set dependencies
                final String dependencies = this.buildDependenciesString();
                pomStr = pomStr.replaceAll(Wizards.IN_FILE_POM_DEPENDENCIES,
                        dependencies);

                if (prefs
                        .getBoolean(ESBDevPrefConstants.OBJECTWEB_MAVEN_REPO_NAME)) {
                    pomStr = pomStr.replaceAll(
                            Wizards.IN_FILE_OBJECTWEB_MAVEN_REPO,
                            Wizards.OBJECTWEB_MAVEN_REPO);
                } else {
                    pomStr = pomStr.replaceAll(
                            Wizards.IN_FILE_OBJECTWEB_MAVEN_REPO, ""); //$NON-NLS-1$
                }

                pomStream = new ByteArrayInputStream(pomStr.getBytes());
            } catch (final IOException e) {
            } finally {
                pomFile.create(pomStream, true, null);

            }
        }
    }

    /**
     * Creates the JBI project.
     * 
     * @param description
     *            Project description
     * @param projectHandle
     *            Project handler
     * @param monitor
     *            Progress Monitor
     * 
     * @throws CoreException
     *             CoreException
     */
    private void createProject(final IProjectDescription description,
            final IProgressMonitor monitor) throws CoreException {

        monitor.subTask(Messages.JobWizardProjectCreation_19);

        this.newProjectHandle.create(description, new SubProgressMonitor(
                monitor, WizardComposites.WORK_TICKS));

        if (monitor.isCanceled()) {
            throw new CoreException(Status.CANCEL_STATUS);
        }

        this.newProjectHandle.open(new SubProgressMonitor(monitor,
                WizardComposites.WORK_TICKS));

        monitor.worked(2);

        if ((this.projectType == ProjectType.SA)
                || (this.projectType == ProjectType.SL)) {
            this.customizeDeployableProject(monitor);
        } else {
            this.customizeComponentProject(monitor);
        }

        // create the POM file of the project
        this.createPomFile();
    }

    /**
     * Creates the eclipse project build path from dependencies. FIXME : PATH
     * MUST BE CREATED ON DEPENDENCIES (DYNAMIC)
     * 
     * @param javaProject
     * @throws JavaModelException
     */
    private void createProjectBuildPath(final IJavaProject javaProject)
            throws JavaModelException {
        final List<IClasspathEntry> classpath = new ArrayList<IClasspathEntry>();

        final IClasspathEntry jreClasspathEntry = JavaCore
                .newContainerEntry(new Path(JarPaths.JRE_CONTAINER));
        classpath.add(jreClasspathEntry);

        // add classpath entries from dependencies
        if (this.dependencies != null) {
            for (final ArtifactInformation info : this.dependencies) {
                if (info.getType().equals("jar")) { //$NON-NLS-1$
                    final Path jar = new Path(JarPaths.buildJarPath(info));
                    final Path src = new Path(JarPaths.buildSourcePath(info));
                    final IClasspathEntry cpEntry = JavaCore.newVariableEntry(
                            jar, src, null);
                    classpath.add(cpEntry);
                }
            }
        }

        final IClasspathEntry jbiClasspathEntry = JavaCore.newVariableEntry(
                new Path(JarPaths.buildJarPath(JarPaths.JBI)), null, null);
        classpath.add(jbiClasspathEntry);

        final IClasspathEntry junitClasspathEntry = JavaCore.newVariableEntry(
                new Path(JarPaths.buildJarPath(JarPaths.JUNIT)), null, null);
        classpath.add(junitClasspathEntry);

        if ((this.cdkArtifact != null) && this.isComponent()) {
            final IClasspathEntry cdkClasspathEntry = JavaCore
                    .newVariableEntry(new Path(JarPaths
                            .buildJarPath(this.cdkArtifact)), null, null);
            classpath.add(cdkClasspathEntry);

            final IClasspathEntry commonClasspathEntry = JavaCore
                    .newVariableEntry(new Path(JarPaths
                            .buildJarPath(JarPaths.PETALS_COMMON)), null, null);
            classpath.add(commonClasspathEntry);
        }

        // set the sources directories (src/main/java, src/main/resources,
        // src/test/java, src/test/resources)
        classpath.add(JavaCore.newSourceEntry(new Path(File.separator
                + this.newProjectHandle.getName() + File.separator
                + Wizards.SRC + File.separator + Wizards.MAIN + File.separator
                + Wizards.JAVA)));
        classpath.add(JavaCore.newSourceEntry(new Path(File.separator
                + this.newProjectHandle.getName() + File.separator
                + Wizards.SRC + File.separator + Wizards.MAIN + File.separator
                + Wizards.RESOURCES)));
        classpath.add(JavaCore.newSourceEntry(new Path(File.separator
                + this.newProjectHandle.getName() + File.separator
                + Wizards.SRC + File.separator + Wizards.TEST + File.separator
                + Wizards.JAVA)));
        classpath.add(JavaCore.newSourceEntry(new Path(File.separator
                + this.newProjectHandle.getName() + File.separator
                + Wizards.SRC + File.separator + Wizards.TEST + File.separator
                + Wizards.RESOURCES)));

        javaProject.setRawClasspath(classpath
                .toArray(new IClasspathEntry[classpath.size()]), null);
    }

    /**
     * Create the main package directory (src/main/java/GROUPID)
     * 
     * @throws CoreException
     */
    private void createRootPackageDirectory() throws CoreException {
        final String packageAsDir = this.getPackageName().replace('.',
                File.separatorChar);
        final String directory = Wizards.JAVA_MAIN_PATH + File.separator
                + packageAsDir;
        BusinessCommon.createDirectories(this.newProjectHandle, directory);
    }

    /**
     * Customizes Binding Component projects. Composes project file system tree.
     * 
     * @param monitor
     *            The progress monitor
     */
    private void customizeComponentProject(final IProgressMonitor monitor)
            throws CoreException {

        monitor.subTask(Messages.JobWizardProjectCreation_22);

        // Sets the java nature project description
        final IProjectDescription description = this.newProjectHandle
                .getDescription();
        description.setNatureIds(new String[] { JavaCore.NATURE_ID, getNatureId() });
        this.newProjectHandle.setDescription(description, null);
        BusinessCommon.createJavaTreeStructure(this.newProjectHandle);

        monitor.worked(3);
        monitor.subTask(Messages.JobWizardProjectCreation_23);

        // create the required directories      
        this.createRootPackageDirectory();
        
        if (this.isComponent()) {
            this.createJbiComponentDescriptorFile();                       
            this.createListenerPackageDirectories();
            
            // create packages with default classes ?
            if( createComponentFile )                 
                this.createComponentFile();                
            
            this.createJbiListenerFile();
            if (this.isBindingComponent()) {
                this.createExternalListenerFile();
            }
        } else {
            // SU creation
            this.createJbiSUDescriptorFile();
        }

        // Create the "Build Path"
        final IJavaProject javaProject = JavaCore.create(this.newProjectHandle);
        this.createProjectBuildPath(javaProject);

        // .class files directory
        if (Activator.getDefault().getPluginPreferences().getBoolean(
                ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            javaProject.setOutputLocation(
                    new Path(File.separator + this.newProjectHandle.getName()
                            + File.separator + Wizards.DIR_TARGET
                            + File.separator + Wizards.DIR_CLASSES), null);
        } else {
            javaProject.setOutputLocation(new Path(File.separator
                    + this.newProjectHandle.getName() + File.separator
                    + Wizards.DIR_BIN), null);
        }

        javaProject.close();
        monitor.worked(4);
    }

    /**
     * Customizes Simple projects. Composes project file system tree.
     * 
     * @param monitor
     *            The progress monitor
     */
    private void customizeDeployableProject(final IProgressMonitor monitor)
            throws CoreException {

        BusinessCommon.createTreeStructure(this.newProjectHandle);

        final IFile jbiFile = this.getJbiIFile();
        InputStream jbiStream = this.getClass().getResourceAsStream(
                this.getJbiResource());

        if ((this.dependencies != null) && (this.dependencies.size() > 0)) {
            String jbiStr = ""; //$NON-NLS-1$
            final StringBuffer suListStr = new StringBuffer(""); //$NON-NLS-1$

            // SU processing
            try {
                jbiStr = InputStreamUtils.inputStreamAsString(jbiStream);

                // set the Meta datas
                jbiStr = jbiStr.replaceAll(Wizards.IN_FILE_JBI_SA_NAME,
                        this.mavenArtifactId);
                // jbiStr = jbiStr.replaceAll(Wizards.IN_FILE_DESCRIPTION,
                // this.description);

                // set the SUs
                for (final ArtifactInformation current : this.dependencies) {
                    // get the component target for the SU from its POM
                    // descriptor
                    final ArtifactInformation componentTarget = PomUtils
                            .getJBIComponentTargetForSU(current.getName());
                    String componentName = Messages.JobWizardProjectCreation_24;
                    if (componentTarget != null) {
                        componentName = componentTarget.getName();
                    }

                    suListStr.append(JbiUtils.getSuElementForSa(current
                            .getName(), current.getName() + "-" //$NON-NLS-1$
                            + current.getVersion() + ".zip", componentName)); //$NON-NLS-1$
                }
                jbiStr = jbiStr.replaceAll(Wizards.IN_FILE_SULIST, suListStr
                        .toString());

                jbiStream = new ByteArrayInputStream(jbiStr.getBytes());

            } catch (final IOException e) {
            } finally {
                jbiFile.create(jbiStream, true, null);
            }
        } else {
            jbiFile.create(jbiStream, true, null);
        }
        monitor.worked(4);
    }

    private String getBootstrapClassNameWithPackage() {
        // TODO : Depends on CDK version...
        return Constants.Templates.DEFAULT_BOOTSTRAP;
    }

    /**
     * 
     * @return
     */
    private String getComponentClassName() {
        String result = this.mavenArtifactId.replaceAll("-", ""); //$NON-NLS-1$ //$NON-NLS-2$
        result = result.substring(0, 1).toUpperCase()
                + result.substring(1, result.length()).toLowerCase();
        return result + Constants.Templates.COMPONENT;
    }

    /**
     * 
     * @return
     */
    private String getComponentClassNameWithPackage() {
        return this.getPackageName() + "." + this.getComponentClassName(); //$NON-NLS-1$
    }

    /**
     * Get the component description. TODO Add composite in new project page...
     * 
     * @return
     */
    private String getComponentDescription() {
        return Messages.JobWizardProjectCreation_30;
    }

    /**
     * 
     * @return
     */
    private Path getComponentMainPath() {
        final String path = Wizards.JAVA_MAIN_PATH + File.separator;
        final String className = this.getComponentClassNameWithPackage()
                .replace('.', File.separatorChar); //$NON-NLS-1$
        return new Path(path + className);
    }

    /**
     * 
     * @return
     */
    private String getComponentName() {        
        return this.mavenArtifactId;
    }

    /**
     * Get the project dependencies.
     * 
     * @return
     */
    private List<ArtifactInformation> getAllDependencies() {
        final List<ArtifactInformation> result = new ArrayList<ArtifactInformation>();

        // get the SU project information
        if (this.dependencies != null) {
            for (final ArtifactInformation su : this.dependencies) {
                result.add(su);
            }
        }

        // get the CDK dependency
        if (this.cdkArtifact != null) {
            result.add(this.cdkArtifact);
        }

        return result;
    }

    private String getExternalListenerNameWithPackage() {
        return this.getListenerPackage() + "." //$NON-NLS-1$
                + Constants.Templates.EXTERNAL_LISTENER;
    }

    private Path getExternalListenerPath() {
        final String path = Wizards.JAVA_MAIN_PATH + File.separator;
        String className = this.getExternalListenerNameWithPackage();
        className = className.replace('.', File.separatorChar); //$NON-NLS-1$
        return new Path(path + className);
    }

    /**
     * @return
     */
    private IFile getJbiIFile() {
        final IFile jbiFile = this.newProjectHandle.getFile(new Path(
                Wizards.SRC + File.separator + Wizards.MAIN + File.separator
                        + Wizards.JBI + File.separator + Wizards.JBI_FILENAME));
        return jbiFile;
    }

    private String getJbiListenerNameWithPackage() {
        return this.getListenerPackage() + "." //$NON-NLS-1$
                + Constants.Templates.JBI_LISTENER;
    }

    private Path getJbiListenerPath() {
        final String path = Wizards.JAVA_MAIN_PATH + File.separator;
        String className = this.getJbiListenerNameWithPackage();
        className = className.replace('.', File.separatorChar); //$NON-NLS-1$
        return new Path(path + className);
    }

    private String getJbiResource() {
        // TODO : Add version
        return "/jbi-" + this.projectType.toString().toLowerCase() + ".xml"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    private String getListenerPackage() {
        return this.getPackageName() + "." //$NON-NLS-1$
                + Constants.Templates.LISTENER_PACKAGE;
    }

    private String getPackageName() {
        return this.mavenGroupId + "." //$NON-NLS-1$
                + this.projectType.toString().toLowerCase();
    }

    private String getPomResource() {
        // TODO : Add version
        return "/pom-" + this.projectType.toString().toLowerCase() + ".xml"; //$NON-NLS-1$ //$NON-NLS-2$
    }

    /**
     * The current project to create is a binding component one
     * 
     * @return
     */
    private boolean isBindingComponent() {
        return this.projectType == ProjectType.BINDING;
    }

    /**
     * The current project is a component one
     * 
     * @return
     */
    private boolean isComponent() {
        return ((this.projectType == ProjectType.BINDING) || (this.projectType == ProjectType.ENGINE));
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    protected IStatus run(final IProgressMonitor monitor) {

        monitor.beginTask(Messages.JobWizardProjectCreation_42,
                JobWizardProjectCreation.TOTAL_WORK_TICKS);

        try {
            monitor.worked(0);
            this.createNewProject(monitor);

            monitor.subTask(Messages.JobWizardProjectCreation_43);

            if (this.specificTypesIndex > -1) {
                this.specificTypesList.get(this.specificTypesIndex)
                        .getSpecificCustomization()
                        .addSpecificRessourcesToProject(this.newProjectHandle,
                                monitor);
            }

        } catch (final CoreException e) {
            return e.getStatus();
        }

        monitor.worked(JobWizardProjectCreation.TOTAL_WORK_TICKS);
        monitor.done();

        return Status.OK_STATUS;
    }

    /**
     * @param cdkArtifact
     *            the cdkArtifact to set
     */
    public void setCdkArtifact(final ArtifactInformation cdkArtifact) {
        this.cdkArtifact = cdkArtifact;
    }

    /**
     * @param locationUri
     *            the locationUri to set
     */
    public void setLocationUri(final URI locationUri) {
        this.locationUri = locationUri;
    }

    public void setMavenArtifactId(final String mavenArtifactId) {
        this.mavenArtifactId = mavenArtifactId;
    }

    public void setMavenGroupId(final String mavenGroupId) {
        this.mavenGroupId = mavenGroupId;
    }

    public void setMavenPackaging(final String mavenPackaging) {
        this.mavenPackaging = mavenPackaging;
    }

    public void setMavenVersion(final String mavenVersion) {
        this.mavenVersion = mavenVersion;
    }

    /**
     * @param newProjectHandle
     *            the newProjectHandle to set
     */
    public void setNewProjectHandle(final IProject newProjectHandle) {
        this.newProjectHandle = newProjectHandle;
    }

    /**
     * @param projectType
     *            the projectType to set
     */
    public void setProjectType(final ProjectType projectType) {
        this.projectType = projectType;
    }

    /**
     * @param specificTypesIndex
     *            the specificTypesIndex to set
     */
    public void setSpecificTypesIndex(final int specificTypesIndex) {
        this.specificTypesIndex = specificTypesIndex;
    }

    /**
     * @param specificTypesList
     *            the specificTypesList to set
     */
    public void setSpecificTypesList(
            final List<WizardCustomizationModel> specificTypesList) {
        this.specificTypesList = specificTypesList;
    }

    /**
     * @param dependencies
     *            the dependencies to set
     */
    public void setDependencies(final List<ArtifactInformation> dependencies) {
        this.dependencies = dependencies;
    }

    /**
     * 
     * @return
     */
    public List<ArtifactInformation> getDependencies() {
        return this.dependencies;
    }

    /**
     * Add a dependency
     * 
     * @param artifact
     */
    public void addDependency(ArtifactInformation artifact) {
        if (artifact == null) {
            return;
        }

        if (this.dependencies == null) {
            this.dependencies = new ArrayList<ArtifactInformation>();
        }

        this.dependencies.add(artifact);
    }

    /**
     * @param useDefaultUri
     *            the useDefaultUri to set
     */
    public void setUseDefaultUri(final boolean useDefaultUri) {
        this.useDefaultUri = useDefaultUri;
    }

    /**
     * @param suList
     *            the suList to set
     */
    public void setSuList(List<ProvidedSuBean> suList) {
        this.suList = suList;
    }

    /**
     * @return the createComponentFile
     */
    public boolean isCreateComponentFile() {
        return createComponentFile;
    }

    /**
     * @param createComponentFile the createComponentFile to set
     */
    public void setCreateComponentFile(boolean createComponentFile) {
        this.createComponentFile = createComponentFile;
    }

}
