/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MavenAction.java,v 1.7 2007/04/13 08:00:51 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.actions;

import java.io.File;
import java.io.FileNotFoundException;
import java.net.URI;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.IJobChangeListener;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.wizards.models.MavenCommandBean;

/**
 * @author LAM
 */
public class MavenAction extends Action {
    
    private static MavenJob job;
    
    private final String maven = Messages.MavenAction_12;
    private final Shell shell;

    private String projectName;
    private String commandArgs;

    /**
     * 
     * Creates a new instance of {@link MavenAction}
     * 
     * @param shell
     */
    public MavenAction(final Shell shell) {
        this.setText(this.maven);
        this.setToolTipText(Messages.MavenAction_13);
        this.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
                Activator.PLUGIN_ID, Messages.MavenAction_15));
        this.shell = shell;
        if (job == null) {
            job = new MavenJob(shell);
        }
    }

    /**
     * @return the projectName
     */
    public String getProjectName() {
        return this.projectName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.Action#run()
     */
    @Override
    public void run() {
        final IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
        try {
            final IProject currentProject = wsRoot.getProject(this.projectName);
            URI projectURI;
            if ((currentProject.getDescription() != null)
                    && (currentProject.getDescription().getLocationURI() != null)) {
                projectURI = currentProject.getDescription().getLocationURI();
            } else {
                projectURI = currentProject.getLocationURI();
            }

            if (projectURI == null) {
                throw new FileNotFoundException();
            }

            final File projectDirectory = new File(projectURI);
            if ((projectDirectory != null) && projectDirectory.isDirectory()) {
                job.getMavenCommandList().add(
                        new MavenCommandBean(commandArgs, projectDirectory));
                
                if (job.getState() != Job.RUNNING) {
                    job.setRule(new JobMavenRule());
                    job.setUser(true);
                    job.schedule();
                }
            }
            
        } catch (final CoreException e) {
            MessageDialog.openError(this.shell, this.maven,
                    Messages.MavenAction_29);
        } catch (final FileNotFoundException e) {
            MessageDialog.openError(this.shell, this.maven,
                    Messages.MavenAction_31);
        }
    }

    /**
     * @param commandArgs
     *            the commandArgs to set in the job
     */
    public void setCommandArgs(final String commandArgs) {
        this.commandArgs = commandArgs;
    }

    /**
     * @param projectName
     *            the projectName to set
     */
    public void setProjectName(final String projectName) {
        this.projectName = projectName;
    }

    /**
     * @return the commandArgs
     */
    public String getCommandArgs() {
        return commandArgs;
    }

}
