/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.actions;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.console.MessageConsoleStream;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.consoles.ConsoleMavenFactory;
import org.objectweb.petals.tools.plugin.development.consoles.MavenOutputConsole;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.StringUtils;
import org.objectweb.petals.tools.plugin.development.wizards.models.MavenCommandBean;

/**
 * @author LA
 *
 */
public class MavenJob extends Job {
    
    private final String maven = Messages.MavenAction_12;
    
    private final Shell shell;
    
    // The process that handles the command
    private Process process;
    
    // The output for the process
    private InputStream out;
    
    private List<MavenCommandBean> mavenCommandList;
    
    private String quotes = ""; //$NON-NLS-1$
    
    /**
     * Default Constructor
     */
    public MavenJob(Shell shell) {
        super(Messages.MavenAction_0);
        this.shell = shell;
        mavenCommandList = new ArrayList<MavenCommandBean>();
        // On Windows system, use ' " ' if the path contains some
        // space-characters
        if (System.getProperty("os.name").toLowerCase().indexOf("windows") > -1) { //$NON-NLS-1$ //$NON-NLS-2$
            quotes = "\""; //$NON-NLS-1$
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    protected IStatus run(final IProgressMonitor monitor) {
        // Get the Maven command from preferences
        final Preferences prefs = Activator.getDefault().getPluginPreferences();
        final String mavenCmd = prefs
                .getString(ESBDevPrefConstants.MAVEN_CMD_NAME);
        
        // Count the number of compilation units
        int totalWorkTasks = mavenCommandList.size();
        monitor.beginTask(Messages.MavenAction_1, totalWorkTasks);

        for (MavenCommandBean current : mavenCommandList) {
            if (StringUtils.isNotEmpty(mavenCmd)) {
                try {                    
                    String fullCommand = quotes + mavenCmd + quotes + " " //$NON-NLS-1$
                        + current.getCommandArgs();
                    // Command options
                    if (MavenVerboseOptionAction.isVerbose()) {
                        fullCommand = fullCommand + Messages.MavenAction_21;
                    }
                    if (MavenSkipTestOptionAction.isSkipped()) {
                        fullCommand = fullCommand + Messages.MavenAction_22;
                    }
                    if (MavenNotRecursiveOptionAction.isRecursive()) {
                        fullCommand = fullCommand + Messages.MavenAction_23;
                    }
                    if (MavenUpdateSnapshotsOptionAction.isUpdateSnapshots()) {
                        fullCommand = fullCommand + Messages.MavenAction_24;
                    }

                    this.process = Runtime.getRuntime().exec(fullCommand, null,
                            current.getDirectory());
                    this.out = this.process.getInputStream();
                    this.process.getOutputStream().close();
                    
                    monitor.setTaskName(Messages.MavenJob_0 + current.getDirectory().getName());
                    monitor.worked(1);

                } catch (final IOException e) {
                    MessageDialog.openError(shell, 
                            maven,
                            Messages.MavenAction_25 + e.getMessage());
                }
            } else {
                MessageDialog.openError(
                        shell,
                        maven,
                        Messages.MavenAction_26
                        + Messages.MavenAction_27
                        + Messages.MavenAction_28);
            }
            
            ConsoleMavenFactory.showConsole();
            final BufferedReader br = new BufferedReader(new InputStreamReader(this.out));
            String s;

            try {
                final MavenOutputConsole console = Activator.getDefault()
                .getConsole();

                while ((s = br.readLine()) != null) {                    
                    final MessageConsoleStream stream = 
                        console.newMessageStream();

                    if (s.contains(Messages.MavenAction_2)) {
                        stream.setColor(MavenOutputConsole.getErrorColor());
                    } else if (s.contains(Messages.MavenAction_3)) {
                        stream.setColor(MavenOutputConsole.getInfoColor());
                    } else if (s.contains(Messages.MavenAction_4)) {
                        stream.setColor(MavenOutputConsole.getDebugColor());
                    } else {
                        stream.setColor(MavenOutputConsole.getStandardColor());
                    }
                    stream.write(s + "\n"); //$NON-NLS-1$

                    if (monitor.isCanceled()) {
                        this.process.destroy();
                        return Status.CANCEL_STATUS;
                    }
                }
            } catch (final IOException e) {
                MessageDialog.openError(shell,
                        maven,
                        Messages.MavenAction_6);
            }
        }

        monitor.worked(mavenCommandList.size());
        monitor.done();
        mavenCommandList.clear();

        return Status.OK_STATUS;
    }

    /**
     * @return the mavenCommandList
     */
    public List<MavenCommandBean> getMavenCommandList() {
        return mavenCommandList;
    }

}
