/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MavenOutputConsole.java,v 1.2 2007/04/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.consoles;

import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsole;
import org.objectweb.petals.tools.plugin.development.Activator;

/**
 * @author LAM
 */
public class MavenOutputConsole extends MessageConsole {

    /**
     * created colors for each line type - must be disposed at shutdown
     */
    private static Color standardColor = 
        new Color(Activator.getStandardDisplay(), 0, 0, 0);

    private static Color debugColor =
        new Color(Activator.getStandardDisplay(), 
                63, 
                127, 
                95);

    private static Color infoColor =
        new Color(Activator.getStandardDisplay(), 0, 0, 128);

    private static Color errorColor = 
        new Color(Activator.getStandardDisplay(), 255, 0, 0);

    /**
     * Used to notify this console of lifecycle methods <code>init()</code>
     * and <code>dispose()</code>.
     */
    public class MyLifecycle implements org.eclipse.ui.console.IConsoleListener {
        public MyLifecycle() {
        }
        
        public void consolesAdded(IConsole[] consoles) {
            for (int i = 0; i < consoles.length; i++) {
                IConsole console = consoles[i];
                if (console == MavenOutputConsole.this) {
                    init();
                }
            }

        }

        public void consolesRemoved(IConsole[] consoles) {
            for (int i = 0; i < consoles.length; i++) {
                IConsole console = consoles[i];
                if (console == MavenOutputConsole.this) {
                    ConsolePlugin.getDefault().getConsoleManager()
                            .removeConsoleListener(this);
                    dispose();
                }
            }
        }
    }

    /**
     * Constructor initializes preferences and colors but doesn't create the
     * console page yet.
     */
    public MavenOutputConsole() {
        super(Messages.getString("MavenOutputConsole_0"), null); //$NON-NLS-1$
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.console.MessageConsole#dispose()
     */
    protected void dispose() {
    }

    /**
     * Clean-up created fonts.
     */
    public void shutdown() {
        // Call super dispose because we want the partitioner to be
        // disconnected.
        super.dispose();
        if (standardColor != null) {
            standardColor.dispose();
        }
        if (debugColor != null) {
            debugColor.dispose();
        }
        if (infoColor != null) {
            infoColor.dispose();
        }
        if (errorColor != null) {
            errorColor.dispose();
        }
    }

    /**
     * @return the debugColor
     */
    public static Color getDebugColor() {
        return debugColor;
    }

    /**
     * @return the errorColor
     */
    public static Color getErrorColor() {
        return errorColor;
    }

    /**
     * @return the infoColor
     */
    public static Color getInfoColor() {
        return infoColor;
    }

    /**
     * @return the standardColor
     */
    public static Color getStandardColor() {
        return standardColor;
    }

}
