/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Constants.java,v 1.4 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.constants;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;

/**
 * General constants class
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public final class Constants {

    /**
     * Relative path in the project to the jbi descriptor file
     */
    public static final String JBI_PATH = "src/main/jbi/jbi.xml"; //$NON-NLS-1$

    /**
     * The path of the project POM file
     */
    public static final String POM_PATH = "pom.xml"; //$NON-NLS-1$

    /**
     * Service Unit Image
     */
    public static final Image SU_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/su.gif").createImage(); //$NON-NLS-1$

    /**
     * No Image
     */
    public static final Image NO_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/no.gif").createImage(); //$NON-NLS-1$
    
    /**
     * Service Assembly Image
     */
    public static final Image SA_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/sa.gif").createImage(); //$NON-NLS-1$

    /**
     * Service Engine Image
     */
    public static final Image SE_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/se.gif").createImage(); //$NON-NLS-1$

    /**
     * Binding Component Image
     */
    public static final Image BC_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/bc.gif").createImage(); //$NON-NLS-1$

    /**
     * Shared Library Image
     */
    public static final Image SL_IMAGE = AbstractUIPlugin
            .imageDescriptorFromPlugin(Activator.PLUGIN_ID,
                    "resources/icons/sl.gif").createImage(); //$NON-NLS-1$

    /**
     * Creates a new instance of {@link Constants}
     * 
     */
    private Constants() {
    }

    public interface Templates {
        public static final String JBI_LISTENER = "JBIListener"; //$NON-NLS-1$

        public static final String EXTERNAL_LISTENER = "ExternalListener"; //$NON-NLS-1$

        public static final String LISTENER_PACKAGE = "listeners"; //$NON-NLS-1$

        public static final String BC_TEMPLATE = "BindingComponent"; //$NON-NLS-1$

        public static final String SE_TEMPLATE = "ServiceEngine"; //$NON-NLS-1$

        public static final String COMPONENT = "Component"; //$NON-NLS-1$

        public static final String BOOTSTRAP = "Bootstrap"; //$NON-NLS-1$

        public static final String NAME = "Name"; //$NON-NLS-1$

        public static final String DESCRIPTION = "Description"; //$NON-NLS-1$

        public static final String DEFAULT_BOOTSTRAP = "org.objectweb.petals.component.framework.Bootstrap"; //$NON-NLS-1$

    }

    public interface PetalsComponentFramework {

        public static ArtifactInformation CDK_12 = new ArtifactInformation(
                "petals-cdk", "org.objectweb.petals", "1.2", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        public static ArtifactInformation CDK_13 = new ArtifactInformation(
                "petals-cdk", "org.objectweb.petals", "1.3", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        public static ArtifactInformation CDK_14 = new ArtifactInformation(
                "petals-cdk", "org.objectweb.petals", "1.4", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        public static ArtifactInformation CDK_141 = new ArtifactInformation(
                "petals-cdk", "org.objectweb.petals", "1.4.1", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        public static ArtifactInformation CDK_20BBETA1 = new ArtifactInformation(
                "petals-cdk", "org.objectweb.petals", "2.0-beta1", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

        public static ArtifactInformation PCF_20 = new ArtifactInformation(
                "petals-component-framework", "org.objectweb.petals", "2.0", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                null);

        public static ArtifactInformation PCF_201 = new ArtifactInformation(
                "petals-component-framework", "org.objectweb.petals", "2.0.1", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
                null);
    }

    public interface ArtifactsType {
        
        public static String JAR = "jar"; //$NON-NLS-1$

        public static String JBI_COMPONENT = "jbi-component"; //$NON-NLS-1$

        public static String JBI_SHARED_LIBRARY = "jbi-shared-library"; //$NON-NLS-1$

        public static String JBI_SERVICE_UNIT = "jbi-service-unit"; //$NON-NLS-1$

        public static String JBI_SERVICE_ASSEMBLY = "jbi-service-assembly"; //$NON-NLS-1$

    }
}