/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JarPaths.java,v 1.1 2007/04/25 14:18:41 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.constants;

import java.io.File;

import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;

/**
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public final class JarPaths {

    public static final String M2_REPO = "M2_REPO"; //$NON-NLS-1$

    /**
     * JRE_CONTAINER = "org.eclipse.jdt.launching.JRE_CONTAINER"
     */
    public static final String JRE_CONTAINER = "org.eclipse.jdt.launching.JRE_CONTAINER"; //$NON-NLS-1$

    public static final ArtifactInformation JBI = new ArtifactInformation(
            "jbi", "javax.jbi", "1.0", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    public static final ArtifactInformation JUNIT = new ArtifactInformation(
            "junit", "junit", "3.8.2", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    public static final ArtifactInformation PETALS_COMMON = new ArtifactInformation(
            "petals-jbi-descriptor", "org.objectweb.petals", "1.0", null); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    /**
     * Build the JAR path for a given artifact
     * 
     * @param artifact
     * @return
     */
    public static final String buildJarPath(ArtifactInformation artifact) {
        StringBuffer sb = new StringBuffer(M2_REPO);
        sb.append(File.separator);
        sb.append(artifact.getGroup().replace('.',File.separatorChar));
        sb.append(File.separator);
        sb.append(artifact.getName());
        sb.append(File.separator);
        sb.append(artifact.getVersion());
        sb.append(File.separator);
        sb.append(artifact.getName());
        sb.append('-');
        sb.append(artifact.getVersion());
        sb.append(".jar"); //$NON-NLS-1$

        return sb.toString();
    }
    
    public static final String buildSourcePath(ArtifactInformation artifact) {
        StringBuffer sb = new StringBuffer(M2_REPO);
        sb.append(File.separator);
        sb.append(artifact.getGroup().replace('.',File.separatorChar));
        sb.append(File.separator);
        sb.append(artifact.getName());
        sb.append(File.separator);
        sb.append(artifact.getVersion());
        sb.append(File.separator);
        sb.append(artifact.getName());
        sb.append('-');
        sb.append(artifact.getVersion());
        sb.append("-sources.jar"); //$NON-NLS-1$
        return sb.toString();
    }
    

    /**
     * 
     * Creates a new instance of {@link JarPaths}
     * 
     */
    private JarPaths() {
    }

}
