/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JbiEditor.java,v 1.8 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor;

import java.io.IOException;
import java.util.ResourceBundle;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.DomUtils;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.InputStreamUtils;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.JbiBeanUtils;
import
    org.objectweb.petals.tools.plugin.development.jbieditor.business.TransformBeanToDom;
import
    org.objectweb.petals.tools.plugin.development.jbieditor.business.TransformDomToBean;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.UpdateJbiBean;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.XmlParsing;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.ClassPathPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.OverviewPage;
import
    org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitConsumesPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitPage;
import
    org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitProvidesPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.SharedLibraryPage;
import org.w3c.dom.Document;

/**
 * Jbi Editor core.<br>
 * In this class are declared the pages for the JBI_EDITOR.
 * 
 * @author LAM
 */
public class JbiEditor extends MultiPageEditorPart implements IResourceChangeListener {

    /**
     * Mode for pages when opened for BC
     */
    public static final int MODE_BC = 0;

    /**
     * Mode for pages when opened for SE
     */
    public static final int MODE_SE = 1;

    /**
     * Mode for pages when opened for SL
     */
    public static final int MODE_SL = 2;

    /**
     * Mode for pages when opened for SA
     */
    public static final int MODE_SA = 3;

    /**
     * Mode for pages when opened for SU
     */
    public static final int MODE_SU = 4;
    
    public static final String JBI_EDITOR = Messages.JbiEditor_0;
    
    private static final int TAB_SOURCE_COMPONENT = 3;
    
    private static String jbiStructureError = Messages.JbiEditor_18;
    
    private static String errorOccuredLoadingPrevious = 
        Messages.JbiEditor_2;
    
    private static String errorLoadingFile = Messages.JbiEditor_3
        + Messages.JbiEditor_4
        + Messages.JbiEditor_5;

    private FormToolkit toolkit;

    private OverviewPage overviewPage;

    private ClassPathPage classPathPage;

    private SharedLibraryPage sharedLibraryPage;

    private ServiceUnitPage serviceUnitPage;

    private ServiceUnitProvidesPage suProvidesPage;

    private ServiceUnitConsumesPage suConsumesPage;

    private XMLEditor editor;

    private Jbi jbiBean;

    private int currentPageIndex;

    // Technical data that specifies sets to false until first pages loading is
    // done
    private boolean lockDirty;

    /**
     * Constructor
     */
    public JbiEditor() {
        super();
        ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
        lockDirty = true;
    }

    /**
     * Creates the source page editor for jbi.xml file
     */
    void createPageSource() {
        try {
            editor = new XMLEditor();
            int index = addPage(editor, getEditorInput());
            setPageText(index, editor.getTitle());
        } catch (PartInitException e) {
            ErrorDialog.openError(getSite().getShell(),
                    Messages.JbiEditor_6, null, e.getStatus());
        }
    }

    /**
     * Creates page overview wich shows details about the component
     */
    void createPageOverview(int mode) {
        overviewPage = new OverviewPage(toolkit, getContainer(), mode);
        int index = addPage(overviewPage.getOverview());
        setPageText(index, Messages.JbiEditor_7);

        SourceChangeListener overviewChangeListener = new SourceChangeListener(
                this, SourceChangeListener.PAGE_OVERVIEW);
        overviewPage.addListeners(jbiBean, overviewChangeListener);
    }

    /**
     * Creates the ClassPath page wich lists the ClassPath and the Bootstrap
     * jars
     */
    void createPageClassPath() {
        classPathPage = new ClassPathPage(toolkit, getContainer());
        int index = addPage(classPathPage.getClassPath());
        setPageText(index, Messages.JbiEditor_8);

        SourceChangeListener classpathChangeListener = new SourceChangeListener(
                this, SourceChangeListener.PAGE_CLASSPATH);
        classPathPage.addListeners(jbiBean, classpathChangeListener);
    }

    /**
     * Creates the SharedLibrary page
     */
    void createPageSharedLibrary(int mode) {
        sharedLibraryPage = new SharedLibraryPage(toolkit, getContainer(), mode);
        int index = addPage(sharedLibraryPage.getSharedLibrary());
        setPageText(index, Messages.JbiEditor_9);

        SourceChangeListener sharedLibraryChangeListener = new SourceChangeListener(
                this, SourceChangeListener.PAGE_SHAREDLIBRARY);
        sharedLibraryPage.addListeners(sharedLibraryChangeListener);
    }

    /**
     * Creates the Service Assembly page
     */
    void createPageServiceUnit() {
        serviceUnitPage = new ServiceUnitPage(toolkit, getContainer());
        int index = addPage(serviceUnitPage.getForm());
        setPageText(index, Messages.JbiEditor_10);

        SourceChangeListener suChangeListener = new SourceChangeListener(this,
                SourceChangeListener.PAGE_SERVICEUNIT);
        serviceUnitPage.addListeners(suChangeListener);
    }

    /**
     * Creates the Provided Service Unit Page
     */
    void createPageServiceUnitProvides() {
        suProvidesPage = new ServiceUnitProvidesPage(toolkit, getContainer());
        int index = addPage(suProvidesPage.getForm());
        setPageText(index, Messages.JbiEditor_11);

        SourceChangeListener suProvidesChangeListener = new SourceChangeListener(
                this, SourceChangeListener.PAGE_SERVICEUNITPROVIDES);
        suProvidesPage.addListeners(suProvidesChangeListener);
    }

    /**
     * Creates the Consumed Service Unit Page
     */
    void createPageServiceUnitConsumes() {
        suConsumesPage = new ServiceUnitConsumesPage(toolkit, getContainer());
        int index = addPage(suConsumesPage.getForm());
        setPageText(index, Messages.JbiEditor_12);

        SourceChangeListener suConsumesChangeListener = new SourceChangeListener(
                this, SourceChangeListener.PAGE_SERVICEUNITCONSUMES);
        suConsumesPage.addListeners(suConsumesChangeListener);
    }

    /**
     * Creates the pages of the jbi Editor multi-page editor
     */
    protected void createPages() {
        toolkit = new FormToolkit(getContainer().getDisplay());
        currentPageIndex = 0;

        try {
            String editorText = 
                InputStreamUtils.inputStreamAsString(
                        ((FileEditorInput) getEditorInput()).getFile().getContents());
            Document jbiDom = XmlParsing.parse(editorText);
            jbiBean = TransformDomToBean.populateJbiBean(jbiDom);
        } catch (CoreException e) {
            MessageDialog.openError(getSite().getShell(),
                    jbiStructureError,
                    errorLoadingFile);
        } catch (IOException e) {
            MessageDialog.openError(getSite().getShell(),
                    jbiStructureError,
                    errorLoadingFile);
        }

        if (jbiBean != null) {
            if (JbiBeanUtils.isSe(jbiBean)) {
                createPageOverview(MODE_SE);
                createPageClassPath();
                createPageSharedLibrary(MODE_SE);
                createPageSource();
            }

            if (JbiBeanUtils.isBc(jbiBean)) {
                createPageOverview(MODE_BC);
                createPageClassPath();
                createPageSharedLibrary(MODE_BC);
                createPageSource();
            }

            if (JbiBeanUtils.isSl(jbiBean)) {
                createPageOverview(MODE_SL);
                createPageSharedLibrary(MODE_SL);
                createPageSource();
            }

            if (JbiBeanUtils.isSa(jbiBean)) {
                createPageOverview(MODE_SA);
                createPageServiceUnit();
                createPageSource();
            }

            if (JbiBeanUtils.isSu(jbiBean)) {
                createPageServiceUnitProvides();
                createPageServiceUnitConsumes();
                createPageSource();
            }
        }
    }

    /**
     * Disposes all
     */
    public void dispose() {
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
        super.dispose();
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
     */
    public void doSave(IProgressMonitor monitor) {
        
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            prepareSaveComponent();
        }
        
        if (JbiBeanUtils.isSl(jbiBean)) {
            prepareSaveSl();
        }
        
        if (JbiBeanUtils.isSa(jbiBean)) {
            prepareSaveSa();
        }
        
        if (JbiBeanUtils.isSu(jbiBean)) {
            prepareSaveSu();
        }

        // Only BC or SE have 4 pages
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            if (currentPageIndex != TAB_SOURCE_COMPONENT) {
                updateSourceTab();
            }
            getEditor(TAB_SOURCE_COMPONENT).doSave(monitor);
        } else {
            if (currentPageIndex != 2) {
                updateSourceTab();
            }
            getEditor(2).doSave(monitor);
        }
    }

    /*
     * (non-Javadoc) Method declared on IEditorPart
     */
    public void gotoMarker(IMarker marker) {
        setActivePage(currentPageIndex);
        IDE.gotoMarker(getEditor(currentPageIndex), marker);
    }

    /**
     * The <code>JbiEditor</code> implementation of this method checks that
     * the input is an instance of <code>IFileEditorInput</code>.
     */
    public void init(IEditorSite site, IEditorInput editorInput)
            throws PartInitException {
        if (!(editorInput instanceof IFileEditorInput)) {
            throw new PartInitException(
                    Messages.JbiEditor_13);
        }
        super.init(site, editorInput);
    }

    /*
     * (non-Javadoc) Method declared on IEditorPart.
     */
    public boolean isSaveAsAllowed() {
        return true;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.MultiPageEditorPart#pageChange(int)
     */
    protected void pageChange(int newPageIndex) {
        super.pageChange(newPageIndex);

        lockDirty = true;

        // 4 pages for the multipage Editor
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            processPageQuitComponent(newPageIndex);
            processPageGoComponent(newPageIndex);
        }

        // 3 Pages for the multipage Editor
        if (JbiBeanUtils.isSl(jbiBean)) {
            processPageQuitSl(newPageIndex);
            processPageGoSl(newPageIndex);
        }

        // 3 Pages for the multipage Editor
        if (JbiBeanUtils.isSa(jbiBean)) {
            processPageQuitSa(newPageIndex);
            processPageGoSa(newPageIndex);
        }

        if (JbiBeanUtils.isSu(jbiBean)) {
            processPageQuitSu(newPageIndex);
            processPageGoSu(newPageIndex);
        }

        // Saves the current page index for the next page change event
        currentPageIndex = newPageIndex;

        if (lockDirty) {
            lockDirty = false;
        }
    }

    /**
     * Closes all project files on project close.
     */
    public void resourceChanged(final IResourceChangeEvent event) {
        if (event.getType() == IResourceChangeEvent.PRE_CLOSE) {
            Display.getDefault().asyncExec(new Runnable() {
                public void run() {
                    IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
                    for (int i = 0; i < pages.length; i++) {
                        if (((FileEditorInput) editor.getEditorInput())
                                .getFile().getProject().equals(
                                        event.getResource())) {
                            IEditorPart editorPart = pages[i].findEditor(editor
                                    .getEditorInput());
                            pages[i].closeEditor(editorPart, true);
                        }
                    }
                }
            });
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.EditorPart#doSaveAs()
     */
    @Override
    public void doSaveAs() {
        // Do nothing, impossible to save jbi.xml to another file name.
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
     */
    @Override
    protected void setInput(IEditorInput input) {
        super.setInput(input);
        if (input instanceof FileEditorInput) {
            setPartName(JBI_EDITOR);
            setTitleToolTip(JBI_EDITOR);
        }
    }

    private void updateSourceTab() {
        Document newSource;
        try {
            newSource = TransformBeanToDom.createDocument(jbiBean);
            String editorText = Jbi.XML_HEADER + "\n" //$NON-NLS-1$
                    + DomUtils.getMyNodeString(newSource);
            editor.getDocumentProvider().getDocument(getEditorInput()).set(
                    editorText);
            ResourceBundle bundle = Activator.getDefault().getResourceBundle();
            TextOperationAction formatAction = new TextOperationAction(bundle,
                    Messages.JbiEditor_15, editor, ISourceViewer.FORMAT);
            formatAction.run();
        } catch (ParserConfigurationException e) {
            MessageDialog
                    .openError(getSite().getShell(), JBI_EDITOR,
                            Messages.JbiEditor_16
                            + Messages.JbiEditor_17);
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void overviewModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
            updateSourceTab();
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void classPathModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, classPathPage);
            updateSourceTab();
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void sharedLibraryModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, sharedLibraryPage);
            updateSourceTab();
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void serviceUnitModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, serviceUnitPage);
            updateSourceTab();
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void serviceUnitProvidesModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suProvidesPage);
            updateSourceTab();
        }
    }

    /**
     * Modify the jbi.xml source code in the editor tab when modification change
     * occurs in a form element of this page
     */
    public void serviceUnitConsumesModified() {
        if (!lockDirty) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suConsumesPage);
            updateSourceTab();
        }
    }
    
    /**
     * Private method for tab change processing :
     * Saves the page information into the bean
     * 
     * @param newPageIndex The index of the selected tab
     */
    private void processPageQuitComponent(int newPageIndex) {
        // First, save the updates from the page to the JBI bean
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 0)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if ((currentPageIndex != newPageIndex) && (currentPageIndex == 1)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, classPathPage);
        } else if ((currentPageIndex != newPageIndex) && (currentPageIndex == 2)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, sharedLibraryPage);
        }
        
        // If we have to parse the new source file
        if ((currentPageIndex != newPageIndex) 
                && (currentPageIndex == TAB_SOURCE_COMPONENT)) {
            String editorText = editor.getDocumentProvider().getDocument(
                    getEditorInput()).get();
            Document jbiDom = XmlParsing.parse(editorText);
            Jbi tempBean = TransformDomToBean.populateJbiBean(jbiDom);

            if (tempBean != null) {
                jbiBean = tempBean;
            } else {
                MessageDialog.openError(getSite().getShell(),
                        jbiStructureError,
                        errorOccuredLoadingPrevious);
            }
        }
    }

    
    /**
     * Private method for tab change processing :
     * Print information from the bean to the selected tab
     * 
     * @param newPageIndex The index of the selected tab to go
     */
    private void processPageGoComponent(int newPageIndex) {
        // Then, change page
        if (newPageIndex == 0) {
            overviewPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 1) {
            classPathPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 2) {
            sharedLibraryPage.updatePageContent(jbiBean);
        } else if (newPageIndex == TAB_SOURCE_COMPONENT) {
            updateSourceTab();
        }
    }
    
    /**
     * Private method for tab change processing :
     * Saves the page information into the bean
     * 
     * @param newPageIndex The index of the selected tab
     */
    private void processPageQuitSl(int newPageIndex) {
        // First, save the updates from the page to the JBI bean
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 0)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if ((currentPageIndex != newPageIndex) && (currentPageIndex == 1)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, sharedLibraryPage);
        }
        
        // If we have to parse the new source file
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 2)) {
            String editorText = editor.getDocumentProvider().getDocument(
                    getEditorInput()).get();
            Document jbiDom = XmlParsing.parse(editorText);
            Jbi tempBean = TransformDomToBean.populateJbiBean(jbiDom);

            if (tempBean != null) {
                jbiBean = tempBean;
            } else {
                MessageDialog.openError(getSite().getShell(),
                        jbiStructureError,
                        errorOccuredLoadingPrevious);
            }
        }
    }

    
    /**
     * Private method for tab change processing :
     * Print information from the bean to the selected tab
     * 
     * @param newPageIndex The index of the selected tab to go
     */
    private void processPageGoSl(int newPageIndex) {
        // Then, change page
        if (newPageIndex == 0) {
            overviewPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 1) {
            sharedLibraryPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 2) {
            updateSourceTab();
        }
    }
    
    /**
     * Private method for tab change processing :
     * Saves the page information into the bean
     * 
     * @param newPageIndex The index of the selected tab
     */
    private void processPageQuitSa(int newPageIndex) {
        // First, save the updates from the page to the JBI bean
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 0)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if ((currentPageIndex != newPageIndex) && (currentPageIndex == 1)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, serviceUnitPage);
        }
        
        // If we have to parse the new source file
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 2)) {
            String editorText = editor.getDocumentProvider().getDocument(
                    getEditorInput()).get();
            Document jbiDom = XmlParsing.parse(editorText);
            Jbi tempBean = TransformDomToBean.populateJbiBean(jbiDom);

            if (tempBean != null) {
                jbiBean = tempBean;
            } else {
                MessageDialog
                        .openError(getSite().getShell(),
                                jbiStructureError,
                                errorOccuredLoadingPrevious);
            }
        }
        // TODO : Connections page
    }

    
    /**
     * Private method for tab change processing :
     * Print information from the bean to the selected tab
     * 
     * @param newPageIndex The index of the selected tab to go
     */
    private void processPageGoSa(int newPageIndex) {
        // Then, change page
        if (newPageIndex == 0) {
            overviewPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 1) {
            serviceUnitPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 2) {
            updateSourceTab();
        }
    }
    
    /**
     * Private method for tab change processing :
     * Saves the page information into the bean
     * 
     * @param newPageIndex The index of the selected tab
     */
    private void processPageQuitSu(int newPageIndex) {
        
        // First, save the updates from the page to the JBI bean
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 0)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suProvidesPage);
        } else if ((currentPageIndex != newPageIndex) && (currentPageIndex == 1)) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suConsumesPage);
        }
        
        // If we have to parse the new source file
        if ((currentPageIndex != newPageIndex) && (currentPageIndex == 2)) {
            String editorText = editor.getDocumentProvider().getDocument(
                    getEditorInput()).get();
            Document jbiDom = XmlParsing.parse(editorText);
            Jbi tempBean = TransformDomToBean.populateJbiBean(jbiDom);

            if (tempBean != null) {
                jbiBean = tempBean;
            } else {
                MessageDialog.openError(getSite().getShell(),
                        jbiStructureError,
                        errorOccuredLoadingPrevious);
            }
        }
    }

    
    /**
     * Private method for tab change processing :
     * Print information from the bean to the selected tab
     * 
     * @param newPageIndex The index of the selected tab to go
     */
    private void processPageGoSu(int newPageIndex) {
        // Then, change page
        if (newPageIndex == 0) {
            suProvidesPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 1) {
            suConsumesPage.updatePageContent(jbiBean);
        } else if (newPageIndex == 2) {
            updateSourceTab();
        }
    }
    
    /**
     * Prepare the jbi.xml file save
     */
    private void prepareSaveComponent() {
        // First, save the updates from the page to the JBI bean
        if (currentPageIndex == 0) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if (currentPageIndex == 1) {
            UpdateJbiBean.updateBeanForPage(jbiBean, classPathPage);
        } else if (currentPageIndex == 2) {
            UpdateJbiBean.updateBeanForPage(jbiBean, sharedLibraryPage);
        }
    }
    
    /**
     * Prepare the jbi.xml file save
     */
    private void prepareSaveSa() {
        // First, save the updates from the page to the JBI bean
        if (currentPageIndex == 0) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if (currentPageIndex == 1) {
            UpdateJbiBean.updateBeanForPage(jbiBean, serviceUnitPage);
        }
        // TODO : Connections page
    }
    
    /**
     * Prepare the jbi.xml file save
     */
    private void prepareSaveSl() {
        // First, save the updates from the page to the JBI bean
        if (currentPageIndex == 0) {
            UpdateJbiBean.updateBeanForPage(jbiBean, overviewPage);
        } else if (currentPageIndex == 1) {
            UpdateJbiBean.updateBeanForPage(jbiBean, sharedLibraryPage);
        }
    }
    
    /**
     * Prepare the jbi.xml file save
     */
    private void prepareSaveSu() {
        // First, JBI bean is updated for the current page
        if (currentPageIndex == 0) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suProvidesPage);
        } else if (currentPageIndex == 1) {
            UpdateJbiBean.updateBeanForPage(jbiBean, suConsumesPage);
        }
    }

}
