/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SourceChangeListener.java,v 1.4 2007/04/16 15:48:45 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor;

import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

/**
 * @author LAM
 */
public class SourceChangeListener implements ModifyListener, SelectionListener {

    public static final int PAGE_OVERVIEW = 0;

    public static final int PAGE_CLASSPATH = 1;

    public static final int PAGE_SHAREDLIBRARY = 2;

    public static final int PAGE_SERVICEUNITCONSUMES = 3;

    public static final int PAGE_SERVICEUNIT = 4;

    public static final int PAGE_SERVICEUNITPROVIDES = 5;

    private JbiEditor jbiEditor;

    // Technical data
    private int pageIndex;

    /**
     * Constructor
     * 
     * @param jbiEditor The JBI Editor
     */
    public SourceChangeListener(JbiEditor jbiEditor, int pageIndex) {
        super();
        this.jbiEditor = jbiEditor;
        this.pageIndex = pageIndex;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
     */
    public void modifyText(ModifyEvent e) {
        notifyModification();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetDefaultSelected(SelectionEvent e) {
        notifyModification();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
     */
    public void widgetSelected(SelectionEvent e) {
        notifyModification();
    }

    private void notifyModification() {        
        if (pageIndex == PAGE_OVERVIEW) {
            jbiEditor.overviewModified();
        } else if (pageIndex == PAGE_CLASSPATH) {
            jbiEditor.classPathModified();
        } else if (pageIndex == PAGE_SHAREDLIBRARY) {
            jbiEditor.sharedLibraryModified();
        } else if (pageIndex == PAGE_SERVICEUNIT) {
            jbiEditor.serviceUnitModified();
        } else if (pageIndex == PAGE_SERVICEUNITCONSUMES) {
            jbiEditor.serviceUnitConsumesModified();
        } else if (pageIndex == PAGE_SERVICEUNITPROVIDES) {
            jbiEditor.serviceUnitProvidesModified();
        }
    }
    
}
