/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLDoubleClickStrategy.java,v 1.3 2007/04/16 07:54:23 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.ITextViewer;

public class XMLDoubleClickStrategy implements ITextDoubleClickStrategy {
    
    private ITextViewer fText;

    public XMLDoubleClickStrategy() {
    }
    
    public void doubleClicked(ITextViewer part) {
        int pos = part.getSelectedRange().x;

        if (pos < 0) {
            return;
        }

        fText = part;

        if (!selectComment(pos)) {
            selectWord(pos);
        }
    }

    protected boolean selectComment(int caretPos) {
        
        IDocument doc = fText.getDocument();
        int startPos;
        int endPos;
        boolean result = false;

        try {
            int pos = caretPos;
            char c = ' ';

            while (pos >= 0) {
                c = doc.getChar(pos);
                if (c == '\\') {
                    pos -= 2;
                    continue;
                }
                if (testBreak(c)) {
                    break;
                }
                --pos;
            }

            if (!(c != '\"')) {
                startPos = pos;
                pos = caretPos;
                int length = doc.getLength();
                c = ' ';

                while (pos < length) {
                    c = doc.getChar(pos);
                    if (testBreak(c)) {
                        break;
                    }
                    ++pos;
                }
                if (c != '\"') {
                    result = false;
                } else {
                    endPos = pos;

                    int offset = startPos + 1;
                    int len = endPos - offset;
                    fText.setSelectedRange(offset, len);
                    result = true;
                }
            }
        } catch (BadLocationException x) {
            result = false;
        }

        return result;
    }
    
    private boolean testBreak(char c) {
        boolean result = false;
        
        if (c == Character.LINE_SEPARATOR || c == '\"') {
            result = true;
        }
        
        return result;
    }

    protected boolean selectWord(int caretPos) {

        IDocument doc = fText.getDocument();
        int startPos;
        int endPos;
        boolean result = false;

        try {

            int pos = caretPos;
            char c;

            while (pos >= 0) {
                c = doc.getChar(pos);
                if (!Character.isJavaIdentifierPart(c)) {
                    break;
                }
                --pos;
            }

            startPos = pos;

            pos = caretPos;
            int length = doc.getLength();

            while (pos < length) {
                c = doc.getChar(pos);
                if (!Character.isJavaIdentifierPart(c)) {
                    break;
                }
                ++pos;
            }

            endPos = pos;
            selectRange(startPos, endPos);
            result = true;

        } catch (BadLocationException x) {
            result = false;
        }

        return result;
    }

    /**
     * @param startPos Start position
     * @param stopPos Stop position
     */
    private void selectRange(int startPos, int stopPos) {
        int offset = startPos + 1;
        int length = stopPos - offset;
        fText.setSelectedRange(offset, length);
    }
}
