/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Component.java,v 1.5 2007/04/16 08:39:55 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.beans;

import java.util.List;

/**
 * Binding for DOM Element tag : jbi &gt; component
 * 
 * @author LAM
 */
public class Component {

    /**
     * Type used for defining a Binding Component project
     */
    public static final String TYPE_BC = "binding-component"; //$NON-NLS-1$

    /**
     * Type used for defining a Service Engine project
     */
    public static final String TYPE_SE = "service-engine"; //$NON-NLS-1$

    /**
     * component-class-loader-delegation attribute in the tag
     */
    private String attribComponentClassLoaderDelegation;

    /**
     * type attribute in the tag
     */
    private String attribType;

    /**
     * bootstrap-class-loader-delegation attribute in the tag
     */
    private String attribBootstrapClassLoaderDelegation;

    /**
     * Reference to the &lt;identification&gt; tag
     */
    private Identification identification;

    /**
     * Reference to the &lt;component-class-name&gt; tag
     */
    private CoComponentClassName componentClassName;

    /**
     * Reference to the bootstrap-class-name tag
     */
    private String bootstrapClassName;

    /**
     * Reference to the &lt;shared-library-list&gt; tag
     */
    private List<CoSlListSl> sharedLibraryList;

    /**
     * List of &lt;component-class-path / path-element&gt; tags
     */
    private List<String> componentClassPaths;

    /**
     * List of &lt;bootstrap-class-path / path-element&gt; tags
     */
    private List<String> bootstrapClassPaths;
    
    /**
     * ESB extensions list
     */
    private StringBuffer other;

    public Component() {
        
    }
    
    /**
     * Getter
     * 
     * @return The component class name
     */
    public CoComponentClassName getComponentClassName() {
        return componentClassName;
    }

    /**
     * Setter
     * 
     * @param componentClassName
     *            The component class name
     */
    public void setComponentClassName(CoComponentClassName componentClassName) {
        this.componentClassName = componentClassName;
    }

    /**
     * Getter
     * 
     * @return The Identification bean
     */
    public Identification getIdentification() {
        return identification;
    }

    /**
     * Setter
     * 
     * @param identification
     *            The Identification bean
     */
    public void setIdentification(Identification identification) {
        this.identification = identification;
    }

    /**
     * Getter
     * 
     * @return The bootstrap class name
     */
    public String getBootstrapClassName() {
        return bootstrapClassName;
    }

    /**
     * Setter
     * 
     * @param bootstrapClassName
     *            The bootstrap class name
     */
    public void setBootstrapClassName(String bootstrapClassName) {
        this.bootstrapClassName = bootstrapClassName;
    }

    /**
     * Getter
     * 
     * @return The shared library list
     */
    public List<CoSlListSl> getSharedLibraryList() {
        return sharedLibraryList;
    }

    /**
     * Getter
     * 
     * @return The component class path list
     */
    public List<String> getComponentClassPaths() {
        return componentClassPaths;
    }

    /**
     * Getter
     * 
     * @return The bootstrap class path list
     */
    public List<String> getBootstrapClassPaths() {
        return bootstrapClassPaths;
    }

    /**
     * Getter
     * 
     * @return The bootstrap-class-loader-delegation attribute
     */
    public String getAttribBootstrapClassLoaderDelegation() {
        return attribBootstrapClassLoaderDelegation;
    }

    /**
     * Setter
     * 
     * @param attribBootstrapClassLoaderDelegation
     *            The bootstrap-class-loader-delegation attribute
     */
    public void setAttribBootstrapClassLoaderDelegation(
            String attribBootstrapClassLoaderDelegation) {
        this.attribBootstrapClassLoaderDelegation = attribBootstrapClassLoaderDelegation;
    }

    /**
     * Getter
     * 
     * @return The component-class-loader-delegation attribute
     */
    public String getAttribComponentClassLoaderDelegation() {
        return attribComponentClassLoaderDelegation;
    }

    /**
     * Setter
     * 
     * @param attribComponentClassLoaderDelegation
     *            The component-class-loader-delegation attribute
     */
    public void setAttribComponentClassLoaderDelegation(
            String attribComponentClassLoaderDelegation) {
        this.attribComponentClassLoaderDelegation = attribComponentClassLoaderDelegation;
    }

    /**
     * Getter
     * 
     * @return The type attribute
     */
    public String getAttribType() {
        return attribType;
    }

    /**
     * Setter
     * 
     * @param attribType
     *            The type attribute
     */
    public void setAttribType(String attribType) {
        this.attribType = attribType;
    }

    /**
     * Setter
     * 
     * @param bootstrapClassPaths
     *            The bootstrap class path list
     */
    public void setBootstrapClassPaths(List<String> bootstrapClassPaths) {
        this.bootstrapClassPaths = bootstrapClassPaths;
    }

    /**
     * Setter
     * 
     * @param componentClassPaths
     *            The component class paths list
     */
    public void setComponentClassPaths(List<String> componentClassPaths) {
        this.componentClassPaths = componentClassPaths;
    }

    /**
     * Setter
     * 
     * @param sharedLibraryList
     *            The shared library list
     */
    public void setSharedLibraryList(
            List<CoSlListSl> sharedLibraryList) {
        this.sharedLibraryList = sharedLibraryList;
    }

    /**
     * @return the other
     */
    public StringBuffer getOther() {
        return other;
    }

    /**
     * @param other the other to set
     */
    public void setOther(StringBuffer other) {
        this.other = other;
    }

}
