/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Jbi.java,v 1.6 2007/04/16 08:39:55 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.beans;

import java.util.ArrayList;
import java.util.List;

/**
 * Root tag for jbi.xml file structure.
 * 
 * @author LAM
 */
public class Jbi {

    /**
     * TYPE_COMPONENT = "component"
     */
    public static final String TYPE_COMPONENT = "component"; //$NON-NLS-1$

    /**
     * TYPE_SL = "shared-library"
     */
    public static final String TYPE_SL = "shared-library"; //$NON-NLS-1$

    /**
     * TYPE_SA = "service-assembly"
     */
    public static final String TYPE_SA = "service-assembly"; //$NON-NLS-1$

    /**
     * TYPE_SERVICES = "services"
     */
    public static final String TYPE_SERVICES = "services"; //$NON-NLS-1$

    /**
     * XML file Header "<?xml version="1.0" encoding="UTF-8"?>"
     */
    public static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"; //$NON-NLS-1$

    private String jbiType;

    private Component component;

    private SharedLibrary sharedLibrary;

    private ServiceAssembly serviceAssembly;

    private Services services;

    private List<Attribute> jbiAttributes;
    
    /**
     * Creates a new instance of {@link Jbi}
     *
     */
    public Jbi() {
    }

    /**
     * Gets attributes for jbi bean tag
     * 
     * @return The jbi tag attributes list
     */
    public List<Attribute> getJbiAttributes() {
        if (jbiAttributes == null) {
            jbiAttributes = new ArrayList<Attribute>();
        }

        return jbiAttributes;
    }

    /**
     * Getter
     * 
     * @return The Jbi component type
     */
    public String getJbiType() {
        return jbiType;
    }

    /**
     * Setter
     * 
     * @param jbiType
     *            The Jbi component type
     */
    public void setJbiType(String jbiType) {
        this.jbiType = jbiType;
    }

    /**
     * Getter
     * 
     * @return The Component bean
     */
    public Component getComponent() {
        return component;
    }

    /**
     * Setter
     * 
     * @param component
     *            The Component bean
     */
    public void setComponent(Component component) {
        this.component = component;
    }

    /**
     * Getter
     * 
     * @return the sharedLibrary The SharedLibrary bean
     */
    public SharedLibrary getSharedLibrary() {
        return sharedLibrary;
    }

    /**
     * Setter
     * 
     * @param sharedLibrary
     *            the SharedLibrary bean to set
     */
    public void setSharedLibrary(SharedLibrary sharedLibrary) {
        this.sharedLibrary = sharedLibrary;
    }

    /**
     * Getter
     * 
     * @return the serviceAssembly The ServiceAssembly bean
     */
    public ServiceAssembly getServiceAssembly() {
        return serviceAssembly;
    }

    /**
     * Setter
     * 
     * @param serviceAssembly
     *            the ServiceAssembly to set
     */
    public void setServiceAssembly(ServiceAssembly serviceAssembly) {
        this.serviceAssembly = serviceAssembly;
    }

    /**
     * Getter
     * 
     * @return the services The Services bean
     */
    public Services getServices() {
        return services;
    }

    /**
     * Setter
     * 
     * @param services
     *            the Services bean to set
     */
    public void setServices(Services services) {
        this.services = services;
    }
}
