/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DomUtils.java,v 1.5 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import org.objectweb.petals.tools.plugin.development.constants.JbiXmlTags;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * DOM class utilities
 * 
 * @author LAM
 */
public final class DomUtils {
    
    private static final String NODE_TEXT = Messages.getString("DomUtils.0"); //$NON-NLS-1$
    
    private static final String NODE_DOCUMENT = Messages.getString("DomUtils_1"); //$NON-NLS-1$
    
    private static final String NODE_COMMENT = Messages.getString("DomUtils_2"); //$NON-NLS-1$
    
    private DomUtils() {
        
    }

    /**
     * Do the conversion from a DOM Node to a String, including tag attributes
     * 
     * @param node
     *            The node to convert
     * 
     * @return The String representing the Node
     */
    public static String getMyNodeString(Node node) {
        StringBuffer nodeString = new StringBuffer();

        if (node.getNodeName() != null) {
            // Tag beginning
            if (testNodeType(node)) {
                nodeString.append(JbiXmlTags.TAG_OPEN_START + node.getNodeName());
                // Attributes for the tag
                NamedNodeMap attrList = node.getAttributes();
                if (attrList != null) {
                    for (int i = 0; i < attrList.getLength(); i++) {
                        nodeString.append(" " + attrList.item(i).getNodeName() //$NON-NLS-1$
                                + "=\"" + attrList.item(i).getNodeValue() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                }
                nodeString.append(JbiXmlTags.TAG_END);
            }
            
            if (isCommentNode(node)) {
                nodeString.append(JbiXmlTags.COMMENT_BEGIN);
            }
            
            // get your node value
            if (node.getNodeValue() != null) {
                nodeString.append(node.getTextContent());
            }

            // do i have a child node to handle?
            if (node.hasChildNodes()) {
                for (int i = 0; i < node.getChildNodes().getLength(); i++) {
                    nodeString.append(getMyNodeString(node.getChildNodes()
                            .item(i)));
                }
            }
            
            // Tag ending
            if (testNodeType(node)) {
                nodeString.append(JbiXmlTags.TAG_CLOSE_START + node.getNodeName() + ">"); //$NON-NLS-1$
            }
            
            if (isCommentNode(node)) {
                nodeString.append(JbiXmlTags.COMMENT_END);
            }
        }

        return nodeString.toString();
    }
    
    /**
     * This method tests if the node is a Text or Document node
     * 
     * @param node The node to test
     * @return true or false
     */
    private static boolean testNodeType(Node node) {
        return (!node.getNodeName().equals(NODE_TEXT))
            && (!node.getNodeName().equals(NODE_DOCUMENT)
            && (!node.getNodeName().equals(NODE_COMMENT)));
    }
    
    /**
     * This method tests if the node is a comment
     * 
     * @param node The node to test
     * @return true or false
     */
    private static boolean isCommentNode(Node node) {
        return node.getNodeName().equals(NODE_COMMENT);
    }
}
