/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JbiBeanUtils.java,v 1.4 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Component;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;

/**
 * Jbi bean utilities
 * 
 * @author LAM
 */
public final class JbiBeanUtils {
    
    private JbiBeanUtils() {
    }

    /**
     * @return Return true if the represented project is a Service Unit, false
     *         if not
     */
    public static boolean isSu(Jbi bean) {
        return bean.getServices() != null;
    }

    /**
     * @return Return true if the represented project is a Shared Library, false
     *         if not
     */
    public static boolean isSl(Jbi bean) {
        return bean.getSharedLibrary() != null;
    }

    /**
     * @return Return true if the represented project is a Service Assembly,
     *         false if not
     */
    public static boolean isSa(Jbi bean) {
        return bean.getServiceAssembly() != null;
    }

    /**
     * @return Return true if the represented project is a Binding Component,
     *         false if not
     */
    public static boolean isBc(Jbi bean) {
        boolean result = false;

        if (bean.getComponent() != null) {
            if (Component.TYPE_BC.equals(bean.getComponent().getAttribType())) {
                result = true;
            }
        }

        return result;
    }

    /**
     * @return Return true if the represented project is a Service Engine, false
     *         if not
     */
    public static boolean isSe(Jbi bean) {
        boolean result = false;

        if (bean.getComponent() != null) {
            if (Component.TYPE_SE.equals(bean.getComponent().getAttribType())) {
                result = true;
            }
        }

        return result;
    }

    /**
     * Getter
     * 
     * @param bean
     *            The jbi bean
     * 
     * @return Returns the Service Unit names list from the jbi SA bean
     */
    public static List<String> getSuList(Jbi bean) {
        List<String> suList = new ArrayList<String>();

        if (JbiBeanUtils.isSa(bean)) {
            for (SaServiceUnit currentSu : bean.getServiceAssembly()
                    .getServiceUnitList()) {
                suList.add(currentSu.getIdentification().getName());
            }
        }

        return suList;
    }
    
    /**
     * Test if the bean is Empty.
     * I.e. if the bean is not any of the components type
     * 
     * @param bean The bean
     * @return True or false
     */
    public static boolean isEmpty(Jbi bean) {
        return (!isBc(bean)
               && !isSa(bean)
               && !isSe(bean)
               && !isSl(bean)
               && !isSu(bean));
    }
}
