/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: TransformBeanToDom.java,v 1.11 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.objectweb.petals.tools.plugin.development.constants.JbiXmlTags;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Attribute;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.CoSlListSl;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.utils.DomUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Tranforms the Jbi bean into a DOM Document Object
 * 
 * @author LAM
 */
public final class TransformBeanToDom {

    /**
     * Creates a new instance of {@link TransformBeanToDom}
     * 
     */
    private TransformBeanToDom() {
    }

    /**
     * Root method for jbi bean transformation
     * 
     * @param jbiBean
     *            The jbi bean to populate
     * 
     * @return The populated Document object
     */
    public static Document createDocument(Jbi jbiBean)
            throws ParserConfigurationException {

        Document document = DomUtils.createEmptyDocument();
        document.setXmlVersion(JbiXmlTags.XML_VERSION);

        Element root = document.createElement(JbiXmlTags.JBI);
        document.appendChild(root);

        for (Attribute current : jbiBean.getJbiAttributes()) {
            root.setAttribute(current.getDescription(), current.getValue());
        }

        // Component bean (BC or SE)
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            populateComponent(jbiBean, document, root);
        }

        // Shared Library bean
        if (JbiBeanUtils.isSl(jbiBean)) {
            populateSl(jbiBean, document, root);
        }

        // Service Assembly bean
        if (JbiBeanUtils.isSa(jbiBean)) {
            populateSa(jbiBean, document, root);
        }

        // Service Unit bean
        if (JbiBeanUtils.isSu(jbiBean)) {
            populateSu(jbiBean, document, root);
        }

        return document;
    }

    /**
     * Populates the service-assembly node
     * 
     * @param serviceAssembly
     *            The node to populate
     * @param list
     *            The SaServiceUnit beans list
     * @param jbiBean
     *            The jbi bean
     * @param document
     *            The document used to create new Element nodes
     */
    public static void populateSaSuList(Element serviceAssembly,
            List<SaServiceUnit> list, Jbi jbiBean, Document document) {
        if ((jbiBean.getServiceAssembly().getServiceUnitList() != null)
                && (jbiBean.getServiceAssembly().getServiceUnitList().size() > 0)) {
            for (SaServiceUnit current : jbiBean.getServiceAssembly()
                    .getServiceUnitList()) {
                Element saSu = document.createElement(JbiXmlTags.SERVICE_UNIT);

                Element saSuIde = document
                        .createElement(JbiXmlTags.IDENTIFICATION);
                Element saSuIdeNam = document.createElement(JbiXmlTags.NAME);
                saSuIdeNam
                        .setTextContent(current.getIdentification().getName());
                saSuIde.appendChild(saSuIdeNam);
                Element saSuIdeDes = document
                        .createElement(JbiXmlTags.DESCRIPTION);
                saSuIdeDes.setTextContent(current.getIdentification()
                        .getDescription());
                saSuIde.appendChild(saSuIdeDes);
                saSu.appendChild(saSuIde);

                Element saSuTar = document.createElement(JbiXmlTags.TARGET);
                Element saSuTarArt = document
                        .createElement(JbiXmlTags.ARTIFACTS_ZIP);
                saSuTarArt
                        .setTextContent(current.getTarget().getArtifactsZip());
                saSuTar.appendChild(saSuTarArt);
                Element suSuTarComNam = document
                        .createElement(JbiXmlTags.COMPONENT_NAME);
                suSuTarComNam.setTextContent(current.getTarget()
                        .getComponentName());
                saSuTar.appendChild(suSuTarComNam);
                saSu.appendChild(saSuTar);

                serviceAssembly.appendChild(saSu);
            }
        }
    }

    /**
     * Populates the document with the JBI bean
     * 
     * @param jbiBean
     *            The JBI bean
     * @param document
     *            The document to populate
     * @param rootElement
     *            The root element of the document
     */
    private static void populateComponent(Jbi jbiBean, Document document,
            Element rootElement) {
        Element componentElement = document.createElement(JbiXmlTags.COMPONENT);
        populateCompAttr(jbiBean, componentElement);
        rootElement.appendChild(componentElement);

        Element identificationElement = document.createElement(JbiXmlTags.IDENTIFICATION);
        componentElement.appendChild(identificationElement);
        Element comIdeNam = document.createElement(JbiXmlTags.NAME);
        comIdeNam.setTextContent(jbiBean.getComponent().getIdentification()
                .getName());
        identificationElement.appendChild(comIdeNam);
        Element comIdeDes = document.createElement(JbiXmlTags.DESCRIPTION);
        comIdeDes.setTextContent(jbiBean.getComponent().getIdentification()
                .getDescription());
        identificationElement.appendChild(comIdeDes);

        // Component class name
        Element componentClassNameElement = createComponentClassNameNode(
                jbiBean, document);
        componentElement.appendChild(componentClassNameElement);

        // Component classpath
        Element componentClassPathElement = createComponentClassPathNode(
                jbiBean, document);
        componentElement.appendChild(componentClassPathElement);

        // bootstrap class name
        Element bsClassNameElement = createBootstrapClassNameNode(jbiBean, document);
        componentElement.appendChild(bsClassNameElement);

        // bootstrap classpath
        Element bsClassPathElement = createBootstrapClassPathNode(jbiBean, document);
        componentElement.appendChild(bsClassPathElement);

        populateCompSl(jbiBean, document, componentElement);
        populateOtherNode(jbiBean, document, componentElement);
    }

    /**
     * @param jbiBean
     * @param document
     * @return
     */
    public static Element createBootstrapClassPathNode(Jbi jbiBean,
            Document document) {
        Element comBooClaPat = document
                .createElement(JbiXmlTags.BOOTSTRAP_CLASS_PATH);
        for (String current : jbiBean.getComponent().getBootstrapClassPaths()) {
            Element comBooClaPatPat = document
                    .createElement(JbiXmlTags.PATH_ELEMENT);
            comBooClaPatPat.setTextContent(current);
            comBooClaPat.appendChild(comBooClaPatPat);
        }
        return comBooClaPat;
    }

    /**
     * @param jbiBean
     * @param document
     * @return
     */
    public static Element createBootstrapClassNameNode(Jbi jbiBean,
            Document document) {
        Element comBooClaNam = document
                .createElement(JbiXmlTags.BOOTSTRAP_CLASS_NAME);
        comBooClaNam.setTextContent(jbiBean.getComponent()
                .getBootstrapClassName());
        return comBooClaNam;
    }

    /**
     * @param jbiBean
     * @param document
     * @return
     */
    public static Element createComponentClassPathNode(Jbi jbiBean,
            Document document) {
        Element componentClassPathElement = document
                .createElement(JbiXmlTags.COMPONENT_CLASS_PATH);
        for (String current : jbiBean.getComponent().getComponentClassPaths()) {
            Element pathElement = document
                    .createElement(JbiXmlTags.PATH_ELEMENT);
            pathElement.setTextContent(current);
            componentClassPathElement.appendChild(pathElement);
        }
        return componentClassPathElement;
    }

    /**
     * @param jbiBean
     * @param document
     * @return
     */
    public static Element createComponentClassNameNode(Jbi jbiBean,
            Document document) {
        Element componentClassNameElement = document
                .createElement(JbiXmlTags.COMPONENT_CLASS_NAME);
        componentClassNameElement.setTextContent(jbiBean.getComponent()
                .getComponentClassName().getValue());
        componentClassNameElement.setAttribute(JbiXmlTags.DESCRIPTION, jbiBean
                .getComponent().getComponentClassName().getDescription());
        return componentClassNameElement;
    }

    /**
     * Imports the Petals extensions node from a String to a node in the
     * document
     * 
     * @param jbiBean
     *            The bean where to find String representation
     * @param document
     *            The document where to import the node
     * @param component
     *            The component Node where to append the node
     */
    private static void populateOtherNode(Jbi jbiBean, Document document,
            Element component) {

        Document otherNodeDom = XmlParsing.parse(jbiBean.getComponent()
                .getOther().toString());

        if (otherNodeDom != null) {
            component.appendChild(document.importNode(otherNodeDom
                    .getDocumentElement(), true));
        }
    }

    /**
     * @param jbiBean
     *            The JBI bean
     * @param document
     *            The Document
     * @param component
     *            The Component Element
     */
    private static void populateCompSl(Jbi jbiBean, Document document,
            Element component) {
        if ((jbiBean.getComponent().getSharedLibraryList() != null)
                && (jbiBean.getComponent().getSharedLibraryList().size() > 0)) {
            for (CoSlListSl current : jbiBean.getComponent()
                    .getSharedLibraryList()) {
                Element currentSl = document
                        .createElement(JbiXmlTags.SHARED_LIBRARY);
                currentSl.setTextContent(current.getValue());

                if (StringUtils.isNotEmpty(current.getVersion())) {
                    currentSl.setAttribute(JbiXmlTags.VERSION, current
                            .getVersion());
                }

                component.appendChild(currentSl);
            }
        }
    }

    /**
     * @param jbiBean
     *            The JBI bean
     * @param component
     *            The Component Element
     */
    private static void populateCompAttr(Jbi jbiBean, Element component) {
        component.setAttribute(JbiXmlTags.TYPE, jbiBean.getComponent()
                .getAttribType());

        if (StringUtils.isNotEmpty(jbiBean.getComponent()
                .getAttribComponentClassLoaderDelegation())) {
            component.setAttribute(
                    JbiXmlTags.COMPONENT_CLASS_LOADER_DELEGATION, jbiBean
                            .getComponent()
                            .getAttribComponentClassLoaderDelegation());
        }

        if (StringUtils.isNotEmpty(jbiBean.getComponent()
                .getAttribBootstrapClassLoaderDelegation())) {
            component.setAttribute(
                    JbiXmlTags.BOOTSTRAP_CLASS_LOADER_DELEGATION, jbiBean
                            .getComponent()
                            .getAttribBootstrapClassLoaderDelegation());
        }
    }

    /**
     * @param jbiBean
     *            The JBI bean
     * @param document
     *            The document to populate
     * @param root
     *            The root element of the document
     */
    private static void populateSl(Jbi jbiBean, Document document, Element root) {
        Element sharedLibrary = document
                .createElement(JbiXmlTags.SHARED_LIBRARY);
        root.appendChild(sharedLibrary);

        if (StringUtils.isNotEmpty(jbiBean.getSharedLibrary()
                .getAttribClassLoaderDelegation())) {
            sharedLibrary
                    .setAttribute(JbiXmlTags.CLASS_LOADER_DELEGATION, jbiBean
                            .getSharedLibrary()
                            .getAttribClassLoaderDelegation());
        }

        if (StringUtils.isNotEmpty(jbiBean.getSharedLibrary()
                .getAttribVersion())) {
            sharedLibrary.setAttribute(JbiXmlTags.VERSION, jbiBean
                    .getSharedLibrary().getAttribVersion());
        }

        Element shaIde = document.createElement(JbiXmlTags.IDENTIFICATION);
        Element shaIdeNam = document.createElement(JbiXmlTags.NAME);
        shaIdeNam.setTextContent(jbiBean.getSharedLibrary().getIdentification()
                .getName());
        shaIde.appendChild(shaIdeNam);
        Element shaIdeDes = document.createElement(JbiXmlTags.DESCRIPTION);
        shaIdeDes.setTextContent(jbiBean.getSharedLibrary().getIdentification()
                .getDescription());
        shaIde.appendChild(shaIdeDes);
        sharedLibrary.appendChild(shaIde);

        Element shaShaLibClaPat = document
                .createElement(JbiXmlTags.SHARED_LIBRARY_CLASS_PATH);
        sharedLibrary.appendChild(shaShaLibClaPat);

        for (String current : jbiBean.getSharedLibrary()
                .getSharedLibraryClassPath()) {
            Element shaShaShaPatEle = document
                    .createElement(JbiXmlTags.PATH_ELEMENT);
            shaShaShaPatEle.setTextContent(current);
            shaShaLibClaPat.appendChild(shaShaShaPatEle);
        }
    }

    /**
     * @param jbiBean
     *            The JBI bean
     * @param document
     *            The document to populate
     * @param root
     *            The root element of the document
     */
    private static void populateSa(Jbi jbiBean, Document document, Element root) {
        Element serviceAssembly = document
                .createElement(JbiXmlTags.SERVICE_ASSEMBLY);
        root.appendChild(serviceAssembly);

        Element saIde = document.createElement(JbiXmlTags.IDENTIFICATION);
        Element saIdeNam = document.createElement(JbiXmlTags.NAME);
        saIdeNam.setTextContent(jbiBean.getServiceAssembly()
                .getIdentification().getName());
        saIde.appendChild(saIdeNam);
        Element saIdeDes = document.createElement(JbiXmlTags.DESCRIPTION);
        saIdeDes.setTextContent(jbiBean.getServiceAssembly()
                .getIdentification().getDescription());
        saIde.appendChild(saIdeDes);
        serviceAssembly.appendChild(saIde);

        populateSaSuList(serviceAssembly, jbiBean.getServiceAssembly()
                .getServiceUnitList(), jbiBean, document);
    }

    /**
     * @param jbiBean
     *            The JBI bean
     * @param document
     *            The document to populate
     * @param rootElement
     *            The root element of the document
     */
    private static void populateSu(Jbi jbiBean, Document document,
            Element rootElement) {
        Element servicesElement = document.createElement(JbiXmlTags.SERVICES);

        if (JbiBeanUtils.isBc(jbiBean)) {
            servicesElement.setAttribute(JbiXmlTags.BINDING_COMPONENT,
                    JbiXmlTags.TRUE);
        } else {
            servicesElement.setAttribute(JbiXmlTags.BINDING_COMPONENT,
                    JbiXmlTags.FALSE);
        }

        if ((jbiBean.getServices().getProvidesList() != null)
                && (jbiBean.getServices().getProvidesList().size() > 0)) {
            for (SeConsumesProvides current : jbiBean.getServices()
                    .getProvidesList()) {
                Element provides = createProvidesNode(document, current);
                servicesElement.appendChild(provides);
            }
        }

        if ((jbiBean.getServices().getConsumesList() != null)
                && (jbiBean.getServices().getConsumesList().size() > 0)) {

            for (SeConsumesProvides current : jbiBean.getServices()
                    .getConsumesList()) {
                Element consumes = createConsumesNode(document, current);
                servicesElement.appendChild(consumes);
            }
        }
        rootElement.appendChild(servicesElement);
    }

    /**
     * @param document
     * @param consumesProvides
     * @return
     */
    public static Element createConsumesNode(Document document,
            SeConsumesProvides consumesProvides) {
        Element consumes = document.createElement(JbiXmlTags.CONSUMES);
        consumes.setAttribute(JbiXmlTags.INTERFACE_NAME, consumesProvides
                .getInterfaceName());

        if (StringUtils.isNotEmpty(consumesProvides.getServiceName())) {
            consumes.setAttribute(JbiXmlTags.SERVICE_NAME, consumesProvides
                    .getServiceName());
        }

        if (StringUtils.isNotEmpty(consumesProvides.getEndpointName())) {
            consumes.setAttribute(JbiXmlTags.ENDPOINT_NAME, consumesProvides
                    .getEndpointName());
        }

        if (StringUtils.isNotEmpty(consumesProvides.getLinkType())) {
            consumes.setAttribute(JbiXmlTags.LINK_TYPE, consumesProvides
                    .getLinkType());
        }

        // extra information (extensions)
        consumes.setTextContent(consumesProvides.getExtensions().toString());
        return consumes;
    }

    /**
     * @param document
     * @param consumesProvides
     * @return
     */
    public static Element createProvidesNode(Document document,
            SeConsumesProvides consumesProvides) {
        Element provides = document.createElement(JbiXmlTags.PROVIDES);
        provides.setAttribute(JbiXmlTags.INTERFACE_NAME, consumesProvides
                .getInterfaceName());
        provides.setAttribute(JbiXmlTags.SERVICE_NAME, consumesProvides
                .getServiceName());
        provides.setAttribute(JbiXmlTags.ENDPOINT_NAME, consumesProvides
                .getEndpointName());
        provides.setTextContent(consumesProvides.getExtensions().toString());
        return provides;
    }

}
