/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: TransformDomToBean.java,v 1.14 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.tools.plugin.development.constants.JbiXmlTags;
import org.objectweb.petals.tools.plugin.development.exceptions.JbiBindingException;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Attribute;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.CoComponentClassName;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.CoSlListSl;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Component;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Identification;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaConnConnConsumer;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaConnConnProvider;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaConnConnection;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaConnections;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaSuTarget;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceAssembly;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Services;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SharedLibrary;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Jbi bean tranformation of the Document object representing jbi.xml
 * 
 * @author LAM
 */
public final class TransformDomToBean {

    /**
     * Creates a new instance of {@link TransformDomToBean}
     *
     */
    private TransformDomToBean() {
    }

    /**
     * Populates the Jbi bean from the Document
     * 
     * @param jbiDom
     *            The jbi.xml Document object representation
     * 
     * @return The populated jbi bean
     */
    public static Jbi populateJbiBean(Document jbiDom) {

        Jbi bean = new Jbi();
        
        if (jbiDom == null) {
            return null;
        }

        try {
            Element root = jbiDom.getDocumentElement();

            Element componentNode = org.objectweb.petals.tools.plugin.development.utils.DomUtils
                    .getChildFromNode(root, JbiXmlTags.COMPONENT);
            Element sLNode = org.objectweb.petals.tools.plugin.development.utils.DomUtils
                    .getChildFromNode(root, JbiXmlTags.SHARED_LIBRARY);
            Element sANode = org.objectweb.petals.tools.plugin.development.utils.DomUtils
                    .getChildFromNode(root, JbiXmlTags.SERVICE_ASSEMBLY);
            Element servicesNode = org.objectweb.petals.tools.plugin.development.utils.DomUtils
                    .getChildFromNode(root, JbiXmlTags.SERVICES);

            // Attributes for the jbi root tag
            NamedNodeMap attrList = root.getAttributes();
            for (int i = 0; i < attrList.getLength(); i++) {
                Attribute newJbiAttr = new Attribute(attrList.item(i)
                        .getNodeName(), attrList.item(i).getNodeValue());
                bean.getJbiAttributes().add(newJbiAttr);
            }

            if (componentNode != null) {
                bean.setJbiType(Jbi.TYPE_COMPONENT);
                bean.setComponent(populateComponent(componentNode));
            }

            if (sLNode != null) {
                bean.setJbiType(Jbi.TYPE_SL);
                bean.setSharedLibrary(populateSharedLibrary(sLNode));
            }

            if (sANode != null) {
                bean.setJbiType(Jbi.TYPE_SA);
                bean.setServiceAssembly(populateServiceAssembly(sANode));
            }

            if (servicesNode != null) {
                bean.setJbiType(Jbi.TYPE_SERVICES);
                bean.setServices(populateServices(servicesNode));
            }
        } catch (JbiBindingException e) {
            // An error has occured, null return is used
            // to avoid later structure problems
            bean = null;
        }

        return bean;
    }

    /**
     * Populates the Component bean from the Element DOM node
     * 
     * @param componentNode
     *            The component DOM node
     * 
     * @return The Component polulated bean
     */
    private static Component populateComponent(Element componentNode)
            throws JbiBindingException {
        Component bean = new Component();

        bean.setAttribType(componentNode.getAttribute(JbiXmlTags.TYPE));
        bean.setAttribBootstrapClassLoaderDelegation(componentNode
                .getAttribute(JbiXmlTags.BOOTSTRAP_CLASS_LOADER_DELEGATION));
        bean.setAttribComponentClassLoaderDelegation(componentNode
                .getAttribute(JbiXmlTags.COMPONENT_CLASS_LOADER_DELEGATION));

        Node identificationNode = componentNode.getElementsByTagName(
                JbiXmlTags.IDENTIFICATION).item(0);
        bean
                .setIdentification(populateIdentification((Element) identificationNode));

        Node ccnNode = componentNode.getElementsByTagName(
                JbiXmlTags.COMPONENT_CLASS_NAME).item(0);
        bean
                .setComponentClassName(populateComponentClassName((Element) ccnNode));

        Node bootstrapClassNameNode = componentNode.getElementsByTagName(
                JbiXmlTags.BOOTSTRAP_CLASS_NAME).item(0);
        bean.setBootstrapClassName(bootstrapClassNameNode.getTextContent());

        Node ccp = componentNode.getElementsByTagName(
                JbiXmlTags.COMPONENT_CLASS_PATH).item(0);
        bean.setComponentClassPaths(populatePathElements((Element) ccp));

        Node bcp = componentNode.getElementsByTagName(
                JbiXmlTags.BOOTSTRAP_CLASS_PATH).item(0);
        bean.setBootstrapClassPaths(populatePathElements((Element) bcp));

        NodeList sll = componentNode
                .getElementsByTagName(JbiXmlTags.SHARED_LIBRARY);
        bean.setSharedLibraryList(populateSharedLibraryElements(sll));

        bean.setOther(populateOtherList(componentNode));

        return bean;
    }

    /**
     * Populates the Identification bean from the Element DOM node
     * 
     * @param identNode
     *            The identification DOM node
     * 
     * @return The Identification populated bean
     */
    private static Identification populateIdentification(Element identNode)
            throws JbiBindingException {
        Identification ident = new Identification();

        Node nameNode = identNode.getElementsByTagName(JbiXmlTags.NAME).item(0);
        ident.setName(nameNode.getTextContent());

        Node descriptionNode = identNode.getElementsByTagName(
                JbiXmlTags.DESCRIPTION).item(0);
        ident.setDescription(descriptionNode.getTextContent());

        return ident;
    }

    /**
     * Populates the CoComponentClassName bean from the Element DOM node
     * 
     * @param ccn
     *            The component-class-name DOM node
     * 
     * @return The CoComponentClassName populated bean
     */
    private static CoComponentClassName populateComponentClassName(Element ccn)
            throws JbiBindingException {
        CoComponentClassName bean = new CoComponentClassName();

        bean.setValue(ccn.getTextContent());
        bean.setDescription(ccn.getAttribute(JbiXmlTags.DESCRIPTION));

        return bean;
    }

    /**
     * Populates a path-element list from the DOM node
     * 
     * @param node
     *            The DOM node
     * 
     * @return The path-element beans list
     */
    private static List<String> populatePathElements(Element node)
            throws JbiBindingException {
        List<String> pathElemList = new ArrayList<String>();

        NodeList pathElemNodeList = node
                .getElementsByTagName(JbiXmlTags.PATH_ELEMENT);

        for (int i = 0; i < pathElemNodeList.getLength(); i++) {
            pathElemList.add(pathElemNodeList.item(i).getTextContent());
        }

        return pathElemList;
    }

    /**
     * Populates a SharedLibrary list from the DOM node
     * 
     * @param nodeList
     *            The shared-library-list DOM node list
     * 
     * @return The CoSlListSl bean list
     */
    private static List<CoSlListSl> populateSharedLibraryElements(
            NodeList nodeList) throws JbiBindingException {
        List<CoSlListSl> sll = new ArrayList<CoSlListSl>();

        if (nodeList != null) {
            for (int i = 0; i < nodeList.getLength(); i++) {
                CoSlListSl newSL = new CoSlListSl();

                Element slElem = (Element) (nodeList.item(i));

                newSL.setValue(slElem.getTextContent());
                newSL.setVersion(slElem.getAttribute(JbiXmlTags.VERSION));

                sll.add(newSL);
            }
        }

        return sll;
    }

    /**
     * Populates the SharedLibrary bean from the Element DOM node
     * 
     * @param slNode
     *            The shared-library DOM node
     * 
     * @return The SharedLibrary populated bean
     */
    private static SharedLibrary populateSharedLibrary(Element slNode)
            throws JbiBindingException {
        SharedLibrary bean = new SharedLibrary();

        bean.setAttribClassLoaderDelegation(slNode
                .getAttribute(JbiXmlTags.CLASS_LOADER_DELEGATION));
        bean.setAttribVersion(slNode.getAttribute(JbiXmlTags.VERSION));

        Node identificationNode = slNode.getElementsByTagName(
                JbiXmlTags.IDENTIFICATION).item(0);
        bean
                .setIdentification(populateIdentification((Element) identificationNode));

        Node slcp = slNode.getElementsByTagName(
                JbiXmlTags.SHARED_LIBRARY_CLASS_PATH).item(0);
        bean.setSharedLibraryClassPath(populatePathElements((Element) slcp));

        return bean;
    }

    /**
     * Populates the ServiceAssembly bean from the Element DOM node
     * 
     * @param saNode
     *            The service-assembly DOM node
     * 
     * @return The ServiceAssembly populated bean
     */
    private static ServiceAssembly populateServiceAssembly(Element saNode)
            throws JbiBindingException {
        ServiceAssembly bean = new ServiceAssembly();

        Node identificationNode = saNode.getElementsByTagName(
                JbiXmlTags.IDENTIFICATION).item(0);
        bean
                .setIdentification(populateIdentification((Element) identificationNode));

        NodeList serviceUnitList = saNode
                .getElementsByTagName(JbiXmlTags.SERVICE_UNIT);
        bean.setServiceUnitList(populateServiceUnitList(serviceUnitList));

        Node connectionNode = saNode.getElementsByTagName(
                JbiXmlTags.CONNECTIONS).item(0);
        bean.setConnections(populateConnections((Element) connectionNode));

        return bean;
    }

    /**
     * Populates a ServiceUnit list from the DOM node
     * 
     * @param nodeList
     *            The service-assembly/service-unit DOM node list
     * 
     * @return The SaServiceUnit populated bean list
     */
    private static List<SaServiceUnit> populateServiceUnitList(NodeList nodeList)
            throws JbiBindingException {
        List<SaServiceUnit> sll = new ArrayList<SaServiceUnit>();

        if (nodeList != null) {
            for (int i = 0; i < nodeList.getLength(); i++) {
                SaServiceUnit newSU = new SaServiceUnit();

                Element currentElement = (Element) nodeList.item(i);

                Node identificationNode = currentElement.getElementsByTagName(
                        JbiXmlTags.IDENTIFICATION).item(0);
                newSU
                        .setIdentification(populateIdentification((Element) identificationNode));

                Node targetNode = currentElement.getElementsByTagName(
                        JbiXmlTags.TARGET).item(0);
                newSU.setTarget(populateTarget((Element) targetNode));

                sll.add(newSU);
            }
        }

        return sll;
    }

    /**
     * Populates the Target from the DOM node
     * 
     * @param node
     *            The target DOM node
     * 
     * @return The SaSuTarget populated bean
     */
    private static SaSuTarget populateTarget(Element node)
            throws JbiBindingException {
        SaSuTarget newTarget = new SaSuTarget();

        newTarget.setArtifactsZip(node.getElementsByTagName(
                JbiXmlTags.ARTIFACTS_ZIP).item(0).getTextContent());
        newTarget.setComponentName(node.getElementsByTagName(
                JbiXmlTags.COMPONENT_NAME).item(0).getTextContent());

        return newTarget;
    }

    /**
     * Populates the Connections from the DOM node
     * 
     * @param node
     *            The connections DOM node
     * 
     * @return The SaConnections populated bean
     */
    private static SaConnections populateConnections(Element node)
            throws JbiBindingException {
        SaConnections connections = new SaConnections();

        if (node != null) {
            NodeList connectionList = node
                    .getElementsByTagName(JbiXmlTags.CONNECTION);
            connections
                    .setConnectionList(populatesConnectionList(connectionList));
        }

        return connections;
    }

    /**
     * Populates the Connection from DOM node
     * 
     * @param nodeList
     *            The service-assembly/connections/connection DOM node list
     * 
     * @return The SaConnConnection populated bean list
     */
    private static List<SaConnConnection> populatesConnectionList(
            NodeList nodeList) throws JbiBindingException {
        List<SaConnConnection> saccl = new ArrayList<SaConnConnection>();

        if (nodeList != null) {
            for (int i = 0; i < nodeList.getLength(); i++) {
                SaConnConnection newConn = new SaConnConnection();

                Node consumerElem = ((Element) (nodeList.item(i)))
                        .getElementsByTagName(JbiXmlTags.CONSUMER).item(0);
                Node providerElem = ((Element) (nodeList.item(i)))
                        .getElementsByTagName(JbiXmlTags.PROVIDER).item(0);

                newConn.setConsumer(populateConsumer((Element) consumerElem));
                newConn.setProvider(populateProvider((Element) providerElem));

                saccl.add(newConn);
            }
        }

        return saccl;
    }

    /**
     * Populates the SaConnConnConsumer from DOM node
     * 
     * @param node
     *            The service-assembly/connections/connection/consumer DOM node
     * 
     * @return The SaConnConnConsumer populated bean
     */
    private static SaConnConnConsumer populateConsumer(Element node)
            throws JbiBindingException {
        SaConnConnConsumer newConsumer = new SaConnConnConsumer();

        newConsumer.setAttribEndpointName(node
                .getAttribute(JbiXmlTags.ENDPOINT_NAME));
        newConsumer.setAttribInterfaceName(node
                .getAttribute(JbiXmlTags.INTERFACE_NAME));
        newConsumer.setAttribServiceName(node
                .getAttribute(JbiXmlTags.SERVICE_NAME));
        newConsumer.setValue(node.getTextContent());

        return newConsumer;
    }

    /**
     * Populates the SaConnConnProvider from DOM node
     * 
     * @param node
     *            The service-assembly/connections/connection/provider DOM node
     * 
     * @return The SaConnConnProvider populated bean
     */
    private static SaConnConnProvider populateProvider(Element node)
            throws JbiBindingException {
        SaConnConnProvider newProvider = new SaConnConnProvider();

        newProvider.setAttribEndpointName(node
                .getAttribute(JbiXmlTags.ENDPOINT_NAME));
        newProvider.setAttribServiceName(node
                .getAttribute(JbiXmlTags.SERVICE_NAME));
        newProvider.setValue(node.getTextContent());

        return newProvider;
    }

    /**
     * Populates the Service from the DOM node
     * 
     * @param node
     *            The services DOM node
     * 
     * @return The Services populated bean
     */
    private static Services populateServices(Element node)
            throws JbiBindingException {
        Services newServices = new Services();

        NodeList providesList = node.getElementsByTagName(JbiXmlTags.PROVIDES);
        newServices.setProvidesList(populateProvidesList(providesList));

        NodeList consumesList = node.getElementsByTagName(JbiXmlTags.CONSUMES);
        newServices.setConsumesList(populateConsumesList(consumesList));

        return newServices;
    }

    /**
     * Populates the ProvidesList from DOM node
     * 
     * @param nodeList
     *            The consumes or provides DOM node list
     * 
     * @return The SeConsumesProvides populated bean list
     */
    private static List<SeConsumesProvides> populateProvidesList(
            NodeList nodeList) throws JbiBindingException {
        List<SeConsumesProvides> sepl = new ArrayList<SeConsumesProvides>();

        if (nodeList != null) {
            for (int i = 0; i < nodeList.getLength(); i++) {
                Element currentElement = (Element) nodeList.item(i);

                SeConsumesProvides newProvides = new SeConsumesProvides();

                newProvides.setEndpointName(currentElement
                        .getAttribute(JbiXmlTags.ENDPOINT_NAME));
                newProvides.setInterfaceName(currentElement
                        .getAttribute(JbiXmlTags.INTERFACE_NAME));
                newProvides.setServiceName(currentElement
                        .getAttribute(JbiXmlTags.SERVICE_NAME));

                newProvides.setExtensions(populateOtherList(currentElement));

                sepl.add(newProvides);
            }
        }

        return sepl;
    }

    /**
     * Populates the Petals extensions into a StringBuffer. Petals extensions
     * are tag elements that are defined as ##other in the jbi.xsd. They are
     * defined by a DOM element and recursively analyzed to be converted into a
     * StringBuffer
     * 
     * @param element
     *            The current DOM element to convert
     * @return The StringBuffer representing the Petals extensions DOM node
     */
    private static StringBuffer populateOtherList(Element element)
            throws JbiBindingException {
        StringBuffer otherList = new StringBuffer();

        NodeList otherNodeList = element
                .getElementsByTagName("extensions:extensions"); //$NON-NLS-1$

        if (otherNodeList != null) {
            for (int i = 0; i < otherNodeList.getLength(); i++) {
                Node currentElement = otherNodeList.item(i);
                String textNode = DomUtils.getMyNodeString(currentElement);
                if (StringUtils.isNotEmpty(textNode)) {
                    otherList.append(textNode);
                }
            }
        }

        return otherList;
    }

    /**
     * Populates the ConsumesList from DOM node
     * 
     * @param nodeList
     *            The provides or comsumes DOM node list
     * 
     * @return The SeConsumesProvides populated bean list
     */
    private static List<SeConsumesProvides> populateConsumesList(
            NodeList nodeList) throws JbiBindingException {
        List<SeConsumesProvides> secl = new ArrayList<SeConsumesProvides>();

        if (nodeList != null) {
            for (int i = 0; i < nodeList.getLength(); i++) {
                Element currentElement = (Element) nodeList.item(i);

                SeConsumesProvides newConsumes = new SeConsumesProvides();

                newConsumes.setEndpointName(currentElement
                        .getAttribute(JbiXmlTags.ENDPOINT_NAME));
                newConsumes.setInterfaceName(currentElement
                        .getAttribute(JbiXmlTags.INTERFACE_NAME));
                newConsumes.setServiceName(currentElement
                        .getAttribute(JbiXmlTags.SERVICE_NAME));
                newConsumes.setLinkType(currentElement
                        .getAttribute(JbiXmlTags.LINK_TYPE));

                newConsumes.setExtensions(populateOtherList(currentElement));

                secl.add(newConsumes);
            }
        }

        return secl;
    }
}
