/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: UpdateJbiBean.java,v 1.10 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.tools.plugin.development.jbieditor.beans.CoSlListSl;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Component;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Identification;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaSuTarget;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceTableLine;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.TableLine;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.ClassPathPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.OverviewPage;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitConsumesPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitPage;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.pages.ServiceUnitProvidesPage;
import org.objectweb.petals.tools.plugin.development.jbieditor.pages.SharedLibraryPage;

/**
 * Updates the Jbi bean when a page change occurs
 * 
 * @author LAM
 */
public final class UpdateJbiBean {
    
    private UpdateJbiBean() {
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean The bean to update
     * @param overviewPage The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean, OverviewPage overviewPage) {
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            jbiBean.getComponent().getIdentification().setName(
                    overviewPage.getName().getText());
            jbiBean.getComponent().getIdentification().setDescription(
                    overviewPage.getDescription().getText());
            jbiBean.getComponent().getComponentClassName().setValue(
                    overviewPage.getCompoClassName().getText());
            jbiBean.getComponent().getComponentClassName().setDescription(
                    overviewPage.getCompoClassDescription().getText());
            jbiBean.getComponent().setBootstrapClassName(
                    overviewPage.getBootstrapClassName().getText());
            jbiBean.getComponent().setAttribComponentClassLoaderDelegation(
                    overviewPage.getComponentClassLoaderDelegation().getText());
            jbiBean.getComponent().setAttribBootstrapClassLoaderDelegation(
                    overviewPage.getBootstrapClassLoaderDelegation().getText());

            if (overviewPage.getTypeBC().getSelection()) {
                jbiBean.getComponent().setAttribType(Component.TYPE_BC);
            } else {
                jbiBean.getComponent().setAttribType(Component.TYPE_SE);
            }
            
            jbiBean.getComponent().setOther(
                    new StringBuffer(overviewPage.getExtensionsTextZone().getText()));
        }
        if (JbiBeanUtils.isSl(jbiBean)) {
            jbiBean.getSharedLibrary().getIdentification().setName(
                    overviewPage.getName().getText());
            jbiBean.getSharedLibrary().getIdentification().setDescription(
                    overviewPage.getDescription().getText());
            jbiBean.getSharedLibrary().setAttribVersion(
                    overviewPage.getVersion().getText());
            jbiBean.getSharedLibrary().setAttribClassLoaderDelegation(
                    overviewPage.getClassLoaderDelegation().getText());
        }
        if (JbiBeanUtils.isSa(jbiBean)) {
            jbiBean.getServiceAssembly().getIdentification().setName(
                    overviewPage.getName().getText());
            jbiBean.getServiceAssembly().getIdentification().setDescription(
                    overviewPage.getDescription().getText());
        }
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean
     *            The bean to update
     * @param classPathPage
     *            The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean,
            ClassPathPage classPathPage) {
        List<String> cpList = new ArrayList<String>();
        List<String> bsList = new ArrayList<String>();

        for (TableLine current : classPathPage.getViewerClassPath()
                .getSimpleTableList().getLines()) {
            cpList.add(current.getName());
        }
        for (TableLine current : classPathPage.getViewerBootstrap()
                .getSimpleTableList().getLines()) {
            bsList.add(current.getName());
        }

        jbiBean.getComponent().setComponentClassPaths(cpList);
        jbiBean.getComponent().setBootstrapClassPaths(bsList);
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean
     *            The bean to update
     * @param sharedLibraryPage
     *            The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean,
            SharedLibraryPage sharedLibraryPage) {
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            List<CoSlListSl> coslList = new ArrayList<CoSlListSl>();
            int i = 0;
            for (TableLine current : sharedLibraryPage.getViewerSharedLibrary()
                    .getSimpleTableList().getLines()) {
                CoSlListSl newSl = new CoSlListSl();
                newSl.setValue(current.getName());
                newSl.setVersion(sharedLibraryPage
                        .getSharedLibraryVersionList().get(i).getVersion());
                coslList.add(newSl);
                i++;
            }

            jbiBean.getComponent().setSharedLibraryList(coslList);
        }

        if (JbiBeanUtils.isSl(jbiBean)) {
            List<String> slslList = new ArrayList<String>();
            for (TableLine current : sharedLibraryPage.getViewerSharedLibrary()
                    .getSimpleTableList().getLines()) {
                slslList.add(current.getName());
            }

            jbiBean.getSharedLibrary().setSharedLibraryClassPath(slslList);
        }
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean
     *            The bean to update
     * @param serviceUnitPage
     *            The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean,
            ServiceUnitPage serviceUnitPage) {
        List<SaServiceUnit> suList = new ArrayList<SaServiceUnit>();

        // new empty SaServiceUnit
        SaServiceUnit newssu;

        int i = 0;
        for (TableLine current : serviceUnitPage.getViewerServiceUnit()
                .getSimpleTableList().getLines()) {
            newssu = new SaServiceUnit();
            Identification newId = new Identification();
            newId.setDescription(serviceUnitPage.getServiceUnitList().get(i)
                    .getIdentification().getDescription());
            newId.setName(current.getName());
            SaSuTarget newTarget = new SaSuTarget();
            newTarget.setArtifactsZip(serviceUnitPage.getServiceUnitList().get(
                    i).getTarget().getArtifactsZip());
            newTarget.setComponentName(serviceUnitPage.getServiceUnitList()
                    .get(i).getTarget().getComponentName());
            newssu.setIdentification(newId);
            newssu.setTarget(newTarget);

            suList.add(newssu);
            i++;
        }

        jbiBean.getServiceAssembly().setServiceUnitList(suList);
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean
     *            The bean to update
     * @param suProvidesPage
     *            The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean,
            ServiceUnitProvidesPage suProvidesPage) {
        List<SeConsumesProvides> seList = new ArrayList<SeConsumesProvides>();

        int i = 0;
        for (ServiceTableLine current : suProvidesPage.getViewer()
                .getServicesTableList().getServices()) {
            SeConsumesProvides newSe = new SeConsumesProvides();
            newSe.setEndpointName(current.getEndpointName());
            newSe.setInterfaceName(current.getInterfaceName());
            newSe.setLinkType(current.getLinkType());
            newSe.setExtensions(suProvidesPage.getSeList().get(i).getExtensions());
            newSe.setServiceName(current.getServiceName());
            seList.add(newSe);
            i++;
        }

        jbiBean.getServices().setProvidesList(seList);
    }

    /**
     * Updates the JBI bean from the page form
     * 
     * @param jbiBean
     *            The bean to update
     * @param suConsumesPage
     *            The page wich contains informations to update
     */
    public static void updateBeanForPage(Jbi jbiBean,
            ServiceUnitConsumesPage suConsumesPage) {
        List<SeConsumesProvides> seList = new ArrayList<SeConsumesProvides>();

        int i = 0;
        for (ServiceTableLine current : suConsumesPage.getViewer()
                .getServicesTableList().getServices()) {
            SeConsumesProvides newSe = new SeConsumesProvides();
            newSe.setEndpointName(current.getEndpointName());
            newSe.setInterfaceName(current.getInterfaceName());
            newSe.setLinkType(current.getLinkType());
            newSe.setExtensions(suConsumesPage.getSeList().get(i).getExtensions());
            newSe.setServiceName(current.getServiceName());
            seList.add(newSe);
            i++;
        }

        jbiBean.getServices().setConsumesList(seList);
    }
}
