/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.maven.PomUtils;
import org.w3c.dom.Document;

/**
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public final class WizardsUtils {

    /**
     * Creates a new instance of {@link WizardsUtils}
     * 
     */
    private WizardsUtils() {
    }

    /**
     * Retrieves the SU list from the projects within the workspace.
     * 
     * @return Su String List
     */
    public static List<String> getSuNamesFromWorkspace() {
        List<String> resultList = new ArrayList<String>();

        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();

        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);

            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);

                    if ((bean != null) && (JbiBeanUtils.isSu(bean))) {
                        resultList.add(currentProject.getName());
                    }
                } catch (CoreException e) {
                }
            }
        }
        return resultList;
    }

    /**
     * Get the components information from the workspace
     * 
     * @return
     */
    public static List<ArtifactInformation> getComponentsFromWorkspace() {
        List<ArtifactInformation> result = new ArrayList<ArtifactInformation>();
        List<String> suNames = getComponentNamesFromWorkspace();
        for (String projectName : suNames) {
            ArtifactInformation info = PomUtils
                    .getProjectInformation(projectName);
            if (info != null) {
                result.add(info);
            }
        }
        return result;
    }

    /**
     * Get the component information from the workspace projects
     * 
     * @param componentName
     * @return null if the component has not been found in the workspace
     */
    public static ArtifactInformation getComponentFromWorkspace(
            String componentName) {
        ArtifactInformation result = null;
        List<ArtifactInformation> list = getComponentsFromWorkspace();

        Iterator<ArtifactInformation> iter = list.iterator();
        while (iter.hasNext() && result == null) {
            ArtifactInformation info = iter.next();
            if (info.getName().equals(componentName)) {
                result = info;
            }
        }
        return result;
    }

    /**
     * 
     * @return
     */
    public static List<ArtifactInformation> getSuFromWorkspace() {
        List<ArtifactInformation> result = new ArrayList<ArtifactInformation>();
        List<String> suNames = getSuNamesFromWorkspace();
        for (String projectName : suNames) {
            ArtifactInformation info = PomUtils
                    .getProjectInformation(projectName);
            if (info != null) {
                result.add(info);
            }
        }
        return result;
    }

    /**
     * Retrieves the Component list from the projects within the workspace
     * 
     * @return Component String List
     */
    public static List<String> getComponentNamesFromWorkspace() {
        List<String> resultList = new ArrayList<String>();

        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();

        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);

                    if ((bean != null)
                            && (JbiBeanUtils.isBc(bean) || JbiBeanUtils
                                    .isSe(bean))) {
                        resultList.add(bean.getComponent().getIdentification()
                                .getName());
                    }
                } catch (CoreException e) {
                }
            }
        }

        return resultList;
    }

}
