/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XmlParsing.java,v 1.4 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.jbieditor.business;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * Xml parsing utilities
 * 
 * @author LAM
 */
public final class XmlParsing {

    /**
     * Creates a new instance of {@link XmlParsing}
     *
     */
    private XmlParsing() {
    }

    /**
     * Converts a String into a DOM Document object
     * 
     * @param xmlFile
     *            String that represents a valid XML stream
     * 
     * @return The corresponding Document
     * 
     * @throws UnsupportedEncodingException
     *             UnsupportedEncodingException
     */
    public static Document parse(String xmlFile) {
        InputStream jbiFileStream;
        Document document = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try {
            jbiFileStream = new ByteArrayInputStream(xmlFile.getBytes());
            builder = factory.newDocumentBuilder();
            document = builder.parse(jbiFileStream);
        } catch (ParserConfigurationException e) {
            document = null;
        } catch (SAXException e) {
            document = null;
        } catch (IOException e) {
            document = null;
        }

        return document;
    }

    /**
     * Converts an InputStream into a DOM Document object
     * 
     * @param xmlStream
     *            The InputStream to convert
     * 
     * @return he corresponding Document
     * 
     * @throws UnsupportedEncodingException
     *             UnsupportedEncodingException
     */
    public static Document parse(InputStream xmlStream) {
        Document document = null;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder;
        try {
            builder = factory.newDocumentBuilder();
            document = builder.parse(xmlStream);
        } catch (ParserConfigurationException e) {
            document = null;
        } catch (SAXException e) {
            document = null;
        } catch (IOException e) {
            document = null;
        }

        return document;
    }
}
