/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLFormattingStrategy.java,v 1.4 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.format;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class XMLFormattingStrategy extends DefaultFormattingStrategy {

    private String initialIndentation;

    private boolean lastTagWasOpening;

    public XMLFormattingStrategy() {
        super();
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.tools.plugin.development.jbieditor.format.DefaultFormattingStrategy#formatterStarts(java.lang.String)
     */
    public void formatterStarts(String initialIndent) {
        this.initialIndentation = initialIndent;
    }

    /* (non-Javadoc)
     * @see org.objectweb.petals.tools.plugin.development.jbieditor.format.DefaultFormattingStrategy#format(java.lang.String, boolean, java.lang.String, int[])
     */
    public String format(String content, boolean isLineStart,
            String indentation, int[] positions) {
        
        String indent = indentation;

        if (isLineStart) {
            indent = initialIndentation;
        }

        content = formatContent(content);

        // if the partition does not contain the start tag then just do
        // indentation
        if (content.indexOf("<") == -1) { //$NON-NLS-1$
            // just check to see whether we need to indent the next tag
            if (content.indexOf("/>") != -1) { //$NON-NLS-1$
                lastTagWasOpening = false;
            } else {
                lastTagWasOpening = true;
            }
            return content;
        }

        // start and end tag
        if (content.indexOf("/>") != -1) { //$NON-NLS-1$

            if (lastTagWasOpening) {
                indent = indent + "\t"; //$NON-NLS-1$
            }
            lastTagWasOpening = false;
            return LINE_SEPARATOR + indent + content;
        }

        // end tag
        if (content.indexOf("</") != -1) { //$NON-NLS-1$
            boolean lastOpening = lastTagWasOpening;
            lastTagWasOpening = false;

            // if the last tag was a closing tag we need to reduce the
            // indentation
            if (!lastOpening) {
                if (indent.length() > 1) {
                    indent = indent.substring(0,
                            indent.length() - 1);
                }
                // add new line after content
                return LINE_SEPARATOR + indent + content;
            } else {
                return content;
            }

        }

        // start tag
        if (content.indexOf("<") != -1) { //$NON-NLS-1$
            // if the last tag was an opening tag we need to reduce the
            // indentation
            if (lastTagWasOpening) {
                indent = indent + "\t"; //$NON-NLS-1$
            }

            lastTagWasOpening = true;

            // add new line after content
            return LINE_SEPARATOR + indent + content;
        }
        return content;
    }

    protected String formatContent(String content) {
        String[] contentParts = content.split("\\s+|\r|\n"); //$NON-NLS-1$
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < contentParts.length; i++) {
            buffer.append(contentParts[i].trim()).append(" "); //$NON-NLS-1$
        }
        buffer.delete(buffer.length() - 1, buffer.length());
        return buffer.toString();
    }

    public void formatterStops() {
    }

}
