/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MarkingErrorHandler.java,v 1.6 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.markers;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITypedRegion;
import org.eclipse.ui.texteditor.MarkerUtilities;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.xml.XMLValidationErrorHandler;
import org.objectweb.petals.tools.plugin.development.jbieditor.xml.XmlValidationError;
import org.xml.sax.SAXParseException;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class MarkingErrorHandler extends XMLValidationErrorHandler {

    public static final String ERROR_MARKER_ID = "org.objectweb.petals.tools.plugin.development.dtderror"; //$NON-NLS-1$

    private IFile file;

    private IDocument document;

    public MarkingErrorHandler(IFile file, IDocument document) {
        super();
        this.file = file;
        this.document = document;
    }

    public void removeExistingMarkers() {
        try {
            file.deleteMarkers(ERROR_MARKER_ID, true, IResource.DEPTH_ZERO);
        } catch (CoreException e1) {
            e1.printStackTrace();
        }
    }

    protected XmlValidationError nextError(SAXParseException e, boolean isFatal) {

        XmlValidationError validationError = super.nextError(e, isFatal);

        Map<String, Object> map = new HashMap<String, Object>();
        int lineNumber = e.getLineNumber();
        int columnNumber = e.getColumnNumber();
        MarkerUtilities.setLineNumber(map, lineNumber);
        MarkerUtilities.setMessage(map, e.getMessage());
        map.put(IMarker.LOCATION, file.getFullPath().toString());

        Integer charStart = getCharStart(lineNumber, columnNumber);
        if (charStart != null) {
            map.put(IMarker.CHAR_START, charStart);
        }

        Integer charEnd = getCharEnd(lineNumber, columnNumber);
        if (charEnd != null) {
            map.put(IMarker.CHAR_END, charEnd);
        }

        map.put(IMarker.SEVERITY, new Integer(IMarker.SEVERITY_ERROR));

        try {
            MarkerUtilities.createMarker(file, map, ERROR_MARKER_ID);
        } catch (CoreException ee) {
            ee.printStackTrace();
        }

        return validationError;
    }

    private Integer getCharEnd(int lineNumber, int columnNumber) {
        try {
            return new Integer(document.getLineOffset(lineNumber - 1)
                    + columnNumber);
        } catch (BadLocationException e) {
            e.printStackTrace();
            return null;
        }
    }

    private Integer getCharStart(int lineNumber, int columnNumber) {
        Integer result = null;
        
        try {
            int lineStartChar = document.getLineOffset(lineNumber - 1);
            Integer charEnd = getCharEnd(lineNumber, columnNumber);
            if (charEnd != null) {
                ITypedRegion typedRegion = document.getPartition(charEnd
                        .intValue() - 2);
                int partitionStartChar = typedRegion.getOffset();
                result = new Integer(partitionStartChar);
            } else {
                result = new Integer(lineStartChar);
            }
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
        
        return result;
    }

}
