/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ClassPathPage.java,v 1.7 2007/04/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.pages;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.objectweb.petals.tools.plugin.development.jbieditor.SourceChangeListener;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.viewers.SimpleTableViewer;

/**
 * ClassPath page
 * 
 * @author LAM
 */
public class ClassPathPage {

    private ScrolledForm form;

    private FormToolkit toolkit;

    private SimpleTableViewer viewerClassPath;

    private SimpleTableViewer viewerBootstrap;

    /**
     * Constructor
     * 
     * @param toolkit
     *            The parent toolkit
     * @param container
     *            The parent container
     */
    public ClassPathPage(FormToolkit toolkit, Composite container) {
        this.toolkit = toolkit;
        form = toolkit.createScrolledForm(container);
        form.setText(Messages.ClassPathPage_0); 
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        form.getBody().setLayout(layout);

        createTableClassPath();
        createTableBootstrap();

        toolkit.paintBordersFor(form.getBody());
    }

    /**
     * Getter
     * 
     * @return the viewerBootstrap
     */
    public SimpleTableViewer getViewerBootstrap() {
        return viewerBootstrap;
    }

    /**
     * Getter
     * 
     * @return the viewerClassPath
     */
    public SimpleTableViewer getViewerClassPath() {
        return viewerClassPath;
    }

    /**
     * Getter
     * 
     * @return The form
     */
    public ScrolledForm getClassPath() {
        return form;
    }

    /**
     * Updates the content of this page from the Jbi bean
     * 
     * @param bean
     *            Jbi bean
     */
    public void updatePageContent(Jbi bean) {
        if (bean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            viewerClassPath.clearTableLines();
            viewerBootstrap.clearTableLines();

            viewerClassPath.setInput(bean.getComponent()
                    .getComponentClassPaths());
            viewerBootstrap.setInput(bean.getComponent()
                    .getBootstrapClassPaths());
        }
    }

    /**
     * Creates the ClassTable section
     */
    private void createTableClassPath() {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        viewerClassPath = new SimpleTableViewer(client, toolkit);

        toolkit.paintBordersFor(client);

        GridData gd = new GridData(GridData.FILL_BOTH);

        section.setText(Messages.ClassPathPage_1); 
        section.setDescription(Messages.ClassPathPage_2); 
        section.setClient(client);
        section.setExpanded(true);
        section.addExpansionListener(new ExpansionAdapter() {
            public void expansionStateChanged(ExpansionEvent e) {
                form.reflow(false);
            }
        });
        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Creates the Bootstrap table section
     */
    private void createTableBootstrap() {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        viewerBootstrap = new SimpleTableViewer(client, toolkit);

        toolkit.paintBordersFor(client);

        GridData gd = new GridData(GridData.FILL_BOTH);
        section.setText(Messages.ClassPathPage_3); 
        section.setDescription(Messages.ClassPathPage_4); 
        section.setClient(client);
        section.setExpanded(true);
        section.addExpansionListener(new ExpansionAdapter() {
            public void expansionStateChanged(ExpansionEvent e) {
                form.reflow(false);
            }
        });
        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Add listeners to forms elements
     * 
     * @param jbiBean
     *            The JBI bean
     * @param changeListener
     *            The listener
     */
    public void addListeners(Jbi jbiBean, SourceChangeListener changeListener) {
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            viewerClassPath.getTable().addSelectionListener(changeListener);
            viewerBootstrap.getTable().addSelectionListener(changeListener);
        }
    }
}
