/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: OverviewPage.java,v 1.10 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.pages;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.objectweb.petals.tools.plugin.development.jbieditor.JbiEditor;
import org.objectweb.petals.tools.plugin.development.jbieditor.SourceChangeListener;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.JbiBeanUtils;

/**
 * @author LAM
 */
public class OverviewPage {
    
    private static final int HEIGHT = 200;

    private static final int NUM_COLS = 3;
    
    private ScrolledForm form;
    private Text name;
    private Text description;
    private Text compoClassName;
    private Text compoClassDescription;
    private Text bootstrapClassName;
    private Button typeBC;
    private Button typeSE;
    private Text classLoaderDelegation;
    private Text version;
    private Text componentClassLoaderDelegation;
    private Text bootstrapClassLoaderDelegation;
    private Text extensionsTextZone;

    /**
     * The creation mode for this page (i.e. component type)
     */
    private int mode;

    /**
     * Constructor
     * 
     * @param toolkit The used toolkit
     * @param container The container
     * @param mode The opening mode for this page (i.e. component type)
     */
    public OverviewPage(FormToolkit toolkit, Composite container, int mode) {
        this.mode = mode;
        form = toolkit.createScrolledForm(container);
        form.setText(Messages.OverviewPage_0);
        TableWrapLayout layout = new TableWrapLayout();
        form.getBody().setLayout(layout);
        layout.numColumns = NUM_COLS;

        Section section = toolkit.createSection(form.getBody(),
                Section.DESCRIPTION | Section.TITLE_BAR | Section.EXPANDED);

        TableWrapData td = new TableWrapData(TableWrapData.FILL_GRAB);
        td.colspan = NUM_COLS;
        section.setLayoutData(td);
        section.setText(Messages.OverviewPage_1);
        if ((mode == JbiEditor.MODE_SE) || (mode == JbiEditor.MODE_BC)) {
            section.setDescription(
                Messages.OverviewPage_2);
        }
        if (mode == JbiEditor.MODE_SL) {
            section.setDescription(
                Messages.OverviewPage_3);
        }
        if (mode == JbiEditor.MODE_SA) {
            section.setDescription(
                Messages.OverviewPage_4);
        }

        Composite sectionClient = toolkit.createComposite(section);
        TableWrapLayout layout2 = new TableWrapLayout();
        layout2.numColumns = NUM_COLS;
        sectionClient.setLayout(layout2);

        if ((mode == JbiEditor.MODE_BC) || (mode == JbiEditor.MODE_SE)) {
            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_5);
            name = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            name.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_7);
            description = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            description.setLayoutData(td);

            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            
            td = new TableWrapData();
            toolkit.createLabel(sectionClient, Messages.OverviewPage_12);
            typeBC = toolkit.createButton(sectionClient, Messages.OverviewPage_13,
                    SWT.RADIO);
            typeSE = toolkit.createButton(sectionClient, Messages.OverviewPage_14,
                    SWT.RADIO);

            if (mode == JbiEditor.MODE_BC) {
                typeBC.setSelection(true);
            } else {
                typeSE.setSelection(true);
            }

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient,
                    Messages.OverviewPage_15);
            componentClassLoaderDelegation = toolkit.createText(sectionClient,
                    ""); //$NON-NLS-1$
            componentClassLoaderDelegation.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient,
                    Messages.OverviewPage_17);
            bootstrapClassLoaderDelegation = toolkit.createText(sectionClient,
                    ""); //$NON-NLS-1$
            bootstrapClassLoaderDelegation.setLayoutData(td);

            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_22);
            compoClassName = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            compoClassName.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_24);
            compoClassDescription = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            compoClassDescription.setLayoutData(td);

            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_29);
            bootstrapClassName = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            bootstrapClassName.setLayoutData(td);
            
            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            td.heightHint = HEIGHT;

            toolkit.createLabel(sectionClient, Messages.OverviewPage_31);
            extensionsTextZone = toolkit.createText(sectionClient, "",  //$NON-NLS-1$
                    SWT.MULTI | SWT.WRAP | SWT.V_SCROLL);
            extensionsTextZone.setLayoutData(td);
        }

        if (mode == JbiEditor.MODE_SL) {
            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_33);
            name = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            name.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_35);
            description = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            description.setLayoutData(td);

            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$
            toolkit.createLabel(sectionClient, ""); //$NON-NLS-1$

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_40);
            classLoaderDelegation = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            classLoaderDelegation.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_42);
            version = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            version.setLayoutData(td);
        }

        if (mode == JbiEditor.MODE_SA) {
            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit.createLabel(sectionClient, Messages.OverviewPage_44);
            name = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            name.setLayoutData(td);

            td = new TableWrapData(TableWrapData.FILL_GRAB);
            td.colspan = 2;
            toolkit
                    .createLabel(sectionClient,
                            Messages.OverviewPage_46);
            description = toolkit.createText(sectionClient, ""); //$NON-NLS-1$
            description.setLayoutData(td);
        }

        toolkit.paintBordersFor(sectionClient);
        section.setClient(sectionClient);
    }

    /**
     * Getter
     * 
     * @return The page form
     */
    public ScrolledForm getOverview() {
        return form;
    }

    /**
     * Updates the page content according to the jbi bean
     * 
     * @param jbiBean
     *            The jbi bean
     */
    public void updatePageContent(Jbi jbiBean) {
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            name.setText(jbiBean.getComponent().getIdentification().getName());
            description.setText(jbiBean.getComponent().getIdentification()
                    .getDescription());
            compoClassName.setText(jbiBean.getComponent()
                    .getComponentClassName().getValue());
            compoClassDescription.setText(jbiBean.getComponent()
                    .getComponentClassName().getDescription());
            bootstrapClassName.setText(jbiBean.getComponent()
                    .getBootstrapClassName());
            componentClassLoaderDelegation.setText(jbiBean.getComponent()
                    .getAttribComponentClassLoaderDelegation());
            bootstrapClassLoaderDelegation.setText(jbiBean.getComponent()
                    .getAttribBootstrapClassLoaderDelegation());
            if (JbiBeanUtils.isBc(jbiBean)) {
                typeBC.setSelection(true);
                typeSE.setSelection(false);
            } else {
                typeBC.setSelection(false);
                typeSE.setSelection(true);
            }
            
            extensionsTextZone.setText(jbiBean.getComponent().getOther().toString());
        }
        if (mode == JbiEditor.MODE_SL) {
            name.setText(jbiBean.getSharedLibrary().getIdentification()
                    .getName());
            description.setText(jbiBean.getSharedLibrary().getIdentification()
                    .getDescription());
            version.setText(jbiBean.getSharedLibrary().getAttribVersion());
            classLoaderDelegation.setText(jbiBean.getSharedLibrary()
                    .getAttribClassLoaderDelegation());
        }
        if (mode == JbiEditor.MODE_SA) {
            name.setText(jbiBean.getServiceAssembly().getIdentification()
                    .getName());
            description.setText(jbiBean.getServiceAssembly()
                    .getIdentification().getDescription());
        }
    }

    /**
     * Getter
     * 
     * @return the typeBC
     */
    public Button getTypeBC() {
        return typeBC;
    }

    /**
     * Getter
     * 
     * @return the typeSE
     */
    public Button getTypeSE() {
        return typeSE;
    }

    /**
     * Getter
     * 
     * @return the bootstrapClassLoaderDelegation
     */
    public Text getBootstrapClassLoaderDelegation() {
        return bootstrapClassLoaderDelegation;
    }

    /**
     * Getter
     * 
     * @return the bootstrapClassName
     */
    public Text getBootstrapClassName() {
        return bootstrapClassName;
    }

    /**
     * Getter
     * 
     * @return the classLoaderDelegation
     */
    public Text getClassLoaderDelegation() {
        return classLoaderDelegation;
    }

    /**
     * Getter
     * 
     * @return the compoClassDescription
     */
    public Text getCompoClassDescription() {
        return compoClassDescription;
    }

    /**
     * Getter
     * 
     * @return the compoClassName
     */
    public Text getCompoClassName() {
        return compoClassName;
    }

    /**
     * Getter
     * 
     * @return the componentClassLoaderDelegation
     */
    public Text getComponentClassLoaderDelegation() {
        return componentClassLoaderDelegation;
    }

    /**
     * Getter
     * 
     * @return the description
     */
    public Text getDescription() {
        return description;
    }

    /**
     * Getter
     * 
     * @return the name
     */
    public Text getName() {
        return name;
    }

    /**
     * Getter
     * 
     * @return the version
     */
    public Text getVersion() {
        return version;
    }

    /**
     * Add listeners to forms elements
     * 
     * @param jbiBean The JBI bean
     * @param changeListener The listener
     */
    public void addListeners(Jbi jbiBean, SourceChangeListener changeListener) {
        if (jbiBean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            name.addModifyListener(changeListener);
            description.addModifyListener(changeListener);
            compoClassName.addModifyListener(changeListener);
            compoClassDescription.addModifyListener(changeListener);
            bootstrapClassName.addModifyListener(changeListener);
            componentClassLoaderDelegation.addModifyListener(changeListener);
            bootstrapClassLoaderDelegation.addModifyListener(changeListener);
            typeBC.addSelectionListener(changeListener);
            typeSE.addSelectionListener(changeListener);
            extensionsTextZone.addModifyListener(changeListener);
        }
        if (mode == JbiEditor.MODE_SL) {
            name.addModifyListener(changeListener);
            description.addModifyListener(changeListener);
            version.addModifyListener(changeListener);
            classLoaderDelegation.addModifyListener(changeListener);
        }
        if (mode == JbiEditor.MODE_SA) {
            name.addModifyListener(changeListener);
            description.addModifyListener(changeListener);
        }
    }

    /**
     * @return the extensionsTextZone
     */
    public Text getExtensionsTextZone() {
        return extensionsTextZone;
    }
}
