/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnitPage.java,v 1.8 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.pages;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.objectweb.petals.tools.plugin.development.jbieditor.SourceChangeListener;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Identification;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaSuTarget;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.WizardsUtils;
import org.objectweb.petals.tools.plugin.development.viewers.SimpleTableViewer;
import org.objectweb.petals.tools.plugin.development.wizards.SelectSuListWizard;

/**
 * The ServiceUnitPage for the JBI Editor
 * 
 * @author LAM
 */
public class ServiceUnitPage {

    private static final String SERVICE_UNIT = Messages.ServiceUnitPage_0;
    private ScrolledForm form;
    private FormToolkit toolkit;
    private SimpleTableViewer viewerServiceUnit;
    private List<SaServiceUnit> serviceUnitList;
    private Text suDescription;
    private Text suArtifact;
    private CCombo suComponentName;
    
    private WizardDialog selectSuListWizard;

    // Technical index for su details modification
    private int suIndex;

    /**
     * Constructor
     * 
     * @param toolkit
     *            The used toolkit
     * @param container
     *            The container
     */
    public ServiceUnitPage(FormToolkit toolkit, Composite container) {
        this.toolkit = toolkit;
        form = toolkit.createScrolledForm(container);
        form.setText(SERVICE_UNIT);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        form.getBody().setLayout(layout);

        createTableSection(SERVICE_UNIT, Messages.ServiceUnitPage_1);
        createDetailSection(Messages.ServiceUnitPage_2, Messages.ServiceUnitPage_3);

        toolkit.paintBordersFor(form.getBody());
    }

    /**
     * Creates the main table section
     * 
     * @param title
     *            Section title
     * @param description
     *            Section description
     */
    private void createTableSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        viewerServiceUnit = new SimpleTableViewer(client, toolkit);

        viewerServiceUnit.removeAddListener();
        
        viewerServiceUnit.getTable().addSelectionListener(
                new SelectionListener() {
                    public void widgetSelected(SelectionEvent e) {
                        int indexSel = viewerServiceUnit.getTable().getSelectionIndex();
                        suDescription.setText(serviceUnitList.get(indexSel)
                                .getIdentification().getDescription());
                        suArtifact.setText(serviceUnitList.get(indexSel)
                                .getTarget().getArtifactsZip());
                        suComponentName.setText(serviceUnitList.get(indexSel)
                                .getTarget().getComponentName());
                        suIndex = indexSel;
                    }

                    public void widgetDefaultSelected(SelectionEvent e) {
                        suDescription.setText(""); //$NON-NLS-1$
                        suArtifact.setText(""); //$NON-NLS-1$
                        suComponentName.setText(""); //$NON-NLS-1$
                    }
                });

        viewerServiceUnit.getAdd().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {                
                SelectSuListWizard selectSuList = new SelectSuListWizard(serviceUnitList);
                selectSuList.setViewerServiceUnit(viewerServiceUnit);

                selectSuListWizard = 
                    new WizardDialog(form.getShell(), selectSuList);

                selectSuListWizard.open();
            }
        });

        viewerServiceUnit.getDelete().addSelectionListener(
                new SelectionAdapter() {
                    @Override
                    public void widgetSelected(SelectionEvent e) {
                        serviceUnitList.remove(suIndex);
                    }
                });

        serviceUnitList = new ArrayList<SaServiceUnit>();
        toolkit.paintBordersFor(client);
        GridData gd = new GridData(GridData.VERTICAL_ALIGN_BEGINNING);
        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);
        section.addExpansionListener(new ExpansionAdapter() {
            public void expansionStateChanged(ExpansionEvent e) {
                form.reflow(false);
            }
        });
        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Creates the Details section that display informations according to the
     * main section selection
     * 
     * @param title
     *            Section title
     * @param description
     *            Section description
     */
    private void createDetailSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        GridData gd = new GridData(GridData.FILL_HORIZONTAL);

        toolkit.createLabel(client, Messages.ServiceUnitPage_7);
        suDescription = toolkit.createText(client, ""); //$NON-NLS-1$
        suDescription.setLayoutData(gd);

        suDescription.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                // Do nothing
            }

            public void keyReleased(KeyEvent e) {
                String str = ((Text) e.getSource()).getText();
                Identification oldIdDesc = serviceUnitList.get(suIndex)
                        .getIdentification();
                oldIdDesc.setDescription(str);
            }
        });

        toolkit.createLabel(client, Messages.ServiceUnitPage_9);
        suArtifact = toolkit.createText(client, ""); //$NON-NLS-1$
        suArtifact.setLayoutData(gd);

        suArtifact.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                // Do nothing
            }

            public void keyReleased(KeyEvent e) {
                String str = ((Text) e.getSource()).getText();
                SaSuTarget oldTargetArt = serviceUnitList.get(suIndex)
                        .getTarget();
                oldTargetArt.setArtifactsZip(str);
            }
        });

        toolkit.createLabel(client, Messages.ServiceUnitPage_11);
        suComponentName = new CCombo(client, SWT.FLAT);
        suComponentName.setLayoutData(gd);
        fillSuComponentNameCombo();

        suComponentName.addSelectionListener(new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
                
            }

            public void widgetSelected(SelectionEvent e) {
                String str = ((CCombo) e.getSource()).getText();
                SaSuTarget oldTargetArt = serviceUnitList.get(suIndex)
                        .getTarget();
                oldTargetArt.setComponentName(str);
            }
            
        });
        
        suComponentName.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                // Do nothing
            }

            public void keyReleased(KeyEvent e) {
                String str = ((CCombo) e.getSource()).getText();
                SaSuTarget oldTargetArt = serviceUnitList.get(suIndex)
                        .getTarget();
                oldTargetArt.setComponentName(str);
            }
        });

        toolkit.paintBordersFor(client);

        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);

        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Getter
     * 
     * @return the form
     */
    public ScrolledForm getForm() {
        return form;
    }

    /**
     * Updates the page content according to the JBI bean
     * 
     * @param bean
     *            The JBI bean
     */
    public void updatePageContent(Jbi bean) {
        viewerServiceUnit.clearTableLines();
        serviceUnitList.clear();
        List<String> suList = new ArrayList<String>();

        for (SaServiceUnit currentSU : bean.getServiceAssembly()
                .getServiceUnitList()) {
            serviceUnitList.add(currentSU);
            suList.add(currentSU.getIdentification().getName());
        }

        viewerServiceUnit.setInput(suList);
    }

    /**
     * Getter
     * 
     * @return the serviceUnitList The SaServiceUnit bean list
     */
    public List<SaServiceUnit> getServiceUnitList() {
        return serviceUnitList;
    }

    /**
     * Getter
     * 
     * @return the viewerServiceUnit
     */
    public SimpleTableViewer getViewerServiceUnit() {
        return viewerServiceUnit;
    }

    /**
     * Add listeners to forms elements
     * 
     * @param changeListener The listener
     */
    public void addListeners(SourceChangeListener changeListener) {
        viewerServiceUnit.getTable().addSelectionListener(changeListener);
        suDescription.addModifyListener(changeListener);
        suArtifact.addModifyListener(changeListener);
        suComponentName.addModifyListener(changeListener);
    }
    
    private void fillSuComponentNameCombo() {
        for (String current : WizardsUtils.getComponentNamesFromWorkspace()) {
            suComponentName.add(current);
        }
    }

}
