/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceUnitProvidesPage.java,v 1.9 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.pages;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.objectweb.petals.tools.plugin.development.jbieditor.SourceChangeListener;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.viewers.ServicesTableViewer;

/**
 * The ServiceUnitProvidesPage for the JBI Editor
 * 
 * @author LAM
 */
public class ServiceUnitProvidesPage {

    private static final int HEIGHT = 200;
    private static final int WIDTH = 100;
    
    private ScrolledForm form;
    private FormToolkit toolkit;
    private Text freeTextZone;
    private ServicesTableViewer viewer;
    private List<SeConsumesProvides> seList;

    // Technical index for other extensions modification
    private int otherIndex;

    /**
     * Constructor
     * 
     * @param toolkit
     *            The used toolkit
     * @param container
     *            The container
     */
    public ServiceUnitProvidesPage(FormToolkit toolkit, Composite container) {
        this.toolkit = toolkit;
        form = toolkit.createScrolledForm(container);
        form.setText(Messages.ServiceUnitProvidesPage_0);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        form.getBody().setLayout(layout);

        createTableSection(Messages.ServiceUnitProvidesPage_1, Messages.ServiceUnitProvidesPage_2);
        createDetailSection(Messages.ServiceUnitProvidesPage_3,
                Messages.ServiceUnitProvidesPage_4);

        toolkit.paintBordersFor(form.getBody());
    }

    /**
     * Creates the main table section
     * 
     * @param title
     *            Section title
     * @param description
     *            Section description
     */
    private void createTableSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        viewer = new ServicesTableViewer(client, toolkit,
                ServicesTableViewer.MODE_PROVIDES);

        viewer.getTable().addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                int indexSel = viewer.getTable().getSelectionIndex();
                freeTextZone
                        .setText(seList.get(indexSel).getExtensions().toString());
                otherIndex = indexSel;
            }

            public void widgetDefaultSelected(SelectionEvent e) {
                freeTextZone.setText(""); //$NON-NLS-1$
                otherIndex = 0;
            }
        });

        viewer.getAdd().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                SeConsumesProvides newSe = new SeConsumesProvides();
                newSe.setEndpointName(""); //$NON-NLS-1$
                newSe.setInterfaceName(""); //$NON-NLS-1$
                newSe.setLinkType(""); //$NON-NLS-1$
                newSe.setExtensions(new StringBuffer());
                newSe.setServiceName(""); //$NON-NLS-1$
                seList.add(newSe);
            }
        });

        viewer.getDelete().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                seList.remove(otherIndex);
            }
        });

        GridData gd = new GridData(GridData.FILL_BOTH);
        gd.verticalSpan = 2;
        gd.heightHint = HEIGHT;
        gd.widthHint = WIDTH;

        toolkit.paintBordersFor(client);

        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);
        section.addExpansionListener(new ExpansionAdapter() {
            public void expansionStateChanged(ExpansionEvent e) {
                form.reflow(false);
            }
        });
        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Creates the Details section that display informations according to the
     * main section selection
     * 
     * @param title
     *            Section title
     * @param description
     *            Section description
     */
    private void createDetailSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        client.setLayout(layout);

        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.heightHint = HEIGHT;

        toolkit.createLabel(client, Messages.ServiceUnitProvidesPage_10);
        freeTextZone = toolkit.createText(client, "", SWT.MULTI | SWT.WRAP //$NON-NLS-1$
                | SWT.V_SCROLL);
        freeTextZone.setLayoutData(gd);

        freeTextZone.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                // Do nothing
            }

            public void keyReleased(KeyEvent e) {
                String newOtherExtensions = ((Text) e.getSource()).getText();
                SeConsumesProvides oldOtherExtensions = seList.get(otherIndex);
                oldOtherExtensions
                        .setExtensions(new StringBuffer(newOtherExtensions));
            }
        });

        toolkit.paintBordersFor(client);

        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);

        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Getter
     * 
     * @return the form
     */
    public ScrolledForm getForm() {
        return form;
    }

    /**
     * Updates the page content according to the JBI bean
     * 
     * @param bean
     *            The JBI bean
     */
    public void updatePageContent(Jbi bean) {
        seList = bean.getServices().getProvidesList();
        viewer.clearServicesTable();
        viewer.setInput(seList);
    }

    /**
     * Getter
     * 
     * @return the viewer
     */
    public ServicesTableViewer getViewer() {
        return viewer;
    }

    /**
     * Getter
     * 
     * @return the SeConsumesProvides bean list
     */
    public List<SeConsumesProvides> getSeList() {
        return seList;
    }

    /**
     * Add listeners to forms elements
     * 
     * @param changeListener
     *            The listener
     */
    public void addListeners(SourceChangeListener changeListener) {
        viewer.getTable().addSelectionListener(changeListener);
        freeTextZone.addModifyListener(changeListener);
    }
}
