/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SharedLibraryPage.java,v 1.10 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.pages;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.objectweb.petals.tools.plugin.development.jbieditor.JbiEditor;
import org.objectweb.petals.tools.plugin.development.jbieditor.SourceChangeListener;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.CoSlListSl;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Version;
import org.objectweb.petals.tools.plugin.development.viewers.SimpleTableViewer;

/**
 * The SharedLibraryPage for the JBI Editor
 * 
 * @author LAM
 */
public class SharedLibraryPage {

    private static final String SHARED_LIBRARY = Messages.SharedLibraryPage_0;
    
    private ScrolledForm form;
    private FormToolkit toolkit;
    private SimpleTableViewer viewerSharedLibrary;
    private List<Version> sharedLibraryVersionList;
    private Text slVersion;
    // Technical index for version modification
    private int versionIndex;
    // The creation mode for this page (i.e. component type)
    private int mode;

    /**
     * Constructor
     * 
     * @param toolkit
     *            The used toolkit
     * @param container
     *            The container
     */
    public SharedLibraryPage(FormToolkit toolkit, Composite container, int mode) {
        this.mode = mode;
        this.toolkit = toolkit;
        form = toolkit.createScrolledForm(container);
        form.setText(SHARED_LIBRARY);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        form.getBody().setLayout(layout);

        createTableSection(SHARED_LIBRARY, Messages.SharedLibraryPage_1);
        if ((mode == JbiEditor.MODE_BC) || (mode == JbiEditor.MODE_SE)) {
            createDetailSection(Messages.SharedLibraryPage_2, Messages.SharedLibraryPage_3);
        }

        toolkit.paintBordersFor(form.getBody());
    }

    /**
     * Getter
     * 
     * @return The page form
     */
    public ScrolledForm getSharedLibrary() {
        return form;
    }

    /**
     * Updates the page content according to the JBI bean
     * 
     * @param bean The JBI bean
     */
    public void updatePageContent(Jbi bean) {
        if (bean.getJbiType().equals(Jbi.TYPE_COMPONENT)) {
            viewerSharedLibrary.clearTableLines();
            sharedLibraryVersionList.clear();
            List<String> slList = new ArrayList<String>();

            Version v;
            for (CoSlListSl currentSL : bean
                    .getComponent().getSharedLibraryList()) {
                v = new Version();
                v.setVersion(currentSL.getVersion());
                sharedLibraryVersionList.add(v);
                slList.add(currentSL.getValue());
            }

            viewerSharedLibrary.setInput(slList);
        }
        if (mode == JbiEditor.MODE_SL) {
            viewerSharedLibrary.clearTableLines();
            viewerSharedLibrary.setInput(bean.getSharedLibrary()
                    .getSharedLibraryClassPath());
        }
    }

    /**
     * Creates the main table section
     * 
     * @param title Section title
     * @param description Section description
     */
    private void createTableSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        viewerSharedLibrary = new SimpleTableViewer(client, toolkit);

        if ((mode == JbiEditor.MODE_BC) || (mode == JbiEditor.MODE_SE)) {
            viewerSharedLibrary.getTable().addSelectionListener(
                    new SelectionListener() {
                        public void widgetSelected(SelectionEvent e) {
                            int indexSel = viewerSharedLibrary.getTable()
                                    .getSelectionIndex();
                            slVersion.setText(sharedLibraryVersionList.get(
                                    indexSel).getVersion());
                            versionIndex = indexSel;
                        }

                        public void widgetDefaultSelected(SelectionEvent e) {
                            slVersion.setText(""); //$NON-NLS-1$
                            versionIndex = 0;
                        }
                    });

            viewerSharedLibrary.getAdd().addSelectionListener(
                    new SelectionAdapter() {
                        @Override
                        public void widgetSelected(SelectionEvent e) {
                            Version newV = new Version();
                            newV.setVersion(""); //$NON-NLS-1$
                            sharedLibraryVersionList.add(newV);
                        }
                    });

            viewerSharedLibrary.getDelete().addSelectionListener(
                    new SelectionAdapter() {
                        @Override
                        public void widgetSelected(SelectionEvent e) {
                            sharedLibraryVersionList.remove(versionIndex);
                        }
                    });
        }

        sharedLibraryVersionList = new ArrayList<Version>();

        toolkit.paintBordersFor(client);

        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);
        section.addExpansionListener(new ExpansionAdapter() {
            public void expansionStateChanged(ExpansionEvent e) {
                form.reflow(false);
            }
        });
        GridData gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Creates the Details section that display informations according to the
     * main section selection
     * 
     * @param title Section title
     * @param description Section description
     */
    private void createDetailSection(String title, String description) {
        Section section = toolkit.createSection(form.getBody(),
                Section.TITLE_BAR | Section.DESCRIPTION);
        section.setActiveToggleColor(toolkit.getHyperlinkGroup()
                .getActiveForeground());
        section.setToggleColor(toolkit.getColors().getColor(
                FormColors.SEPARATOR));
        Composite client = toolkit.createComposite(section, SWT.WRAP);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        client.setLayout(layout);

        GridData gd = new GridData(GridData.FILL_HORIZONTAL);

        toolkit.createLabel(client, Messages.SharedLibraryPage_6);
        slVersion = toolkit.createText(client, ""); //$NON-NLS-1$
        slVersion.setLayoutData(gd);

        slVersion.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent e) {
                // Do nothing
            }

            public void keyReleased(KeyEvent e) {
                String newVersion = ((Text) e.getSource()).getText();
                Version oldVersion = sharedLibraryVersionList.get(versionIndex);
                oldVersion.setVersion(newVersion);
            }
        });

        toolkit.paintBordersFor(client);

        section.setText(title);
        section.setDescription(description);
        section.setClient(client);
        section.setExpanded(true);

        gd = new GridData(GridData.FILL_BOTH);
        section.setLayoutData(gd);
    }

    /**
     * Getter
     * 
     * @return The page form
     */
    public ScrolledForm getForm() {
        return form;
    }

    /**
     * Getter
     * 
     * @return the viewerSharedLibrary
     */
    public SimpleTableViewer getViewerSharedLibrary() {
        return viewerSharedLibrary;
    }

    /**
     * Getter
     * 
     * @return The sharedLibraryVersionList
     */
    public List<Version> getSharedLibraryVersionList() {
        return sharedLibraryVersionList;
    }

    /**
     * Add listeners to forms elements
     * 
     * @param changeListener
     *            The listener
     */
    public void addListeners(SourceChangeListener changeListener) {
        viewerSharedLibrary.getTable().addSelectionListener(changeListener);
        if ((mode == JbiEditor.MODE_BC) || (mode == JbiEditor.MODE_SE)) {
            slVersion.addModifyListener(changeListener);
        }
    }

}
