/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CDataRule.java,v 1.4 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.rules;

import org.eclipse.jface.text.rules.ICharacterScanner;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.Token;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class CDataRule implements IRule {

    private static final String START_MATCH_STRING = "<![CDATA["; //$NON-NLS-1$

    private static final String END_MATCH_STRING = "]]>"; //$NON-NLS-1$
    
    private IToken fToken;

    private StringBuffer buffer = new StringBuffer();

    private int charsRead = 0;

    private String matchString;

    public CDataRule(IToken token, boolean start) {
        super();
        this.fToken = token;
        if (start) {
            this.matchString = START_MATCH_STRING;
        } else {
            this.matchString = END_MATCH_STRING;
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.text.rules.IRule#evaluate(org.eclipse.jface.text.rules.ICharacterScanner)
     */
    public IToken evaluate(ICharacterScanner scanner) {

        IToken result = Token.UNDEFINED;
        buffer.setLength(0);

        charsRead = 0;
        int c = read(scanner);

        if (c == matchString.charAt(0)) {
            do {
                c = read(scanner);
            } while (isOK((char) c));

            if (charsRead == matchString.length()) {
                result = fToken;
            } else {
                rewind(scanner);
                result = Token.UNDEFINED;
            }

        }

        scanner.unread();
        return result;
    }

    private void rewind(ICharacterScanner scanner) {
        int rewindLength = charsRead;
        while (rewindLength > 0) {
            scanner.unread();
            rewindLength--;
        }
    }

    private int read(ICharacterScanner scanner) {
        int c = scanner.read();
        buffer.append((char) c);
        charsRead++;
        return c;
    }

    private boolean isOK(char c) {
        if (charsRead >= matchString.length()) {
            return false;
        }
        return matchString.charAt(charsRead - 1) == c;
    }
}
