/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLPartitionScanner.java,v 1.4 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.scanners;

import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;
import org.objectweb.petals.tools.plugin.development.constants.JbiXmlTags;
import org.objectweb.petals.tools.plugin.development.jbieditor.rules.NonMatchingRule;
import org.objectweb.petals.tools.plugin.development.jbieditor.rules.StartTagRule;
import org.objectweb.petals.tools.plugin.development.jbieditor.rules.XMLTextPredicateRule;

public class XMLPartitionScanner extends RuleBasedPartitionScanner {

    public static final String XML_DEFAULT = "__xml_default"; //$NON-NLS-1$

    public static final String XML_COMMENT = "__xml_comment"; //$NON-NLS-1$

    public static final String XML_PI = "__xml_pi"; //$NON-NLS-1$

    public static final String XML_DOCTYPE = "__xml_doctype"; //$NON-NLS-1$

    public static final String XML_CDATA = "__xml_cdata"; //$NON-NLS-1$

    public static final String XML_START_TAG = "__xml_start_tag"; //$NON-NLS-1$

    public static final String XML_END_TAG = "__xml_end_tag"; //$NON-NLS-1$

    public static final String XML_TEXT = "__xml_text"; //$NON-NLS-1$
    
    private static final int RULES_NUMBER = 7;
    private static final int RULE_COMMENT = 1;
    private static final int RULE_XMLPI = 2;
    private static final int RULE_ENDTAG = 3;
    private static final int RULE_STARTTAG = 4;
    private static final int RULE_DOCTYPE = 5;
    private static final int RULE_TEXT = 6;

    public XMLPartitionScanner() {
        IToken xmlComment = new Token(XML_COMMENT);
        IToken xmlPI = new Token(XML_PI);
        IToken startTag = new Token(XML_START_TAG);
        IToken endTag = new Token(XML_END_TAG);
        IToken docType = new Token(XML_DOCTYPE);
        IToken text = new Token(XML_TEXT);

        IPredicateRule[] rules = new IPredicateRule[RULES_NUMBER];

        rules[0] = new NonMatchingRule();
        rules[RULE_COMMENT] = new MultiLineRule("<!--", "-->", xmlComment); //$NON-NLS-1$ //$NON-NLS-2$
        rules[RULE_XMLPI] = new MultiLineRule("<?", "?>", xmlPI); //$NON-NLS-1$ //$NON-NLS-2$
        rules[RULE_ENDTAG] = 
            new MultiLineRule(JbiXmlTags.TAG_CLOSE_START, JbiXmlTags.TAG_END, endTag);
        rules[RULE_STARTTAG] = new StartTagRule(startTag);
        rules[RULE_DOCTYPE] = new MultiLineRule("<!DOCTYPE", JbiXmlTags.TAG_END, docType); //$NON-NLS-1$
        rules[RULE_TEXT] = new XMLTextPredicateRule(text);

        setPredicateRules(rules);
    }
}
