/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLTagScanner.java,v 1.3 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.scanners;

import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.objectweb.petals.tools.plugin.development.jbieditor.ColorManager;
import org.objectweb.petals.tools.plugin.development.jbieditor.IXMLColorConstants;
import org.objectweb.petals.tools.plugin.development.jbieditor.XMLWhitespaceDetector;

public class XMLTagScanner extends RuleBasedScanner {
    
    private static final int RULES_NUMBER = 3;

    public XMLTagScanner(ColorManager manager) {
        IToken string = new Token(new TextAttribute(manager
                .getColor(IXMLColorConstants.STRING)));

        IRule[] rules = new IRule[RULES_NUMBER];

        // Add rule for double quotes
        rules[0] = new SingleLineRule("\"", "\"", string, '\\'); //$NON-NLS-1$ //$NON-NLS-2$
        // Add a rule for single quotes
        rules[1] = new SingleLineRule("'", "'", string, '\\'); //$NON-NLS-1$ //$NON-NLS-2$
        // Add generic whitespace rule.
        rules[2] = new WhitespaceRule(new XMLWhitespaceDetector());

        setRules(rules);
    }
}
