/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLElement.java,v 1.3 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.xml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.text.Position;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class XMLElement {

    private List<XMLElement> elementChildren = new ArrayList<XMLElement>();

    private List<XMLAttribute> attributeChildren = new ArrayList<XMLAttribute>();

    private String name;

    private XMLElement parent;

    private Position position;

    public XMLElement(String name) {
        super();
        this.name = name;
    }

    public List getChildrenDTDElements() {
        return elementChildren;
    }

    public XMLElement addChildElement(XMLElement element) {
        elementChildren.add(element);
        element.setParent(this);
        return this;
    }

    public void setParent(XMLElement element) {
        this.parent = element;
    }

    public XMLElement getParent() {
        return parent;
    }

    public XMLElement addChildAttribute(XMLAttribute attribute) {
        attributeChildren.add(attribute);
        return this;
    }

    public String getName() {
        return name;
    }

    public String getAttributeValue(String localName) {
        for (Iterator iter = attributeChildren.iterator(); iter.hasNext();) {
            XMLAttribute attribute = (XMLAttribute) iter.next();
            if (attribute.getName().equals(localName)) {
                return attribute.getValue();
            }
        }
        return null;
    }

    public void clear() {
        elementChildren.clear();
        attributeChildren.clear();
    }

    public void setPosition(Position position) {
        this.position = position;
    }

    public Position getPosition() {
        return position;
    }

}
