/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: XMLValidationErrorHandler.java,v 1.5 2007/04/17 09:49:35 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.jbieditor.xml;

import java.util.ArrayList;
import java.util.List;

import org.xml.sax.ErrorHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Based on a tutorial wich can be found here :
 * http://www.realsolve.co.uk/site/tech/jface-text.php
 * 
 * @author Phil Zoio
 */
public class XMLValidationErrorHandler implements ErrorHandler {

    private List<XmlValidationError> errorList = new ArrayList<XmlValidationError>();

    private Locator locator;

    public XMLValidationErrorHandler() {
    }

    /**
     * @param e
     * @throws SAXException
     */
    public void error(SAXParseException e) throws SAXException {
        handleError(e, false);
    }

    public void setDocumentLocator(Locator loc) {
        this.locator = loc;
    }

    private void handleError(SAXParseException e, boolean isFatal) {
        XmlValidationError validationError = nextError(e, isFatal);
        errorList.add(validationError);
    }

    protected XmlValidationError nextError(SAXParseException e, boolean isFatal) {
        String errorMessage = e.getMessage();

        int lineNumber = locator.getLineNumber();
        int columnNumber = locator.getColumnNumber();
        String isFatalStr;
        if (isFatal) {
            isFatalStr = Messages.XMLValidationErrorHandler_0;
        } else {
            isFatalStr = Messages.XMLValidationErrorHandler_1;
        }

        log(this, isFatalStr + Messages.XMLValidationErrorHandler_2
                + lineNumber + Messages.XMLValidationErrorHandler_3 + columnNumber + ": " + errorMessage); //$NON-NLS-2$

        XmlValidationError validationError = new XmlValidationError();
        validationError.setLineNumber(lineNumber);
        validationError.setColumnNumber(columnNumber);
        validationError.setErrorMessage(errorMessage);
        return validationError;
    }

    private void log(XMLValidationErrorHandler handler, String string) {
    }

    public void fatalError(SAXParseException e) throws SAXParseException {
        handleError(e, true);
    }

    public List<XmlValidationError> getErrorList() {
        return errorList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
     */
    public void warning(SAXParseException arg0) throws SAXException {
    }

}
