/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsDevelopmentPerspective.java,v 1.9 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.perspective;

import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;
import org.eclipse.ui.console.IConsoleConstants;
import org.objectweb.petals.tools.plugin.development.views.PetalsBrowserView;

/**
 * Perspective for Petals Development. Includes standard Java editor and views : -
 * Petals Browser - Problems - Servers - Console - Properties
 * 
 * @author LAM
 */
public class PetalsDevelopmentPerspective implements IPerspectiveFactory {

    /**
     * ID = "org.objectweb.petals.tools.plugin.development.petalsdevpersp"
     */
    public static final String ID =
        "org.objectweb.petals.tools.plugin.development.petalsdevpersp"; //$NON-NLS-1$
    
    private static String leftTopPos = Messages.PetalsDevelopmentPerspective_1;
    private static final float RATIO_LEFT = 0.2f;
    private static final float RATIO_TOP = 0.7f;
    
    public PetalsDevelopmentPerspective() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IPerspectiveFactory#createInitialLayout(org.eclipse.ui.IPageLayout)
     */
    public void createInitialLayout(IPageLayout layout) {
        String editorArea = layout.getEditorArea();
        layout.setEditorAreaVisible(true);
        String serversViewId = "org.eclipse.wst.server.ui.ServersView"; //$NON-NLS-1$

        IFolderLayout leftTop = layout.createFolder(leftTopPos,
                IPageLayout.LEFT, RATIO_LEFT, editorArea);
        leftTop.addView(JavaUI.ID_PACKAGES);
        layout.getViewLayout(JavaUI.ID_PACKAGES).setCloseable(true);

        IFolderLayout leftBottom = layout.createFolder(Messages.PetalsDevelopmentPerspective_3,
                IPageLayout.BOTTOM, RATIO_TOP, leftTopPos);
        leftBottom.addView(PetalsBrowserView.ID);
        layout.getViewLayout(PetalsBrowserView.ID).setCloseable(true);

        IFolderLayout bottom = layout.createFolder(Messages.PetalsDevelopmentPerspective_4,
                IPageLayout.BOTTOM, RATIO_TOP, editorArea);
        bottom.addView(IPageLayout.ID_PROBLEM_VIEW);
        layout.getViewLayout(IPageLayout.ID_PROBLEM_VIEW).setCloseable(true);
        bottom.addView(serversViewId);
        layout.getViewLayout(serversViewId).setCloseable(true);
        bottom.addView(IConsoleConstants.ID_CONSOLE_VIEW);
        layout.getViewLayout(IConsoleConstants.ID_CONSOLE_VIEW).setCloseable(
                true);
        bottom.addView(IPageLayout.ID_PROP_SHEET);
        layout.getViewLayout(IPageLayout.ID_PROP_SHEET).setCloseable(true);

        layout.addPerspectiveShortcut(ID);
        layout.addShowViewShortcut(PetalsBrowserView.ID);
    }

}
