/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ESBDevelopmentPreferencesPage.java,v 1.5 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.preferences;

import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;

/**
 * @author LAM
 */
public class ESBDevelopmentPreferencesPage extends FieldEditorPreferencePage
        implements IWorkbenchPreferencePage {

    private String browse = Messages.ESBDevelopmentPreferencesPage_0;
    private FileFieldEditor ffeJbiXsd;
    private FileFieldEditor ffeMvn;
    private BooleanFieldEditor bfeUseMavenForBuilding;
    private StringFieldEditor sfeMavenPetalsPluginVersion;
    private BooleanFieldEditor bfeOWMavenRepo;

    /**
     * Constructor
     */
    public ESBDevelopmentPreferencesPage() {
        super(GRID);
        setPreferenceStore(Activator.getDefault().getPreferenceStore());
        setDescription(
                Messages.ESBDevelopmentPreferencesPage_1);
    }

    @Override
    protected void initialize() {
        super.initialize();
        ffeMvn.setEnabled(
                bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
        sfeMavenPetalsPluginVersion.setEnabled(
                bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
        bfeOWMavenRepo.setEnabled(
                bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
     */
    @Override
    protected void createFieldEditors() {
        ffeJbiXsd = new FileFieldEditor(
                ESBDevPrefConstants.JBI_XSD_PATH_NAME,
                ESBDevPrefConstants.JBI_XSD_PATH_LABEL,
                getFieldEditorParent());
        ffeJbiXsd.setChangeButtonText(browse);
        ffeJbiXsd.setFileExtensions(new String[] { "jbi.xsd" }); //$NON-NLS-1$
        addField(ffeJbiXsd);

        bfeUseMavenForBuilding = 
            new BooleanFieldEditor(ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME,
                    ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_LABEL,
                    getFieldEditorParent());
        addField(bfeUseMavenForBuilding);

        ffeMvn = new FileFieldEditor(
                ESBDevPrefConstants.MAVEN_CMD_NAME,
                ESBDevPrefConstants.MAVEN_CMD_LABEL,
                getFieldEditorParent());
        ffeMvn.setChangeButtonText(browse);
        addField(ffeMvn);

        sfeMavenPetalsPluginVersion = new StringFieldEditor(
                ESBDevPrefConstants.MAVEN_PETALS_PLUGIN_VERSION_NAME,
                ESBDevPrefConstants.MAVEN_PETALS_PLUGIN_VERSION_LABEL,
                getFieldEditorParent());
        addField(sfeMavenPetalsPluginVersion);

        bfeOWMavenRepo = 
            new BooleanFieldEditor(ESBDevPrefConstants.OBJECTWEB_MAVEN_REPO_NAME,
                    ESBDevPrefConstants.OBJECTWEB_MAVEN_REPO_LABEL,
                    getFieldEditorParent());
        addField(bfeOWMavenRepo);
    }

    @Override
    protected void checkState() {
        super.checkState();
        if (!isValid()) {
            return;
        }
        
        setErrorMessage(null);
        setValid(true);
        
        if ((bfeUseMavenForBuilding.getBooleanValue()) 
                && (ffeMvn.getStringValue().length() == 0)) {
            setErrorMessage(Messages.ESBDevelopmentPreferencesPage_3);
            setValid(false);
        }
    }

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        super.propertyChange(event);
        if (event.getProperty().equals(FieldEditor.VALUE)) {
            if (event.getSource() == bfeUseMavenForBuilding) {
                ffeMvn.setEnabled(
                        bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
                sfeMavenPetalsPluginVersion.setEnabled(
                        bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
                bfeOWMavenRepo.setEnabled(
                        bfeUseMavenForBuilding.getBooleanValue(), getFieldEditorParent());
            }
            checkState();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init(IWorkbench workbench) { }

}
