/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServicesTableContentProvider.java,v 1.6 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.providers;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceTableLine;
import org.objectweb.petals.tools.plugin.development.viewers.IServicesTableListViewer;

/**
 * 
 * Acts as a proxy for the ServicesTableList providing content for the Table. It
 * implements the IServicesTableListViewer interface since it must register
 * changeListeners with the ServicesTableList
 * 
 * @author LAM
 */
public class ServicesTableContentProvider implements
        IStructuredContentProvider, IServicesTableListViewer {
    private ServicesTableList servicesList;

    private TableViewer tableViewer;

    /**
     * Constructor
     * 
     * @param servicesTableList
     *            The ServicesTableList
     * @param tableViewer
     *            The TableViewer
     */
    public ServicesTableContentProvider(ServicesTableList servicesTableList,
            TableViewer tableViewer) {
        super();
        this.servicesList = servicesTableList;
        this.tableViewer = tableViewer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
        if (newInput != null) {
            ((ServicesTableList) newInput).addChangeListener(this);
        }
        if (oldInput != null) {
            ((ServicesTableList) oldInput).removeChangeListener(this);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
        servicesList.removeChangeListener(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object parent) {
        return servicesList.getServices().toArray();
    }

    /*
     * (non-Javadoc)
     * 
     * @see IServicesTableListViewer#addTask(ServiceTableLine)
     */
    public void addService(ServiceTableLine task) {
        tableViewer.add(task);
    }

    /*
     * (non-Javadoc)
     * 
     * @see IServicesTableListViewer#removeTask(ServiceTableLine)
     */
    public void removeService(ServiceTableLine task) {
        tableViewer.remove(task);
    }

    /*
     * (non-Javadoc)
     * 
     * @see IServicesTableListViewer#updateTask(ServiceTableLine)
     */
    public void updateService(ServiceTableLine task) {
        tableViewer.update(task, null);
    }
}
