/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServicesTableList.java,v 1.8 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.providers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceTableLine;
import org.objectweb.petals.tools.plugin.development.viewers.IServicesTableListViewer;

/**
 * Class that plays the role of the domain model in the ServicesTableViewer
 */
public class ServicesTableList {

    private List<ServiceTableLine> services = new ArrayList<ServiceTableLine>();

    private Set<IServicesTableListViewer> changeListeners = 
        new HashSet<IServicesTableListViewer>();

    /**
     * Constructor
     */
    public ServicesTableList() {
        super();
    }

    public ServicesTableList(List<SeConsumesProvides> seList) {
        super();
        if (seList.size() > 0) {
            services = new Vector<ServiceTableLine>(seList.size());
            ServiceTableLine serviceTableLine;
            for (SeConsumesProvides current : seList) {
                serviceTableLine = new ServiceTableLine(current
                        .getInterfaceName(), current.getServiceName(), current
                        .getEndpointName(), current.getLinkType(), current
                        .getExtensions());
                services.add(serviceTableLine);
            }
        }
    }

    /**
     * Return the collection of services
     */
    public List<ServiceTableLine> getServices() {
        return services;
    }

    /**
     * Add a new service to the collection of services
     */
    public void addService() {
        String defaultNew = Messages.ServicesTableList_0;
        
        ServiceTableLine service =
            new ServiceTableLine(
                    defaultNew, defaultNew, defaultNew, defaultNew, new StringBuffer());
        services.add(services.size(), service);
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((IServicesTableListViewer) iterator.next()).addService(service);
        }
    }

    /**
     * @param service
     */
    public void removeService(ServiceTableLine service) {
        services.remove(service);
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((IServicesTableListViewer) iterator.next()).removeService(service);
        }
    }

    /**
     * @param service
     */
    public void serviceChanged(ServiceTableLine service) {
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((IServicesTableListViewer) iterator.next()).updateService(service);
        }
    }

    /**
     * @param viewer
     */
    public void removeChangeListener(IServicesTableListViewer viewer) {
        changeListeners.remove(viewer);
    }

    /**
     * @param viewer
     */
    public void addChangeListener(IServicesTableListViewer viewer) {
        changeListeners.add(viewer);
    }

}
