/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SimpleTableContentProvider.java,v 1.4 2007/04/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.providers;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.TableLine;
import org.objectweb.petals.tools.plugin.development.viewers.ILinesTableListViewer;

/**
 * 
 * Acts as a proxy for the ServicesTableList providing content for the Table. It
 * implements the IServicesTableListViewer interface since it must register
 * changeListeners with the ServicesTableList
 * 
 * @author LAM
 * 
 */
public class SimpleTableContentProvider implements IStructuredContentProvider,
        ILinesTableListViewer {
    private SimpleTableList simpleList;

    private TableViewer tableViewer;

    /**
     * Constructor
     * 
     * @param simpleList
     * @param tableViewer
     */
    public SimpleTableContentProvider(SimpleTableList simpleList,
            TableViewer tableViewer) {
        super();
        this.simpleList = simpleList;
        this.tableViewer = tableViewer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
        if (newInput != null) {
            ((SimpleTableList) newInput).addChangeListener(this);
        }
        if (oldInput != null) {
            ((SimpleTableList) oldInput).removeChangeListener(this);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
        simpleList.removeChangeListener(this);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object parent) {
        return simpleList.getLines().toArray();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.tools.plugin.development.viewers.ILinesTableListViewer#addLine(org.objectweb.petals.tools.plugin.development.providers.TableLine)
     */
    public void addLine(TableLine line) {
        tableViewer.add(line);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.tools.plugin.development.viewers.ILinesTableListViewer#removeLine(org.objectweb.petals.tools.plugin.development.providers.TableLine)
     */
    public void removeLine(TableLine line) {
        tableViewer.remove(line);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.tools.plugin.development.viewers.ILinesTableListViewer#updateLine(org.objectweb.petals.tools.plugin.development.providers.TableLine)
     */
    public void updateLine(TableLine line) {
        tableViewer.update(line, null);
    }
}
