/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SimpleTableList.java,v 1.5 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.providers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.objectweb.petals.tools.plugin.development.jbieditor.beans.TableLine;
import org.objectweb.petals.tools.plugin.development.viewers.ILinesTableListViewer;

/**
 * Model of a record for the SimpleTableViewer
 */
public class SimpleTableList {

    private List<TableLine> lines;

    private Set<ILinesTableListViewer> changeListeners =
        new HashSet<ILinesTableListViewer>();

    /**
     * Constructor
     */
    public SimpleTableList() {
        super();
        lines = new ArrayList<TableLine>();
    }

    public SimpleTableList(List<String> linesList) {
        super();
        if (linesList.size() > 0) {
            lines = new ArrayList<TableLine>(linesList.size());
            TableLine line;
            for (String current : linesList) {
                line = new TableLine();
                line.setName(current);
                lines.add(line);
            }
        }
    }

    /**
     * Return the collection of lines
     */
    public List<TableLine> getLines() {
        return lines;
    }

    /**
     * Add a new line to the collection of lines
     */
    public void addLine() {
        String newLine = Messages.SimpleTableList_0;
        TableLine line = new TableLine(newLine);
        lines.add(line);
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((ILinesTableListViewer) iterator.next()).addLine(line);
        }
    }
    
    /**
     * Add a new line to the collection of lines
     * 
     * @param name
     */
    public void addLine(String name) {
        String newLine = name;
        TableLine line = new TableLine(newLine);
        lines.add(line);
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((ILinesTableListViewer) iterator.next()).addLine(line);
        }
    }

    /**
     * @param line
     */
    public void removeLine(TableLine line) {
        lines.remove(line);
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((ILinesTableListViewer) iterator.next()).removeLine(line);
        }
    }

    /**
     * @param line
     */
    public void lineChanged(TableLine line) {
        Iterator iterator = changeListeners.iterator();
        while (iterator.hasNext()) {
            ((ILinesTableListViewer) iterator.next()).updateLine(line);
        }
    }

    /**
     * @param viewer
     */
    public void removeChangeListener(ILinesTableListViewer viewer) {
        changeListeners.remove(viewer);
    }

    /**
     * @param viewer
     */
    public void addChangeListener(ILinesTableListViewer viewer) {
        changeListeners.add(viewer);
    }

}
