/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ViewContentProvider.java,v 1.7 2007/04/26 10:45:31 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.providers;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IWorkbenchPartSite;
import org.objectweb.petals.tools.plugin.development.actions.ShowOrphanSuAction;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.JbiBeanUtils;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.business.TransformDomToBean;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.XmlParsing;
import org.objectweb.petals.tools.plugin.development.views.PetalsTreeObject;
import org.objectweb.petals.tools.plugin.development.views.PetalsTreeParent;
import org.w3c.dom.Document;

/**
 * Content provider for the Petals Component Dependencies view
 * 
 * @author LAM
 */
public class ViewContentProvider implements IStructuredContentProvider,
        ITreeContentProvider {

    private PetalsTreeParent invisibleRoot = new PetalsTreeParent(""); //$NON-NLS-1$
    private IWorkbenchPartSite site;

    /**
     * Constructor
     * 
     * @param site The site
     */
    public ViewContentProvider(IWorkbenchPartSite site) {
        super();
        this.site = site;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
     *      java.lang.Object, java.lang.Object)
     */
    public void inputChanged(Viewer v, Object oldInput, Object newInput) {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object parent) {
        if (!(parent instanceof PetalsTreeObject)) {
            if (!invisibleRoot.hasChildren()) {
                initialize();
            }
            return getChildren(invisibleRoot);
        }
        return getChildren(parent);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
     */
    public Object getParent(Object child) {
        if (child instanceof PetalsTreeObject) {
            return ((PetalsTreeObject) child).getParent();
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
     */
    public Object[] getChildren(Object parent) {
        if (parent instanceof PetalsTreeParent) {
            return ((PetalsTreeParent) parent).getChildren();
        }
        return new Object[0];
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
     */
    public boolean hasChildren(Object parent) {
        if (parent instanceof PetalsTreeParent) {
            return ((PetalsTreeParent) parent).hasChildren();
        }
        return false;
    }

    /**
     * initialize
     */
    private void initialize() {

        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();

        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);

                    if (bean != null) {
                        PetalsTreeParent p1 = new PetalsTreeParent(
                                currentProject.getName());
                        invisibleRoot.addChild(p1);

                        p1.setJbiBean(bean);
                    }
                } catch (CoreException e) {
                    MessageDialog.openError(site.getShell(),
                        Messages.ViewContentProvider_1,
                        Messages.ViewContentProvider_2
                        + e.getMessage());
                }
            }
        }
        
        if (ShowOrphanSuAction.isShowSu()) {
            for (PetalsTreeObject currentSa : getSaNodes()) {
                for (PetalsTreeObject currentSu : getSuOfSa(currentSa)) {
                    PetalsTreeObject p2 = new PetalsTreeObject(currentSu.getName());
                    p2.setJbiBean(currentSu.getJbiBean());
                    ((PetalsTreeParent) currentSa).addChild(p2);
                    invisibleRoot.removeChild(currentSu);
                }
            }
        }
    }
    
    /**
     * @return The SA PetalsTreeObject nodes list
     */
    private List<PetalsTreeObject> getSaNodes() {
        List<PetalsTreeObject> saNodes = new ArrayList<PetalsTreeObject>();
        for (PetalsTreeObject current : invisibleRoot.getChildren()) {
            if (JbiBeanUtils.isSa(current.getJbiBean())) {
                saNodes.add(current);
            }
        }
        
        return saNodes;
    }
    
    /**
     * @return The SA PetalsTreeObject nodes list
     */
    private List<PetalsTreeObject> getSuNodes() {
        List<PetalsTreeObject> suNodes = new ArrayList<PetalsTreeObject>();
        for (PetalsTreeObject current : invisibleRoot.getChildren()) {
            if (JbiBeanUtils.isSu(current.getJbiBean())) {
                suNodes.add(current);
            }
        }
        
        return suNodes;
    }
    
    /**
     * @param saTreeObject SA that contains SU's
     * @return PetalsTreeObject list of SU's that are referenced in the Sa
     */
    private List<PetalsTreeObject> getSuOfSa(PetalsTreeObject saTreeObject) {
        List<PetalsTreeObject> suNodes = new ArrayList<PetalsTreeObject>();
        
        for (String currentSuName : JbiBeanUtils.getSuList(saTreeObject.getJbiBean())) {
            for (PetalsTreeObject currentSu : getSuNodes()) {
                if (currentSuName.equals(currentSu.getName())) {
                    suNodes.add(currentSu);
                }
            }
        }
        
        return suNodes;
    }
    
}
