/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServicesTableCellModifier.java,v 1.8 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.viewers;

import org.eclipse.jface.viewers.ICellModifier;
import org.eclipse.swt.widgets.TableItem;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.ServiceTableLine;
import org.objectweb.petals.tools.plugin.development.providers.ServicesTableLabelProvider;

/**
 * This class implements an ICellModifier An ICellModifier is called when the
 * user modifes a cell in the tableViewer
 */
public class ServicesTableCellModifier implements ICellModifier {

    private ServicesTableViewer servicesTableViewer;

    /**
     * Constructor
     * 
     * @param tableViewerExample
     *            an instance of a ServicesTableViewer
     */
    public ServicesTableCellModifier(ServicesTableViewer tableViewerExample) {
        super();
        this.servicesTableViewer = tableViewerExample;
    }

    /**
     * @see org.eclipse.jface.viewers.ICellModifier#canModify(java.lang.Object,
     *      java.lang.String)
     */
    public boolean canModify(Object element, String property) {
        return true;
    }

    /**
     * @see org.eclipse.jface.viewers.ICellModifier#getValue(java.lang.Object,
     *      java.lang.String)
     */
    public String getValue(Object element, String property) {

        // Find the index of the column
        int columnIndex = servicesTableViewer.getColumnNames()
                .indexOf(property);

        String result = null;
        ServiceTableLine service = (ServiceTableLine) element;

        if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_INTERFACE) {
            result = service.getInterfaceName();
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_SERVICE) {
            result = service.getServiceName();
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_ENDPOINT) {
            result = service.getEndpointName();
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_LINK) {
            result = service.getLinkType();
        } else {
            result = ""; //$NON-NLS-1$
        }
        
        return result;
    }

    /**
     * @see org.eclipse.jface.viewers.ICellModifier#modify(java.lang.Object,
     *      java.lang.String, java.lang.Object)
     */
    public void modify(Object element, String property, Object value) {

        // Find the index of the column
        int columnIndex = servicesTableViewer.getColumnNames()
                .indexOf(property);

        TableItem item = (TableItem) element;
        ServiceTableLine service = (ServiceTableLine) item.getData();
        String valueString = ((String) value).trim();

        if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_INTERFACE) {
            service.setInterfaceName(valueString);
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_SERVICE) {
            service.setServiceName(valueString);
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_ENDPOINT) {
            service.setEndpointName(valueString);
        } else if (columnIndex == ServicesTableLabelProvider.ID_COLUMN_LINK) {
            service.setLinkType(valueString);
        }
        
        servicesTableViewer.getServiceList().serviceChanged(service);
    }
}
