/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsBrowserView.java,v 1.16 2007/04/27 07:57:51 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.views;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.actions.JbiEditorAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenCleanAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenCleanInstallAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenInstallAction;
import 
    org.objectweb.petals.tools.plugin.development.actions.MavenNotRecursiveOptionAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenSkipTestOptionAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenVerboseOptionAction;
import org.objectweb.petals.tools.plugin.development.actions.RefreshAction;
import org.objectweb.petals.tools.plugin.development.actions.ShowOrphanSuAction;
import org.objectweb.petals.tools.plugin.development.actions.MavenUpdateSnapshotsOptionAction;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.providers.ViewContentProvider;
import org.objectweb.petals.tools.plugin.development.providers.ViewLabelProvider;

/**
 * This sample class demonstrates how to plug-in a new workbench view. The view
 * shows data obtained from the model. The sample creates a dummy model on the
 * fly, but a real implementation would connect to the model available either in
 * this or another plug-in (e.g. the workspace). The view is connected to the
 * model using a content provider.
 * <p>
 * The view uses a label provider to define how model objects should be
 * presented in the view. Each view can present the same model objects using
 * different labels and icons, if needed. Alternatively, a single label provider
 * can be shared between views in order to ensure that objects of the same type
 * are presented in the same way everywhere.
 * <p>
 */

public class PetalsBrowserView extends ViewPart {

    public static final String ID = "org.objectweb.petals.tools.plugin.development.views.PetalsBrowserView"; //$NON-NLS-1$

    private TreeViewer viewer;

    private Action jbiEditorAction;
    private Action refreshAction;
    private MavenAction mavenInstallAction;
    private MavenAction mavenCleanAction;
    private MavenAction mavenCleanInstallAction;
    private Action mavenVerboseAction;
    private Action mavenSkipTestOptionAction;
    private Action mavenNotRecursiveOptionAction;
    private Action mavenUpdateSnapshotsOptionAction;
    private Action showOrphanSuAction;
    
    private Preferences prefs;

    private IResourceChangeListener resourceChangeListener;

    /**
     * The constructor.
     */
    public PetalsBrowserView() {
        
    }

    /**
     * This is a callback that will allow us to create the viewer and initialize
     * it.
     */
    public void createPartControl(Composite parent) {
        prefs = Activator.getDefault().getPluginPreferences();
        
        viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
        viewer.setContentProvider(new ViewContentProvider(getSite()));
        viewer.setLabelProvider(new ViewLabelProvider());
        viewer.setInput(getViewSite());
        viewer.expandAll();
        makeActions();
        hookContextMenu();
        hookToolbarMenu();

        resourceChangeListener = new IResourceChangeListener() {
            public void resourceChanged(IResourceChangeEvent event) {
                boolean doWeHaveToRefresh = false;
                if (event.getType() != IResourceChangeEvent.POST_CHANGE) { return; }
                IResourceDelta[] deltas = event.getDelta().getAffectedChildren();
                if (deltas.length > 0) {
                    for (int i = 0; i < deltas.length; i++) {
                        IResourceDelta[] deltas2 = deltas[i].getAffectedChildren();
                        for (int j = 0; j < deltas2.length; j++) {
                            String projectFilename =
                                deltas2[j].getProjectRelativePath().toString();
                            if (projectFilename.equals(".project")) { //$NON-NLS-1$
                                doWeHaveToRefresh = true;
                            }
                        }
                    }
                }
                if (doWeHaveToRefresh) { refreshAction.run(); }
            }
        };
        ResourcesPlugin.getWorkspace().addResourceChangeListener(
                resourceChangeListener);
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        viewer.getControl().setFocus();
    }

    /**
     * Instanciate all actions used in the dependancies treeview
     */
    private void makeActions() {
        jbiEditorAction = new JbiEditorAction(getViewSite());
        refreshAction = new RefreshAction(viewer, getSite());
        mavenInstallAction = new MavenInstallAction(getViewSite().getShell());
        mavenCleanAction = new MavenCleanAction(getViewSite());
        mavenCleanInstallAction = new MavenCleanInstallAction(getViewSite());
        mavenVerboseAction = new MavenVerboseOptionAction();
        mavenSkipTestOptionAction = new MavenSkipTestOptionAction();
        mavenNotRecursiveOptionAction = new MavenNotRecursiveOptionAction();
        showOrphanSuAction = new ShowOrphanSuAction(viewer, getSite());
        mavenUpdateSnapshotsOptionAction = 
            new MavenUpdateSnapshotsOptionAction();
    }

    private void hookContextMenu() {
        MenuManager menuMgr = new MenuManager(Messages.PetalsBrowserView_2);
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                PetalsBrowserView.this.fillContextMenu(manager);
            }
        });
        Menu menu = menuMgr.createContextMenu(viewer.getControl());
        viewer.getControl().setMenu(menu);
        getSite().registerContextMenu(menuMgr, viewer);
    }

    private void hookToolbarMenu() {
        IActionBars actionBars = getViewSite().getActionBars();
        IToolBarManager toolBar = actionBars.getToolBarManager();
        toolBar.add(showOrphanSuAction);
        toolBar.add(refreshAction);
    }

    /**
     * @param manager Popupmenu
     */
    private void fillContextMenu(IMenuManager manager) {
        Object selectedObject = ((IStructuredSelection) viewer.getSelection())
                .getFirstElement();
        Object obj = (PetalsTreeObject) selectedObject;

        if (obj instanceof PetalsTreeObject) {
            String projectName = ((PetalsTreeObject) obj).getName();

            ((JbiEditorAction) jbiEditorAction).setProjectName(projectName);
            manager.add(jbiEditorAction);
            
            if (prefs.getBoolean(ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
                manager.add(new Separator());
                mavenInstallAction.setProjectName(projectName);
                manager.add(mavenInstallAction);
                mavenCleanInstallAction.setProjectName(projectName);
                manager.add(mavenCleanInstallAction);
                mavenCleanAction.setProjectName(projectName);
                manager.add(mavenCleanAction);
                manager.add(mavenVerboseAction);
                manager.add(mavenSkipTestOptionAction);
                manager.add(mavenNotRecursiveOptionAction);
                manager.add(mavenUpdateSnapshotsOptionAction);
            }
        }

    }

    public void refreshViewer() {
        viewer.setContentProvider(new ViewContentProvider(getSite()));
        viewer.setLabelProvider(new ViewLabelProvider());
        viewer.refresh();
        viewer.expandAll();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.part.WorkbenchPart#dispose()
     */
    @Override
    public void dispose() {
        super.dispose();
        ResourcesPlugin.getWorkspace().removeResourceChangeListener(
                resourceChangeListener);

    }
}
