/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ProjectUtils.java,v 1.2 2007/05/15 13:29:18 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.business;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.objectweb.petals.tools.plugin.development.constants.Wizards;
import org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.Activator;
import org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.beans.ProvidedSuBean;
import org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.customization.Constants;
import org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.customization.Version;
import org.objectweb.petals.tools.plugin.development.wizards.models.ProvidedSuBeans;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author alouis
 * @author chamerling - eBM WebSourcing
 */
public final class ProjectUtils {

    /**
     * 
     * Creates a new instance of {@link ProjectUtils}
     *
     */
    private ProjectUtils() {
    }

    /**
     * Cr\u00e9ation du fichier jbi.xml pour le projet sp\u00e9cifi\u00e9. Le
     * template \u00e0 utiliser est sp\u00e9cifi\u00e9 par le param\u00e8tre
     * jbiRessourceName
     * 
     * @param project
     *            Le projet pour lequel cr\u00e9er un fichier jbi.xml
     * @param jbiRessourceName
     *            Le nom du template \u00e0 utiliser
     * @param suList
     *            La liste de beans de services fournis
     * @throws CoreException
     *             CoreException
     */
    public static void processJbi(IProject project, String jbiRessourceName, boolean consumes,
            List<ProvidedSuBean> suList, ProvidedSuBeans suBean, String directoryName)
            throws CoreException {

        IFile jbiFile = project.getFile(new Path(Wizards.SRC + File.separator + Wizards.MAIN
                + File.separator + Wizards.JBI + File.separator + Constants.JBI_FILE));
        jbiFile.delete(true, null);

        // Then we add custom jbi file
        InputStream jbiXmlStream = Activator.getDefault().getClass().getResourceAsStream(
                jbiRessourceName);

        String jbiStr = ""; //$NON-NLS-1$
        try {
            jbiStr = InputStreamUtils.inputStreamAsString(jbiXmlStream);
            if (consumes) {
                jbiStr = jbiStr.replaceAll(Constants.IN_FILE_XMLNS,
                        getXmlnsDeclarationsFromSuList(suList));
                jbiStr = jbiStr.replaceAll(Constants.IN_FILE_SERVICES,
                        getConsumedServicesListFromSuList(suList, directoryName));
            } else { // provides
                jbiStr = jbiStr.replaceAll(Constants.IN_FILE_XMLNS,
                        getXmlnsDeclarationsFromSuBean(suBean));
                jbiStr = jbiStr.replaceAll(Constants.IN_FILE_SERVICES,
                        getProvidedServiceFromSuBean(suBean, directoryName));

            }
            ByteArrayInputStream bais = new ByteArrayInputStream(jbiStr.getBytes());

            // Then we create the file in the new project
            jbiFile.create(bais, true, null);
        } catch (IOException e) {
            // Something went wrong, we save the file anyway but the
            // user will have to manually edit service name
            jbiFile.create(jbiXmlStream, true, null);
        }
    }

    /**
     * Remove the JUNIT dependency and add the component one
     * 
     * @param project
     * @param componentVersion
     * @throws CoreException
     */
    public static void processPom(IProject project, Version componentVersion) throws CoreException {

        IFile pomFile = project.getFile(Constants.POM_FILE);
        InputStream jbiPomStream = pomFile.getContents();

        String pomStr = ""; //$NON-NLS-1$
        Document pomDom = DomUtils.xmlParsing(jbiPomStream);

        if (pomDom != null) {
            Element root = pomDom.getDocumentElement();
            NodeList nodeList = root.getChildNodes();

            // Remove the JUNIT dependency
            Node dependenciesNode = null;
            for (int i = 0; i < nodeList.getLength(); i++) {
                if (nodeList.item(i).getNodeName().equals(Constants.DEPENDENCIES_TAG)) {
                    dependenciesNode = nodeList.item(i);
                    NodeList dependenciesList = nodeList.item(i).getChildNodes();
                    Node dependencyToRemove = null;
                    for (int j = 0; j < dependenciesList.getLength(); j++) {
                        if (dependenciesList.item(j).getTextContent().contains(Constants.JUNIT)) {
                            dependencyToRemove = dependenciesList.item(j);
                        }
                    }

                    if (dependencyToRemove != null) {
                        nodeList.item(i).removeChild(dependencyToRemove);
                    }
                }
            }

            // add the component dependency
            if (dependenciesNode == null) {
                dependenciesNode = pomDom.createElement("dependencies"); //$NON-NLS-1$
                pomDom.appendChild(dependenciesNode);
            }

            Element dependencyElement = pomDom.createElement("dependency"); //$NON-NLS-1$
            Element artifactElement = pomDom.createElement("artifactId"); //$NON-NLS-1$
            artifactElement.setTextContent(componentVersion.getName());
            dependencyElement.appendChild(artifactElement);

            Element groupElement = pomDom.createElement("groupId"); //$NON-NLS-1$
            groupElement.setTextContent(componentVersion.getGroup());
            dependencyElement.appendChild(groupElement);

            Element versionElement = pomDom.createElement("version"); //$NON-NLS-1$
            versionElement.setTextContent(componentVersion.getVersion());
            dependencyElement.appendChild(versionElement);

            Element typeElement = pomDom.createElement("type"); //$NON-NLS-1$
            typeElement.setTextContent("jbi-component"); //$NON-NLS-1$
            dependencyElement.appendChild(typeElement);

            Element scopeElement = pomDom.createElement("scope"); //$NON-NLS-1$
            scopeElement.setTextContent("runtime"); //$NON-NLS-1$
            dependencyElement.appendChild(scopeElement);

            dependenciesNode.appendChild(dependencyElement);
        }

        pomStr = DomUtils.convertDom(pomDom);

        if (pomStr != null && pomStr.length() > 0) {
            ByteArrayInputStream bais = new ByteArrayInputStream(pomStr.getBytes());

            // Then we erase than create again the file in the new project
            pomFile.delete(true, null);
            pomFile.create(bais, true, null);
        }
    }

    private static String getXmlnsDeclarationsFromSuBean(ProvidedSuBeans suBean) {
        String xmlnsPrefix = "xmlns:"; //$NON-NLS-1$
        String xmlnsDeclaration = ""; //$NON-NLS-1$
        String quotes = "\""; //$NON-NLS-1$
        String equal = "="; //$NON-NLS-1$
        String spacer = " "; //$NON-NLS-1$

        if (!xmlnsDeclaration.contains(suBean.getInterfaceName().getNamespaceUri().getLocalPart())) {
            xmlnsDeclaration = xmlnsDeclaration + xmlnsPrefix
                    + suBean.getInterfaceName().getNamespaceUri().getLocalPart() + equal + quotes
                    + suBean.getInterfaceName().getNamespaceUri().getNamespaceURI() + quotes
                    + spacer;
        }
        if (!xmlnsDeclaration.contains(suBean.getServiceName().getNamespaceUri().getLocalPart())) {
            xmlnsDeclaration = xmlnsDeclaration + xmlnsPrefix
                    + suBean.getServiceName().getNamespaceUri().getLocalPart() + equal + quotes
                    + suBean.getServiceName().getNamespaceUri().getNamespaceURI() + quotes + spacer;
        }
        return xmlnsDeclaration;
    }

    private static String getProvidedServiceFromSuBean(ProvidedSuBeans suBean, String directoryName) {
        String dots = ":"; //$NON-NLS-1$
        String part0 = "<provides interface-name=\""; //$NON-NLS-1$
        String part1 = "\" service-name=\""; //$NON-NLS-1$
        String part2 = "\" endpoint-name=\""; //$NON-NLS-1$
        String part3 = "\"><extensions:extensions><keyvalue:extension><address>"; //$NON-NLS-1$
        String part4 = "</address></keyvalue:extension></extensions:extensions></provides>"; //$NON-NLS-1$

        String providedService = ""; //$NON-NLS-1$

        String tnsInterface;
        String tnsServiceName;

        if (suBean.getInterfaceName().getNamespaceUri() == null) {
            tnsInterface = suBean.getInterfaceName().getValue();
        } else {
            tnsInterface = suBean.getInterfaceName().getNamespaceUri().getLocalPart();
        }

        if (suBean.getServiceName().getNamespaceUri() == null) {
            tnsServiceName = suBean.getServiceName().getValue();
        } else {
            tnsServiceName = suBean.getServiceName().getNamespaceUri().getLocalPart();
        }

        providedService = providedService + part0 + tnsInterface + dots
                + suBean.getInterfaceName().getValue() + part1 + tnsServiceName + dots
                + suBean.getServiceName().getValue() + part2 + suBean.getEndpointName() + part3
                + directoryName + part4;

        return providedService;
    }

    /**
     * Parse the list and generates the xmlns declaration
     * 
     * @param suList
     *            The SU provided services to consume
     * @return The xmlns declaration
     */
    private static String getXmlnsDeclarationsFromSuList(List<ProvidedSuBean> suList) {
        String xmlnsPrefix = "xmlns:"; //$NON-NLS-1$
        String xmlnsDeclaration = ""; //$NON-NLS-1$
        String quotes = "\""; //$NON-NLS-1$
        String equal = "="; //$NON-NLS-1$
        String spacer = " "; //$NON-NLS-1$

        for (ProvidedSuBean current : suList) {
            if (!xmlnsDeclaration.contains(current.getProvidedBean().getInterfaceName()
                    .getNamespaceUri().getLocalPart())) {
                xmlnsDeclaration = xmlnsDeclaration
                        + xmlnsPrefix
                        + current.getProvidedBean().getInterfaceName().getNamespaceUri()
                                .getLocalPart()
                        + equal
                        + quotes
                        + current.getProvidedBean().getInterfaceName().getNamespaceUri()
                                .getNamespaceURI() + quotes + spacer;
            }
            if (!xmlnsDeclaration.contains(current.getProvidedBean().getServiceName()
                    .getNamespaceUri().getLocalPart())) {
                xmlnsDeclaration = xmlnsDeclaration
                        + xmlnsPrefix
                        + current.getProvidedBean().getServiceName().getNamespaceUri()
                                .getLocalPart()
                        + equal
                        + quotes
                        + current.getProvidedBean().getServiceName().getNamespaceUri()
                                .getNamespaceURI() + quotes + spacer;
            }
        }

        return xmlnsDeclaration;
    }

    /**
     * Populates the consumed services list as it will be printed in jbi
     * descriptor file
     * 
     * @param suList
     *            The consumed Su list
     * @return The populated list for jbi descriptor file
     */
    private static String getConsumedServicesListFromSuList(List<ProvidedSuBean> suList, String directoryName) {
        String dots = ":"; //$NON-NLS-1$
        String part0 = "<consumes interface-name=\""; //$NON-NLS-1$
        String part1 = "\" service-name=\""; //$NON-NLS-1$
        String part2 = "\" endpoint-name=\""; //$NON-NLS-1$
        String part3 = "\"><extensions:extensions><keyvalue:extension><address>"; //$NON-NLS-1$
        String part4 = "</address></keyvalue:extension></extensions:extensions></consumes>"; //$NON-NLS-1$

        String populatedList = ""; //$NON-NLS-1$

        for (ProvidedSuBean current : suList) {
            String tnsInterface;
            String tnsServiceName;

            if (current.getProvidedBean().getInterfaceName().getNamespaceUri() == null) {
                tnsInterface = current.getProvidedBean().getInterfaceName().getValue();
            } else {
                tnsInterface = current.getProvidedBean().getInterfaceName().getNamespaceUri()
                        .getLocalPart();
            }

            if (current.getProvidedBean().getServiceName().getNamespaceUri() == null) {
                tnsServiceName = current.getProvidedBean().getServiceName().getValue();
            } else {
                tnsServiceName = current.getProvidedBean().getServiceName().getNamespaceUri()
                        .getLocalPart();
            }

            populatedList = populatedList + part0 + tnsInterface + dots
                    + current.getProvidedBean().getInterfaceName().getValue() + part1
                    + tnsServiceName + dots + current.getProvidedBean().getServiceName().getValue()
                    + part2 + current.getProvidedBean().getEndpointName() + part3
                    + directoryName + part4;
        }

        return populatedList;
    }

    /**
     * Clean des r�pertoires maven
     * 
     * @param project
     *            Le projectHandler
     */
    public static void processCleanClasspath(IProject project) throws CoreException {

        IFolder sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN + File.separator + Wizards.JAVA));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator + Wizards.MAIN
                + File.separator + Wizards.RESOURCES));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator + Wizards.TEST
                + File.separator + Wizards.JAVA));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator + Wizards.TEST
                + File.separator + Wizards.RESOURCES));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator + Wizards.TEST));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.DIR_TARGET + File.separator
                + Wizards.DIR_CLASSES));
        sourceFolder.delete(true, null);

        sourceFolder = project.getFolder(new Path(Wizards.DIR_TARGET));
        sourceFolder.delete(true, null);

        // Clean du classpath projet pour le SU
        List<IClasspathEntry> classpath = new ArrayList<IClasspathEntry>();
        IJavaProject javaProject = JavaCore.create(project);

        IClasspathEntry[] classpathEntryArray = (IClasspathEntry[]) classpath
                .toArray(new IClasspathEntry[classpath.size()]);

        javaProject.setRawClasspath(classpathEntryArray, null);

        javaProject.setOutputLocation(new Path(File.separator + project.getName() + File.separator
                + Wizards.DIR_BIN), null);

        javaProject.close();

        // Sets the java nature project description
        IProjectDescription description = project.getDescription();
        description.setNatureIds(new String[] {});
        project.setDescription(description, null);
    }

}
