/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.customization;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.petals.tools.plugin.development.wizards.AbstractWizardPage;
import org.objectweb.petals.tools.plugin.development.wizards.NewServiceUnitProjectWizard;
import org.objectweb.petals.tools.plugin.development.wizards.utils.WizardsUtils;

/**
 * @author alouis
 */
public class ExtensionsWizardPage extends AbstractWizardPage {
    
    /**
     * The page NAME = "FileTranfer SU Project"
     */
    public static final String NAME = Messages.ExtensionsWizardPage_0;
    
    private ExtensionComposite composite;

    /**
     * @param pageName
     */
    public ExtensionsWizardPage(String pageName) {
        super(pageName);
        setTitle(pageName);
        setDescription(pageName);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        final GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        composite = new ExtensionComposite(parent, SWT.NULL);
        composite.setLayoutData(gridData);
        
        composite.getTextServiceName().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                boolean validate = validatePage();
                setPageComplete(validate);
            }
            
        });
        
        composite.getTextInterfaceName().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                boolean validate = validatePage();
                setPageComplete(validate);
            }
            
        });
        
        composite.getTextEndpointName().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                boolean validate = validatePage();
                setPageComplete(validate);
            }
            
        });
        
        composite.getComboLinkType().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                boolean validate = validatePage();
                setPageComplete(validate);
            }
            
        });
        
        composite.getDirectoryName().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                boolean validate = validatePage();
                setPageComplete(validate);
            }
            
        });
        
        setControl(composite);
        
        composite.setSuList(WizardsUtils.getProvidedSuFromWorkspace());
        
        boolean validate = validatePage();
        setPageComplete(validate);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
     */
    @Override
    public IWizardPage getNextPage() {
        // We don't want this page to have a next page
        return null;
    }
    
    /**
     * Returns whether this page's controls currently all contain valid values.
     * Only consumed interface name and URL are required.
     * 
     * @return <code>true</code> if all controls are valid, and
     *         <code>false</code> if at least one is invalid
     */
    protected boolean doValidatePage() {
    	boolean result = true;
        
        // Reset
        setMessage(null);
        setErrorMessage(null);
        
        String urlContents = composite.getDirectoryName().getText();
        if ("".equals(urlContents)) { //$NON-NLS-1$
        	setMessage(Messages.ExtensionsWizardPage_2);
        	result = false;
        }

        String serviceName = composite.getTextServiceName().getText();
        if ("".equals(serviceName)) { //$NON-NLS-1$
        	setMessage(Messages.ExtensionsWizardPage_4);
        	result = false;
        }

        String interfaceNameContents = composite.getTextInterfaceName().getText();
        if ("".equals(interfaceNameContents)) { //$NON-NLS-1$
        	setMessage(Messages.ExtensionsWizardPage_6);
        	result = false;
        }
                        
        String endpointNameContents = composite.getTextEndpointName().getText();
        if ("".equals(endpointNameContents)) { //$NON-NLS-1$
        	setMessage(Messages.ExtensionsWizardPage_8);
        	result = false;
        }
            
        if (result) {
        	FileTransferWizardCustomization.setData(composite.getConsumesButton().getSelection(), 
        			composite.getSuList(), 
        			composite.getSuBean(),
        			composite.getDirectoryName().getText());
        }	

        NewServiceUnitProjectWizard wizard = (NewServiceUnitProjectWizard) getWizard();
        if (wizard != null) {
            wizard.setCompleted(result);
        }
        return result;
    }

}
