/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizard.su.filetransfer.customization;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/**
 * Enumeration for the FILETRANSFER component supported versions.
 * 
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public enum Version {

    FT_14("petals-binding-filetransfer", "org.objectweb.petals", "1.4"), FT_20( //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            "petals-bc-filetransfer", "org.objectweb.petals", "2.0"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

    /**
     * The default version to be used
     */
    public static final String DEFAULT = "1.4"; //$NON-NLS-1$

    /**
     * Get all the versions of the SOAP component
     * 
     * @return
     */
    public static List<String> getVersions() {
        final List<String> list = new LinkedList<String>();
        for (final Version version : Version.values()) {
            list.add(version.version);
        }
        Collections.sort(list);
        return list;
    }

    /**
     * Get the version information from its version name.
     * 
     * @param versionName
     * @return null if the version has not been found
     */
    public static Version getVersion(String versionName) {
        for (final Version version : Version.values()) {
            if (version.version.equals(versionName)) {
                return version;
            }
        }
        return null;
    }

    private String name;

    private String group;

    private String version;

    /**
     * Creates a new instance of {@link Version}
     * 
     * @param name
     * @param group
     * @param version
     */
    private Version(String name, String group, String version) {
        this.name = name;
        this.group = group;
        this.version = version;
    }

    /**
     * 
     * @return
     */
    public boolean isDefaultVersion() {
        return this.version.equals(DEFAULT);
    }

    /**
     * @return the name
     */
    public String getName() {
        return this.name;
    }

    /**
     * @return the group
     */
    public String getGroup() {
        return this.group;
    }

    /**
     * @return the version
     */
    public String getVersion() {
        return this.version;
    }
}
