/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizard.su.soap.customization;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.petals.tools.plugin.development.wizards.AbstractWizardPage;
import org.objectweb.petals.tools.plugin.development.wizards.NewServiceUnitProjectWizard;
import org.objectweb.petals.tools.plugin.development.wizards.utils.WizardsUtils;

/**
 * @author LAM
 */
public class ExtensionsWizardPage extends AbstractWizardPage {

    /**
     * The page NAME = "Soap SU Project"
     */
    public static final String NAME = Messages.ExtensionsWizardPage_0;

    private ExtensionComposite composite;

    /**
     * Creates a new instance of {@link ExtensionsWizardPage}
     * 
     * @param pageName
     */
    public ExtensionsWizardPage(final String pageName) {
        super(pageName);
        this.setTitle(pageName);
        this.setDescription(pageName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(final Composite parent) {
        final GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        this.composite = new ExtensionComposite(parent, SWT.NULL);
        this.composite.setLayoutData(gridData);

        // ModifyListener modificationListener = new ModifyListener() {
        // public void modifyText(final ModifyEvent e) {
        // final boolean validate = ExtensionsWizardPage.this.validatePage();
        // ExtensionsWizardPage.this.setPageComplete(validate);
        // }
        // };

        this.composite.getTextConsumesServiceName().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                final boolean validate = ExtensionsWizardPage.this.validatePage();
                ExtensionsWizardPage.this.setPageComplete(validate);
            }
        });
        this.composite.getTextInterfaceName().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                final boolean validate = ExtensionsWizardPage.this.validatePage();
                ExtensionsWizardPage.this.setPageComplete(validate);
            }
        });
        this.composite.getTextEndpointName().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                final boolean validate = ExtensionsWizardPage.this.validatePage();
                ExtensionsWizardPage.this.setPageComplete(validate);
            }
        });
        this.composite.getComboLinkType().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                final boolean validate = ExtensionsWizardPage.this.validatePage();
                ExtensionsWizardPage.this.setPageComplete(validate);
            }
        });
        this.composite.getTextUrl().addModifyListener(new ModifyListener() {
            public void modifyText(final ModifyEvent e) {
                final boolean validate = ExtensionsWizardPage.this.validatePage();
                ExtensionsWizardPage.this.setPageComplete(validate);
            }
        });

        this.setControl(this.composite);

        this.composite.setSuList(WizardsUtils.getProvidedSuFromWorkspace());

        final boolean validate = this.validatePage();
        this.setPageComplete(validate);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
     */
    @Override
    public IWizardPage getNextPage() {
        // We don't want this page to have a next page
        return null;
    }

    /**
     * Returns whether this page's controls currently all contain valid values.
     * Only consumed interface name and URL are required.
     * 
     * @return <code>true</code> if all controls are valid, and
     *         <code>false</code> if at least one is invalid
     */
    protected boolean doValidatePage() {
        boolean valid = true;

        // Reset
        this.setMessage(null);
        this.setErrorMessage(null);

        if (this.composite.getSuList().size() > 0) {
            final String urlContents = this.composite.getTextUrl().getText();
            if (urlContents.equals("")) { //$NON-NLS-1$
                this.setErrorMessage(Messages.ExtensionsWizardPage_2);
                valid = false;
            }

            final String interfaceNameContents = this.composite.getTextInterfaceName().getText();
            if (interfaceNameContents.equals("")) { //$NON-NLS-1$
                this.setErrorMessage(Messages.ExtensionsWizardPage_4);
                valid = false;
            }

            if (valid) {
                SoapWizardCustomization.setSuList(this.composite.getSuList());
                SoapWizardCustomization.setTargetComponentVersion(this.composite
                        .getTextComponentVersion());
            }
        } else {
            this.setMessage(Messages.ExtensionsWizardPage_5);
            valid = false;
        }

        NewServiceUnitProjectWizard wizard = (NewServiceUnitProjectWizard) getWizard();
        if (wizard != null) {
            wizard.setCompleted(valid);
        }
        return valid;
    }

}
