/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: BusinessCommon.java,v 1.1 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.constants.Wizards;
import org.objectweb.petals.tools.plugin.development.wizards.interfaces.IWizardCustomization;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * @author LAM
 */
public final class BusinessCommon {

    private BusinessCommon() {

    }

    /**
     * Create the directories denoted by the given path. A path equals to
     * src/main/java will create src/main/java in the {@link IProject}.
     * 
     * @param project
     *            the project to create directories into
     * @param path
     *            The full path to create. The intermediate directories will be
     *            created if they do not exists. The path is like
     *            'src/main/java' where the directory separator is the default
     *            platform one ('/')
     */
    public static void createDirectories(IProject project, String path)
            throws CoreException {
        StringTokenizer tokenizer = new StringTokenizer(path, File.separator);
        String target = ""; //$NON-NLS-1$
        while (tokenizer.hasMoreElements()) {
            target = target + tokenizer.nextToken() + File.separator;
            IFolder folder = project.getFolder(new Path(target));
            if (!folder.exists()) {
                folder.create(false, true, null);
            }
        }
    }

    /**
     * Creates the Java project tree structure for BC, SE and SU
     * 
     * @param project
     *            The java project
     * @throws CoreException
     *             CoreException
     */
    public static void createJavaTreeStructure(IProject project)
            throws CoreException {
        // Tree project creation
        IFolder sourceFolder = project.getFolder(new Path(Wizards.SRC));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN + File.separator + Wizards.JAVA));
        sourceFolder.create(false, true, null);
        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN + File.separator + Wizards.RESOURCES));
        sourceFolder.create(false, true, null);
        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN + File.separator + Wizards.JBI));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.TEST));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.TEST + File.separator + Wizards.JAVA));
        sourceFolder.create(false, true, null);
        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.TEST + File.separator + Wizards.RESOURCES));
        sourceFolder.create(false, true, null);

        if (Activator.getDefault().getPluginPreferences().getBoolean(
                ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            sourceFolder = project.getFolder(new Path(Wizards.DIR_TARGET));
            sourceFolder.create(false, true, null);
            sourceFolder = project.getFolder(new Path(Wizards.DIR_TARGET
                    + File.separator + Wizards.DIR_CLASSES));
            sourceFolder.create(false, true, null);
        } else {
            sourceFolder = project.getFolder(new Path(Wizards.DIR_BIN));
            sourceFolder.create(false, true, null);
        }
    }

    /**
     * Creates the tree structure for SA and SL projects
     * 
     * @param project
     *            The project
     * @throws CoreException
     *             CoreException
     */
    public static void createTreeStructure(IProject project)
            throws CoreException {
        // Tree project creation
        IFolder sourceFolder = project.getFolder(new Path(Wizards.SRC));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN));
        sourceFolder.create(false, true, null);

        sourceFolder = project.getFolder(new Path(Wizards.SRC + File.separator
                + Wizards.MAIN + File.separator + Wizards.JBI));
        sourceFolder.create(false, true, null);
    }

    /**
     * Populates the customs extensions list for the component type of selected
     * wizard
     * 
     * @param componentType
     *            The component type (BC, SE, SU, SA or SL)
     * @return The WizardCustomizationModel list for this component type
     */
    public static List<WizardCustomizationModel> searchForCustomExtensions(
            String componentType) {
        IConfigurationElement[] decls = Platform
                .getExtensionRegistry()
                .getConfigurationElementsFor(
                        "org.objectweb.petals.tools.plugin.development.customwizard"); //$NON-NLS-1$

        List<WizardCustomizationModel> list = new ArrayList<WizardCustomizationModel>();
        IWizardCustomization iwc;

        try {
            for (IConfigurationElement current : decls) {
                if (componentType.equals(current.getAttribute(Messages.BusinessCommon_2))) {
                    WizardCustomizationModel newType = new WizardCustomizationModel();
                    newType.setSpecificProjectType(current
                            .getAttribute(Messages.BusinessCommon_3));
                    iwc = (IWizardCustomization) current
                            .createExecutableExtension(Messages.BusinessCommon_4);
                    newType.setSpecificCustomization(iwc);
                    list.add(newType);
                }
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }

        return list;
    }

}
