/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

/**
 * The composite used to choose the component framework name and version to use.
 * 
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public class ComponentFrameworkComposite extends Composite {
    private Group groupComponentFramework = null;

    private Label labelComponentFrameworkGroup = null;

    private Combo comboComponentFramework = null;

    private Label labelComponentFrameworkGroupId = null;

    private Text textComponentFrameworkGroupId = null;

    private Label labelComponentFrameworkVersion = null;

    private Combo comboComponentFrameworkVersion = null;

    private Label labelComponentType;

    private Button buttonBinding;

    private Button buttonEngine;
    
    private Composite buttonComposite; 

    public static final String[] FRAMEWORK_NAMES = {
            Messages.ComponentFrameworkComposite_0, Messages.ComponentFrameworkComposite_1 };

    public static final String[] CDK_VERSIONS = { "1.2", "1.3", "1.4", "1.4.1", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
            "1.4.2", "2.0-beta1" }; //$NON-NLS-1$ //$NON-NLS-2$

    public static final String[] PCF_VERSIONS = { "2.0", "2.0.1" }; //$NON-NLS-1$ //$NON-NLS-2$

    /** Objects listening for changements on the combo defining CDK versions. */
    private ArrayList<Object> ExternalListeners = new ArrayList<Object>();
    
    private static Map<String, String[]> CDK_MAP = new HashMap<String, String[]>();
    static {
        CDK_MAP.put(FRAMEWORK_NAMES[0], PCF_VERSIONS);
        CDK_MAP.put(FRAMEWORK_NAMES[1], CDK_VERSIONS);
    }

    /**
     * Creates a new instance of {@link ComponentFrameworkComposite}
     * 
     * @param parent
     * @param style
     */
    public ComponentFrameworkComposite(Composite parent, int style) {
        super(parent, style);
        initialize();
    }

    /**
     * 
     */
    private void initialize() {
        setLayout(new GridLayout());
        createJBIComponent();
    }

    /**
     * This method initializes groupMavenId
     * 
     */
    private void createJBIComponent() {
        GridData gridData11 = new GridData();
        gridData11.horizontalAlignment = GridData.FILL;
        gridData11.grabExcessHorizontalSpace = true;
        gridData11.verticalAlignment = GridData.CENTER;

        GridData gridData1 = new GridData();
        gridData1.horizontalAlignment = GridData.FILL;
        gridData1.grabExcessHorizontalSpace = true;
        gridData1.verticalAlignment = GridData.CENTER;

        GridData gridData2 = new GridData();
        gridData2.horizontalAlignment = GridData.FILL;
        gridData2.grabExcessHorizontalSpace = true;
        gridData2.verticalAlignment = GridData.CENTER;

        GridData gridDataChoiceButton = new GridData();
        gridDataChoiceButton.horizontalAlignment = GridData.FILL;
        gridDataChoiceButton.grabExcessHorizontalSpace = true;
        gridDataChoiceButton.verticalAlignment = GridData.CENTER;

        GridData gridData3 = new GridData();
        gridData3.horizontalAlignment = GridData.FILL;
        gridData3.grabExcessHorizontalSpace = true;
        gridData3.verticalAlignment = GridData.CENTER;

        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = GridData.CENTER;

        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;

        groupComponentFramework = new Group(this, SWT.NONE);
        groupComponentFramework.setText(Messages.ComponentFrameworkComposite_10);
        groupComponentFramework.setLayoutData(gridData11);
        groupComponentFramework.setLayout(gridLayout);

        // component type
        // BC of SE choice
        labelComponentType = new Label(groupComponentFramework, SWT.NONE);
        labelComponentType.setText(Messages.ComponentFrameworkComposite_11);
        buttonComposite = new Composite(groupComponentFramework,
                SWT.NONE);
        buttonComposite.setLayout(new GridLayout(2, true));
        
        Listener listener = new Listener() {
            public void handleEvent(Event e) {
                Control[] children = buttonComposite.getChildren();
                for (int i = 0; i < children.length; i++) {
                    Control child = children[i];
                    // unselect others
                    if (e.widget != child && child instanceof Button
                            && (child.getStyle() & SWT.RADIO) != 0) {
                        ((Button) child).setSelection(false);
                    }
                }
                if (!((Button) e.widget).getSelection()) {
                    ((Button) e.widget).setSelection(true);
                }                
                
                // notify listeners
                if( e.widget instanceof Button ) {
                    tellExternalListeners();
                }
            }
        };

        buttonBinding = new Button(buttonComposite, SWT.RADIO);
        buttonBinding.setText(Messages.ComponentFrameworkComposite_12);
        buttonBinding.setSelection(true);
        buttonBinding.setLayoutData(gridDataChoiceButton);
        buttonBinding.addListener(SWT.Selection, listener);
        // TODO: review the way these buttons are managed
        buttonBinding.setData( "BC" );  // "BC" as data to differentiate it from the other one //$NON-NLS-1$

        buttonEngine = new Button(buttonComposite, SWT.RADIO);
        buttonEngine.setText(Messages.ComponentFrameworkComposite_13);
        buttonEngine.setSelection(false);
        buttonEngine.setLayoutData(gridDataChoiceButton);
        buttonEngine.addListener(SWT.Selection, listener);
        // TODO: review the way these buttons are managed
        buttonEngine.setData( "SE" );  // "SE" as data to differentiate it from the other one //$NON-NLS-1$

        // Framework Name
        labelComponentFrameworkGroup = new Label(groupComponentFramework,
                SWT.NONE);
        labelComponentFrameworkGroup.setText(Messages.ComponentFrameworkComposite_14);
        comboComponentFramework = new Combo(groupComponentFramework, SWT.BORDER);
        comboComponentFramework.setItems(FRAMEWORK_NAMES);

        comboComponentFramework.addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }

            public void widgetSelected(SelectionEvent e) {
                comboComponentFrameworkVersion.setItems(CDK_MAP
                        .get(comboComponentFramework.getText()));
                comboComponentFrameworkVersion.select(0);
            }
        });

        comboComponentFramework.select(0);
        comboComponentFramework.setLayoutData(gridData);

        labelComponentFrameworkGroupId = new Label(groupComponentFramework,
                SWT.NONE);
        labelComponentFrameworkGroupId.setText(Messages.ComponentFrameworkComposite_15);
        textComponentFrameworkGroupId = new Text(groupComponentFramework,
                SWT.BORDER);
        textComponentFrameworkGroupId.setText("org.objectweb.petals"); //$NON-NLS-1$
        textComponentFrameworkGroupId.setLayoutData(gridData2);

        labelComponentFrameworkVersion = new Label(groupComponentFramework,
                SWT.NONE);
        
        // combo associated with the CDK versions
        labelComponentFrameworkVersion.setText(Messages.ComponentFrameworkComposite_17);
        comboComponentFrameworkVersion = new Combo(groupComponentFramework,
                SWT.BORDER);
        comboComponentFrameworkVersion
                .setItems(CDK_MAP.get(FRAMEWORK_NAMES[0]));
        comboComponentFrameworkVersion.setLayoutData(gridData3);
        comboComponentFrameworkVersion.select(0);
        
        // add a listener to detect changes in the combo
        comboComponentFrameworkVersion.addSelectionListener( new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
            }

            public void widgetSelected(SelectionEvent e) {                
                // tell the listeners a change happened
                tellExternalListeners ();
            }            
        });
    }

    public String getComponentFramework() {
        return comboComponentFramework.getText();
    }
    
    public boolean isBindingComponent() {
        return buttonBinding.getSelection();
    }

    public String getComponentFrameworkVersion() {
        return comboComponentFrameworkVersion.getText();
    }

    public void setComponentFrameworkVersion(String text) {
        comboComponentFrameworkVersion.setText(text);
    }

    /**
     * @return the textComponentFrameworkGroupId
     */
    public String getComponentFrameworkGroupId() {
        return this.textComponentFrameworkGroupId.getText();
    }
    
    /**
     * @param WannaListen the object that wants to listen to the version combo.
     */    
    public void addExternalListener( Object wannaListen ) {
        ExternalListeners.add( wannaListen );      
    }
    
    /**
     * @param wannaLeave the object that do not want to listen to the version combo anymore.
     */
    public void removeExternalListener( Object wannaLeave ) {
        ExternalListeners.remove( wannaLeave );
    }
    
    /**
     * When a cdk version is selected in the combo, or when a new component type
     * is chosen, this method is called.
     * 
     * It notifies listeners which registered through addExternalListener
     * that a change was done. This method also provides to the listeners
     * the selected version.
     */
    private void tellExternalListeners () {
        for( Object o : ExternalListeners ) {
            // here, we cast objects to classes allowing the notification
            // the changes only have to be done in the 'tellExternalListeners' methods
            if( o instanceof NewComponentProjectWizardComposite ) {
                NewComponentProjectWizardComposite ncpw =
                    (NewComponentProjectWizardComposite) o;
                ncpw.cdkVersionOrComponentTypeHasChanged();         
            }
        }
    }
}
