/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ExtensionsComposite.java,v 1.1 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * Composite for extensions
 * 
 * @author LAM
 */
public class ExtensionsComposite extends Composite {

    private static final int MARGIN_HEIGHT = 55;

    private Group specificGroup = null;

    private Label label = null;

    private Combo specificCombo = null;

    private Composite composite = null;

    /**
     * Default constructor
     * 
     * @param parent
     *            The parent Composite
     * @param style
     *            The style
     */
    public ExtensionsComposite(final Composite parent, final int style) {
        super(parent, style);
        this.initialize();
    }

    /**
     * This method initializes composite
     * 
     */
    private void createComposite() {
        final GridData gridData2 = new GridData();
        gridData2.horizontalAlignment = GridData.FILL;
        gridData2.grabExcessHorizontalSpace = true;
        gridData2.grabExcessVerticalSpace = false;
        gridData2.heightHint = ExtensionsComposite.MARGIN_HEIGHT;
        gridData2.verticalAlignment = GridData.CENTER;
        final GridLayout gridLayout3 = new GridLayout();
        gridLayout3.marginWidth = 0;
        gridLayout3.marginHeight = 0;
        
        this.composite = new Composite(this, SWT.NONE);
        this.createSpecificGroup();
        this.composite.setLayoutData(gridData2);
        this.composite.setLayout(gridLayout3);
    }

    /**
     * This method initializes specificCombo
     * 
     */
    private void createSpecificCombo() {
        final GridData gridData = new GridData();
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = GridData.CENTER;
        gridData.horizontalAlignment = GridData.FILL;
        this.specificCombo = new Combo(this.specificGroup, SWT.NONE);
        this.specificCombo.setLayoutData(gridData);
    }

    /**
     * This method initializes specificGroup
     * 
     */
    private void createSpecificGroup() {
        final GridData gridData1 = new GridData();
        gridData1.horizontalAlignment = GridData.FILL;
        gridData1.grabExcessHorizontalSpace = true;
        gridData1.grabExcessVerticalSpace = true;
        gridData1.heightHint = -1;
        gridData1.horizontalIndent = 0;
        gridData1.verticalAlignment = GridData.END;
        final GridLayout gridLayout1 = new GridLayout();
        gridLayout1.numColumns = 2;
        final GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        this.specificGroup = new Group(this.composite, SWT.NONE);
        this.specificGroup.setText(Messages.ExtensionsComposite_0);
        this.specificGroup.setLayoutData(gridData1);
        this.specificGroup.setLayout(gridLayout1);
        this.label = new Label(this.specificGroup, SWT.NONE);
        this.label.setText(Messages.ExtensionsComposite_1);
        this.createSpecificCombo();
        this.specificGroup.setLayout(gridLayout);
    }

    /**
     * Fill the combo with specific project types
     * 
     * @param list
     *            The WizardCustomizationModel list for this Composite
     */
    public void fillSpecificCombo(final List<WizardCustomizationModel> list) {
        if ((list != null) && (!list.isEmpty())) {
            for (final WizardCustomizationModel current : list) {
                this.specificCombo.add(current.getSpecificProjectType());
            }
        }
    }

    /**
     * Returns the Extensions combo value
     * 
     * @return The selected Extension
     */
    public String getComboValue() {
        return this.specificCombo.getText();
    }

    /**
     * @return the specificCombo
     */
    public Combo getSpecificCombo() {
        return this.specificCombo;
    }

    /**
     * This method initializes this
     * 
     */
    private void initialize() {
        final GridLayout gridLayout2 = new GridLayout();
        this.createComposite();
        this.setLayout(gridLayout2);
    }

}
