/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewBindingComponentProjectWizard.java,v 1.12 2007/04/25 14:51:30 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.actions.JobWizardProjectCreation;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.constants.ProjectType;
import org.objectweb.petals.tools.plugin.development.constants.Wizards;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * New Petals Binding Component Wizard. Creates a new BC project in user's
 * workspace.
 * 
 * Extends BasicNewProjectResourceWizard but overrides some functions to match a
 * custom Binding Component Java project structure.
 * 
 * @author LAM
 */
public class NewComponentProjectWizard extends
        BasicNewProjectResourceWizard {

    private List<WizardCustomizationModel> specificTypesList;

    private int specificTypesIndex;

    /**
     * Main page of the wizard
     */
    private NewComponentProjectWizardPage mainPage;

    /**
     * Creates a new instance of {@link NewComponentProjectWizard}
     * 
     */
    public NewComponentProjectWizard() {

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        super.init(workbench, selection);

        setWindowTitle(Messages.NewComponentProjectWizard_0);
        specificTypesList = BusinessCommon
                .searchForCustomExtensions(Wizards.WIZARD_CUSTOM_BC);
        for (WizardCustomizationModel current : BusinessCommon
                .searchForCustomExtensions(Wizards.WIZARD_CUSTOM_SE)) {
            specificTypesList.add(current);
        }
        specificTypesIndex = -1;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#addPages()
     */
    public void addPages() {
        mainPage = new NewComponentProjectWizardPage(Messages.NewComponentProjectWizard_1, this);
        addPage(mainPage);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#performFinish()
     */
    public boolean performFinish() {
        final JobWizardProjectCreation job = new JobWizardProjectCreation();
        String name = mainPage.getProjectName();
        String group = name;
        String tmp = mainPage.getBcComposite().getMavenComposite().getNamespace();
        if (tmp != null && tmp.trim().length() > 0) {
            group = tmp;
        }
        
        // determine if we should create a component file or not
        boolean createComponentFile = mainPage.isComponentToGenerate();        
        job.setCreateComponentFile( createComponentFile );

        job.setNewProjectHandle(mainPage.getProjectHandle());

        job.setProjectType(getComponentType());
        job.setUseDefaultUri(mainPage.useDefaults());
        job.setLocationUri(mainPage.getLocationURI());
        job.setSpecificTypesList(specificTypesList);
        job.setSpecificTypesIndex(specificTypesIndex);

        if (Activator.getDefault().getPluginPreferences().getBoolean(
                ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            job.setMavenArtifactId(name);
            job.setMavenGroupId(group);
            job.setMavenVersion(mainPage.getBcComposite().getMavenComposite()
                    .getVersion());
            job.setMavenPackaging(Constants.ArtifactsType.JBI_COMPONENT);

            // set the CDK information
            String artifactName = mainPage.getBcComposite()
                    .getComponentFrameworkComposite().getComponentFramework();
            String artifactGroup = mainPage.getBcComposite()
                    .getComponentFrameworkComposite()
                    .getComponentFrameworkGroupId();
            String artifactVersion = mainPage.getBcComposite()
                    .getComponentFrameworkComposite()
                    .getComponentFrameworkVersion();
            ArtifactInformation cdkInfo = new ArtifactInformation(artifactName,
                    artifactGroup, artifactVersion, null);
            job.setCdkArtifact(cdkInfo);

        }

        job.setRule(ResourcesPlugin.getWorkspace().getRoot());
        job.setUser(true);
        job.schedule();

        return true;
    }

    /**
     * Get the component type to create
     * 
     * @return
     */
    private ProjectType getComponentType() {
        ProjectType result = ProjectType.ENGINE;
        if (mainPage.getBcComposite().getComponentFrameworkComposite()
                .isBindingComponent()) {
            result = ProjectType.BINDING;
        }
        return result;
    }

    /**
     * @return the mainPage
     */
    public NewComponentProjectWizardPage getMainPage() {
        return mainPage;
    }

    /**
     * @return the specificTypesList
     */
    public List<WizardCustomizationModel> getSpecificTypesList() {
        return specificTypesList;
    }

    /**
     * @return the specificTypesIndex
     */
    public int getSpecificTypesIndex() {
        return specificTypesIndex;
    }

    /**
     * @param specificTypesIndex
     *            the specificTypesIndex to set
     */
    public void setSpecificTypesIndex(int specificTypesIdex) {
        this.specificTypesIndex = specificTypesIdex;
    }
}
