/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewBindingComponentProjectWizardComposite.java,v 1.5 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.wizards;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;

/**
 * @author LAM
 * @author Vincent Zurczak
 */
public class NewComponentProjectWizardComposite extends Composite {

    private MavenIdComposite mavenComposite = null;
    private ComponentFrameworkComposite componentFrameworkComposite = null;
    private ExtensionsComposite extensionsComposite = null;
    private NewComponentProjectWizard wizard;
    /** The composite used to determine which artifacts must be generated. */
    private GeneratedArtifactsComposite generatedArtifactsComposite;
    /** */
    private Preferences prefs;
    

    /**
     * @param parent The parent Composite
     * @param style The SWT style
     */
    public NewComponentProjectWizardComposite(Composite parent, int style,
            NewComponentProjectWizard wizard) {
        super(parent, style);
        this.wizard = wizard;
        prefs = Activator.getDefault().getPluginPreferences();
        initialize();
    }

    /**
     * This method initializes this
     *
     */
    private void initialize() {
        this.setLayout(new GridLayout());
        createExtensionsComposite();
        extensionsComposite.fillSpecificCombo(wizard.getSpecificTypesList());
    }

    /**
     * This method initializes extensionsComposite
     *
     */
    private void createExtensionsComposite() {
        GridLayout gridLayout = new GridLayout();
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;

        // --> create layout data for the maven part of the wizard
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = GridData.CENTER;
        
        // create the maven part of the wizard
        if (prefs.getBoolean(ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            mavenComposite = new MavenIdComposite(this, SWT.NONE);
            mavenComposite.setLayout(gridLayout);
            mavenComposite.setLayoutData(gridData);
        }

        // --> create layout data for the cdk part of the wizard
        GridData gridData2 = new GridData();
        gridData2.horizontalAlignment = GridData.FILL;
        gridData2.grabExcessHorizontalSpace = true;
        gridData2.verticalAlignment = GridData.CENTER;
        
        // create the cdk part of the wizard
        componentFrameworkComposite = new ComponentFrameworkComposite(this, SWT.NONE);
        componentFrameworkComposite.setLayoutData(gridData2);
        componentFrameworkComposite.setLayout(gridLayout);
        // register itself as a listener - to be notified when another version of the CDK is selected
        componentFrameworkComposite.addExternalListener( this );
        
        // --> create layout data for the extension part of the wizard
        GridData gridData3 = new GridData();
        gridData3.horizontalAlignment = GridData.FILL;
        gridData3.grabExcessHorizontalSpace = true;
        gridData3.verticalAlignment = GridData.CENTER;

        // create the extension part of the wizard
        extensionsComposite = new ExtensionsComposite(this, SWT.NONE);
        extensionsComposite.setLayoutData(gridData3);
        extensionsComposite.setLayout(gridLayout);
        
        // --> create "generated artifacts" part of the wizard 
        generatedArtifactsComposite = new GeneratedArtifactsComposite( this, SWT.NONE );
        
        // define its layout data
        GridData gacGridData = new GridData();
        gacGridData.horizontalAlignment = GridData.FILL;
        gacGridData.grabExcessHorizontalSpace = true;
        gacGridData.verticalAlignment = GridData.CENTER;
        generatedArtifactsComposite.setLayoutData( gacGridData );
        
        // override its layout
        generatedArtifactsComposite.setLayout( gridLayout );
    }

    /**
     * @return the specificCombo
     */
    public Combo getSpecificCombo() {
        return extensionsComposite.getSpecificCombo();
    }

    public MavenIdComposite getMavenComposite() {
        return mavenComposite;
    }

    /**
     * @return the componentFrameworkComposite
     */
    public ComponentFrameworkComposite getComponentFrameworkComposite() {
        return this.componentFrameworkComposite;
    }

    /**
     * Update generatedArtifacts part of the wizard when a version of the cdk or
     * a new type of component is selected.
     *  
     * Pre-condition: generatedArtifactsComposite is neither null, nor disposed.
     *      "generatedArtifactsComposite != null && !generatedArtifactsComposite.isDisposed()"
     * Pre-condition: same thing for 'componentFrameworkComposite'.
     */
    public void cdkVersionOrComponentTypeHasChanged () {
        generatedArtifactsComposite.changeComponentType( 
                componentFrameworkComposite.isBindingComponent(), 
                componentFrameworkComposite.getComponentFrameworkVersion());
    }

    /**
     * @return true if the "generate default artifacts" checkbox is checked when the user clicks on the "finish" button.
     */
    public boolean isComponentToGenerate() {
        return generatedArtifactsComposite.isGenerateDefaultComponent();
    }
}  //  @jve:decl-index=0:visual-constraint="63,180"
