/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewBindingComponentProjectWizardPage.java,v 1.8 2007/04/25 14:18:41 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * @author LAM
 */
public class NewComponentProjectWizardPage extends WizardNewProjectCreationPage {

    private IWizardPage nextPage;
    private NewComponentProjectWizard wizard;

    /**
     * The BC wizard page Composite
     */
    private NewComponentProjectWizardComposite bcComposite;

    /**
     * @param pageName
     */
    public NewComponentProjectWizardPage(String pageName,
            NewComponentProjectWizard wizard) {
        super(pageName);
        this.wizard = wizard;
        setTitle(pageName);
        setDescription(Messages.NewComponentProjectWizardPage_0);
        setPageComplete(false);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        
        bcComposite = new NewComponentProjectWizardComposite(
                (Composite) getControl(), SWT.NONE, wizard);
        bcComposite.setLayoutData(gridData);

        bcComposite.getSpecificCombo().addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) { }
            public void widgetSelected(SelectionEvent e) {
                int idSelItem = ((Combo) e.getSource()).getSelectionIndex();
                String nameSelItem = ((Combo) e.getSource()).getItem(idSelItem);
                nextPage = null;
                wizard.setSpecificTypesIndex(-1);
                int i = 0;
                for (WizardCustomizationModel current : wizard.getSpecificTypesList()) {
                    if (current.getSpecificProjectType().equals(nameSelItem)) {
                        nextPage =
                            current.getSpecificCustomization().getExtensionsWizardPage();
                        wizard.setSpecificTypesIndex(i);
                        wizard.addPage(nextPage);
                    }
                    i++;
                }
                setPageComplete(validatePage());
            }
        });

        bcComposite.getSpecificCombo().addModifyListener(new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                String nameSelItem = ((Combo) e.getSource()).getText();
                wizard.setSpecificTypesIndex(-1);
                nextPage = null;
                int i = 0;
                for (WizardCustomizationModel current : wizard.getSpecificTypesList()) {
                    if (current.getSpecificProjectType().equals(nameSelItem)) {
                        nextPage =
                            current.getSpecificCustomization().getExtensionsWizardPage();
                        wizard.setSpecificTypesIndex(i);
                        wizard.addPage(nextPage);
                    }
                    i++;
                }
                setPageComplete(validatePage());
            }

        });
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#getProjectHandle()
     */
    @Override
    public IProject getProjectHandle() {
        return super.getProjectHandle();
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
     */
    @Override
    public IWizardPage getNextPage() {
        return nextPage;
    }

    public NewComponentProjectWizardComposite getBcComposite() {
        return bcComposite;
    }

    /**
     * @return true if the "generate default artifacts" checkbox is checked when the user clicks on the "finish" button.
     */
    public boolean isComponentToGenerate() {
        return bcComposite.isComponentToGenerate();
    }

}
