/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewServiceAssemblyProjectWizardComposite.java,v 1.5 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;

/**
 * Composite for SA project creation
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class NewServiceAssemblyProjectWizardComposite extends Composite {

    private static final int TABLE_WIDTH = 200;

    private static final int TABLE_HEIGHT = 150;

    private Composite compositeSuSelection = null;

    private Group groupSuSelec = null;

    private Label labelSelectSu = null;

    private Table tableSuList = null;

    /**
     * Creates a new instance of
     * {@link NewServiceAssemblyProjectWizardComposite}
     * 
     * @param parent
     * @param style
     */
    public NewServiceAssemblyProjectWizardComposite(final Composite parent,
            final int style) {
        super(parent, style);
        this.initialize();
    }

    /**
     * This method initializes compositeSuSelection
     * 
     */
    private void createCompositeSuSelection() {
        final GridData gridData4 = new GridData();
        gridData4.horizontalAlignment = GridData.FILL;
        gridData4.grabExcessHorizontalSpace = true;

        this.compositeSuSelection = new Composite(this, SWT.NONE);
        this.compositeSuSelection.setLayout(new GridLayout());
        this.compositeSuSelection.setLayoutData(gridData4);
        this.createGroupSuSelec();
    }

    /**
     * This method initializes groupSuSelec
     * 
     */
    private void createGroupSuSelec() {
        final GridData gridData3 = new GridData();
        gridData3.heightHint = NewServiceAssemblyProjectWizardComposite.TABLE_HEIGHT;
        gridData3.horizontalAlignment = GridData.BEGINNING;
        gridData3.verticalAlignment = GridData.FILL;
        gridData3.grabExcessVerticalSpace = true;
        gridData3.grabExcessHorizontalSpace = false;
        gridData3.widthHint = NewServiceAssemblyProjectWizardComposite.TABLE_WIDTH;
        final GridData gridData1 = new GridData();
        gridData1.horizontalAlignment = GridData.FILL;
        gridData1.grabExcessHorizontalSpace = true;
        gridData1.verticalAlignment = GridData.CENTER;

        final GridLayout gridLayout1 = new GridLayout();
        this.groupSuSelec = new Group(this.compositeSuSelection, SWT.NONE);
        this.groupSuSelec.setText(Messages.NewServiceAssemblyProjectWizardComposite_0);
        this.groupSuSelec.setLayout(gridLayout1);
        this.groupSuSelec.setLayoutData(gridData1);

        this.labelSelectSu = new Label(this.groupSuSelec, SWT.NONE);
        this.labelSelectSu.setText(Messages.NewServiceAssemblyProjectWizardComposite_1);
        this.labelSelectSu.setLayoutData(gridData1);
        this.tableSuList = new Table(this.groupSuSelec, SWT.BORDER
                | SWT.V_SCROLL | SWT.H_SCROLL | SWT.CHECK);
        this.tableSuList.setLayoutData(gridData3);
        this.tableSuList.setHeaderVisible(false);
        this.tableSuList.setLinesVisible(false);

        final TableColumn colonne1 = new TableColumn(this.tableSuList, SWT.LEFT);
        colonne1.setWidth(NewServiceAssemblyProjectWizardComposite.TABLE_WIDTH);
    }

    /**
     * Fill SU selection list
     * 
     * @param suList
     */
    public void fillSuTable(final List<ArtifactInformation> suList) {
        this.tableSuList.removeAll();

        for (final ArtifactInformation current : suList) {
            final TableItem line = new TableItem(this.tableSuList, SWT.NONE);
            line.setText(current.getName());
            // add information as line data
            line.setData(current);
        }
    }

    /**
     * @return the list of selected SU
     */
    public List<ArtifactInformation> getSelectedSuList() {
        final List<ArtifactInformation> resultList = new ArrayList<ArtifactInformation>();

        if (this.tableSuList.getItemCount() > 0) {
            for (final TableItem current : this.tableSuList.getItems()) {
                if (current.getChecked()) {
                    resultList.add((ArtifactInformation) current.getData());
                }
            }
        }
        return resultList;
    }

    /**
     * Initialization method
     */
    private void initialize() {
        final GridLayout gridLayout = new GridLayout();
        gridLayout.marginWidth = 0;

        this.setLayout(gridLayout);
        this.createCompositeSuSelection();
    }

}
