/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewServiceAssemblyProjectWizardPage.java,v 1.5 2007/04/16 14:43:44 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.WizardsUtils;

/**
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class NewServiceAssemblyProjectWizardPage extends
        WizardNewProjectCreationPage {

    /**
     * The SU wizard page Composite
     */
    private NewServiceAssemblyProjectWizardComposite saComposite;

    /**
     * The maven composite
     */
    private MavenIdComposite mavenComposite = null;

    /**
     * Eclipse preferences
     */
    private Preferences prefs;

    /**
     * Creates a new instance of {@link NewServiceAssemblyProjectWizardPage}
     * 
     * @param pageName
     */
    public NewServiceAssemblyProjectWizardPage(String pageName) {
        super(pageName);
        setTitle(pageName);
        setDescription(Messages.NewServiceAssemblyProjectWizardPage_0);
        setPageComplete(false);
        prefs = Activator.getDefault().getPluginPreferences();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);

        if (prefs.getBoolean(ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            GridData gridData2 = new GridData();
            gridData2.horizontalAlignment = GridData.FILL;
            gridData2.grabExcessHorizontalSpace = true;
            mavenComposite = new MavenIdComposite((Composite) getControl(),
                    SWT.NONE);
            mavenComposite.setLayoutData(gridData2);
        }

        GridData gd = new GridData();
        gd.horizontalAlignment = GridData.FILL;
        gd.grabExcessHorizontalSpace = true;
        gd.verticalAlignment = GridData.FILL;
        gd.grabExcessVerticalSpace = true;
        saComposite = new NewServiceAssemblyProjectWizardComposite(
                (Composite) getControl(), SWT.NONE);

        saComposite.setLayoutData(gd);
        saComposite.fillSuTable(WizardsUtils.getSuFromWorkspace());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#getProjectHandle()
     */
    @Override
    public IProject getProjectHandle() {
        return super.getProjectHandle();
    }

    /**
     * @return the saComposite
     */
    public NewServiceAssemblyProjectWizardComposite getSaComposite() {
        return saComposite;
    }

    /**
     * @return the mavenComposite
     */
    public MavenIdComposite getMavenComposite() {
        return mavenComposite;
    }

}
