/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewServiceUnitProjectWizard.java,v 1.10 2007/04/25 14:51:30 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.List;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.actions.JobWizardProjectCreation;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.constants.ProjectType;
import org.objectweb.petals.tools.plugin.development.constants.Wizards;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.WizardsUtils;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.wizards.custom.ServiceUnitCustomizationWizardPage;
import org.objectweb.petals.tools.plugin.development.wizards.models.ProvidedSuBean;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * Project wizard for JBI Service Unit
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class NewServiceUnitProjectWizard extends AbstractNewJBIProjectWizard {

    /**
     * The wizard customization list for Service Units
     */
    private List<WizardCustomizationModel> specificTypesList;
    
    private List<ProvidedSuBean> suList;

    private int specificTypesIndex;

    private String workspaceComponentName;

    /**
     * Main page of the wizard
     */
    private NewServiceUnitProjectWizardPage mainPage;
    
    protected ServiceUnitCustomizationWizardPage suBlankPage;
    
    /**
     * Creates a new instance of {@link NewServiceUnitProjectWizard}
     * 
     */
    public NewServiceUnitProjectWizard() {
        super();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#addPages()
     */
    @Override
    public void addPages() {
        this.mainPage = new NewServiceUnitProjectWizardPage(
                Messages.NewServiceUnitProjectWizard_0, this);
        addPage(this.mainPage);
        
        this.suBlankPage = new ServiceUnitCustomizationWizardPage(Messages.NewServiceUnitProjectWizard_1, this);
        addPage(this.suBlankPage);
        
        // add the wizard pages
        for (WizardCustomizationModel wizard : this.specificTypesList) {
            addPage(wizard.getSpecificCustomization().getExtensionsWizardPage());
        }
    }

    /**
     * @return the specificTypesIndex
     */
    public int getSpecificTypesIndex() {
        return this.specificTypesIndex;
    }

    /**
     * @return the specificTypesList
     */
    public List<WizardCustomizationModel> getSpecificTypesList() {
        return this.specificTypesList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    @Override
    public void init(final IWorkbench workbench,
            final IStructuredSelection currentSelection) {
        super.init(workbench, currentSelection);
        this.setWindowTitle(Messages.NewServiceUnitProjectWizard_2);
        this.specificTypesList = BusinessCommon
                .searchForCustomExtensions(Wizards.WIZARD_CUSTOM_SU);
        this.specificTypesIndex = -1;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#performFinish()
     */
    @Override
    public boolean performFinish() {

        final JobWizardProjectCreation job = new JobWizardProjectCreation();
        String name = mainPage.getProjectName();
        String group = name;
        String tmp = mainPage.getSuComposite().getMavenComposite()
                .getNamespace();
        if (tmp != null && tmp.trim().length() > 0) {
            group = tmp;
        }

        job.setNewProjectHandle(this.mainPage.getProjectHandle());
        job.setProjectType(ProjectType.SU);
        job.setUseDefaultUri(this.mainPage.useDefaults());
        job.setLocationUri(this.mainPage.getLocationURI());

        if (mainPage.getSuComposite().isWorkspaceResource()) {
            ArtifactInformation info = WizardsUtils
                    .getComponentFromWorkspace(workspaceComponentName);
            job.addDependency(info);
        } else if (mainPage.getSuComposite().isExtensionResource()) {
            job.setSpecificTypesList(this.specificTypesList);
            job.setSpecificTypesIndex(this.specificTypesIndex);
        } else if (mainPage.getSuComposite().isNoneResource()) {
            job.setSuList(suList);
        }

        if (Activator.getDefault().getPluginPreferences().getBoolean(
                ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            job.setMavenArtifactId(name);
            job.setMavenGroupId(group);
            job.setMavenVersion(this.mainPage.getSuComposite()
                    .getMavenComposite().getVersion());
            job.setMavenPackaging(Constants.ArtifactsType.JBI_SERVICE_UNIT);
        }

        job.setRule(ResourcesPlugin.getWorkspace().getRoot());
        job.setUser(true);
        job.schedule();

        return true;
    }
    
    /**
     * @param specificTypesIndex
     *            the specificTypesIndex to set
     */
    public void setSpecificTypesIndex(final int specificTypesIdex) {
        this.specificTypesIndex = specificTypesIdex;
    }

    /**
     * @param workspaceComponentName
     *            the workspaceComponentName to set
     */
    public void setWorkspaceComponentName(String workspaceComponentName) {
        this.workspaceComponentName = workspaceComponentName;
    }

    /**
     * 
     * @param suList
     */
    public void setSuList(List<ProvidedSuBean> suList) {
        this.suList = suList;
    }

}
