/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewServiceUnitProjectWizardComposite.java,v 1.6 2007/04/24 07:26:57 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.List;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.WizardsUtils;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * Composite for Service Unit
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class NewServiceUnitProjectWizardComposite extends Composite {

    public static enum TARGET_TYPE {
        extensions, workspace, none
    }

    private MavenIdComposite mavenComposite;

    private final NewServiceUnitProjectWizard wizard;

    private final Preferences prefs;

    protected Button buttonWorkspace;

    protected Button buttonWizard;

    protected Button buttonNew;

    private Composite buttonComposite;

    private Combo comboWorkspace;

    private Combo comboExtensions;

    private boolean noWorkspaceResource;

    private TARGET_TYPE suTarget;

    /**
     * Creates a new instance of {@link NewServiceUnitProjectWizardComposite}
     * 
     * @param parent
     * @param style
     * @param wizard
     */
    public NewServiceUnitProjectWizardComposite(final Composite parent,
            final int style, final NewServiceUnitProjectWizard wizard) {
        super(parent, style);
        this.wizard = wizard;
        this.prefs = Activator.getDefault().getPluginPreferences();
        this.initialize();
    }

    /**
     * This method initializes extensionsComposite
     * 
     */
    private void createExtensionsComposite() {
        final GridLayout gridLayout = new GridLayout();
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        gridLayout.horizontalSpacing = 0;
        gridLayout.marginWidth = 0;
        gridLayout.numColumns = 2;

        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = GridData.CENTER;

        GridData gridData11 = new GridData();
        gridData11.horizontalAlignment = GridData.FILL;
        gridData11.grabExcessHorizontalSpace = true;
        gridData11.verticalAlignment = GridData.CENTER;

        GridData gridDataChoiceButton = new GridData();
        gridDataChoiceButton.horizontalAlignment = GridData.FILL;
        gridDataChoiceButton.grabExcessHorizontalSpace = true;
        gridDataChoiceButton.verticalAlignment = GridData.CENTER;

        GridData gridDataChoiceButton1 = new GridData();
        gridDataChoiceButton1.horizontalAlignment = GridData.FILL;
        gridDataChoiceButton1.grabExcessHorizontalSpace = true;
        gridDataChoiceButton1.verticalAlignment = GridData.CENTER;

        GridData gridDataChoiceButton2 = new GridData();
        gridDataChoiceButton2.horizontalAlignment = GridData.FILL;
        gridDataChoiceButton2.grabExcessHorizontalSpace = true;
        gridDataChoiceButton2.verticalAlignment = GridData.CENTER;

        final GridData gridData22 = new GridData();
        gridData22.horizontalAlignment = GridData.FILL;
        gridData22.grabExcessHorizontalSpace = true;
        gridData22.verticalAlignment = GridData.CENTER;

        if (this.prefs
                .getBoolean(ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            final GridData gridData1 = new GridData();
            gridData1.horizontalAlignment = GridData.FILL;
            gridData1.grabExcessHorizontalSpace = true;
            gridData1.verticalAlignment = GridData.CENTER;

            this.mavenComposite = new MavenIdComposite(this, SWT.NONE);
            this.mavenComposite.setLayoutData(gridData1);
            this.mavenComposite.setLayout(gridLayout);
        }

        Group groupTargetComponent = new Group(this, SWT.NONE);
        groupTargetComponent.setText(Messages.NewServiceUnitProjectWizardComposite_0);
        groupTargetComponent.setLayoutData(gridData11);
        groupTargetComponent.setLayout(gridLayout);

        // component type
        // BC of SE choice
        Label labelComponentType = new Label(groupTargetComponent, SWT.NONE);
        labelComponentType.setText(Messages.NewServiceUnitProjectWizardComposite_1);
        buttonComposite = new Composite(groupTargetComponent, SWT.NONE);
        buttonComposite.setLayout(new GridLayout(3, true));

        Listener toggleListener = new Listener() {
            public void handleEvent(Event e) {
                Control[] children = buttonComposite.getChildren();
                for (int i = 0; i < children.length; i++) {
                    Control child = children[i];
                    // unselect others
                    if (e.widget != child && child instanceof Button
                            && (child.getStyle() & SWT.RADIO) != 0) {
                        Button button = (Button) child;
                        button.setSelection(false);
                    }
                }

                Button selected = (Button) e.widget;
                if (!selected.getSelection()) {
                    selected.setSelection(true);
                }
                // if (!selected.getSelection() ){//&& selected !=
                // buttonWorkspace) {
                // selected.setSelection(true);

                if (selected == buttonWorkspace) {
                    if (!noWorkspaceResource) {
                        comboWorkspace.setEnabled(true);
                    }
                    comboExtensions.setEnabled(false);
                    suTarget = TARGET_TYPE.workspace;
                } else if (selected == buttonNew) {
                    comboWorkspace.setEnabled(false);
                    comboExtensions.setEnabled(false);
                    suTarget = TARGET_TYPE.none;
                } else if (selected == buttonWizard) {
                    comboWorkspace.setEnabled(false);
                    comboExtensions.setEnabled(true);
                    suTarget = TARGET_TYPE.extensions;
                }
            }
            
            // }
        };

        // buttons
        buttonNew = new Button(buttonComposite, SWT.RADIO);
        buttonNew.setText(Messages.NewServiceUnitProjectWizardComposite_2);
        buttonNew.setLayoutData(gridDataChoiceButton);
        buttonNew.setToolTipText(Messages.NewServiceUnitProjectWizardComposite_3);
        buttonNew.addListener(SWT.Selection, toggleListener);

        buttonWorkspace = new Button(buttonComposite, SWT.RADIO);
        buttonWorkspace.setText(Messages.NewServiceUnitProjectWizardComposite_4);
        buttonWorkspace.setLayoutData(gridDataChoiceButton2);
        buttonWorkspace
                .setToolTipText(Messages.NewServiceUnitProjectWizardComposite_5);
        buttonWorkspace.addListener(SWT.Selection, toggleListener);

        buttonWizard = new Button(buttonComposite, SWT.RADIO);
        buttonWizard.setText(Messages.NewServiceUnitProjectWizardComposite_6);
        buttonWizard.setLayoutData(gridDataChoiceButton1);
        buttonWizard
                .setToolTipText(Messages.NewServiceUnitProjectWizardComposite_7);
        buttonWizard.addListener(SWT.Selection, toggleListener);

        Label labelWorkspace = new Label(groupTargetComponent, SWT.NONE);
        labelWorkspace.setText(Messages.NewServiceUnitProjectWizardComposite_8);
        comboWorkspace = new Combo(groupTargetComponent, SWT.BORDER
                | SWT.READ_ONLY);
        List<ArtifactInformation> artifacts = WizardsUtils
                .getComponentsFromWorkspace();
        if (artifacts.size() == 0) {
            noWorkspaceResource = true;
        } else {
            comboWorkspace.add(""); //$NON-NLS-1$
            for (ArtifactInformation artifactInformation : artifacts) {
                comboWorkspace.add(artifactInformation.getName());
            }
            comboWorkspace.select(0);
        }
        comboWorkspace.setLayoutData(gridData);

        // Extensions
        Label labelExtensions = new Label(groupTargetComponent, SWT.NONE);
        labelExtensions.setText(Messages.NewServiceUnitProjectWizardComposite_10);

        comboExtensions = new Combo(groupTargetComponent, SWT.BORDER
                | SWT.READ_ONLY);
        List<WizardCustomizationModel> extensionList = this.wizard
                .getSpecificTypesList();
        comboExtensions.add(""); //$NON-NLS-1$
        for (WizardCustomizationModel custom : extensionList) {
            comboExtensions.add(custom.getSpecificProjectType());
        }
        comboExtensions.select(0);
        comboExtensions.setLayoutData(gridData22);

        // initial selection
        buttonNew.setSelection(false);
        buttonWorkspace.setSelection(false);
        if (noWorkspaceResource) {
            buttonWorkspace.setEnabled(false);
        }
        buttonWizard.setSelection(false);
        comboWorkspace.setEnabled(false);
        comboExtensions.setEnabled(false);
        suTarget = TARGET_TYPE.none;
    }

    public MavenIdComposite getMavenComposite() {
        return this.mavenComposite;
    }

    /**
     * @return the specificCombo
     */
    public Combo getExtensionsCombo() {
        return this.comboExtensions;
    }

    /**
     * 
     * @return
     */
    public Combo getWorkspaceCombo() {
        return this.comboWorkspace;
    }

    /**
     * This method initializes this
     * 
     */
    private void initialize() {
        this.setLayout(new GridLayout());
        this.createExtensionsComposite();
    }

    /**
     * @return the workspaceResource
     */
    public boolean isWorkspaceResource() {
        return suTarget == TARGET_TYPE.workspace;
    }

    public boolean isExtensionResource() {
        return suTarget == TARGET_TYPE.extensions;
    }

    public boolean isNoneResource() {
        return suTarget == TARGET_TYPE.none;
    }

    /**
     * @return the buttonWorkspace
     */
    public Button getButtonWorkspace() {
        return this.buttonWorkspace;
    }

    /**
     * @return the buttonWizard
     */
    public Button getButtonWizard() {
        return this.buttonWizard;
    }

    /**
     * @return the buttonNew
     */
    public Button getButtonNew() {
        return this.buttonNew;
    }

}
