/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewServiceUnitProjectWizardPage.java,v 1.7 2007/04/25 14:18:41 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.objectweb.petals.tools.plugin.development.wizards.models.WizardCustomizationModel;

/**
 * Wizard page for Service Unit
 * 
 * @author LAM
 * @author chamerling - eBM WebSourcing
 */
public class NewServiceUnitProjectWizardPage extends
        WizardNewProjectCreationPage {

    private IWizardPage nextPage;

    /**
     * The wizard which use this page
     */
    private final NewServiceUnitProjectWizard suWizard;

    /**
     * This page composite
     */
    private NewServiceUnitProjectWizardComposite suWizardComposite;

    /**
     * @param pageName
     *            The page name
     */
    public NewServiceUnitProjectWizardPage(final String pageName,
            final NewServiceUnitProjectWizard wizard) {
        super(pageName);
        this.suWizard = wizard;
        this.setTitle(pageName);
        this.setDescription(Messages.NewServiceUnitProjectWizardPage_0);
        this.setPageComplete(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createControl(final Composite parent) {
        super.createControl(parent);
        final GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        this.suWizardComposite = new NewServiceUnitProjectWizardComposite(
                (Composite) this.getControl(), SWT.NONE, this.suWizard);
        this.suWizardComposite.setLayoutData(gridData);

        // set listeners
        this.configureListeners();

    }

    private void configureListeners() {
        // extensions
        this.suWizardComposite.getExtensionsCombo().addSelectionListener(
                new SelectionListener() {
                    public void widgetDefaultSelected(final SelectionEvent e) {
                    }

                    public void widgetSelected(final SelectionEvent e) {
                        validateExtensionsChoice();
                    }
                });

        this.suWizardComposite.getButtonWizard().addSelectionListener(
                new SelectionListener() {
                    public void widgetDefaultSelected(final SelectionEvent e) {
                    }

                    public void widgetSelected(final SelectionEvent e) {
                        validateExtensionsChoice();
                    }
                });

        this.suWizardComposite.getExtensionsCombo().addModifyListener(
                new ModifyListener() {

                    public void modifyText(final ModifyEvent e) {
                        validateExtensionsChoice();
                    }
                });

        // workspace
        this.suWizardComposite.getButtonWorkspace().addSelectionListener(
                new SelectionListener() {
                    public void widgetDefaultSelected(final SelectionEvent e) {
                    }

                    public void widgetSelected(final SelectionEvent e) {
                        validateWorkspaceChoice();
                    }
                });

        this.suWizardComposite.getWorkspaceCombo().addSelectionListener(
                new SelectionListener() {
                    public void widgetDefaultSelected(final SelectionEvent e) {
                    }

                    public void widgetSelected(final SelectionEvent e) {
                        validateWorkspaceChoice();
                    }
                });

        // new
        this.suWizardComposite.getButtonNew().addSelectionListener(
                new SelectionListener() {

                    public void widgetDefaultSelected(SelectionEvent e) {
                    }

                    public void widgetSelected(SelectionEvent e) {
                        validateNewChoice();
                    }
                });

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
     */
    @Override
    public IWizardPage getNextPage() {
        return this.nextPage;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#validatePage()
     */
    @Override
    protected boolean validatePage() {
        this.suWizard.setCompleted(false);
        boolean result = super.validatePage();

        // validate the user fields
        boolean userValidation = false;
        if (suWizardComposite != null) {
            boolean enabled1 = suWizardComposite.buttonNew.isEnabled();
            boolean selection1 = suWizardComposite.buttonNew.getSelection();
            boolean newValid = enabled1 && selection1;

            boolean wizardValid = suWizardComposite.buttonWizard.isEnabled()
                    && suWizardComposite.buttonWizard.getSelection()
                    && suWizardComposite.getExtensionsCombo()
                            .getSelectionIndex() > 0;
            boolean workspaceValid = suWizardComposite.buttonWorkspace
                    .isEnabled()
                    && suWizardComposite.buttonWorkspace.getSelection()
                    && suWizardComposite.getWorkspaceCombo()
                            .getSelectionIndex() > 0;
            userValidation = newValid || workspaceValid || wizardValid;
        }
        return result && userValidation;
    }

    /**
     * 
     * @return
     */
    private boolean validateWorkspaceChoice() {

        boolean result = false;
        Combo combo = this.suWizardComposite.getWorkspaceCombo();
        Button button = this.suWizardComposite.getButtonWorkspace();
        result = button.isEnabled() && button.getSelection()
                && combo.getSelectionIndex() > 0;

        if (result) {
            int id = combo.getSelectionIndex();
            String item = combo.getItem(id);
            this.suWizard.setWorkspaceComponentName(item);
            this.nextPage = ((NewServiceUnitProjectWizard) getWizard()).suBlankPage;
        } else {
            this.nextPage = null;
        }

        // update page values, will update buttons...
        result = result && validatePage();

        this.setPageComplete(result);

        return result;
    }

    /**
     * 
     * @return
     */
    private boolean validateExtensionsChoice() {

        boolean result = false;
        Combo combo = this.suWizardComposite.getExtensionsCombo();
        Button button = this.suWizardComposite.getButtonWizard();

        result = button.isEnabled() && button.getSelection()
                && combo.getSelectionIndex() > 0;

        if (result) {

            int id = combo.getSelectionIndex();
            String item = combo.getItem(id);
            int i = 0;
            for (final WizardCustomizationModel current : NewServiceUnitProjectWizardPage.this.suWizard
                    .getSpecificTypesList()) {
                if (current.getSpecificProjectType().equals(item)) {
                    NewServiceUnitProjectWizardPage.this.nextPage = current
                            .getSpecificCustomization()
                            .getExtensionsWizardPage();
                    NewServiceUnitProjectWizardPage.this.suWizard
                            .setSpecificTypesIndex(i);
                }
                i++;
            }
        } else {
            this.nextPage = null;
        }

        // update page values, will update buttons...
        result = result && validatePage();
        this.setPageComplete(result);

        return result;
    }

    /**
     * 
     * @return
     */
    private boolean validateNewChoice() {
        boolean result = false;
        Button button = this.suWizardComposite.getButtonNew();

        result = button.isEnabled() && button.getSelection();

        if (result) {
            this.nextPage = ((NewServiceUnitProjectWizard) getWizard()).suBlankPage;
        } else {
            this.nextPage = null;
        }

        // update page values, will update buttons...
        result = result && validatePage();

        this.setPageComplete(result);
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.dialogs.WizardNewProjectCreationPage#getProjectHandle()
     */
    @Override
    public IProject getProjectHandle() {
        return super.getProjectHandle();
    }

    public NewServiceUnitProjectWizardComposite getSuComposite() {
        return this.suWizardComposite;
    }

}
