/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewSharedLibraryProjectWizard.java,v 1.8 2007/04/25 14:51:30 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.objectweb.petals.tools.plugin.development.Activator;
import org.objectweb.petals.tools.plugin.development.actions.JobWizardProjectCreation;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.constants.ESBDevPrefConstants;
import org.objectweb.petals.tools.plugin.development.constants.ProjectType;

/**
 * @author LAM
 */
public class NewSharedLibraryProjectWizard extends
        BasicNewProjectResourceWizard {

    /**
     * Main page of the wizard
     */
    private NewSharedLibraryProjectWizardPage mainPage;

    public NewSharedLibraryProjectWizard() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#addPages()
     */
    @Override
    public void addPages() {
        mainPage = new NewSharedLibraryProjectWizardPage(Messages.NewSharedLibraryProjectWizard_0);

        super.addPage(mainPage);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#performFinish()
     */
    @Override
    public boolean performFinish() {
        final JobWizardProjectCreation job = new JobWizardProjectCreation();
        String name = mainPage.getProjectName();
        String group = name;
        String tmp = mainPage.getSlComposite().getMavenComposite()
                .getNamespace();
        if (tmp != null && tmp.trim().length() > 0) {
            group = tmp;
        }

        job.setNewProjectHandle(mainPage.getProjectHandle());
        job.setProjectType(ProjectType.SA);
        job.setUseDefaultUri(mainPage.useDefaults());
        job.setLocationUri(mainPage.getLocationURI());
        job.setSpecificTypesIndex(-1);

        if (Activator.getDefault().getPluginPreferences().getBoolean(
                ESBDevPrefConstants.USE_MAVEN_FOR_BUILDING_NAME)) {
            job.setMavenArtifactId(name);
            job.setMavenGroupId(group);
            job.setMavenVersion(mainPage.getSlComposite().getMavenComposite()
                    .getVersion());
            job.setMavenPackaging(Constants.ArtifactsType.JBI_SHARED_LIBRARY);
        }

        job.setRule(ResourcesPlugin.getWorkspace().getRoot());
        job.setUser(true);
        job.schedule();

        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    @Override
    public void init(IWorkbench workbench, IStructuredSelection currentSelection) {
        super.init(workbench, currentSelection);
        setNeedsProgressMonitor(true);
        setWindowTitle(Messages.NewSharedLibraryProjectWizard_1);
    }

}
