/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards;

import java.util.List;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Identification;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaSuTarget;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.viewers.SimpleTableViewer;

/**
 * @author LAM
 */
public class SelectSuListWizard extends Wizard implements INewWizard {

    private SelectSuListPage mainPage;

    private SimpleTableViewer viewerServiceUnit;

    private List<SaServiceUnit> serviceUnitList;

    /**
     * Creates a new instance of {@link SelectSuListWizard}
     *
     * @param serviceUnitList
     */
    public SelectSuListWizard(List<SaServiceUnit> serviceUnitList) {
        super();
        this.serviceUnitList = serviceUnitList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
     *      org.eclipse.jface.viewers.IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard#addPages()
     */
    public void addPages() {
        mainPage = new SelectSuListPage(Messages.SelectSuListWizard_0);
        addPage(mainPage);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.Wizard#performFinish()
     */
    @Override
    public boolean performFinish() {
        if (viewerServiceUnit != null) {
            for (ArtifactInformation info : mainPage.getSaComposite()
                    .getSelectedSuList()) {
                String suName = info.getName();

                if (!alreadyInSuList(suName, serviceUnitList)) {
                    // new empty SaServiceUnit
                    SaServiceUnit newssu = new SaServiceUnit();

                    Identification newId = new Identification();
                    newId.setDescription(""); //$NON-NLS-1$
                    newId.setName(suName);
                    SaSuTarget newTarget = new SaSuTarget();
                    newTarget.setArtifactsZip(""); //$NON-NLS-1$
                    newTarget.setComponentName(""); //$NON-NLS-1$
                    newssu.setIdentification(newId);
                    newssu.setTarget(newTarget);

                    serviceUnitList.add(newssu);

                    viewerServiceUnit.getSimpleTableList().addLine(suName);
                }
            }
        }

        return true;
    }

    /**
     * @param viewerServiceUnit
     *            the viewerServiceUnit to set
     */
    public void setViewerServiceUnit(SimpleTableViewer viewerServiceUnit) {
        this.viewerServiceUnit = viewerServiceUnit;
    }

    /**
     * Tests if the new Su is already in the list
     * 
     * @param newSu
     * @param serviceUnitList
     * @return
     */
    private static boolean alreadyInSuList(String newSu,
            List<SaServiceUnit> serviceUnitList) {

        for (SaServiceUnit current : serviceUnitList) {
            if (current.getIdentification().getName().equals(newSu)) {
                return true;
            }
        }

        return false;
    }

}
