/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards.custom;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.petals.tools.plugin.development.wizards.AbstractWizardPage;
import org.objectweb.petals.tools.plugin.development.wizards.NewServiceUnitProjectWizard;
import org.objectweb.petals.tools.plugin.development.wizards.utils.WizardsUtils;

/**
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 * 
 */
public class ServiceUnitCustomizationWizardPage extends AbstractWizardPage {

    enum ServiceMode {
        consumes, provides
    }

    /**
     * This page composite
     */
    private ServiceUnitCustomizationComposite composite;

    /**
     * The root wizard
     */
    private NewServiceUnitProjectWizard suWizard;

    /**
     * Creates a new instance of {@link ServiceUnitCustomizationWizardPage}
     * 
     * @param pageName
     * @param wizard
     */
    public ServiceUnitCustomizationWizardPage(final String pageName,
            final NewServiceUnitProjectWizard wizard) {
        super(pageName);
        this.suWizard = wizard;
        this.setTitle(pageName);
        this.setDescription(pageName);
        this.setPageComplete(false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(final Composite parent) {
        // super.createControl(parent);
        final GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        this.composite = new ServiceUnitCustomizationComposite(parent,
                SWT.NULL, suWizard);
        this.composite.setLayoutData(gridData);

        this.composite.getTextServiceName().addModifyListener(
                new ModifyListener() {

                    public void modifyText(final ModifyEvent e) {
                        final boolean validate = ServiceUnitCustomizationWizardPage.this
                                .validatePage();
                        ServiceUnitCustomizationWizardPage.this
                                .setPageComplete(validate);
                    }

                });

        this.composite.getTextInterfaceName().addModifyListener(
                new ModifyListener() {

                    public void modifyText(final ModifyEvent e) {
                        final boolean validate = ServiceUnitCustomizationWizardPage.this
                                .validatePage();
                        ServiceUnitCustomizationWizardPage.this
                                .setPageComplete(validate);
                    }

                });

        this.composite.getTextEndpointName().addModifyListener(
                new ModifyListener() {

                    public void modifyText(final ModifyEvent e) {
                        final boolean validate = ServiceUnitCustomizationWizardPage.this
                                .validatePage();
                        ServiceUnitCustomizationWizardPage.this
                                .setPageComplete(validate);
                    }

                });

        this.composite.getComboLinkType().addModifyListener(
                new ModifyListener() {

                    public void modifyText(final ModifyEvent e) {
                        final boolean validate = ServiceUnitCustomizationWizardPage.this
                                .validatePage();
                        ServiceUnitCustomizationWizardPage.this
                                .setPageComplete(validate);
                    }

                });

        // this.composite.getExtensions().addModifyListener(new ModifyListener()
        // {
        //
        // public void modifyText(final ModifyEvent e) {
        // final boolean validate = ServiceUnitCustomizationWizardPage.this
        // .validatePage();
        // ServiceUnitCustomizationWizardPage.this
        // .setPageComplete(validate);
        // }
        //
        // });

        this.setControl(this.composite);

        this.composite.setSuList(WizardsUtils.getProvidedSuFromWorkspace());

        final boolean validate = this.validatePage();
        this.setPageComplete(validate);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.wizard.WizardPage#getNextPage()
     */
    @Override
    public IWizardPage getNextPage() {
        // We don't want this page to have a next page
        return null;
    }

    /**
     * Returns whether this page's controls currently all contain valid values.
     * 
     * @return <code>true</code> if all controls are valid, and
     *         <code>false</code> if at least one is invalid
     */
    protected boolean doValidatePage() {

        boolean result = true;
        List<String> errorMessages = new ArrayList<String>();

        // Clean
        this.setMessage(null);
        this.setErrorMessage(null);

        final String serviceName = this.composite.getTextServiceName()
                .getText();
        if ("".equals(serviceName)) { //$NON-NLS-1$
            errorMessages.add(Messages.ServiceUnitCustomizationWizardPage_1);
            result = false;
        }

        final String interfaceNameContents = this.composite
                .getTextInterfaceName().getText();
        if ("".equals(interfaceNameContents)) { //$NON-NLS-1$
            errorMessages.add(Messages.ServiceUnitCustomizationWizardPage_3);
            result = false;
        }

        final String endpointNameContents = this.composite
                .getTextEndpointName().getText();
        if ("".equals(endpointNameContents)) { //$NON-NLS-1$
            errorMessages.add(Messages.ServiceUnitCustomizationWizardPage_5);
            result = false;
        }

        if (result) {
            // consumes or provides mode
            if (composite.getConsumesButton().getSelection()) {
                suWizard.setSuList(this.composite.getSuList());

            } else if (composite.getProvidesButton().getSelection()) {
                // create the provided service bean
                suWizard.setSuList(this.composite.getCurrentSuBeans());
            }

        } else {
            if (errorMessages.size() > 0) {
                String message = Messages.ServiceUnitCustomizationWizardPage_6;
                for (int i = 0; i < errorMessages.size(); i++) {
                    message += errorMessages.get(i);
                    if (i < errorMessages.size() - 1) {
                        message += ", "; //$NON-NLS-1$
                    }
                    this.setMessage(message, ERROR);
                }
            }
        }

        // suWizard.setCompleted(result);
        return result;
    }
}
