/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: IWizardCustomization.java,v 1.2 2007/04/25 08:25:47 lam Exp $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.development.wizards.interfaces;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.wizard.IWizardPage;

/**
 * Extension point for New Components Wizards customization.
 * By default, projects are created with classic tree directory structure.
 * This extension point will give ability to add specific file templates, folders or
 * classpath entries to the wizards.
 * 
 * @author LAM
 */
public interface IWizardCustomization {
    
    /**
     * @param fieldValues The values for all fields represented by the fieldsList
     * attribute in the customizationWizard element extension wizard
     * @param project The project to add IFiles and IFolders
     * @param monitor The progress monitor used. This method can use
     * from monitor.worked(5) to monitor.worked(20) to indicate job progression
     * @throws CoreException CoreException
     */
    void addSpecificRessourcesToProject(IProject project, IProgressMonitor monitor) 
        throws CoreException;
    
    /**
     * Gets the specific Extension Wizard Page for the wizard.
     * The Page returned will be added to the wizard and may include
     *  any editable controls.
     * 
     * @return The specific Wizard Page
     */
    IWizardPage getExtensionsWizardPage();
    
    /**
     * Gets the Extension Wizard Page name.
     * Usefull to determine if the page was already added to the wizard
     * 
     * @return The Extension Wizard Page name
     */
    String getExtensionPageName();
    
}
