/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.plugin.development.wizards.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.objectweb.petals.tools.plugin.development.constants.Constants;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Attribute;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.Jbi;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SaServiceUnit;
import org.objectweb.petals.tools.plugin.development.jbieditor.beans.SeConsumesProvides;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.JbiBeanUtils;
import 
    org.objectweb.petals.tools.plugin.development.jbieditor.business.TransformDomToBean;
import org.objectweb.petals.tools.plugin.development.jbieditor.business.XmlParsing;
import org.objectweb.petals.tools.plugin.development.maven.ArtifactInformation;
import org.objectweb.petals.tools.plugin.development.maven.PomUtils;
import org.objectweb.petals.tools.plugin.development.wizards.models.MavenSaBean;
import org.objectweb.petals.tools.plugin.development.wizards.models.NamespaceBean;
import org.objectweb.petals.tools.plugin.development.wizards.models.ProvidedSuBeans;
import org.w3c.dom.Document;

/**
 * @author LAM
 */
public final class WizardsUtils {

    private WizardsUtils() { }

    /**
     * Get all the providers from the workspace
     * 
     * @return Provided SU beans List
     */
    public static List<ProvidedSuBeans> getProvidedSuFromWorkspace() {
        List<ProvidedSuBeans> resultList = new ArrayList<ProvidedSuBeans>();

        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();

        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);

                    if ((bean != null) && (JbiBeanUtils.isSu(bean))) {
                        for (SeConsumesProvides current : bean.getServices()
                                .getProvidesList()) {
                            ProvidedSuBeans newProvidedServiceBean = 
                                new ProvidedSuBeans();
                            newProvidedServiceBean.setEndpointName(current
                                    .getEndpointName());
                            newProvidedServiceBean
                                    .setInterfaceName(getNamespaceBeanFromString(
                                            bean, current.getInterfaceName()));
                            newProvidedServiceBean.setLinkType(current
                                    .getLinkType());
                            newProvidedServiceBean
                                    .setServiceName(getNamespaceBeanFromString(
                                            bean, current.getServiceName()));
                            newProvidedServiceBean
                                    .setProjectName(currentProject.getName());
                            newProvidedServiceBean
                                    .setProjectPath(currentProject
                                            .getLocation().toOSString());
                            resultList.add(newProvidedServiceBean);
                        }
                    }

                } catch (CoreException e) { }
            }
        }

        return resultList;
    }

    /**
     * Retrieves the namespace list and values for the String
     * 
     * @param bean
     *            The jbi bean that contains the QNames attributes
     * @param string
     *            The String to analyse
     * @return The NamespaceBean
     */
    private static NamespaceBean getNamespaceBeanFromString(Jbi bean,
            String string) {
        NamespaceBean nsBean = new NamespaceBean();
        nsBean.setValue(string);
        nsBean.setNamespaceUri(new QName("", "")); //$NON-NLS-1$ //$NON-NLS-2$
        String separatorChar = Messages.WizardsUtils_0;

        // if the string is a QName format
        if (string.contains(separatorChar)) {
            String localName = string.substring(0, string
                    .indexOf(separatorChar));
            String value = string.substring(string.indexOf(separatorChar) + 1,
                    string.length());

            // Set the value
            nsBean.setValue(value);
            // searches the namespaceUrl that refers to the local name
            for (Attribute attribute : bean.getJbiAttributes()) {
                if (attribute.getDescription().contains(localName)) {
                    nsBean.setNamespaceUri(new QName(attribute.getValue(),
                            localName));
                }
            }
        }

        return nsBean;
    }
    
    /**
     * Gets a bean list that represent all SA projects in the WorkSpace
     * 
     * @return The SaBean list
     */
    public static List<MavenSaBean> getMavenSaFromWorkspace() {
        List<MavenSaBean> foundSa = new ArrayList<MavenSaBean>();
        
        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
        
        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);
    
                    if ((bean != null) && (JbiBeanUtils.isSa(bean))) {
                        // General informations
                        MavenSaBean newSa = new MavenSaBean();
                        newSa.setProjectDirectory(
                                new File(currentProject.getLocationURI()));
                        newSa.setProjectName(currentProject.getName());
                        
                        // Pom.xml Maven informations
                        ArtifactInformation ai = PomUtils.getProjectInformation(currentProject.getName());
                        if (ai != null) {
                            newSa.setArtifactId(ai.getName());
                            newSa.setVersion(ai.getVersion());
                        }
                        
                        // Gets the SU list
                        for (SaServiceUnit current 
                                : bean.getServiceAssembly().getServiceUnitList()) {
                            File currentSuDir = 
                                new File(wsRoot.getProject(
                                        current.getIdentification().
                                        getName()).getLocationURI());
                            newSa.getSuProjectDirectories().add(currentSuDir);
                        }
                        
                        foundSa.add(newSa);
                    }
                } catch (CoreException e) { }
            }
        }
        
        return foundSa;
    }

    /**
     * Retrieves the SU list from the projects within the WorkSpace
     * 
     * @return SU String List
     */
    public static List<String> getSuNamesFromWorkspace() {
        List<String> resultList = new ArrayList<String>();
    
        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
    
        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
    
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);
    
                    if ((bean != null) && (JbiBeanUtils.isSu(bean))) {
                        resultList.add(currentProject.getName());
                    }
                } catch (CoreException e) { }
            }
        }
        return resultList;
    }

    /**
     * Get the components information from the WorkSpace
     * 
     * @return
     */
    public static List<ArtifactInformation> getComponentsFromWorkspace() {
        List<ArtifactInformation> result = new ArrayList<ArtifactInformation>();
        List<String> suNames = getComponentNamesFromWorkspace();
        for (String projectName : suNames) {
            ArtifactInformation info = PomUtils
                    .getProjectInformation(projectName);
            if (info != null) {
                result.add(info);
            }
        }
        return result;
    }

    /**
     * Get the component information from the workspace projects
     * 
     * @param componentName
     * @return null if the component has not been found in the workspace
     */
    public static ArtifactInformation getComponentFromWorkspace(
            String componentName) {
        ArtifactInformation result = null;
        List<ArtifactInformation> list = getComponentsFromWorkspace();
    
        Iterator<ArtifactInformation> iter = list.iterator();
        while (iter.hasNext() && result == null) {
            ArtifactInformation info = iter.next();
            if (info.getName().equals(componentName)) {
                result = info;
            }
        }
        return result;
    }

    /**
     * 
     * @return
     */
    public static List<ArtifactInformation> getSuFromWorkspace() {
        List<ArtifactInformation> result = new ArrayList<ArtifactInformation>();
        List<String> suNames = getSuNamesFromWorkspace();
        for (String projectName : suNames) {
            ArtifactInformation info = PomUtils
                    .getProjectInformation(projectName);
            if (info != null) {
                result.add(info);
            }
        }
        return result;
    }

    /**
     * Retrieves the Component list from the projects within the workspace
     * 
     * @return Component String List
     */
    public static List<String> getComponentNamesFromWorkspace() {
        List<String> resultList = new ArrayList<String>();
    
        IWorkspaceRoot wsRoot = ResourcesPlugin.getWorkspace().getRoot();
    
        for (IProject currentProject : wsRoot.getProjects()) {
            IFile jbiFile = currentProject.getFile(Constants.JBI_PATH);
            if (jbiFile.exists()) {
                try {
                    Document jbiDom = XmlParsing.parse(jbiFile.getContents());
                    Jbi bean = TransformDomToBean.populateJbiBean(jbiDom);
    
                    if ((bean != null)
                            && (JbiBeanUtils.isBc(bean) || JbiBeanUtils
                                    .isSe(bean))) {
                        resultList.add(bean.getComponent().getIdentification()
                                .getName());
                    }
                } catch (CoreException e) { }
            }
        }
        
        return resultList;
    }

}
