/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.views.MonitoringView;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;

/**
 * @author LAM
 */
public class DropDownMonitoredServersAction extends Action implements
        IMenuCreator {
    
    final Image CONNECTED_SERVER = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/connectedserver.gif").createImage();
    
    private Menu menu;
    
    public DropDownMonitoredServersAction() {
        setToolTipText("Liste des PetalsServer monitors");
        ImageDescriptor imageDescriptor = new ImageDescriptor() {
            @Override
            public ImageData getImageData() {
                return CONNECTED_SERVER.getImageData();
            }
        };
        setImageDescriptor(imageDescriptor);
        setMenuCreator(this);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IMenuCreator#dispose()
     */
    public void dispose() {
        if (menu != null) {
            menu.dispose();
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Control)
     */
    public Menu getMenu(Control parent) {
        if (menu != null) {
            menu.dispose();
        }
        
        menu = new Menu(parent);
        
        PetalsServer currentPs = null;
        if (MonitoringView.getInstance() != null) {
            currentPs = MonitoringView.getInstance().getPetalsServer();
        }
        
        if (PetalsServersView.getInstance() != null) {
            // Recuperation de la liste complete des serveurs
            Object[] listeRoot = PetalsServersView.getInstance().getProvider().getElements(null);
            for (int i=0; i<listeRoot.length; i++) {
                if (((PetalsServer) listeRoot[i]).isConnected()
                        && ((PetalsServer) listeRoot[i]).isMonitored()) {
                    Action action = new ShowMonitoredServerAction((PetalsServer)listeRoot[i]);
                    action.setChecked(((PetalsServer) listeRoot[i]).equals(currentPs));
                    addActionToMenu(menu, action, i + 1);
                }
            }
        }
        return menu;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.action.IMenuCreator#getMenu(org.eclipse.swt.widgets.Menu)
     */
    public Menu getMenu(Menu parent) {
        return null;
    }
    
    private void addActionToMenu(Menu parent, Action action, int accelerator) {
        if (accelerator < 10) {
            StringBuffer label= new StringBuffer();
            //add the numerical accelerator
            label.append('&');
            label.append(accelerator);
            label.append(' ');
            label.append(action.getText());
            action.setText(label.toString());
        }
        ActionContributionItem item= new ActionContributionItem(action);
        item.fill(parent, -1);
    }

}
