package org.objectweb.petals.tools.plugin.monitor.actions;

import java.net.URL;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.TreeViewer;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentAnalyzer;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentType;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 13 mars 07
 */
public class InstallComponentByMenuAction extends ViewerAction {

	private URL url;
	private PetalsServer petalsServer;
	private ComponentType type = ComponentType.unknown;
	private String name;
	
	public InstallComponentByMenuAction(TreeViewer viewer) {
		super(viewer);
	}

	@Override
	public void run() {
		if (url != null && petalsServer != null) {
			ComponentParent component = petalsServer.install(url);
			if (component != null) {
				petalsServer.add(component);	
			}	
			refreshViewer();
		}
	}

	public void setPetalsServer(PetalsServer petalsServer) {
		this.petalsServer = petalsServer;
	}

	public void setUrl(URL url) {
		this.url = url;
		if (url != null) {
			ComponentAnalyzer analyzer = new ComponentAnalyzer(url.toString());
			analyzer.analyze();
			type = analyzer.getType();
			setImageDescriptor(ImageDescriptor.createFromImage(type.getImage()));	
			String description = analyzer.getDescription();
			setDescription(description != null ? description : "Installe le " + type.toString() + " depuis " + url);
			name = analyzer.getName();
			if (name != null) {
				setText(name + (description != null ? " (" + description + ")" : ""));
			} else {
				String filepath = url.getFile().toLowerCase();
				setText(filepath.substring(filepath.lastIndexOf("/")+1, filepath.indexOf(".zip")));
			}
		}
	}
	
	public boolean isActionable() {
		return type != ComponentType.unknown;
	}

	public ComponentType getType() {
		return type;
	}
	
	public String getName() {
		return name;
	}
	
}
