/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.actions;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Timestamp;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.nmr.Message;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.views.MonitoringView;

/**
 * @author LAM
 */
public class MonitoringExportCsvAction extends Action {

    private PetalsServer currentServer;
    private IViewSite site;
    
    /**
     * Default constructor
     * 
     * @param exportTable
     * @param site
     */
    public MonitoringExportCsvAction(IViewSite site) {
        setText("Exporter en CSV");
        setToolTipText("Exporter la liste des messages dans un fichier CSV");
        setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/export.gif"));
        this.site = site;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run() {
        currentServer = MonitoringView.getInstance().getPetalsServer();
        
        FileDialog fDialog = new FileDialog(site.getShell(), SWT.SAVE);
        fDialog.setFilterExtensions(new String[] { "*.csv" });
        fDialog.setText("Enregistrer dans un fichier CSV");
        
        fDialog.open();
        String selectedFilePath = fDialog.getFilterPath();
        String selectedFileName = fDialog.getFileName();
        
        try {
            if (StringUtils.isNotEmpty(selectedFileName)) {
                FileWriter csvFile = new FileWriter(selectedFilePath + File.separator + selectedFileName);
                csvFile.write(formatNotifsDatas().toString());
                csvFile.close();
            }
        } catch (IOException e) {
            MessageDialog.openError(site.getShell(), "Exportation impossible", 
                    "Une erreur s'est produite dans l'exportation CSV");
        }
    }
    
    /**
     * Table datas extraction and csv formating
     * 
     * @return The StringBuffer that represent CSV datas
     */
    private StringBuffer formatNotifsDatas() {
        StringBuffer tableDatas = new StringBuffer();
        String sep = ",";
        String endOfLine = "\n";

        for (Message current : currentServer.getMessagesCollector().getMessagesList()) {
            Timestamp tsTime = new Timestamp(current.getTime());
            String columnTimestamp = tsTime.toString();
            String columnComponent = current.getComponent();
            String columnRole = current.getRole();
            String columnType = current.getType();
            String columnEPN = current.getEndpointName();
            String columnServiceName = current.getServiceName();
            String clolumnMessageContent = current.getContent();

            tableDatas.append(
                    columnTimestamp
                    + sep
                    + columnComponent
                    + sep
                    + columnRole
                    + sep
                    + columnType
                    + sep
                    + columnEPN
                    + sep
                    + columnServiceName
                    + sep
                    + "\""
                    + clolumnMessageContent
                    + "\""
                    + endOfLine);
        }

        return tableDatas;
    }

}
