package org.objectweb.petals.tools.plugin.monitor.actions;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.providers.ViewContentProvider;
import org.objectweb.petals.tools.plugin.monitor.wizards.NewComponentWizard;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 13 mars 07
 */
public class NewComponentAction extends ViewerAction {

	private IViewSite site;
	protected List<String> history = new ArrayList<String>();
	
	public NewComponentAction(TreeViewer viewer, IViewSite site) {
		super(viewer);
		setText("Nouveau");
		setToolTipText("Ajoute un nouveau composant");
		setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/new_component.gif"));
		this.site = site;
	}

	public void run() {
		ISelection selection = viewer.getSelection();
		List<PetalsServer> servers = new ArrayList<PetalsServer>();
		for (Object obj: ((IStructuredSelection)selection).toList()) {
			Object object = (TreeObject)obj;
			if (object instanceof PetalsServer && ((PetalsServer)object).isConnected()) {
				servers.add((PetalsServer)object);
			}
		}

		// si aucun serveur s\u00e9lectionn\u00e9, tous sont propos\u00e9s par d\u00e9faut
		if (servers.size() == 0) {
			for (TreeObject root :((ViewContentProvider)viewer.getContentProvider()).getChildren(null)) {
				if (root instanceof PetalsServer) {
					servers.add((PetalsServer)root);
				}
			}
		}
		
		NewComponentWizard wizard = new NewComponentWizard();
		wizard.init(site.getWorkbenchWindow().getWorkbench(),(IStructuredSelection)viewer.getSelection());
		WizardDialog dialog = new WizardDialog(site.getShell(), wizard);
		dialog.create();
//			TODO: init history
		wizard.setComponentsFilenames(history.toArray(new String[history.size()]));
		wizard.setServers(servers);
		if (dialog.open() == WizardDialog.OK) {
			String url = wizard.getUrl();
			if (!history.contains(url)) {
//					TODO: save new URL
				history.add(url);	
			}
			
			for (PetalsServer ps: wizard.getServers()) {
				try {
					ComponentParent component = ps.install(new URL(url));
					if (component != null) {
						ps.add(component);
					}
				} catch (MalformedURLException e) {
					MessageDialog.openError(null, "Installation d'un composant", "L'URL du ZIP du composant '" + url + "' est malform\u00e9e");
				}
			}
			
		}
			
			
			
		
		refreshViewer();
	}

}
