/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.actions;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.views.PerformanceNotifierView;

/**
 * @author LAM
 */
public class PerfNotifExportCsvAction extends Action {

    private List<String[]> notifDatas;
    private IViewSite site;
    
    /**
     * Default constructor
     * 
     * @param exportTable
     * @param site
     */
    public PerfNotifExportCsvAction(IViewSite site) {
        setText("Exporter en CSV");
        setToolTipText("Exporter la liste des notifications dans un fichier CSV");
        setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/export.gif"));
        this.site = site;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.action.Action#run()
     */
    public void run() {
        notifDatas = PerformanceNotifierView.getInstance().getUserData();
        
        FileDialog fDialog = new FileDialog(site.getShell(), SWT.SAVE);
        fDialog.setFilterExtensions(new String[] { "*.csv" });
        fDialog.setText("Enregistrer dans un fichier CSV");
        
        fDialog.open();
        String selectedFilePath = fDialog.getFilterPath();
        String selectedFileName = fDialog.getFileName();
        
        try {
            if (StringUtils.isNotEmpty(selectedFileName)) {
                FileWriter csvFile = new FileWriter(selectedFilePath + File.separator + selectedFileName);
                csvFile.write(formatNotifsDatas().toString());
                csvFile.close();
            }
        } catch (IOException e) {
            MessageDialog.openError(site.getShell(), "Exportation impossible", 
                    "Une erreur s'est produite dans l'exportation CSV");
        }
    }
    
    /**
     * Table datas extraction and csv formating
     * 
     * @return The StringBuffer that represent CSV datas
     */
    private StringBuffer formatNotifsDatas() {
        StringBuffer tableDatas = new StringBuffer();
        String sep = ",";
        String endOfLine = "\n";

        for (String[] current : notifDatas) {
            String columnId = current[0];
            String columnTimestamp = current[1];
            String columnOwner = current[2];
            String columnType = current[3];
            String columnDuration = current[4];
            String columnUserdata = current[5];

            tableDatas.append(
                    columnId
                    + sep
                    + columnTimestamp
                    + sep
                    + columnOwner
                    + sep
                    + columnType
                    + sep
                    + columnDuration
                    + sep
                    + columnUserdata
                    + endOfLine);
        }

        return tableDatas;
    }

}
