package org.objectweb.petals.tools.plugin.monitor.actions;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.CheckedTreeSelectionDialog;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.providers.ViewContentProvider;


public class SearchPetalsServersAction extends ViewerAction {

    private Shell shell;
    private ViewContentProvider provider;
    
    public SearchPetalsServersAction(TreeViewer viewer, Shell shell, ViewContentProvider provider) {
        super(viewer);
        setText("Recherche");
        setToolTipText("Ajoute les serveurs PETALS d\u00e9clar\u00e9s depuis celui s\u00e9lectionn\u00e9");
        setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
                Activator.PLUGIN_ID, "icons/search.gif"));
        this.shell = shell;
        this.provider = provider;
    }

    @SuppressWarnings("unchecked")
    public void run() {
        Set<Map<String,String>> discoveredServers = new HashSet<Map<String,String>>() {
            private static final long serialVersionUID = -6417024831021104084L;

            @SuppressWarnings("unchecked")
            @Override
            public boolean contains(Object o) {
                if (o instanceof Map) {
                    Map<String,String> m  = (Map<String,String>)o;
                    String uid  = m.get(PetalsServer.CONF_UID);
                    if (uid == null) {
                        return false;
                    }
                    for (Map<String,String> om: this) {
                        if (uid.equals(om.get(PetalsServer.CONF_UID))) {
                            return true;
                        }
                    }
                }
                return false;
            }
        };
        ISelection selection = viewer.getSelection();
        for (Object obj: ((IStructuredSelection)selection).toList()) {
            TreeObject tpo = (TreeObject)obj;
            Object object = (TreeObject)tpo;
            if (object instanceof PetalsServer) {
                PetalsServer ps = ((PetalsServer)object);
                for (Map<String,String> discoveredServer: ps.getDiscoveredRemoteServers()) {
                    if (!discoveredServers.contains(discoveredServer)) {
                        discoveredServers.add(discoveredServer);
                    }
                }
            }
        }

            
        ITreeContentProvider contentProvider = new ITreeContentProvider() {

            public Object[] getChildren(Object parentElement) {
                return null;
            }

            public Object getParent(Object element) {
                return null;
            }

            public boolean hasChildren(Object element) {
                return false;
            }

            @SuppressWarnings("unchecked")
            public Object[] getElements(Object inputElement) {
                Set<Map<String,String>>set = (Set<Map<String,String>>)inputElement;
                return  set.toArray(new Map[set.size()]);
            }

            public void dispose() {
            }

            public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
            }
            
        };
        
        LabelProvider labelProvider  = new LabelProvider() {

            final Image SERVER_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/connectedserver.gif").createImage();
            
            @Override
            public Image getImage(Object element) {
                return SERVER_IMAGE;
            }

            @Override
            public String getText(Object element) {
                Map<String, String> m = (Map<String,String>)element;
                String ip = m.get(PetalsServer.CONF_HOST);
                String host = null;
                try {
                    host = InetAddress.getByName(ip).getHostName();
                } catch (UnknownHostException e) {
                    host = null;
                }
                return (host != null ? host + " (" + ip + ")" : ip) + " - " + m.get(PetalsServer.CONF_NAME);
            }           
        };
        
        CheckedTreeSelectionDialog ctsd = new CheckedTreeSelectionDialog(shell, labelProvider, contentProvider);
        ctsd.setMessage("S\u00e9lectionner les serveurs Petals distants \u00e0 placer sous monitoring:");
        ctsd.setBlockOnOpen(true);
        ctsd.setTitle("S\u00e9lection des serveurs");
        ctsd.setImage(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/search.gif").createImage());
        ctsd.setInput(discoveredServers);
        ctsd.setInitialSelections(discoveredServers.toArray(new Map[discoveredServers.size()]));
        if (ctsd.open() == CheckedTreeSelectionDialog.OK) {
            String host;
            String ip;
            for (Object r: ctsd.getResult()) {
                Map<String, String> m = (Map<String,String>)r;
                ip = m.get(PetalsServer.CONF_HOST);
                try {
                    host = InetAddress.getByName(ip).getHostName();
                } catch (UnknownHostException e) {
                    host = null;
                }
                System.out.println((host != null ? host + " (" + ip + ")" : ip) + " - " + m.get(PetalsServer.CONF_NAME));
                PetalsServer ps = new PetalsServer(m);
                provider.add(ps);
                //TODO: delete duplicate entry
            }
        }
        refreshViewer();
    }

}
