package org.objectweb.petals.tools.plugin.monitor.configure;

import java.util.ArrayList;
import java.util.List;

import javax.management.Attribute;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;


public class AttributeContainer {

	private final static Image DEFAULT_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/attribute.gif").createImage();
	
	transient protected MBeanAttributeInfo attributeInfo;
	transient protected MBeanServerConnection connection;
	transient protected ObjectName objectName;
	protected List<ModifyModelListener> modifyModelListeners;

	public AttributeContainer(MBeanServerConnection connection, ObjectName objectName, MBeanAttributeInfo attributeInfo) {
		if (connection == null || attributeInfo == null || objectName == null) {
			throw new RuntimeException("Connection, objectName and/or attributeInfo can't be null-reference");
		}
		this.connection = connection;
		this.attributeInfo = attributeInfo;
		this.objectName = objectName;
		modifyModelListeners = new ArrayList<ModifyModelListener>();
	}
	
	public Object getValue() {
		if (attributeInfo.isReadable()) {
			try {
				return connection.getAttribute(objectName, attributeInfo.getName());
			} catch (Exception e) {
				return null;
			} 
		}
		return null;
	}
	
	public void setValue(Object newValue) {
		if (attributeInfo.isWritable()) {
			try {
				connection.setAttribute(objectName, new Attribute(attributeInfo.getName(), newValue));
				notifyListeners();
			} catch (Exception e) {
				MessageDialog.openError(null, "Modification attributs", "Impossible de MAJ l'attribut " + attributeInfo.getName());
			} 
		}
	}
	
	public String getName() {
		String description = attributeInfo.getDescription();
		return attributeInfo.getName() + (description != null ? " (" + description + ")" : "");
	}
	
	public Image getImage() {
		return DEFAULT_IMAGE;	
	}
	
	public interface ModifyModelListener {
		
		public void modifiedModel(AttributeContainer attributeContainer); 
		
	}
	
	public void addModifyModelListener(ModifyModelListener modifyModelListener) {
		if (modifyModelListener != null) {
			modifyModelListeners.add(modifyModelListener);	
		}
	}

	public void removeModifyModelListener(ModifyModelListener modifyModelListener) {
		if (modifyModelListener != null) {
			modifyModelListeners.remove(modifyModelListener);	
		}
	}

	public void clearModifyModelListeners() {
		modifyModelListeners.clear();
	}
	
	public void notifyListeners() {
		for (ModifyModelListener listener: modifyModelListeners) {
			listener.modifiedModel(this);
		}	
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof AttributeContainer) {
			AttributeContainer attributeContainer = (AttributeContainer)obj;
			return attributeContainer.connection == this.connection && attributeContainer.attributeInfo == this.attributeInfo;
		}
		return false;
	}
	
	public String getType() {
		return attributeInfo.getType(); 
	}
	
	public String getShortName() {
		return attributeInfo.getName(); 
	}
	
	public boolean isModifiable() {
		return attributeInfo.isWritable();
	}
	
}
