package org.objectweb.petals.tools.plugin.monitor.configure;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;


public class ComponentSettingsEditor extends FormEditor implements ITabbedPropertySheetPageContributor {
	
	private GeneralPage commonPage;
	private AttributesPage attributesPage;
	private MethodsPage methodsPage;
	private NotificationsPage notificationsPage;
		
	private FormColors formColors;
	
	public ComponentSettingsEditor() {
		super();
		setInput(getEditorInput());
	}

	FormColors getFormColors(Display display) {
		if (formColors == null) {
			formColors = new FormColors(display);
			formColors.markShared();
		}
		return formColors;
	}
	
	protected FormToolkit createToolkit(Display display) {
		return new FormToolkit(getFormColors(display));
	}
	
	@Override
	protected void addPages() {
		commonPage = new GeneralPage(this);
		attributesPage = new AttributesPage(this);
		methodsPage = new MethodsPage(this);
		notificationsPage = new NotificationsPage(this);
		
		try {
			addPage(commonPage);
			addPage(attributesPage);
			addPage(methodsPage);
			addPage(notificationsPage);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void doSaveAs() {
		// TODO Raccord de m\u00e9thode auto-g\u00e9n\u00e9r\u00e9
		System.out.println();
	}

	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}

	@Override
	protected void setInput(IEditorInput input) {
		super.setInput(input);
		if (input instanceof PetalsComponentInput) {
			PetalsComponent component = ((PetalsComponentInput)input).getComponent();
			setPartName(component.getName());
			setTitleToolTip("Configuration de " + component.getName());
		}
	}
	
	@Override
	public Object getAdapter(Class adapter) {
			if (adapter == IPropertySheetPage.class) {
				//return new PropertySheetPage();
				return new TabbedPropertySheetPage(this);
			}
			return super.getAdapter(adapter);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor#getContributorId()
	 */
	public String getContributorId() {
		return getSite().getId();
	}

	@Override
	public void editorDirtyStateChanged() {
		// TODO Raccord de m\u00e9thode auto-g\u00e9n\u00e9r\u00e9
		super.editorDirtyStateChanged();
	}

	@Override
	public boolean isDirty() {
		// TODO Raccord de m\u00e9thode auto-g\u00e9n\u00e9r\u00e9
		return super.isDirty();
	}

	@Override
	protected void pageChange(int newPageIndex) {
		// TODO Raccord de m\u00e9thode auto-g\u00e9n\u00e9r\u00e9
		super.pageChange(newPageIndex);
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		// TODO Raccord de m\u00e9thode auto-g\u00e9n\u00e9r\u00e9
		attributesPage.doSave(monitor);
	}
	
} 
