package org.objectweb.petals.tools.plugin.monitor.configure;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;


public class GeneralPage extends CommonPage {
	
	private Text componentNameText;
	private Text componentTypeText;
	private Text componentDescriptionText;
	private Text componentClassText;
	
	public GeneralPage(FormEditor formEditor) {
		super(formEditor, "common", "G\u00e9n\u00e9ral");
	}

	@Override
	protected void createFormContent(IManagedForm managedForm) {
		super.createFormContent(managedForm);
		FormToolkit toolkit = managedForm.getToolkit();
		form.setBackgroundImage(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/form_banner.gif").createImage());
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		form.getBody().setLayout(layout);
		createTableSection(form, toolkit, "G\u00e9n\u00e9ralit\u00e9s");
		updatePage();
	}

	@Override
	public void updatePage() {
		form.setText("G\u00e9n\u00e9ralit\u00e9s sur " + component.getName());
		componentNameText.setText(component.getName() != null ? component.getName() : "");
		componentTypeText.setText(component.getType() != null ? component.getType() : "");
		String beanClassName = component.getConfigurationMBean().getClassName();
		componentClassText.setText(beanClassName != null ? beanClassName : "");
		String beanDescription = component.getConfigurationMBean().getDescription();
		componentDescriptionText.setText(beanDescription != null ? beanDescription : "");
	}
	
	
	private void createTableSection(final ScrolledForm form, FormToolkit toolkit, String title) {
		Section section = toolkit.createSection(form.getBody(), Section.TWISTIE | Section.DESCRIPTION);
		section.setActiveToggleColor(toolkit.getHyperlinkGroup().getActiveForeground());
		section.setToggleColor(toolkit.getColors().getColor(FormColors.SEPARATOR));
		toolkit.createCompositeSeparator(section);
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		
		client.setLayout(layout);

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalAlignment = GridData.FILL;
        gd.grabExcessHorizontalSpace = true;
        
        toolkit.createLabel(client, "Nom");
		componentNameText = toolkit.createText(client, "");
		componentNameText.setEditable(false);
		componentNameText.setLayoutData(gd);
		
		toolkit.createLabel(client, "Type");
		componentTypeText = toolkit.createText(client, "");
		componentTypeText.setEditable(false);
		componentTypeText.setLayoutData(gd);
		
		toolkit.createLabel(client, "Classe");
		componentClassText = toolkit.createText(client, "");
		componentClassText.setEditable(false);
		componentClassText.setLayoutData(gd);
		
		gd = new GridData();
		gd.verticalAlignment = SWT.TOP;
		toolkit.createLabel(client, "Description").setLayoutData(gd);
		
		componentDescriptionText = toolkit.createText(client, "", SWT.MULTI | SWT.WRAP);
		componentDescriptionText.setEditable(false);
		gd = new GridData(GridData.FILL_BOTH);
		componentDescriptionText.setLayoutData(gd);
		
		toolkit.paintBordersFor(client);
		section.setText(title);
		section.setDescription("Informations g\u00e9n\u00e9rales sur le " + component.getType() + " " +component.getName());
		section.setClient(client);
		section.setExpanded(true);
		section.addExpansionListener(new ExpansionAdapter() {
			public void expansionStateChanged(ExpansionEvent e) {
				form.reflow(false);
			}
		});
		gd = new GridData(GridData.FILL_BOTH);
		section.setLayoutData(gd);
	}
	
	

} 
