package org.objectweb.petals.tools.plugin.monitor.configure;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.FormColors;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.events.ExpansionAdapter;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.views.NotificationChangedInterface;
import org.objectweb.petals.tools.plugin.monitor.views.NotificationsView;


public class NotificationsPage extends CommonPage implements NotificationChangedInterface {
	
	private TableViewer tableViewer;
	
	public NotificationsPage(FormEditor formEditor) {
		super(formEditor, "notifications", "Notifications");
	}
	
	@Override
	public void updatePage() {
		form.setText("Notifications de " + component.getName());
	}
	
	protected void createFormContent(IManagedForm managedForm) {
		super.createFormContent(managedForm);
		FormToolkit toolkit = managedForm.getToolkit();
		form.setBackgroundImage(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/form_banner.gif").createImage());
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		form.getBody().setLayout(layout);
		createTableSection(form, toolkit, "Notifications");
		component.addNotificationListener(this);
		component.subscribe();
		updatePage();
	}
	
	private void createTableSection(final ScrolledForm form, FormToolkit toolkit, String title) {
		Section section = toolkit.createSection(form.getBody(), Section.TWISTIE | Section.DESCRIPTION);
		section.setActiveToggleColor(toolkit.getHyperlinkGroup().getActiveForeground());
		section.setToggleColor(toolkit.getColors().getColor(FormColors.SEPARATOR));
		toolkit.createCompositeSeparator(section);
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		
		client.setLayout(layout);

		GridData gd = new GridData(GridData.FILL_BOTH);
        gd.horizontalAlignment = GridData.FILL;
        gd.grabExcessHorizontalSpace = true;
        gd.grabExcessVerticalSpace = true;
        gd.verticalAlignment = GridData.FILL;
		
		tableViewer = NotificationsView.getInstance().createTable(client);
		
		gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 200;
		gd.widthHint = 100;
		tableViewer.getTable().setLayoutData(gd);
		toolkit.paintBordersFor(client);
						
		section.setText(title);
		section.setDescription("Liste des notifications du composant " + component.getName());
		section.setClient(client);
		section.setExpanded(true);
		section.addExpansionListener(new ExpansionAdapter() {
			public void expansionStateChanged(ExpansionEvent e) {
				form.reflow(false);
			}
		});
		gd = new GridData(GridData.FILL_BOTH);
		section.setLayoutData(gd);
	}
	

	@Override
	public void dispose() {
		component.removeNotificationListener(this);
		component.unsubscribe();
	}

	public void changedNotifications() {
		try {
			tableViewer.setInput(component.getNotification());
		} catch (Exception e) {
			MessageDialog.openError(null, "Notifications", "Impossible de retirer les notifications de le bean d'informations de l'objet JMX de ce composant");
		} 
	}
	
} 
