package org.objectweb.petals.tools.plugin.monitor.configure;

import java.util.ArrayList;
import java.util.List;

import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class ScrolledAttributesBlock extends MasterDetailsBlock implements AttributeContainer.ModifyModelListener {
	
	private AttributesPage page;
	private TableViewer viewer;
	private PetalsComponent component;
	private Table table;
	
	public ScrolledAttributesBlock(AttributesPage page) {
		this.page = page;
		component = page.component;
	}

	/**
	 * @param id
	 * @param title
	 */
	class MasterContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof PetalsComponentInput) {
				PetalsComponentInput input = (PetalsComponentInput) page.getEditor().getEditorInput();
				MBeanInfo confBean = input.getComponent().getConfigurationMBean();
				List<AttributeContainer> containers = new ArrayList<AttributeContainer>();
				for (MBeanAttributeInfo mAttributeInfo: confBean.getAttributes()) {
					AttributeContainer aContainer = new AttributeContainer(component.getConnection(), component.getObjectName(), mAttributeInfo);
					aContainer.addModifyModelListener(ScrolledAttributesBlock.this);
					containers.add(aContainer);
				}
				return containers.toArray(new AttributeContainer[containers.size()]);
			}
			return new Object[0];
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	class MasterLabelProvider extends LabelProvider implements ITableLabelProvider {
		public String getColumnText(Object obj, int index) {
			if (obj instanceof AttributeContainer) {
				return ((AttributeContainer)obj).getName();				
			}
			return obj.toString();
		}

		public Image getColumnImage(Object obj, int index) {
			if (obj instanceof AttributeContainer) {
				return 	((AttributeContainer)obj).getImage();
			}
			return null;
		}
	}

	protected void createMasterPart(final IManagedForm managedForm, Composite parent) {
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.DESCRIPTION | Section.TITLE_BAR);
		section.setText("Attributs");
		section.setDescription("Attributs disponibles dans le " + component.getType() + " " + component.getName());
		section.marginWidth = 10;
		section.marginHeight = 5;
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		client.setLayout(layout);
		table = toolkit.createTable(client, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 20;
		gd.widthHint = 100;
		table.setLayoutData(gd);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		final SectionPart spart = new SectionPart(section);
		managedForm.addPart(spart);
		viewer = new TableViewer(table);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				managedForm.fireSelectionChanged(spart, event.getSelection());
			}
		});
		viewer.setContentProvider(new MasterContentProvider());
		viewer.setLabelProvider(new MasterLabelProvider());
		viewer.setInput(page.getEditor().getEditorInput());
	}

	protected void createToolBarActions(IManagedForm managedForm) {
		final ScrolledForm form = managedForm.getForm();
		Action haction = new Action("hor", Action.AS_RADIO_BUTTON) { //$NON-NLS-1$
			public void run() {
				sashForm.setOrientation(SWT.HORIZONTAL);
				form.reflow(true);
			}
		};
		haction.setChecked(true);
		haction.setToolTipText("Disposition horizontale");
		haction.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
				Activator.PLUGIN_ID, "icons/horizontal.gif"));

		Action vaction = new Action("ver", Action.AS_RADIO_BUTTON) {
			public void run() {
				sashForm.setOrientation(SWT.VERTICAL);
				form.reflow(true);
			}
		};
		vaction.setChecked(false);
		vaction.setToolTipText("Disposition verticale");
		vaction.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
				Activator.PLUGIN_ID, "icons/vertical.gif"));
		form.getToolBarManager().add(haction);
		form.getToolBarManager().add(vaction);
	}

	protected void registerPages(DetailsPart detailsPart) {
		detailsPart.registerPage(AttributeContainer.class, new TypeMBeanAttributeDetailsPage());
	}
	
	public void refresh() {
		if (viewer != null) {
			viewer.refresh();
		}
	}
	
	public void update() {
		
	}
	
	public boolean isDirty() {
		if (detailsPart == null) {
			return false;
		}
		return detailsPart.isDirty();
	}

	public void modifiedModel(AttributeContainer attributeContainer) {
		if (table != null) {
			//TODO
			table.update();
		}
	}
	
	public void doSave(IProgressMonitor monitor) {
		if (detailsPart != null) {
			detailsPart.commit(true);
		}
	}
	
}