package org.objectweb.petals.tools.plugin.monitor.configure;

import java.util.List;

import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class ScrolledMethodsBlock extends MasterDetailsBlock {
	
	private MethodsPage page;
	private TableViewer viewer;
	private PetalsComponent component;
	private Table table;
	
	public ScrolledMethodsBlock(MethodsPage page) {
		this.page = page;
		component = page.component;
	}

	/**
	 * @param id
	 * @param title
	 */
	class MasterContentProvider implements IStructuredContentProvider {
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof PetalsComponentInput) {
				PetalsComponentInput input = (PetalsComponentInput) page.getEditor().getEditorInput();
				MBeanInfo confBean = input.getComponent().getConfigurationMBean();
				return (MBeanOperationInfo[])confBean.getOperations(); 
			}
			return new Object[0];
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	class MasterLabelProvider extends LabelProvider implements ITableLabelProvider {
		
		final Image DEFAULT_METHOD = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/method.gif").createImage();
		
		public String getColumnText(Object obj, int index) {
			if (obj instanceof MBeanOperationInfo) {
				return ((MBeanOperationInfo)obj).getName();			
			}
			return obj.toString();
		}

		public Image getColumnImage(Object obj, int index) {
			if (obj instanceof MBeanOperationInfo) {
				return 	DEFAULT_METHOD;
			}
			return null;
		}
	}

	protected void createMasterPart(final IManagedForm managedForm,
			Composite parent) {
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.DESCRIPTION
				| Section.TITLE_BAR);
		section.setText("M\u00e9thodes");
		section.setDescription("Methods disponibles dans le " + component.getType() + " " + component.getName() + ":");
		section.marginWidth = 10;
		section.marginHeight = 5;
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		client.setLayout(layout);
		table = toolkit.createTable(client, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.heightHint = 20;
		gd.widthHint = 70;
		table.setLayoutData(gd);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		final SectionPart spart = new SectionPart(section);		
		managedForm.addPart(spart);
		viewer = new TableViewer(table);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				managedForm.fireSelectionChanged(spart, event.getSelection());
			}
		});
		viewer.setContentProvider(new MasterContentProvider());
		viewer.setLabelProvider(new MasterLabelProvider());
		viewer.setInput(page.getEditor().getEditorInput());
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				 invokeMethod();
			}
		});
		
		
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.add(new Action() {
	
			public String getText() {
				return "Invoquer";
			}

			public void run() {
				invokeMethod();				
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		
	}

	protected void createToolBarActions(IManagedForm managedForm) {
		final ScrolledForm form = managedForm.getForm();
		Action clearAction = new Action("clear", Action.AS_PUSH_BUTTON) { 
			public void run() {
				
			}
		};
		clearAction.setChecked(true);
		clearAction.setToolTipText("Effacer la valeur de tous les arguments");
		clearAction.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(
				Activator.PLUGIN_ID, "icons/clear.gif"));

		Action invokeAction = new Action("invoke", Action.AS_PUSH_BUTTON) {
			public void run() {
				invokeMethod();
			}
		};
		invokeAction.setChecked(false);
		invokeAction.setToolTipText("Invoker la signature de m\u00e9thode s\u00e9lectionn\u00e9e et renseign\u00e9e");
		invokeAction.setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/invoke.gif"));
		form.getToolBarManager().add(clearAction);
		form.getToolBarManager().add(invokeAction);
	}

	protected void clearAguments() {
		
	
	}
	
	protected void invokeMethod() {
		List selection = ((IStructuredSelection)viewer.getSelection()).toList();
		if (!selection.isEmpty()) {
			MBeanOperationInfo mBeanOperationInfo = ((MBeanOperationInfo)selection.get(0));
			try {
				if (mBeanOperationInfo.getSignature().length == 0) {
					component.getConnection().invoke(component.getObjectName(), mBeanOperationInfo.getName(), new Object[0], new String[0]);
//					TODO: gestion de la valeur de retour
					PetalsServersView.refresh();
				}
			} catch (Exception e) {
				MessageDialog.openError(null, "Appel d'une m\u00e9thode", "Impossible d'appeler la m\u00e9thode " + mBeanOperationInfo.getName());
			} 
		}
	}
	
	protected void registerPages(DetailsPart detailsPart) {
		detailsPart.registerPage(MBeanOperationInfo.class, new TypeMBeanMethodDetailsPage());
	}
	
	public void refresh() {
		if (viewer != null) {
			viewer.refresh();
		}
	}
	
	public void update() {
		
	}
	
	public boolean isDirty() {
		if (detailsPart == null) {
			return false;
		}
		return detailsPart.isDirty();
	}

	public void doSave(IProgressMonitor monitor) {
		if (detailsPart != null) {
			detailsPart.commit(true);
		}
	}
	
}