package org.objectweb.petals.tools.plugin.monitor.configure;

import java.lang.reflect.Constructor;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class TypeMBeanAttributeDetailsPage implements IDetailsPage, ModifyListener {

	private IManagedForm mform;
	private AttributeContainer attribute;
	private Text attributeValue;
	private Label attributeName;
	private Section section;
	private boolean dirty;
	private Composite parent;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#initialize(org.eclipse.ui.forms.IManagedForm)
	 */
	public void initialize(IManagedForm mform) {
		this.mform = mform;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		this.parent = parent;
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 5;
		layout.leftMargin = 5;
		layout.rightMargin = 2;
		layout.bottomMargin = 2;
		parent.setLayout(layout);

		FormToolkit toolkit = mform.getToolkit();
		section = toolkit.createSection(parent, Section.DESCRIPTION | Section.TITLE_BAR);
		section.marginWidth = 10;
		section.setText("Valeur");
		section.setDescription("Valeur du composant '" + "comp1" + "' de type '" + "type" + "': "); 
		TableWrapData td = new TableWrapData(TableWrapData.FILL, TableWrapData.TOP);
		td.grabHorizontal = true;
		section.setLayoutData(td);
		Composite client = toolkit.createComposite(section);
		GridLayout glayout = new GridLayout();
		glayout.marginWidth = glayout.marginHeight = 0;
		glayout.numColumns = 2;
		client.setLayout(glayout);		
		createSpacer(toolkit, client, 2);	//spacer
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
		attributeName = toolkit.createLabel(client, "");
		attributeValue = toolkit.createText(client, "", SWT.SINGLE);
		
		gd.widthHint = 40;
		attributeValue.setLayoutData(gd);

		toolkit.paintBordersFor(section);
		section.setClient(client);
	}

	private void createSpacer(FormToolkit toolkit, Composite parent, int span) {
		Label spacer = toolkit.createLabel(parent, "");
		GridData gd = new GridData();
		gd.horizontalSpan = span;
		spacer.setLayoutData(gd);
	}

	private void update() {
		section.setDescription("Valeur pour le param\u00e8tre "+ (!attribute.isModifiable() ? "en lecture seule " : "") + attribute.getName() + " de type " + attribute.getType() + ": ");
		attributeName.setText(attribute.getShortName());
		attributeValue.removeModifyListener(this);
		attributeValue.setText(attribute.getValue() != null ? valueToString(attribute.getValue()) : "");		// entr\u00e9e de la valeur
		attributeValue.setEditable(attribute.isModifiable());
		attributeValue.addModifyListener(this);
		parent.pack();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#inputChanged(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void selectionChanged(IFormPart part, ISelection selection) {
		IStructuredSelection ssel = (IStructuredSelection) selection;
		if (ssel.size() == 1) {
			attribute = (AttributeContainer) ssel.getFirstElement();
			update();
		} else {
			attribute = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#commit()
	 */
	public void commit(boolean onSave) {
		if (onSave) {
			attribute.setValue(stringToValue(attributeValue.getText(), attribute.getType()));	
			dirty = false;
			mform.dirtyStateChanged();	
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#setFocus()
	 */
	public void setFocus() {
		attributeValue.setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}

	public boolean isStale() {
		if (attribute == null) {
			return false;
		}
		String attributeValueString = attributeValue.getText();
		String modelValue = valueToString(attribute.getValue());
		return attributeValueString == null && attributeValueString == null
				|| attributeValueString != null && modelValue != null
				&& attributeValueString.equals(modelValue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#refresh()
	 */
	public void refresh() {
		update();
	}

	public boolean setFormInput(Object input) {
		return false;
	}
	
	public void modifyText(ModifyEvent e) {
		dirty = true;
		mform.dirtyStateChanged();
	}
	
	public String valueToString(Object value) {
		if (value == null) {
			return "";
		}
		return value.toString();
	}
	
	public Object stringToValue(String str, String classname) {
		try {
			Class clazz = Class.forName(classname);
			Constructor constructor = clazz.getConstructor(new Class[]{String.class});
			return constructor.newInstance(new Object[] {str});
		} catch (Exception e) {
			MessageDialog.openError(null, "Conversion valeur textuel de l'attribut vers objet", "Erreur de conversion");
			return null;
		}
		
		
	}
}
