package org.objectweb.petals.tools.plugin.monitor.configure;

import java.lang.reflect.Constructor;

import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Hyperlink;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class TypeMBeanMethodDetailsPage implements IDetailsPage {

	private IManagedForm mform;
	private MBeanOperationInfo method;
	private Section signSection;
	private boolean dirty;
	private Composite parent;
	private FormToolkit toolkit;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#initialize(org.eclipse.ui.forms.IManagedForm)
	 */
	public void initialize(IManagedForm mform) {
		this.mform = mform;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		this.parent = parent;
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 5;
		layout.leftMargin = 5;
		layout.rightMargin = 2;
		layout.bottomMargin = 2;
		parent.setLayout(layout);
		toolkit = mform.getToolkit();
		
		//signature de la m\u00e9thode
		signSection = toolkit.createSection(parent, Section.DESCRIPTION | Section.TITLE_BAR);
		signSection.marginWidth = 10;
		signSection.setText("Signature"); 
		TableWrapData td = new TableWrapData(TableWrapData.FILL, TableWrapData.TOP);
		td.grabHorizontal = true;
		signSection.setLayoutData(td);
		toolkit.paintBordersFor(signSection);
	}

	public void createMethodSign() {
		Composite signClientComposite = toolkit.createComposite(signSection);
		GridLayout glayout = new GridLayout();
		glayout.marginWidth = glayout.marginHeight = 0;
		glayout.marginLeft = 20;
		glayout.marginTop = 10;
		int argsSize = method.getSignature().length;
		glayout.numColumns = 4+(argsSize > 0 ? 2*argsSize-1 : 0);
		signClientComposite.setLayout(glayout);		
		toolkit.createLabel(signClientComposite, method.getReturnType());
		Hyperlink methodNameLink = toolkit.createHyperlink(signClientComposite, method.getName(), SWT.WRAP);
		toolkit.createLabel(signClientComposite, "(");
		methodNameLink.addHyperlinkListener(new IHyperlinkListener() {

			public void linkActivated(HyperlinkEvent e) {
				((MethodsPage)mform.getContainer()).getBlock().invokeMethod();
			}

			public void linkEntered(HyperlinkEvent e) {
			}

			public void linkExited(HyperlinkEvent e) {
			}
			
		});
		for (MBeanParameterInfo parameter: method.getSignature()) {
			toolkit.createHyperlink(signClientComposite, parameter.getType(), SWT.WRAP);
			toolkit.createLabel(signClientComposite, ",");
		}		
		toolkit.createLabel(signClientComposite, ")");
		signSection.setClient(signClientComposite);
	}

	private void update() {
		if (signSection != null) {
			signSection.setDescription("Signature de la m\u00e9thode " + method.getName() + ":");
			createMethodSign();
		}		
		parent.pack();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#inputChanged(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void selectionChanged(IFormPart part, ISelection selection) {
		IStructuredSelection ssel = (IStructuredSelection) selection;
		if (ssel.size() == 1) {
			method = (MBeanOperationInfo) ssel.getFirstElement();
			update();
		} else {
			method = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#commit()
	 */
	public void commit(boolean onSave) {
		if (onSave) {
			//TODO: write argument value	
			dirty = false;
			mform.dirtyStateChanged();	
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#setFocus()
	 */
	public void setFocus() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#dispose()
	 */
	public void dispose() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#isDirty()
	 */
	public boolean isDirty() {
		return dirty;
	}

	public boolean isStale() {
		//TODO
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#refresh()
	 */
	public void refresh() {
		update();
	}

	public boolean setFormInput(Object input) {
		return false;
	}
	
	public void modifyText(ModifyEvent e) {
		dirty = true;
		mform.dirtyStateChanged();
	}
	
	public String valueToString(Object value) {
		if (value == null) {
			return "";
		}
		return value.toString();
	}
	
	public Object stringToValue(String str, String classname) {
		try {
			Class clazz = Class.forName(classname);
			Constructor constructor = clazz.getConstructor(new Class[]{String.class});
			return constructor.newInstance(new Object[] {str});
		} catch (Exception e) {
			MessageDialog.openError(null, "Conversion valeur textuel de l'attribut vers objet", "Erreur de conversion");
			return null;
		}
		
		
	}
	
	
}
