package org.objectweb.petals.tools.plugin.monitor.dnd;

import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DragSourceListener;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceAssembly;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceInterface;
import org.objectweb.petals.tools.plugin.monitor.model.SharedLibrary;


public class ComponentDragSource implements DragSourceListener {

	private TreeViewer viewer;
	
	public ComponentDragSource(TreeViewer viewer) {
		this.viewer = viewer;
		DragSource source = new DragSource(viewer.getControl(), DND.DROP_COPY);
		source.setTransfer(new Transfer[]{TextTransfer.getInstance()});
		source.addDragListener(this);
	}

	public void dragFinished(DragSourceEvent event) {
		List selectedItems = ((IStructuredSelection)viewer.getSelection()).toList();
		for (Object o: selectedItems) {
			try {
				if (o instanceof PetalsComponent) {
					((PetalsComponent)o).uninstall();
				} else if (o instanceof SharedLibrary) {
					((SharedLibrary)o).unload();
				} else if (o instanceof ServiceInterface) {
					((ServiceInterface)o).undeploy();
				}
				if (o instanceof SharedLibrary || o instanceof PetalsComponent || o instanceof ServiceAssembly) {
					//((TreeObject)o).getParent().removeChild((TreeObject)o);	
					((ComponentParent)o).getServer().remove((ComponentParent)o);
				}
				viewer.refresh();
			} catch(Exception e) {
				MessageDialog.openError(null, "D\u00e9sinstallation par DnD","Impossible de replier le SA (" + e.getMessage() + ")");
			}
		}
		event.doit = true;
	}

	public void dragSetData(DragSourceEvent event) {
		
	}

	public void dragStart(DragSourceEvent event) {
		List selectedItems = ((IStructuredSelection)viewer.getSelection()).toList();
		event.doit = selectedItems.size() > 0;
	}

}
