package org.objectweb.petals.tools.plugin.monitor.dnd;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 27 f\u00e9vr. 07
 */
public class ComponentDropTarget extends DropTargetAdapter {
	
	private TreeViewer viewer;
 	
	public ComponentDropTarget(TreeViewer viewer) {
		 this.viewer = viewer;
		 DropTarget target = new DropTarget(viewer.getControl(), DND.DROP_COPY);
		 target.setTransfer(new Transfer[] {TextTransfer.getInstance(), FileTransfer.getInstance()});
		 target.addDropListener(this);
	}

	@Override
	public void dragEnter(DropTargetEvent event) {
		/*if (event.detail == DND.DROP_COPY || event.detail == DND.DROP_DEFAULT) {
			if ((event.operations & DND.DROP_COPY) != 0) {
				event.detail = DND.DROP_COPY;
			} else {
				event.detail = DND.DROP_NONE;
			}
		}*/
		event.detail = DND.DROP_COPY;
	}

	@Override
	public void drop(DropTargetEvent event) {
		PetalsServer ps = null;
		if (event.item.getData() instanceof ComponentParent) {
			ps = ((ComponentParent)event.item.getData()).getServer();
		}
		if (ps != null) {
			URL url;
			for (String data: (String[])event.data) {
				try {
					url = new File(data).toURL();
					ComponentParent component = ps.install(url);
					if (component == null) {
						continue;
					}
					ps.add(component);					
				} catch (MalformedURLException e) {
					MessageDialog.openError(null, "Installation composant", "Le composant d\u00e9pos\u00e9 a une URL malform\u00e9e");
				}
			}
			viewer.refresh();
		} else {
			MessageDialog.openError(null, "Installation composant", "Le composant doit \u00eatre d\u00e9pos\u00e9 sur un serveur Petals ou l'un de ses sous-composants");
		}
	}

	@Override
	public void dropAccept(DropTargetEvent event) {
		if (!(event.currentDataType instanceof TransferData)) {
			event.detail = DND.DROP_NONE;
		}
		if (event.item.getData() instanceof ComponentParent) {
			if (!((ComponentParent)event.item.getData()).getServer().isConnected()) {
				event.detail = DND.DROP_NONE;
			}
		}
	}
}
