package org.objectweb.petals.tools.plugin.monitor.model;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class ComponentAnalyzer {
	
	/* 2 acc\u00e8s possibles au descripteur JBI, selon comment le packager a zipp\u00e9 le composant ( \ ou /) */
	public final static String JBI_DESCRIPTOR_FILENAME1 = "META-INF/jbi.xml";
	public final static String JBI_DESCRIPTOR_FILENAME2 = "META-INF\\jbi.xml";
	
	private URL url;
	private ComponentType type;
	private String description;
	private String name;
	
	public ComponentAnalyzer(String zipUrl) {
		setUrl(zipUrl);
	}
	
	public void setUrl(String zipUrl) {
		try {
			url = new URL(zipUrl);
		} catch (MalformedURLException e) {
			url = null;
		}
		type = null;
	}
	
	public ComponentType getType() {
		if (type == null) {
			analyze();
		}		
		return type;
	}
	
	public String getTypeLabel() {
		return getType().toString();
	}
	
	/**
	 * D\u00e9termine, le type (BC, SE, SL, SA), le nom 
	 * et la description du composant
	 */
	public void analyze() {
		type = ComponentType.unknown;
		if (url == null) {
			return;
		}
		try {
			InputStream is = url.openStream();
			ZipInputStream zis = new ZipInputStream(is);
			ZipEntry entry;			
			while ((entry = zis.getNextEntry()) != null) {
				//System.out.println(entry.getName());
				if (entry.getName().equals(JBI_DESCRIPTOR_FILENAME1) || entry.getName().equals(JBI_DESCRIPTOR_FILENAME2)) {
					StringBuffer sb = new StringBuffer();
					int c;
					while ((c = zis.read()) != -1) {
						sb.append((char)c);	
					}
					//System.out.println("[" + sb + "]");
					DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	                DocumentBuilder builder = factory.newDocumentBuilder();
	                Document document = builder.parse(new ByteArrayInputStream(sb.toString().getBytes()));
	                Element root = document.getDocumentElement();
	                NodeList nl = root.getElementsByTagName("component");
	                NodeList ids;
	                if (nl.getLength() > 0) {
	                	Element e = (Element)nl.item(0);
	                	String t  = e.getAttribute("type");
						if (type != null) {
							if (t.equals("service-engine")) {
								type = ComponentType.serviceEngine;
							} else if (t.equals("binding-component")) {
								type = ComponentType.bindingComponent;
							} else {
								System.out.println("New type found: " + type);
								type = ComponentType.unknown;
							}
						}
						ids = e.getElementsByTagName("identification");
	                } else if ((ids = root.getElementsByTagName("shared-library")).getLength() > 0) {
	                	type = ComponentType.sharedLibrary;
	                	ids = ((Element)ids.item(0)).getElementsByTagName("identification");
	                } else if ((ids = root.getElementsByTagName("service-assembly")).getLength() > 0) {
	                	type = ComponentType.serviceAssembly;
	                	ids = ((Element)ids.item(0)).getElementsByTagName("identification");
	                }
                	if (ids.getLength() > 0) {
						extractNameAndDescription((Element)ids.item(0));
					}
	                break;
				}
			}
			zis.close();
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}
	
	/**
	 * Extrait le nom et la description depuis le noeud
	 * identification
	 * @param idElement Noeud identification
	 */
	void extractNameAndDescription(Element idElement) {
		NodeList nodeList = idElement.getElementsByTagName("name");
		if (nodeList.getLength() > 0) {
			name = nodeList.item(0).getTextContent();	
		}
		nodeList = idElement.getElementsByTagName("description");
		if (nodeList.getLength() > 0) {
			description = nodeList.item(0).getTextContent();	
		}
	}
	
	public String getDescription() {
		return description;
	}
	
	public String getName() {
		return name;
	}
	
//	public static void main(String[] args) {
//		// just for debugging
//		System.out.println(new ComponentAnalyzer("file:///c:/petals-1.1-last/installed/" + args[0]).getType());
//	}
	
}
