package org.objectweb.petals.tools.plugin.monitor.model;

import java.util.List;

import javax.management.ObjectName;

import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeParent;


public class ComponentParent extends TreeParent {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2747057016524335506L;
	public final static String ADDED_TAG = "addedTag";
	public final static String REMOVED_TAG = "removedTag";
	
	transient protected ObjectName objectName;
	protected String name;
	protected String type;
	protected String description;
	protected PetalsServer server;
	protected String url;
	protected String tag;
	
	public String getTag() {
		return tag;
	}

	public void setTag(String tag) {
		this.tag = tag;
	}

	public boolean isNotTagged() {
		return tag == null;
	}
	
	public boolean isAddedTagged() {
		return !isNotTagged() && tag.equals(ADDED_TAG);
	}
	
	public boolean isRemovedTagged() {
		return !isNotTagged() && tag.equals(REMOVED_TAG);
	}
	
	public ComponentParent() {
	
	}
	
	public ComponentParent(ObjectName objectName) {
		super();
		this.objectName = objectName;
		this.name = objectName.getKeyProperty("name");
		this.type = objectName.getKeyProperty("type");
		if (type == null) {
			type = "unknown component";
		}
	}
	
	public ComponentParent(String name) {
		super();
		this.name = name;
	}
	
	public ComponentParent(String name, String description) {
		this(name);
		this.description = description;
	}
	
	public ComponentParent(String name, List<? extends TreeObject> children) {
		this.name = name;
		if (children != null) {
			addChildren(children.toArray(new TreeObject[children.size()]));	
		}		
	}
	
	public String toString() {
		return (name != null ? name : "unknown " + getType()) + (description != null ? " (" + description + ")" : "")+  (hasChildren() ? " (" + getChildren().length + ")" : ""); 
	}

	public String getName() {
		return name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public ObjectName getObjectName() {
		return objectName;
	}

	public void setObjectName(ObjectName objectName) {
		this.objectName = objectName;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}
	
	public void setServer(PetalsServer server) {
		this.server = server;
	}

	public PetalsServer getServer() {
		return server;
	}
	
	public String getUrl() {
		return url;
	}
	
	public void setUrl(String componentZipUrl) {
		this.url = componentZipUrl;
	}
		
	public boolean isUpdate() {
		return url != null && !url.equals("");
	}
	
}
