package org.objectweb.petals.tools.plugin.monitor.model;

import java.io.IOException;
import java.lang.reflect.UndeclaredThrowableException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.DeploymentServiceMBean;
import javax.jbi.management.InstallationServiceMBean;
import javax.jbi.management.InstallerMBean;
import javax.management.Attribute;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.views.properties.IPropertySource;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.preferences.PreferenceConstants;
import org.objectweb.petals.tools.plugin.monitor.properties.PetalsComponentPropertySource;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeParent;
import org.objectweb.petals.tools.plugin.monitor.views.NotificationChangedInterface;
import org.objectweb.petals.tools.plugin.monitor.views.NotificationsView;
import org.objectweb.petals.tools.plugin.monitor.views.PerformanceNotifierView;


public abstract class PetalsComponent extends ComponentParent {

	transient protected ComponentLifeCycleMBean lifecycle;
	
	transient private DeploymentServiceMBean deploy;
	transient private InstallationServiceMBean install;
	transient private InstallerMBean installer;
	transient private MBeanServerConnection connection;
	transient private NotificationListener notificationListener;
    transient private NotificationListener perfNotifsListener;
	transient private List<Notification> notifications;
	transient private List<NotificationChangedInterface> listeners;
	transient private PetalsComponentPropertySource properties;
	transient private IEditorPart editor;
	transient private Preferences prefs;
    
    transient private ObjectName perfMBeanObjectName;

	@Override
	public Object getAdapter(Class key) {
		if (key == IPropertySource.class) {
			if (properties == null) {
				properties = new PetalsComponentPropertySource(this);
			}
			return properties;
		}
		return super.getAdapter(key);
	}

	/**
	 * Verifie si le composant est installe
	 * @return vrai si le composant est installe
	 */
	public boolean isInstalled() {
		if (installer != null) {
			return installer.isInstalled();
		}
		return false;
	}
	
	/**
	 * Construit un nouveau composant PEtals (SE ou BC) 
	 * @param objectName objet JMX du composant
	 * @param lifecycle cycle de vie du composant
	 */
	public PetalsComponent(ObjectName objectName, ComponentLifeCycleMBean lifecycle) {
		super(objectName);
		setLifecycle(lifecycle);
		prefs = Activator.getDefault().getPluginPreferences();
		notifications = new ArrayList<Notification>();
		listeners = new ArrayList<NotificationChangedInterface>();
	}
		
	/**
	 * Definit le cycle de vie du composant
	 * @param lifecycle Cycle de vie du composant
	 */
	public void setLifecycle(ComponentLifeCycleMBean lifecycle) {
		this.lifecycle = lifecycle;
	}
	
	/**
	 * Installe le composant a partir de son URL
	 */
	public void install() {
		try {
			if (lifecycle != null) {
				if (!isInstalled()) {
					if (installer == null && getUrl() != null) {
						 installer =  (InstallerMBean)MBeanServerInvocationHandler.newProxyInstance(connection, install.loadNewInstaller(getUrl().toString()), InstallerMBean.class, true);
					}
					ObjectName lifecycleObjectName = installer.install();
					lifecycle = (ComponentLifeCycleMBean)MBeanServerInvocationHandler.newProxyInstance(connection, lifecycleObjectName, ComponentLifeCycleMBean.class, true);						
				} else {
					MessageDialog.openInformation(null, "Installation composant", "Le " + getType() + " '" + getName() + "' est d\u00e9j\u00e0 install\u00e9");
				}
			}
		} catch (Exception e) {
			System.err.println("Unable to install BC '" + name + "' (" + e.getMessage() + ")");
			e.printStackTrace();
		}
	}
			
	/**
	 * Desinstalle le composant donne (BC ou SE)
	 */
	public void uninstall() {
		/* FIXME: vire le noeud graphique meme qd on clique sur non car uninstall() est dissoci\u00e9e de 
		 * la MAJ UI --> fusionner MAJ UI et actions de changement d'\u00e9tat
		if (exitOnEditingConfiguration()) {
			return;
		}
		*/
		try {
			//TODO:123 uninstall associated Installer
			if (lifecycle != null) {
				if (isInstalled()) {
					if (isStarted()) {
						stop();
						shutdown();
					}
					if (isSubscribed()) {
						unsubscribe();
					}
					installer.uninstall();
					install.unloadInstaller(name, true);
					installer = null;
				} else {
					MessageDialog.openInformation(null, "D\u00e9sinstallation composant", "Le " + getType() + " '" + getName() + "' n'est pas install\u00e9 ou a d\u00e9j\u00e0 \u00e9t\u00e9 install\u00e9");
				}
			}
		} catch (Exception e) {
			System.err.println("Unable to uninstall BC '" + name + "' (" + e.getMessage() + ")");
			e.printStackTrace();
		} finally {
			notificationListener = null;
		}
	}

	/**
	 * Eteint le composant donne (BC ou SE)
	 */
	public void shutdown() {
		try {
			if (lifecycle != null) {
				if (!isStarted()) {
					lifecycle.shutDown();
				} 
			}
		} catch (Exception e) {
			MessageDialog.openError(null, "Arr\u00eat composant", "Impossible d'arr\u00eater le " + getType() +  " '" + name + "' (" + e.getMessage() + ")");
			e.printStackTrace();
		}
	}
	
	/**
	 * Demarre le composant donne (BC ou SE)
	 */
	public void start() {
		
		if (exitOnEditingConfiguration()) {
			return;
		}
		
		try {
			if (lifecycle != null) {
				if (!isStarted()) {
					lifecycle.start();
				} else {
					MessageDialog.openInformation(null, "D\u00e9marrage composant", "Le " + getType() + " '" + getName() + "' est d\u00e9j\u00e0 d\u00e9marr\u00e9");
				}
			}
		} catch (Exception e) {
			MessageDialog.openError(null, "D\u00e9marrage composant", "Impossible de d\u00e9marrer le " + getType() +  " '" + name + "' (" + e.getMessage() + ")");
			e.printStackTrace();
		}
	}
	
	public void update(){
		if (isUpdate()) {
			uninstall();
			install();			
		}
	}
	
	boolean exitOnEditingConfiguration() {
		if (isEdited() && !MessageDialog.openQuestion(null, "Changement d'\u00e9tat du composant", "La configuration de ce composant est en cours d'\u00e9dition, le changement d'\u00e9tat, fermera la page de configuration. tes-vous d'accord pour continuer ?")) {			
			return true;
		} else {
			closeEditor();
			return false;
		}
	}
	
	/**
	 * Arrete le composant donne (BC ou SE) 
	 */
	public void stop() {
		
		if (exitOnEditingConfiguration()) {
			return;
		}
		
		try {
			if (lifecycle != null) {
				if (isStarted()) {
					lifecycle.stop();
				} else {
					MessageDialog.openInformation(null, "Arr\u00eat composant", "Le " + getType() + " '" + getName() + "' est d\u00e9j\u00e0 stopp\u00e9");
				}
			}
		} catch (Exception e) {
			MessageDialog.openError(null, "Arr\u00eat composant", "Impossible d'arr\u00eater le " + getType() +  " '" + name + "' (" + e.getMessage() + ")");			
		}		
	}
	
	/**
	 * Verifie si le composant est demarre
	 * @return vrai si le composant est demarre
	 */
	public boolean isStarted() {
		if (lifecycle == null) {
			return false;
		}
		try {
			String currentState = lifecycle.getCurrentState();
			return currentState.equalsIgnoreCase(ComponentLifeCycleMBean.STARTED);
		} catch(UndeclaredThrowableException e) {
			return false;
		}
	}
	
	/**
	 * Verifie si le composant est un Service Engine
	 * @return true si le composant est un SE
	 */
	public abstract boolean isEngine();
	
	/**
	 * Verifie si le composant est un Binding Component
	 * @return true si le composant est un BC
	 */
	public abstract boolean isBinding();
	
	/** Renvoie le format textuel du type du composant (BC ou SE)
	 * @return Type texte du composant
	 */
	public abstract String getType();

	/**
	 * Verifie si le composant est eteint
	 * @return vrai si le composant est eteint
	 */
	public boolean isShutdown() {
		return lifecycle != null && lifecycle.getCurrentState().equalsIgnoreCase(ComponentLifeCycleMBean.SHUTDOWN);
	}

	/**
	 * Verifie si le composant est stoppe
	 * @return vrai si le composant est stoppe
	 */
	public boolean isStopped() {
		return lifecycle != null && lifecycle.getCurrentState().equalsIgnoreCase(ComponentLifeCycleMBean.STOPPED);
	}
	
	/**
	 * Verifie si le composant est dans un etat inconnu
	 * @return vrai si le composant est dans un etat inconnu
	 */
	public boolean isUnknown() {
		return lifecycle != null && lifecycle.getCurrentState().equalsIgnoreCase(ComponentLifeCycleMBean.UNKNOWN);
	}

	/**
	 * Definit le bean d'installation du composant
	 * @param install Bean d'installation du composant
	 */
	public void setInstall(InstallationServiceMBean install) {
		this.install = install;
		if (install != null) {
			ObjectName installerObjectName = install.loadInstaller(name);
			if (connection != null) {
				installer = (InstallerMBean)MBeanServerInvocationHandler.newProxyInstance(connection, installerObjectName, InstallerMBean.class, true);	
			}
		}
	}

	/**
	 * Recupere la connection JMX
	 * @return Connexion JMX
	 */
	public MBeanServerConnection getConnection() {
		return connection;
	}

	/**
	 * Definit la connection JMX au travers de la laquelle les appels
	 * JMX seront effectues
	 * @param connection
	 */
	public void setConnection(MBeanServerConnection connection) {
		this.connection = connection;
		if (prefs.getBoolean(PreferenceConstants.AUTO_SUBSCRIBE_COMPONENTS)) {
            subscribe();
		}
	}

	@Override
	public TreeObject[] getChildren() {
//		IPreferenceStore store = Activator.getDefault().getPreferenceStore();
//		// Les SA sont elles affich\u00e9es sous les BC ou SE ?
//		boolean showAssemblies = store.getBoolean(PreferenceConstants.SHOW_SA);
		boolean showAssemblies = false;
		
		List<TreeObject> cs = new ArrayList<TreeObject>();
		if (deploy != null) {
			if (deploy.canDeployToComponent(name) && this.isStarted()) {
				try {
					for (String sa: deploy.getDeployedServiceAssembliesForComponent(name)) {
						ServiceAssembly nsa = new ServiceAssembly(sa);
						nsa.setComponentName(name);
						nsa.setDeploy(deploy);
						if (showAssemblies) {
							cs.add(nsa);
						} else {
							cs.addAll(Arrays.asList(nsa.getChildren()));
						}
					}
				} catch (Exception e) {
					MessageDialog.openError(null, "Service Assemblies", "Impossible d'extraire les services assemblies du " + getType() +  " '" + getName() + "' (" + e.getMessage() + ")");
					e.printStackTrace();
				}	
			}
			else if (this.isStopped() && this.isInstalled()) {
				try {
					for (String sa: deploy.getDeployedServiceAssembliesForComponent(name)) {
						ServiceAssembly nsa = new ServiceAssembly(sa);
						nsa.setComponentName(name);
						nsa.setDeploy(deploy);
						if (showAssemblies) {
							cs.add(nsa);
						} else {
							cs.addAll(Arrays.asList(nsa.getChildren()));
						}
					}
				} catch (Exception e) {
					MessageDialog.openError(null, "Service Assemblies", "Impossible d'extraire les services assemblies du " + getType() +  " '" + getName() + "' (" + e.getMessage() + ")");
					e.printStackTrace();
				}
			}
		}
		return cs.toArray(new TreeParent[cs.size()]);
	}

	@Override
	public boolean hasChildren() {
		try {
			String[] deployedSA = deploy.getDeployedServiceAssembliesForComponent(name);
			boolean dsa = deploy != null && deployedSA.length > 0;
			return dsa;
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * D\u00e9finit le bean de d\u00e9ploiement du composant
	 * @param deploy Bean de d\u00e9ploiement
	 */
	public void setDeploy(DeploymentServiceMBean deploy) {
		this.deploy = deploy;
	}
	
	/**
	 * V\u00e9rifie si le composant est r\u00e9installable
	 * ie si le composant a son installateur et son URL
	 * de rechargement
	 * @return vrai si le composant est r\u00e9installable
	 */
	public boolean isUpdate() {
		return installer != null && url != null;
	}
	
	/**
	 * Souscris ce composant  la reception de notifications le concernant
	 */
	public void subscribe() {
	    notificationListener = new NotificationListener() {
	        public void handleNotification(Notification notification, Object arg1) {
	            System.out.println("Notification: " + notification);
	            notifications.add(notification);
	            Display.getDefault().asyncExec(new Runnable() {
	                public void run() {
	                    processListeners();
	                }
	            });
	        }
	    };
	    try {
	        if (connection != null) {
	            connection.addNotificationListener(objectName, notificationListener, null, null);
	            System.out.println("Abonnement du " + getType() + " " + getName());
	            processListeners();				
	        }
	    } catch (Exception e) {
	        MessageDialog.openError(null, "Abonnement", "Impossible d'abonner le " + getType()  + " " + getName() + " aux notifications (" + e.getMessage() + ")");
	    } 
	}
    
    /**
     * Performances notifications subscription
     */
    public void subscribePerfNotifs() {
        try {
            // Ajout d'un listener
            perfNotifsListener = new NotificationListener() {
                public void handleNotification(Notification notification, Object handback) {
                    if (notification.getType().equals("performanceNotificationData") 
                            && PerformanceNotifierView.getInstance() != null) {
                        try {
                            Timestamp tsTime = new Timestamp(Long.parseLong(((String[])notification.getUserData())[1]));
                            String[] userDatasWithSeq = 
                                new String[] { ((String[])notification.getUserData())[0],
                                    tsTime.toString(),
                                    ((String[])notification.getUserData())[2],
                                    ((String[])notification.getUserData())[3],
                                    ((String[])notification.getUserData())[4],
                                    ((String[])notification.getUserData())[5] };
                            
                            PerformanceNotifierView.getInstance().getUserData().add(userDatasWithSeq);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                }
            };
            // Permet d'obtenir la liste des composants a tracer
            Set names = connection.queryNames(null, null);
            for (Iterator i = names.iterator(); i.hasNext();) {
                ObjectName onName = (ObjectName) i.next();
                if (onName.toString().toLowerCase().equals("org.objectweb.petals:type=custom,name=performance_notifier_" + this.getName())) {
                    perfMBeanObjectName = onName;
                }
            }
            if (perfMBeanObjectName != null) {
                Object isMBeanActivatedO = connection.getAttribute(perfMBeanObjectName, "notificationsEnabled");
                Boolean isMBeanActivatedB = new Boolean(isMBeanActivatedO.toString());
                if (! isMBeanActivatedB.booleanValue()) {
                    connection.setAttribute(perfMBeanObjectName, new Attribute("notificationsEnabled", true));
                }
                connection.addNotificationListener(perfMBeanObjectName, perfNotifsListener, null, null);
            }
            else {
                MessageDialog.openError(null, "Abonnement aux notifications de performances",
                        "Impossible d'abonner le " + getType()  + " " + getName()
                        + " aux notifications de performances, "
                        + "car il ne poss\u00e8de pas de MBean JMX appropri\u00e9");
            }
        } catch (Exception e) {
            MessageDialog.openError(null, "Abonnement aux notifications de performances",
                    "Impossible d'abonner le " + getType()  + " " + getName()
                    + " aux notifications de performances (" + e.getMessage() + ")");
        }
    }
    
    /**
     * Desinscrit le composant aux notifications de performances
     */
    public void unsubscribePerfNotifs() {
        if (perfNotifsListener != null && connection != null) {
            try {
                connection.removeNotificationListener(perfMBeanObjectName, perfNotifsListener);
                perfNotifsListener = null;
                NotificationsView nview = NotificationsView.getInstance();
                removeNotificationListener(nview);
                processListeners();
            } catch(IOException e) { 
            } catch (Exception e) {
                MessageDialog.openError(null, "Abonnement aux notifications de performances",
                        "Impossible de d\u00e9sabonner le " + getType()  + " " + getName()
                        + " aux notifications de performances (" + e.getMessage() + ")");
            } 
        }
    }
	
	/**
	 * Desinscrit le composant aux notifications
	 */
	public void unsubscribe() {
		if (notificationListener != null && connection != null) {
			try {
				connection.removeNotificationListener(objectName, notificationListener);
				notificationListener = null;
				NotificationsView nview = NotificationsView.getInstance();
				removeNotificationListener(nview);
				System.out.println("D\u00e9sabonnement du " + getType() + " " + getName());
				processListeners();
			} catch(IOException e) { 
			} catch (Exception e) {
				MessageDialog.openError(null, "Abonnement", "Impossible de d\u00e9sabonner le " + getType()  + " " + getName() + " aux notifications (" + e.getMessage() + ")");
			} 
		}
	}
	
	/**
	 * Verifie si le composant est abonn\u00e9 aux notifications
	 * @return vrai si le composant est abonn\u00e9 aux notifications
	 */
	public boolean isSubscribed() {
		return notificationListener != null;
	}
    
    /**
     * Check if the composant is registered to get the performances notifications
     * 
     * @return true or false
     */
    public boolean isPrefNotifSuscribed() {
        return perfNotifsListener != null;
    }

	/**
	 * R\u00e9cup\u00e8re la liste des notifications de ce composant
	 * @return Liste des notifications de ce composant
	 */
	public List<Notification> getNotification() {
		return notifications;
	}
	
	/**
	 * Vide toutes les notifications recues par ce composant
	 */
	public void clearNotifications() {
		notifications.clear();
		processListeners();
	}
	
	/**
	 * Declenche une notification a tous les abonnes de
	 * l'arrivee d'une nouvelle notification Petals
	 */
	public void processListeners() {
		if (listeners != null) {
			for (NotificationChangedInterface nci: listeners) {
				nci.changedNotifications();
			}	
		}	
	}
	
	/**
	 * Soucrit l'abonne bci aux notifications
	 * @param nci Abonne a inscrire
	 */
	public void addNotificationListener(NotificationChangedInterface nci) {
		if (listeners != null && nci != null && !listeners.contains(nci)) {
			listeners.add(nci);
		}
	}
	
	/**
	 * Desinscrit l'abonne bci des notifications
	 * @param nci Abonne a desinscrire
	 */
	public void removeNotificationListener(NotificationChangedInterface nci) {
		if (nci != null && listeners.contains(nci)) {
			listeners.remove(nci);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.objectweb.petals.tools.plugin.monitor.model.ComponentParent#setServer(org.objectweb.petals.tools.plugin.monitor.model.PetalsServer)
	 */
	public void setServer(PetalsServer server) {
		this.server = server;
	}

	/**
	 * D\u00e9finit l'installateur de ce composant Petals
	 * @param installer du composant
	 */
	public void setInstaller(InstallerMBean installer) {
		this.installer = installer;
	}
	
	/**
	 * R\u00e9cup\u00e8re le bean de configuration personnalis\u00e9e 
	 * du composant
	 * @return bean de configuration personnalis\u00e9e
	 */
	public MBeanInfo getConfigurationMBean() {
		if (installer != null && connection != null) {
			try {
				ObjectName on = installer.getInstallerConfigurationMBean();
				if (on == null) {
					//FIXME: only for debug purpose
					on = objectName;
				}
				return connection.getMBeanInfo(on); 
			} catch (Exception e) {
				return null;
			}
		}
		return null;
	}

	/**
	 * V\u00e9rifie si le composant est configurable 
	 * ie le composant doit \u00eatre \u00e9teint et fournir 
	 * un bean de configuration personnalis\u00e9e
	 * @return true si le composant est configurable
	 */
	public boolean isConfigurable() {
		return isShutdown() &&  getConfigurationMBean() != null;
	}

	public IEditorPart getEditor() {
		return editor;
	}

	public void setEditor(IEditorPart editor) {
		this.editor = editor;
	}
	
	public boolean isEdited()  {
		return editor != null && editor.getEditorSite().getPage().isPartVisible(editor);
	}
	
	public void closeEditor() {
		if (editor != null) {
			editor.getEditorSite().getPage().closeEditor(editor, true);
			editor = null;	
		}
	}
	
}
