package org.objectweb.petals.tools.plugin.monitor.model;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;

import javax.jbi.management.DeploymentServiceMBean;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.views.properties.IPropertySource;
import org.objectweb.petals.tools.plugin.monitor.properties.ServiceAssemblyPropertySource;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class ServiceAssembly extends ComponentParent implements ServiceInterface {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = -8849326274182973992L;
	transient private DeploymentServiceMBean deploy;
	private List<ServiceUnit> suList;
	private String componentName;
	private String zipUrl;
	private ServiceAssemblyPropertySource properties;
	private String returnValue;
	
	private final static String END_PATTERN = "</jbi>";
	
	public void setDeploy(DeploymentServiceMBean deploy) {
		this.deploy = deploy;
		if (deploy != null) {
			try {
				String descriptor = deploy.getServiceAssemblyDescriptor(name);
				descriptor = descriptor.substring(0,descriptor.lastIndexOf(END_PATTERN)+END_PATTERN.length());
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				factory.setIgnoringElementContentWhitespace(true);
                DocumentBuilder builder = factory.newDocumentBuilder();
                Document document = builder.parse(new ByteArrayInputStream(descriptor.getBytes("UTF-8")));
                Element root = document.getDocumentElement();
                NodeList nl = root.getElementsByTagName("service-assembly");
                Element saNode = ((Element)nl.item(0));
                
                nl = saNode.getElementsByTagName("identification");
                setDescription(((Element)((Element)nl.item(0)).getElementsByTagName("description").item(0)).getTextContent());
                
                nl = saNode.getElementsByTagName("service-unit");
                String suName = null, suDescription;
                Element currentNode;
                Element currentTargetNode;
                for (int i=0;i<nl.getLength();i++) {
                	currentNode = (Element)((Element)nl.item(i)).getElementsByTagName("identification").item(0);
                	suName = currentNode.getElementsByTagName("name").item(0).getTextContent();
                	suDescription = currentNode.getElementsByTagName("description").item(0).getTextContent();;
                	System.out.println(suName + " / " + suDescription);
                	ServiceUnit su = new ServiceUnit(suName, suDescription);
                	// Soit le nom du composant est connu par le noeud SA
                	if (componentName != null) {
                		su.setComponentName(componentName);
                	}
                	// Soit il est connu dans le jbi.xml
                	else {
                		currentTargetNode = (Element)((Element)nl.item(i)).getElementsByTagName("target").item(0);
                		String suComponentName = currentTargetNode.getElementsByTagName("component-name").item(0).getTextContent();
                		su.setComponentName(suComponentName);
                	}
                	su.setServiceAssembly(this);
                	su.setDeploy(deploy);
                	su.setServer(server);
                	suList.add(su);
                }
			} catch (Exception e) {
				MessageDialog.openError(null, "Descripteur assembly", "Impossible d'ouvrir le descripteur du service assembly '" + getName() + "' (" + e.getMessage() + ")");
				e.printStackTrace();
			}
		}
	}

	public ServiceAssembly(String name) {
		super(name);
		suList = new ArrayList<ServiceUnit>();
	}

	@Override
	public TreeObject[] getChildren() {
		return suList.toArray(new ServiceUnit[suList.size()]); 
	}

	@Override
	public boolean hasChildren() {
		return !suList.isEmpty();
	}
	
	public boolean isStarted() {
		try {
			return DeploymentServiceMBean.STARTED.equalsIgnoreCase(deploy.getState(name));
		} catch (Exception e) {
			return false;
		}
	}
	
	public boolean isShutdown() {
		try {
			return DeploymentServiceMBean.SHUTDOWN.equalsIgnoreCase(deploy.getState(name));
		} catch (Exception e) {
			return false;
		}
	}
	
	public boolean isStopped() {
		try {
			return DeploymentServiceMBean.STOPPED.equalsIgnoreCase(deploy.getState(name));
		} catch (Exception e) {
			return false;
		}
	}

	 public void stop() {
		 try {
			deploy.stop(name);
		} catch (Exception e) {
			MessageDialog.openError(null, "Arr\u00eat service assembly", "Impossible d'arr\u00eater le " + getType() + " '" + getName() + "' (" + e.getMessage() + ")");
			e.printStackTrace();
		}
	 }

	 public void start() {
		 try {
				deploy.start(name);
			} catch (Exception e) {
				MessageDialog.openError(null, "D\u00e9marrage service assembly", "Impossible de d\u00e9marrer le " + getType() + " '" + getName() + "' (" + e.getMessage() + ")");
				e.printStackTrace();
			}
	 }
	 
	 public void shutdown() {
		 try {
				deploy.shutDown(name);
			} catch (Exception e) {
				MessageDialog.openError(null, "Arr\u00eat service assembly", "Impossible d'\u00e9teindre le " + getType() + " '" + getName() + "' (" + e.getMessage() + ")");
				e.printStackTrace();
			}
	 }
	 
	@Override
	public String getType() {
		return "service assembly";
	}
	
	public boolean isDeployed() {
		try {
			if (StringUtils.isNotEmpty(componentName)) {
				for (String dsa: deploy.getDeployedServiceAssembliesForComponent(componentName)) {
					if (dsa.equals(name)) {
						return true;
					}
				}
			}
			else {
				for (String dsa: deploy.getDeployedServiceAssemblies()) {
					if (dsa.equals(name)) {
						return true;
					}
				}
			}
		} catch (Exception e) {
			return false;
		} 
		return false;
	}

	public String getComponentName() {
		return componentName;
	}

	public void setComponentName(String componentName) {
		this.componentName = componentName;
	}

	public void deploy(String url) {
		if (isDeployable()) {
			try {
				deploy.deploy(url);
			} catch (Exception e) {
				MessageDialog.openError(null, "D\u00e9ploiement service assembly", "Impossible de d\u00e9ployer le " + getType() + " '" + getName() + "' (" + e.getMessage() + ")");
				e.printStackTrace();
			}
		}
	}

	public void undeploy() throws Exception {
		try {
			deploy.undeploy(name);
		} catch (Exception e) {
			MessageDialog.openError(null, "Repliement service assembly", "Impossible de replier le " + getType() + " '" + getName() + "' (" + e.getMessage() + ")");
			e.printStackTrace();
			throw e;
		}
	}

	public boolean isDeployable() {
		boolean result = false;
		
		if (StringUtils.isNotEmpty(componentName)) {
			result = (deploy.canDeployToComponent(componentName) && zipUrl != null);
		}
		else {
			result = (zipUrl != null);
		}
		
		return result;
	}

	public String getZipUrl() {
		return zipUrl;
	}

	public void setZipUrl(String zipUrl) {
		this.zipUrl = zipUrl;
	}

	public List<ServiceUnit> getSuList() {
		return suList;
	}
	
	@Override
	public Object getAdapter(Class key) {
		if (key == IPropertySource.class) {
			if (properties == null) {
				properties = new ServiceAssemblyPropertySource(this);
			}
			return properties;
		}
		return super.getAdapter(key);
	}

	public String getReturnValue() {
		return returnValue;
	}

	public void setReturnValue(String returnValue) {
		this.returnValue = returnValue;
	}
	
	public void redeploy() {
		try {
			undeploy();
			deploy(url);
		} catch (Exception e) {
			MessageDialog.openError(null, "Red\u00e9ploiement", "Impossible de red\u00e9ployer le SA " + name);
		}
		
	}
	
}
