package org.objectweb.petals.tools.plugin.monitor.model;

import javax.jbi.management.DeploymentServiceMBean;

import org.eclipse.ui.views.properties.IPropertySource;
import org.objectweb.petals.tools.plugin.monitor.properties.ServiceUnitPropertySource;



public class ServiceUnit extends ComponentParent implements ServiceInterface  {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5667570127042481657L;
	private ServiceAssembly serviceAssembly;
	transient private DeploymentServiceMBean deploy;
	private String componentName;
	private ServiceUnitPropertySource properties;
	
	public void setComponentName(String componentName) {
		this.componentName = componentName;
	}

	public void setServiceAssembly(ServiceAssembly serviceAssembly) {
		this.serviceAssembly = serviceAssembly;
	}

	public ServiceUnit(String name, String description) {
		super(name, description);
	}

	public void setDeploy(DeploymentServiceMBean deploy) {
		this.deploy = deploy;
	}

	public boolean isDeployed() {
		try {
			for (String dsu: deploy.getDeployedServiceUnitList(componentName)) {
				if (dsu.equalsIgnoreCase(name)) {
					return true;
				}
			}
		} catch (Exception e) {
			return false;
		}
		return false;
	}

	public boolean isShutdown() {
		if (serviceAssembly != null) {
			return serviceAssembly.isShutdown();
		}
		return true;
	}

	public boolean isStopped() {
		if (serviceAssembly != null) {
			return serviceAssembly.isStopped();
		}
		return true;
	}

	public void shutdown() {
		if (serviceAssembly != null) {
			serviceAssembly.shutdown();
		}
	}

	public void start() {
		if (serviceAssembly != null) {
			serviceAssembly.start();
		}		
	}

	public void stop() {
		if (serviceAssembly != null) {
			serviceAssembly.stop();
		}
	}

	public boolean isStarted() {
		if (serviceAssembly != null) {
			return serviceAssembly.isStarted();
		}
		return false;
	}

	public void deploy(String url) {
		if (serviceAssembly != null) {
			serviceAssembly.deploy(url);
		}
	}

	public void undeploy() throws Exception {
		if (serviceAssembly != null) {
			serviceAssembly.undeploy();
		}
	}

	public boolean isDeployable() {
		if (serviceAssembly != null) {
			serviceAssembly.setComponentName(this.componentName);
			return serviceAssembly.isDeployable();
		}
		return false;
	}

	@Override
	public String toString() {
		return (name != null ? name : "unknown " + getType()) + (description != null ? " (" + description + ")" : "");
	}

	public String getComponentName() {
		return componentName;
	}
	
	@Override
	public Object getAdapter(Class key) {
		if (key == IPropertySource.class) {
			if (properties == null) {
				properties = new ServiceUnitPropertySource(this);
			}
			return properties;
		}
		return super.getAdapter(key);
	}
	
}
