/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Message.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.nmr;

import java.util.HashMap;
import java.util.Map;

/**
 * represents a message : HashMap containings time, component, role, type,
 */
public class Message extends HashMap<String, Object> {

    private static final String COMPONENT = "component";
    private static final String CONTENT = "content";
    private static final String ENDPOINT_NAME = "endpointName";
    private static final String ROLE = "role";
    private static final long serialVersionUID = 1702374180586290279L;
    private static final String SERVICE_NAME = "serviceName";
    private static final String TIME = "time";
    private static final String TYPE = "type";
    
    private static final String EMPTY = "";

    /**
     * Creates a new Message with a HashMap containing base informations
     * 
     * @param hashMessage Map representation of the message
     */
    public Message(Map<String, Object> hashMessage) {
        this.put(TIME, hashMessage.get(TIME));
        this.put(COMPONENT, hashMessage.get(COMPONENT));
        this.put(ROLE, hashMessage.get(ROLE));
        this.put(TYPE, hashMessage.get(TYPE));
        this.put(CONTENT, hashMessage.get(CONTENT));
        this.put(SERVICE_NAME, hashMessage.get(SERVICE_NAME));
        this.put(ENDPOINT_NAME, hashMessage.get(ENDPOINT_NAME));
    }

    /**
     * Returns the component name of the message
     * 
     * @return the component name as a String
     */
    public String getComponent() {
        return this.get(COMPONENT).toString();
    }

    /**
     * Returns the content of the message
     * 
     * @return the content as a String, null if message has no content
     */
    public String getContent() {
        if (this.get(CONTENT) != null) {
            return this.get(CONTENT).toString();
        } else {
            return EMPTY;
        }
    }

    /**
     * Returns the name of the endpoint of the message
     * 
     * @return a String representation of the name of the endpoint
     */
    public String getEndpointName() {
        return this.get(ENDPOINT_NAME).toString();
    }

    /**
     * Returns the role of the message
     * 
     * @return the role as a string
     */
    public String getRole() {
        return this.get(ROLE).toString();
    }

    /**
     * Returns the name of the endpoint of the message
     * 
     * @return a String representation of the endpoint's name
     */
    public String getServiceName() {
        return this.get(SERVICE_NAME).toString();
    }

    /**
     * Returns the timestamp of the message
     * 
     * @return a timestamp
     */
    public long getTime() {
        return new Long(this.get(TIME).toString()).longValue();
    }

    /**
     * Returns the type of the message
     * 
     * @return the type as a String, null if message has no type
     */
    public String getType() {
        if (this.get(TYPE) != null) {
            return this.get(TYPE).toString();
        } else {
            return EMPTY;
        }
    }

    /**
     * Sets the component's name
     * 
     * @param component
     *            the string that is to be the component's name
     */
    public void setComponent(String component) {
        this.put(COMPONENT, component);
    }

    /**
     * Sets the message's content
     * 
     * @param content
     *            the content, as a String
     */
    public void setContent(String content) {
        this.put(CONTENT, content);
    }

    /**
     * Sets the new name of the endpoint of the message
     * 
     * @param serviceName
     *            a String representation of the new endpoint
     */
    public void setEndpointName(String endpointName) {
        this.put(ENDPOINT_NAME, endpointName);
    }

    /**
     * Sets the message's role
     * 
     * @param role
     *            the string that is to be the message's role
     */
    public void setRole(String role) {
        this.put(ROLE, role);
    }

    /**
     * Sets the new name of the service of the message
     * 
     * @param serviceName
     *            a String representation of the new name
     */
    public void setServiceName(String serviceName) {
        this.put(SERVICE_NAME, serviceName);
    }

    /**
     * Sets the message's time
     * 
     * @param time a timestamp
     */
    public void setTime(long time) {
        this.put(TIME, time);
    }

    /**
     * Sets the message's type
     * 
     * @param type the string that is to be the message's type
     */
    public void setType(String type) {
        this.put(TYPE, type);
    }

}
