/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.nmr;

import java.util.ArrayList;
import java.util.List;

/**
 * This class instance represent the collected messages for a PetalsServer instance.
 * The messages are stored in a circular list, wich have a maximum size (selectable).
 * When size list reaches the maximum selected size, the last element is deleted.
 * 
 *  This list is used to print collected NMR messages in the specific plugin view.
 * 
 * @author LAM
 */
public class MessagesCollector {
    
    public static final int DEFAULT_SIZE_MAX = 1000;
    
    private List<Message> messagesList;
    private int sizeMax;

    /**
     * Default Constructor
     */
    public MessagesCollector() {
        messagesList = new ArrayList<Message>();
        sizeMax = DEFAULT_SIZE_MAX;
    }

    /**
     * @return the messagesList
     */
    public List<Message> getMessagesList() {
        return messagesList;
    }

    /**
     * @return the sizeMax
     */
    public int getSizeMax() {
        return sizeMax;
    }

    /**
     * @param sizeMax the sizeMax to set
     */
    public void setSizeMax(int sizeMax) {
        this.sizeMax = sizeMax;
    }
    
    /**
     * We add the new message at the end of the list.
     * If the list size exceed sizeMax we remove the first element.
     * 
     * @param message The new Message to be added
     */
    public void addMessage(Message message) {
        messagesList.add(message);
        
        if (messagesList.size() > sizeMax) {
            messagesList.remove(0);
        }
    }
    
    /**
     * Clear the messagesList content
     */
    public void clearMessagesList() {
        messagesList = new ArrayList<Message>();
    }

}
