/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MessageListener.java $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.nmr;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.management.Notification;
import javax.management.NotificationListener;

import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;

public class MessagesListener implements NotificationListener {

    PetalsServer collector;

    // object for dates formating
    private static final DateFormat DATE_FORMATER = new SimpleDateFormat(
        "dd MMM yyyy, HH:mm:ss");

    public MessagesListener(PetalsServer collector) {
        this.collector = collector;
    }

    /**
     * @see NotificationListener#handleNotification(Notification, Object)
     */
    @SuppressWarnings("unchecked")
    public void handleNotification(Notification notification, Object handback) {
        Object userData = notification.getUserData();
        if (userData instanceof ArrayList) {
            List<Map<String, Object>> messages = (ArrayList<Map<String, Object>>) userData;
            collector.collectMessages(rawToMessageList(messages));
        } else {
            log("ERROR", "Notification #" + notification.getSequenceNumber()
                + " : userData aren't messages - datas instance of "
                + userData.getClass().getName());
        }
    }

    /**
     * Transforms a list of Map<String, Object> representations of messages
     * into a list of Message object representations of messages
     * 
     * @param rawList the list of Map messages
     * @return a list of Message
     */
    private List<Message> rawToMessageList(List<Map<String, Object>> rawList) {
        List<Message> messagesList = new ArrayList<Message>();
        for (Map<String, Object> message : rawList) {
            messagesList.add(new Message(message));
        }
        return messagesList;
    }

    private void log(String level, String message) {
        System.out.println("[WEBADMIN " + level + "] "
            + DATE_FORMATER.format(new Date().getTime()) + " : " + message);
    }

}
