/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.nmr;

import java.lang.reflect.UndeclaredThrowableException;

import javax.management.InstanceNotFoundException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.DetailsPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.MasterDetailsBlock;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.binding.servicesexplorer.ServicesExplorerMBean;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;


/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class ScrolledEndpointBlock extends MasterDetailsBlock  {
	
	private PetalsServer server;
	private TableViewer viewer;
	private Table table;
	private NMRPage page;
	private ServicesExplorerMBean nmrBean;
	private Text serviceValue;
	private Text descriptionValue;

	public ScrolledEndpointBlock(NMRPage page) {
		server = page.server;
		this.page = page;
		PetalsServerInput input = (PetalsServerInput) page.getEditor().getEditorInput();
		nmrBean = input.getServer().getNMRBean();
	}

	class MasterContentProvider implements IStructuredContentProvider {
        
		public Object[] getElements(Object inputElement) {
		    if (inputElement instanceof PetalsServerInput) {
		        return nmrBean.getEndpointsNames();
		    }
            return new Object[0];
		}

		public void dispose() { }

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) { }
        
	}

	class MasterLabelProvider extends LabelProvider implements ITableLabelProvider {
		
		final Image DEFAULT_ENDPOINT = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/endpoint.gif").createImage();
		
		public String getColumnText(Object obj, int index) {
			return obj.toString();
		}

		public Image getColumnImage(Object obj, int index) {
			return DEFAULT_ENDPOINT;
		}
        
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.MasterDetailsBlock#createMasterPart(org.eclipse.ui.forms.IManagedForm, org.eclipse.swt.widgets.Composite)
	 */
	protected void createMasterPart(final IManagedForm managedForm, Composite parent) {
		FormToolkit toolkit = managedForm.getToolkit();
		Section section = toolkit.createSection(parent, Section.DESCRIPTION | Section.TITLE_BAR);
		section.setText("Endpoints");
		section.setDescription("Endpoints disponibles depuis le serveur " + server.getName() + ":");
		section.marginWidth = 10;
		section.marginHeight = 5;
		Composite client = toolkit.createComposite(section, SWT.WRAP);
		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		layout.marginWidth = 2;
		layout.marginHeight = 2;
		client.setLayout(layout);
		
        table = toolkit.createTable(client, SWT.NULL);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalAlignment = GridData.FILL;
        gd.grabExcessHorizontalSpace = true;
		gd.heightHint = 50;
		gd.widthHint = 200;
		gd.horizontalSpan = 1;
		table.setLayoutData(gd);
		toolkit.paintBordersFor(client);
		section.setClient(client);
		final SectionPart spart = new SectionPart(section);
		managedForm.addPart(spart);
		viewer = new TableViewer(table);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				managedForm.fireSelectionChanged(spart, event.getSelection());
				chooseEndpoint((String)((IStructuredSelection)event.getSelection()).getFirstElement());
			}
		});
		viewer.setContentProvider(new MasterContentProvider());
		viewer.setLabelProvider(new MasterLabelProvider());
        try {
            viewer.setInput(page.getEditor().getEditorInput());
        } catch (UndeclaredThrowableException ute) {
            if (ute.getCause() instanceof InstanceNotFoundException) {
                ute.printStackTrace();
            }
        }
		
		toolkit.createLabel(client, "Service:");
		serviceValue = toolkit.createText(client, "", SWT.SINGLE);
		serviceValue.setEditable(false);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		serviceValue.setLayoutData(gd);
		

		toolkit.createLabel(client, "Description:");
		descriptionValue = toolkit.createText(client, "", SWT.MULTI | SWT.WRAP);
		descriptionValue.setEditable(false);
		gd = new GridData(GridData.FILL_BOTH);
		descriptionValue.setLayoutData(gd);
		
	}

	protected void chooseEndpoint(String endpointName) {
		serviceValue.setText(nmrBean.getEndpointInterfacesFromName(endpointName));
		descriptionValue.setText(nmrBean.getEndpointDescriptionFromName(endpointName));
	}

	protected void registerPages(DetailsPart detailsPart) {
        detailsPart.registerPage(String.class, new TypeMBeanNMRDetailsPage());
	}
	
	public void refresh() {

	}
	
	public void update() {
		
	}
	
	public boolean isDirty() {
		if (detailsPart == null) {
			return false;
		}
		return detailsPart.isDirty();
	}
	
	public void doSave(IProgressMonitor monitor) {
		if (detailsPart != null) {
			detailsPart.commit(true);
		}
	}

	@Override
	protected void createToolBarActions(IManagedForm managedForm) {
		
	}
	
}