/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.nmr;

import java.io.IOException;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IDetailsPage;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.objectweb.petals.binding.servicesexplorer.ServicesExplorerMBean;

/**
 * @author JR (RAVOIRE Julien - CAPGEMINI)
 * 
 * 12 mars 07
 */
public class TypeMBeanNMRDetailsPage implements IDetailsPage, NotificationListener {

	private IManagedForm mform;
	private String endpointName;
	private Section section;
	//private Composite parent;
	private Text operationValue;
	private Text inValue;
	private Text outValue;
	private Text syncValue;
	private Button isSyncValue;
	private Button invokeButton;
	private Button inOnlyValue;
	private Button inOutValue;
	private Button inOptionalOutValue;
	private Button robustInOnlyValue;
	private ServicesExplorerMBean nmrBean;
	private MBeanServerConnection connection; 
	private ObjectName seObjectName;
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#initialize(org.eclipse.ui.forms.IManagedForm)
	 */
	public void initialize(IManagedForm mform) {
		this.mform = mform;
		nmrBean = ((NMRPage)mform.getContainer()).server.getNMRBean();
		connection = ((NMRPage)mform.getContainer()).server.getConnection();
		try {
			seObjectName = new ObjectName("ServicesExplorerAgent:name=servicesexplorer");
			if (connection != null) {
				connection.addNotificationListener(seObjectName, this, null, null);
			}
		} catch (InstanceNotFoundException infe) {
            infe.printStackTrace();
            MessageDialog.openError(((NMRPage)mform.getContainer()).getSite().getShell(), 
                    "Client NMR", 
                    "Le MBean \"servicesexplorer\" n'est pas pr\u00e9sent ou n'est pas d\u00e9marr\u00e9 sur le serveur.\n"
                    + "Le client NMR ne pourra pas fonctionner correctement.");
		} catch (MalformedObjectNameException mfone) {
            mfone.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	public void createContents(Composite parent) {
		//this.parent = parent;
		TableWrapLayout layout = new TableWrapLayout();
		layout.topMargin = 5;
		layout.leftMargin = 5;
		layout.rightMargin = 2;
		layout.bottomMargin = 2;
		parent.setLayout(layout);

		FormToolkit toolkit = mform.getToolkit();
		section = toolkit.createSection(parent, Section.DESCRIPTION | Section.TITLE_BAR);
		section.marginWidth = 10;
		section.setText("Invocation");
		section.setDescription("S\u00e9lectionner l'op\u00e9ration, le type de l'entr\u00e9e, la valeur d entr\u00e9e et l'asynchronisme de l'appel: "); 
		TableWrapData td = new TableWrapData(TableWrapData.FILL, TableWrapData.TOP);
		td.grabHorizontal = true;
		section.setLayoutData(td);
		Composite client = toolkit.createComposite(section);
		GridLayout glayout = new GridLayout();
		glayout.marginWidth = glayout.marginHeight = 0;
		glayout.numColumns = 3;
		client.setLayout(glayout);		
		createSpacer(toolkit, client, 3);	//spacer

        toolkit.createLabel(client, "Op\u00e9ration:").setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        gd.horizontalSpan = 3;
        gd.heightHint = 16;
		operationValue = toolkit.createText(client, "", SWT.SINGLE);
		operationValue.setLayoutData(gd);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 3;
		toolkit.createLabel(client, "Type:").setLayoutData(gd);
		
		GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
		gd1.horizontalIndent = 15;
		inOnlyValue = toolkit.createButton(client, ServicesExplorerMBean.INONLY, SWT.RADIO);
		inOnlyValue.setSelection(true);
		inOnlyValue.setLayoutData(gd1);
		GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
		gd2.horizontalSpan = 2;
		inOutValue = toolkit.createButton(client, ServicesExplorerMBean.INOUT, SWT.RADIO);
		inOutValue.setLayoutData(gd2);
		inOptionalOutValue = toolkit.createButton(client, ServicesExplorerMBean.INOPTIONALOUT, SWT.RADIO);
		inOptionalOutValue.setLayoutData(gd1);
		robustInOnlyValue = toolkit.createButton(client, ServicesExplorerMBean.ROBUSTINONLY, SWT.RADIO);
		robustInOnlyValue.setLayoutData(gd2);
		
		
		gd = new GridData(GridData.FILL_BOTH);
		
		gd.horizontalSpan = 3;
		gd.grabExcessVerticalSpace = true;
		gd.heightHint = 80;
		
		toolkit.createLabel(client, "In:");
		inValue = toolkit.createText(client, "", SWT.MULTI | SWT.WRAP);
		inValue.setLayoutData(gd);
		
		toolkit.createLabel(client, "Out:");
		outValue = toolkit.createText(client, "", SWT.MULTI | SWT.WRAP);
		outValue.setEditable(false);
		outValue.setLayoutData(gd);
		
		createSpacer(toolkit, client, 3);
		
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 1;
        gd.heightHint = 16;
		invokeButton = toolkit.createButton(client, "Invoke", SWT.PUSH);
		invokeButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {	
			}

			public void widgetSelected(SelectionEvent e) {
				Display.getDefault().syncExec(new Runnable() {
		            public void run() {
		            	invoke();
		            }
		         });
			}			
		});
		invokeButton.setLayoutData(gd);
		isSyncValue = toolkit.createButton(client, "Sync:", SWT.CHECK);
		isSyncValue.setLayoutData(gd);
		isSyncValue.setSelection(true);
		isSyncValue.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				syncValue.setEnabled(isSyncValue.getSelection());
				if (isSyncValue.getSelection()) {
					syncValue.setFocus();
				} 
			}
			
		});
		syncValue = toolkit.createText(client, "0", SWT.MULTI | SWT.WRAP);
		syncValue.setLayoutData(gd);
		
		toolkit.paintBordersFor(section);
		section.setClient(client);
	}

	private void createSpacer(FormToolkit toolkit, Composite parent, int span) {
		Label spacer = toolkit.createLabel(parent, "");
		GridData gd = new GridData();
		gd.horizontalSpan = span;
		spacer.setLayoutData(gd);
	}

	private void update() {
		section.setDescription("Invocation sur le endpoint " + endpointName + ":");
		//parent.pack();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#inputChanged(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void selectionChanged(IFormPart part, ISelection selection) {
		IStructuredSelection ssel = (IStructuredSelection) selection;
		if (ssel.size() == 1) {
			endpointName = (String) ssel.getFirstElement();
			update();
		} else {
			endpointName = null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#commit()
	 */
	public void commit(boolean onSave) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#setFocus()
	 */
	public void setFocus() {
		operationValue.setFocus();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#dispose()
	 */
	public void dispose() {
		if (connection != null) {
			try {
				connection.removeNotificationListener(seObjectName, this);
			} catch (Exception e) {
				e.printStackTrace();
			} 
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.forms.IDetailsPage#isDirty()
	 */
	public boolean isDirty() {
		return false;
	}

	public boolean isStale() {
		return false;
	}

	public void refresh() {
		update();
	}

	public boolean setFormInput(Object input) {
		return false;
	}
	
	String getType() {
		if (inOnlyValue.getSelection()) {
			return ServicesExplorerMBean.INONLY;
		} else if (inOutValue.getSelection()) {
			return ServicesExplorerMBean.INOUT;
		} else if (inOptionalOutValue.getSelection()) {
			return ServicesExplorerMBean.INOPTIONALOUT;
		} else if (robustInOnlyValue.getSelection()) {
			return ServicesExplorerMBean.ROBUSTINONLY;
		}
		return null;
	}
	
	void invoke() {
		if (operationValue.getText().equals("")) {
			MessageDialog.openError(null, "Op\u00e9ration", "Vous devez sp\u00e9cifier l'op\u00e9ration \u00e0 invoquer");
			return;
		}
		
		nmrBean.send(nmrBean.getEndpointInterfacesFromName(endpointName), 
				operationValue.getText(), 
				endpointName, 
				inValue.getText(), 
				getType(), 
				isSyncValue.getSelection() ? Long.parseLong(syncValue.getText()) : -1);
	}

	public void handleNotification(Notification notification, Object handback) {
		final String response = notification.getUserData().toString();
		Display.getDefault().asyncExec(new Runnable() {
            public void run() {
            	outValue.setText(response);
            }
         });
	}
	
}
