package org.objectweb.petals.tools.plugin.monitor.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceAssembly;


public class ServiceAssemblyPropertySource implements IPropertySource {

	protected ServiceAssembly serviceAssembly;
	
	private static final String PROPERTY_INSTANCE = "serviceAssembly.instance";
	private static final String PROPERTY_NAME = "serviceAssembly.name";
	private static final String PROPERTY_DESCRIPTION = "serviceAssembly.description";
	private static final String PROPERTY_TYPE = "serviceAssembly.type";
	private static final String PROPERTY_CONTAINERNAME = "serviceAssembly.containerName";
	private static final String PROPERTY_ZIPURL = "serviceAssembly.zipurl";
	
	private static final String CATEGORY_GENERAL = "G\u00e9n\u00e9ral";
	private static final String CATEGORY_DIVERS = "Divers";
	
	private IPropertyDescriptor[] descriptors;
	
	public ServiceAssemblyPropertySource(ServiceAssembly serviceAssembly) {
		super();
		this.serviceAssembly = serviceAssembly;
	}

	public Object getEditableValue() {
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (descriptors == null) {
			PropertyDescriptor nameDescriptor = new PropertyDescriptor(PROPERTY_NAME, "Nom");
			nameDescriptor.setCategory(CATEGORY_GENERAL);
			PropertyDescriptor containerDescriptor = new PropertyDescriptor(PROPERTY_CONTAINERNAME, "Nom du conteneur");
			containerDescriptor.setCategory(CATEGORY_GENERAL);
			PropertyDescriptor descriptionDescriptor = new PropertyDescriptor(PROPERTY_DESCRIPTION, "Description");
			descriptionDescriptor.setCategory(CATEGORY_DIVERS);
			PropertyDescriptor typeDescriptor = new PropertyDescriptor(PROPERTY_TYPE, "Type");
			typeDescriptor.setCategory(CATEGORY_DIVERS);
			PropertyDescriptor zipurlDescriptor = new PropertyDescriptor(PROPERTY_ZIPURL, "Url du fichier zip");
			zipurlDescriptor.setCategory(CATEGORY_DIVERS);
			descriptors = new IPropertyDescriptor[] {nameDescriptor, descriptionDescriptor, containerDescriptor, typeDescriptor, zipurlDescriptor};
		}
		return descriptors;
	}

	public Object getPropertyValue(Object id) {
		if (id != null) {
			if (id.equals(PROPERTY_NAME)) {
				return serviceAssembly.getName();
			} else if (id.equals(PROPERTY_DESCRIPTION)) {
				return serviceAssembly.getDescription();
			} else if (id.equals(PROPERTY_INSTANCE)) {
				return serviceAssembly;
			} else if (id.equals(PROPERTY_TYPE)) {
				return serviceAssembly.getType();
			} else if (id.equals(PROPERTY_CONTAINERNAME)) {
				return serviceAssembly.getComponentName();
			} else if (id.equals(PROPERTY_ZIPURL)) {
				return serviceAssembly.getZipUrl();
			}
		}
		return null;
	}

	public boolean isPropertySet(Object id) {
		if (id == null) {
			return false;
		}
		return (id.equals(PROPERTY_NAME) ||
			id.equals(PROPERTY_DESCRIPTION) || 
			id.equals(PROPERTY_TYPE) ||
			id.equals(PROPERTY_CONTAINERNAME) ||
			id.equals(PROPERTY_ZIPURL));
	}

	public void resetPropertyValue(Object id) {
		
	}

	public void setPropertyValue(Object id, Object value) {
		if (value != null) {
			if (id.equals(PROPERTY_NAME)) {
				serviceAssembly.setName(value.toString());
			} else if (id.equals(PROPERTY_DESCRIPTION)) {
				serviceAssembly.setDescription(value.toString());
			} else if (id.equals(PROPERTY_TYPE)) {
				serviceAssembly.setType(value.toString());
			} else if (id.equals(PROPERTY_CONTAINERNAME)) {
				serviceAssembly.setComponentName(value.toString());
			} else if (id.equals(PROPERTY_ZIPURL)) {
				serviceAssembly.setZipUrl(value.toString());
			}
		}
	}

}
