package org.objectweb.petals.tools.plugin.monitor.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Text;
import org.objectweb.petals.tools.plugin.monitor.model.SharedLibrary;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


public class SharedLibraryComposite extends Composite {
	
	private Link refreshLink = null;
	private Label componentTypeImage = null;
	private Label componentTypeLabel = null;
	private Link unloadLink = null;
	private Text descriptionTextArea = null;
	
	protected SharedLibrary sharedLibrary;  //  @jve:decl-index=0:
	private Label componentNameText = null;
	
	private static SharedLibraryComposite instance;
	
	public static SharedLibraryComposite getInstance() {
		return instance;
	}
	
	public void setSharedLibrary(SharedLibrary sharedLibrary) {
		this.sharedLibrary = sharedLibrary;
	}
	
	public SharedLibraryComposite(Composite parent, int style) {
		super(parent, style);
		initialize();
		instance = this;
	}

	private void initialize() {
		GridData gridData12 = new GridData();
		gridData12.horizontalAlignment = GridData.FILL;
		gridData12.verticalAlignment = GridData.CENTER;
		gridData12.grabExcessHorizontalSpace = true;
		GridData gridData42 = new GridData();
		gridData42.horizontalAlignment = GridData.CENTER;
		gridData42.verticalAlignment = GridData.CENTER;
		GridData gridData38 = new GridData();
		gridData38.verticalAlignment = GridData.CENTER;
		gridData38.horizontalAlignment = GridData.END;
		GridData gridData6 = new GridData();
		gridData6.verticalAlignment = GridData.CENTER;
		gridData6.horizontalAlignment = GridData.BEGINNING;
		GridData gridData2 = new GridData();
		gridData2.verticalAlignment = GridData.BEGINNING;
		gridData2.grabExcessHorizontalSpace = false;
		gridData2.horizontalAlignment = GridData.FILL;
		GridData gridData1 = new GridData();
		gridData1.verticalAlignment = GridData.CENTER;
		gridData1.verticalSpan = 2;
		gridData1.horizontalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.makeColumnsEqualWidth = false;
		GridData gridData3 = new GridData();
		gridData3.horizontalAlignment = GridData.FILL;
		gridData3.grabExcessVerticalSpace = true;
		gridData3.horizontalSpan = 2;
		gridData3.verticalAlignment = GridData.FILL;
		componentNameText = new Label(this, SWT.NONE);
		componentNameText.setFont(new Font(Display.getDefault(), "Arial Rounded MT Bold", 14, SWT.ITALIC));
		componentNameText.setText("Nom du composant inconnu");
		componentNameText.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		componentNameText.setLayoutData(gridData12);
		componentTypeImage = new Label(this, SWT.NONE);
		componentTypeImage.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		componentTypeImage.setLayoutData(gridData1);
		componentTypeImage.setImage(new Image(Display.getCurrent(), getClass().getResourceAsStream("/newcomponent_wiz.png")));
		componentTypeLabel = new Label(this, SWT.NONE);
		componentTypeLabel.setText("Type du composant inconnu");
		componentTypeLabel.setFont(new Font(Display.getDefault(), "Segoe UI", 8, SWT.ITALIC));
		componentTypeLabel.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		componentTypeLabel.setLayoutData(gridData2);
		descriptionTextArea = new Text(this, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER);
		descriptionTextArea.setToolTipText("Description du composant");
		descriptionTextArea.setLayoutData(gridData3);
		descriptionTextArea.setText("Description indisponible");
		descriptionTextArea.setFont(new Font(Display.getDefault(), "Segoe UI", 8, SWT.ITALIC));
		unloadLink = new Link(this, SWT.NONE);
		unloadLink.setText("<a>D\u00e9charger</a>");
		unloadLink.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		unloadLink.setLayoutData(gridData6);
		unloadLink.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				if (sharedLibrary != null) {
					sharedLibrary.unload();

					refresh();
					PetalsServersView.refresh();
				}
			}
		});
		refreshLink = new Link(this, SWT.NONE);
		refreshLink.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		refreshLink.setLayoutData(gridData38);
		refreshLink.setText("<a>Rafrachir</a>");
		refreshLink.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				refresh();
				PetalsServersView.refresh();
			}
		});
		this.setLayout(gridLayout);
		this.setLayout(gridLayout);
		this.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		this.setToolTipText("Type du composant");
		this.setSize(new Point(650, 204));
	}
	
	public void fillFields() {
		if (sharedLibrary != null) {
			StringBuffer suDescription = new StringBuffer();
			
			if (StringUtils.isNotEmpty(sharedLibrary.getName())) {
				componentNameText.setText(sharedLibrary.getName());
			}
			if (StringUtils.isNotEmpty(sharedLibrary.getType())) {
				componentTypeLabel.setText(sharedLibrary.getType());
			}
			if (StringUtils.isNotEmpty(sharedLibrary.getDescription())) {
				suDescription.append(sharedLibrary.getDescription());
				suDescription.append("\n");
			}
			
			if (StringUtils.isNotEmpty(suDescription.toString())) {
				descriptionTextArea.setText(suDescription.toString());
			}
			
			layout();
		}
	}
	
	public void refresh() {
		new Thread(new Runnable() {
			public void run() {
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						fillFields();
					}
				});
			}
		}).start();
	}
	
	public void apply() {
		if (sharedLibrary != null) {
			sharedLibrary.setName(componentNameText.getText());
			refresh();
		}
	}
	
}  //  @jve:decl-index=0:visual-constraint="61,43"
