package org.objectweb.petals.tools.plugin.monitor.properties;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.objectweb.petals.tools.plugin.monitor.model.SharedLibrary;


public class SharedLibraryPropertySource implements IPropertySource {

	protected SharedLibrary sharedLibrary;
	
	private static final String PROPERTY_INSTANCE = "serviceUnit.instance";
	private static final String PROPERTY_NAME = "serviceUnit.name";
	private static final String PROPERTY_DESCRIPTION = "serviceUnit.description";
	private static final String PROPERTY_TYPE = "serviceUnit.type";
	
	private static final String CATEGORY_GENERAL = "G\u00e9n\u00e9ral";
	private static final String CATEGORY_DIVERS = "Divers";
	
	private IPropertyDescriptor[] descriptors;
	
	public SharedLibraryPropertySource(SharedLibrary sharedLibrary) {
		super();
		this.sharedLibrary = sharedLibrary;
	}

	public Object getEditableValue() {
		return null;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (descriptors == null) {
			PropertyDescriptor nameDescriptor = new PropertyDescriptor(PROPERTY_NAME, "Nom");
			nameDescriptor.setCategory(CATEGORY_GENERAL);
			PropertyDescriptor descriptionDescriptor = new PropertyDescriptor(PROPERTY_DESCRIPTION, "Description");
			descriptionDescriptor.setCategory(CATEGORY_DIVERS);
			PropertyDescriptor typeDescriptor = new PropertyDescriptor(PROPERTY_TYPE, "Type");
			typeDescriptor.setCategory(CATEGORY_DIVERS);
			descriptors = new IPropertyDescriptor[] {nameDescriptor, descriptionDescriptor, typeDescriptor};
		}
		return descriptors;
	}

	public Object getPropertyValue(Object id) {
		if (id != null) {
			if (id.equals(PROPERTY_NAME)) {
				return sharedLibrary.getName();
			} else if (id.equals(PROPERTY_DESCRIPTION)) {
				return sharedLibrary.getDescription();
			} else if (id.equals(PROPERTY_INSTANCE)) {
				return sharedLibrary;
			} else if (id.equals(PROPERTY_TYPE)) {
				return sharedLibrary.getType();
			}
		}
		return null;
	}

	public boolean isPropertySet(Object id) {
		if (id == null) {
			return false;
		}
		return (id.equals(PROPERTY_NAME) ||
			id.equals(PROPERTY_DESCRIPTION) || 
			id.equals(PROPERTY_TYPE));
	}

	public void resetPropertyValue(Object id) {
		
	}

	public void setPropertyValue(Object id, Object value) {
		if (value != null) {
			if (id.equals(PROPERTY_NAME)) {
				sharedLibrary.setName(value.toString());
			} else if (id.equals(PROPERTY_DESCRIPTION)) {
				sharedLibrary.setDescription(value.toString());
			} else if (id.equals(PROPERTY_TYPE)) {
				sharedLibrary.setType(value.toString());
			}
		}
	}

}
