package org.objectweb.petals.tools.plugin.monitor.providers;

import org.eclipse.jface.viewers.ILabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.BindingComponent;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceAssembly;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceEngine;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceInterface;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceUnit;
import org.objectweb.petals.tools.plugin.monitor.model.SharedLibrary;
import org.objectweb.petals.tools.plugin.monitor.utils.OverlayImageIcon;

public class ViewLabelProvider extends LabelProvider implements ILabelDecorator {
	
	public static Image CONNECTED_PETALS_SERVER_IMAGE;
	public static Image DISCONNECTED_PETALS_SERVER_IMAGE;
	public static Image BC_IMAGE;
	public static Image SE_IMAGE;
	public static Image SE_IMAGE_OFF;
	public static Image SA_IMAGE;
	public static Image SU_IMAGE;
	public static Image SL_IMAGE;
	public static Image FOLDER_IMAGE;
	
	public static Image RUNNING_IMAGE;
	public static Image SHUTDOWN_IMAGE;
	public static Image STOPPED_IMAGE;
	public static Image UNKNOWN_IMAGE;
	public static Image ADDED_TAG_IMAGE;
	public static Image REMOVED_TAG_IMAGE;
	
	public ViewLabelProvider() {
		CONNECTED_PETALS_SERVER_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/connectedserver.gif").createImage();
		DISCONNECTED_PETALS_SERVER_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/disconnectedserver.gif").createImage();
		BC_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/bc.gif").createImage();
		SE_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/se.gif").createImage();
		SE_IMAGE_OFF = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/se_off.gif").createImage();
		SA_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/sa.gif").createImage();
		SU_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/su.gif").createImage();
		SL_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/sl.gif").createImage();
		RUNNING_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/running.gif").createImage();
		SHUTDOWN_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/stopped.gif").createImage();
		STOPPED_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/paused.gif").createImage();
		UNKNOWN_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/unknown.gif").createImage();
		FOLDER_IMAGE = PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_FOLDER);
		ADDED_TAG_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/addedtag.gif").createImage();
		REMOVED_TAG_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/ovr/removedtag.gif").createImage();
	}
	
	public String getText(Object obj) {
		return obj.toString();
	}
	
	public Image getImage(Object object) {
		if (object instanceof PetalsServer) {
			return ((PetalsServer)object).isConnected() ? CONNECTED_PETALS_SERVER_IMAGE : DISCONNECTED_PETALS_SERVER_IMAGE;
		} else if (object instanceof BindingComponent) {
			return BC_IMAGE;
		} else if (object instanceof ServiceEngine) {
			return SE_IMAGE;
		} else if (object instanceof ServiceAssembly) {
			return SA_IMAGE;
		} else if (object instanceof SharedLibrary) {
			return SL_IMAGE;
		} else if (object instanceof ServiceUnit) {
			return SU_IMAGE;
		}
		return FOLDER_IMAGE;
	}

	public Image decorateImage(Image image, Object element) {
		if (element instanceof PetalsServer) {
			PetalsServer ps = (PetalsServer)element;
			return  OverlayImageIcon.getOverlay(image, ps.isConnected() ? RUNNING_IMAGE : SHUTDOWN_IMAGE);	
		}
		
		Image tagImage = null;
		if (element instanceof ComponentParent) {
			ComponentParent cp = (ComponentParent)element;
			if (cp.isAddedTagged()) {
				tagImage = ADDED_TAG_IMAGE;
			} else if (cp.isRemovedTagged()) {
				tagImage = REMOVED_TAG_IMAGE;
			} 
		} else {
			return null;
		}

		Image overlayImage = null;
		if (element instanceof PetalsComponent) {
			PetalsComponent pc = (PetalsComponent)element;
			if (pc.isStarted()) {
				overlayImage = RUNNING_IMAGE;
			} else if (pc.isStopped()) {
				overlayImage = STOPPED_IMAGE;
			} else if (pc.isShutdown()) {
				overlayImage = SHUTDOWN_IMAGE;
			} else {
				overlayImage = UNKNOWN_IMAGE;
			}
				
		}
		
		if (element instanceof ServiceAssembly || element instanceof ServiceUnit) {
			ServiceInterface si = (ServiceInterface)element;
			if (si.isStarted()) {
				overlayImage = RUNNING_IMAGE;
			} else if (si.isStopped()) {
				overlayImage = STOPPED_IMAGE;
			} else if (si.isShutdown()) {
				overlayImage = SHUTDOWN_IMAGE;
			} else {
				overlayImage = UNKNOWN_IMAGE;
			}
				

		}	
		
		
		return ((ComponentParent)element).isNotTagged() ? OverlayImageIcon.getOverlay(image, overlayImage) : OverlayImageIcon.getOverlay(image, overlayImage, tagImage);
	}

	public String decorateText(String text, Object element) {
		return text;
	}

}
