package org.objectweb.petals.tools.plugin.monitor.utils;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.model.XmlExportFileConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import fr.acoss.dataacquisition.securite.DaeBlowFishCrypting;

/**
 * Classe d'utilitaires pour la manipulation de Documents DOM.
 * 
 * @author LA
 *
 */
public class DomUtils {

	/**
	 * Parcourt le Document et renvoie la liste des PetalsServer trouv\u00e9s
	 * 
	 * @param annuaire Le Document contenant la liste des serveurs Petals
	 */
	public static List<PetalsServer> populateServersListFromDom(Document annuaire) {
		// La liste des serveurs \u00e0 remplir
		List<PetalsServer> serverList = new ArrayList<PetalsServer>();
		
		// M\u00e9canisme de d\u00e9cryptage du password
		DaeBlowFishCrypting dbfc = DaeBlowFishCrypting.getInstance();
		
		Element racine = annuaire.getDocumentElement();
		NodeList serverNodes = racine.getElementsByTagName(XmlExportFileConstants.TAG_SERVER);

		// Parcourt des \u00e9l\u00e9ments PetalsServer
		for (int i=0; i<serverNodes.getLength(); i++) {
			String txtJmxUrl = null;
			String txtDescription = null;
			String txtLogin = null;
			String txtPassword = null;

			NodeList childrenList = serverNodes.item(i).getChildNodes();
			// Parcourt des sous \u00e9l\u00e9ments
			for (int j=0; j<childrenList.getLength(); j++) {
				if (childrenList.item(j).getNodeName().equals(XmlExportFileConstants.TAG_SERVER_JMXURL)) {
					txtJmxUrl = childrenList.item(j).getTextContent();
				}
				if (childrenList.item(j).getNodeName().equals(XmlExportFileConstants.TAG_SERVER_DESCRIPTION)) {
					txtDescription = childrenList.item(j).getTextContent();
				}
				if (childrenList.item(j).getNodeName().equals(XmlExportFileConstants.TAG_SERVER_JMXLOGIN)) {
					txtLogin = childrenList.item(j).getTextContent();
				}
				if (childrenList.item(j).getNodeName().equals(XmlExportFileConstants.TAG_SERVER_JMXPASSWORD)) {
					txtPassword = dbfc.decrypt(childrenList.item(j).getTextContent());
				}
			}

			String texteLigne = txtJmxUrl;
			PetalsServer nouveauServeur = new PetalsServer(txtJmxUrl);

			if (StringUtils.isNotEmpty(txtDescription)) {
				texteLigne = texteLigne + " (" + txtDescription + ")";
				nouveauServeur.setDescription(txtDescription);
			}
			if (StringUtils.isNotEmpty(txtLogin)) {
				nouveauServeur.setJmxLogin(txtLogin);
			}
			if (StringUtils.isNotEmpty(txtPassword)) {
				nouveauServeur.setJmxPassword(txtPassword);
			}

			serverList.add(nouveauServeur);
		}
		
		return serverList;
	}
}
