package org.objectweb.petals.tools.plugin.monitor.utils;

import org.eclipse.jface.resource.CompositeImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;

/**
 * @author JR (RAVOIRE Julien - Intervenant CAPGEMINI au CNIR/SUD)
 * 
 */
public class OverlayImageIcon extends CompositeImageDescriptor {

	private Image baseImage;

	private Point sizeOfImage;

	private Image overlayImage;
	
	private Image tagImage;

	private int location;

	public static final int TOP_LEFT = 1;

	public static final int TOP_RIGHT = 2;

	public static final int BOTTOM_LEFT = 3;

	public static final int BOTTOM_RIGHT = 4;

	public OverlayImageIcon(Image baseImage, Image overlayImage, int location) {
		this.baseImage = baseImage;
		this.overlayImage = overlayImage;
		this.sizeOfImage = new Point(baseImage.getBounds().width, baseImage
				.getBounds().height);
		this.location = location;
	}

	public OverlayImageIcon(Image baseImage, Image overlayImage, Image tagImage) {
		this.baseImage = baseImage;
		this.overlayImage = overlayImage;
		this.sizeOfImage = new Point(baseImage.getBounds().width, baseImage
				.getBounds().height);
		this.tagImage = tagImage;
	}
	
	
	protected void drawCompositeImage(int arg0, int arg1) {
		drawImage(baseImage.getImageData(), 0, 0);
		if (overlayImage == null) {
			overlayImage = baseImage;
		}
		ImageData imageData = overlayImage.getImageData();
		switch (location) {
			case TOP_LEFT:
				drawImage(imageData, 0, 0);
				break;
			case TOP_RIGHT:
				drawImage(imageData, sizeOfImage.x - imageData.width, 0);
				break;
			case BOTTOM_LEFT:
				drawImage(imageData, 0, sizeOfImage.y - imageData.height);
				break;
			case BOTTOM_RIGHT:
				drawImage(imageData, sizeOfImage.x - imageData.width, sizeOfImage.y
						- imageData.height);
				break;
			default:
				drawImage(imageData, sizeOfImage.x - imageData.width, sizeOfImage.y - imageData.height);
				imageData = tagImage.getImageData();
				drawImage(imageData, 0, 0);
				break;
		}

	}

	protected Point getSize() {
		return sizeOfImage;
	}

	public Image getImage() {
		return createImage();
	}

	public final static Image getOverlay(Image baseImage, Image overlayImage) {
		return new OverlayImageIcon(baseImage, overlayImage, BOTTOM_RIGHT).createImage();
	}
	
	public final static Image getOverlay(Image baseImage, Image overlayImage, Image tagImage) {
		return new OverlayImageIcon(baseImage, overlayImage, tagImage).createImage();
	}
	
}
