package org.objectweb.petals.tools.plugin.monitor.views;

import java.sql.Timestamp;
import java.util.List;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearAllMonitoredServersAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearMonitoredServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.DropDownMonitoredServersAction;
import org.objectweb.petals.tools.plugin.monitor.actions.MonitoringExportCsvAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SetMessagesThresholdAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SetTimeSendLimitAction;
import org.objectweb.petals.tools.plugin.monitor.actions.WatchNMRMessageAction;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.nmr.Message;


public class MonitoringView extends ViewPart implements IAdaptable {

	public final static String ID = "org.objectweb.petals.tools.plugin.monitor.monitoringView";
	
    private static MonitoringView instance;
    
	private Composite top = null;
	private Table monitoringTable = null;
	private PetalsServer petalsServer;
    
    private DropDownMonitoredServersAction dropDownMonitoredServersAction;
    private ClearMonitoredServerAction clearMonitoredServerAction;
    private ClearAllMonitoredServersAction clearAllMonitoredServersAction;
    private SetMessagesThresholdAction setMessagesThresholdAction;
    private SetTimeSendLimitAction setTimeSendLimitAction;
    private WatchNMRMessageAction watchNMRMessageAction;
    private MonitoringExportCsvAction monitoringExportCsvAction;

	private TableViewer tableViewer = null;
	
	public static MonitoringView getInstance() {
		return instance;
	}
	
	public MonitoringView() {
		instance = this;
	}
	
	@Override
	public void createPartControl(Composite parent) {
        createActions();
        IToolBarManager tbm = getViewSite().getActionBars().getToolBarManager();
        configureToolBar(tbm);
        GridLayout gridLayout = new GridLayout();
        gridLayout.horizontalSpacing = 0;
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        top = new Composite(parent, SWT.NONE);
        top.setLayout(gridLayout);
        
        tableViewer = createTable(top);
		monitoringTable = tableViewer.getTable();
        
        tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                // find the selected row in table
                Object theEvent = event.getSource();
                int selectedRow = -1;
                if (theEvent instanceof TableViewer) {
                    TableViewer tv = (TableViewer) theEvent;
                    selectedRow = tv.getTable().getSelectionIndex();
                }

                if (selectedRow > -1) {
                    watchNMRMessageAction.setNmrMessage(
                            petalsServer.getMessagesCollector().getMessagesList().get(selectedRow));
                }
            }
        });
	}
	
	@Override
	public void setFocus() {
		monitoringTable.setFocus();
	}

	/**
	 * Define the PetalsServer to monitor
	 * @param petalsServer
	 */
	public void setPetalsServer(PetalsServer petalsServer) {
		this.petalsServer = petalsServer;
		
		if ((petalsServer != null) && (petalsServer.isMonitored())) {
            refreshViewer();
		} else {
            tableViewer.setInput(null);
        }
	}
			
	class MonitoringContentProvider implements IStructuredContentProvider {

		private Object[] messages;
		
		public Object[] getElements(Object inputElement) {
			return messages;
		}

		public void dispose() {
			messages = null;
		}

		@SuppressWarnings("unchecked")
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			List<Message> _messages = (List<Message>)newInput;
			if (_messages == null) {
				messages = new Object[0];
				return;
			}
			messages = _messages.toArray(new Message[_messages.size()]);
		}
		
	}
	
	class MonitoringLabelProvider extends LabelProvider implements ITableLabelProvider {

		final Image IN_IMAGE = 
            AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/in.gif").createImage();
		final Image OUT_IMAGE = 
            AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/out.gif").createImage();
        
		public Image getColumnImage(Object element, int columnIndex) {
            Message message = (Message)element;
			switch(columnIndex) {
				case 3: 
					if (message.getType().equalsIgnoreCase("in")) {
						return IN_IMAGE;
					}
                    if (message.getType().equalsIgnoreCase("out")) {
                        return OUT_IMAGE;
                    }
                default: return null;
			}
		}

		public String getColumnText(Object element, int columnIndex) {
            Message message = (Message)element;
			switch(columnIndex) {
				case 0: 
                    Timestamp tsTime = new Timestamp(message.getTime());
                    return tsTime.toString();
				case 1: return message.getComponent();
                case 2: return message.getRole();
                case 4: return message.getEndpointName();
                case 5: return message.getServiceName();
                case 6: return message.getContent();
                default: return null;
			}
		}

		public void addListener(ILabelProviderListener listener) { }

		public void dispose() { }

		public boolean isLabelProperty(Object element, String property) {
			return false;
		}

		public void removeListener(ILabelProviderListener listener) { }
		
	}
	
	public TableViewer createTable(Composite client) {
		TableViewer tableViewer = new TableViewer(client, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
        tableViewer.setContentProvider(new MonitoringContentProvider());
		tableViewer.setLabelProvider(new MonitoringLabelProvider());
		
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.grabExcessVerticalSpace = true;
        gridData.verticalAlignment = GridData.FILL;
		
        Table messagesTable = tableViewer.getTable();
        messagesTable.setHeaderVisible(true);
        messagesTable.setLayoutData(gridData);
        messagesTable.setLinesVisible(true);
        messagesTable.setEnabled(true);
        
        TableColumn tsColumn = new TableColumn(messagesTable, SWT.LEFT, 0);
        tsColumn.setText("Timestamp");
        tsColumn.setWidth(150);

        TableColumn componentColumn = new TableColumn(messagesTable, SWT.LEFT, 1);
        componentColumn.setText("Composant");
        componentColumn.setWidth(200);
        
        TableColumn roleColumn = new TableColumn(messagesTable, SWT.LEFT, 2);
        roleColumn.setText("Role");
        roleColumn.setWidth(90);
        
        TableColumn typeColumn = new TableColumn(messagesTable, SWT.CENTER, 3);
        typeColumn.setText("Type");
        typeColumn.setWidth(36);
        
        TableColumn epnColumn = new TableColumn(messagesTable, SWT.LEFT, 4);
        epnColumn.setText("Endpoint name");
        epnColumn.setWidth(200);
        
        TableColumn snColumn = new TableColumn(messagesTable, SWT.LEFT, 5);
        snColumn.setText("Service name");
        snColumn.setWidth(200);
        
        TableColumn messageColumn = new TableColumn(messagesTable, SWT.LEFT, 6);
        messageColumn.setText("Message");
        messageColumn.setWidth(400);
        
        return tableViewer;
	}

    /**
     * @return the petalsServer
     */
    public PetalsServer getPetalsServer() {
        return petalsServer;
    }
    
    protected void createActions() {
        dropDownMonitoredServersAction = new DropDownMonitoredServersAction();
        clearMonitoredServerAction = new ClearMonitoredServerAction();
        clearAllMonitoredServersAction = new ClearAllMonitoredServersAction();
        setMessagesThresholdAction = new SetMessagesThresholdAction(this.getViewSite().getShell());
        setTimeSendLimitAction = new SetTimeSendLimitAction(this.getViewSite().getShell());
        watchNMRMessageAction = new WatchNMRMessageAction();
        monitoringExportCsvAction = new MonitoringExportCsvAction(this.getViewSite());
    }
    
    protected void configureToolBar(IToolBarManager mgr) {
        mgr.add(watchNMRMessageAction);
        mgr.add(monitoringExportCsvAction);
        mgr.add(new Separator());
        mgr.add(clearMonitoredServerAction);
        mgr.add(clearAllMonitoredServersAction);
        mgr.add(new Separator());
        mgr.add(setMessagesThresholdAction);
        mgr.add(setTimeSendLimitAction);
        mgr.add(new Separator());
        mgr.add(dropDownMonitoredServersAction);
    }
    
    /**
     * Rafraichit le contenu de l'arbre des serveurs
     */
    public void refreshViewer() {
        new Thread(new Runnable() {
            public void run() {
                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        tableViewer.setInput(petalsServer.getMessagesCollector().getMessagesList());
                        tableViewer.refresh();
                    }
                });
            }
        }).start();
    }
    
}

