package org.objectweb.petals.tools.plugin.monitor.views;

import java.sql.Timestamp;
import java.util.List;

import javax.management.Notification;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearNotificationsAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SubscribeNotificationsAction;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;


public class NotificationsView extends ViewPart implements NotificationChangedInterface, IAdaptable {

	public final static String ID = "org.objectweb.petals.tools.plugin.monitor.notificationsView";
	
	private Composite top = null;
	private Table notificationsTable = null;
	private PetalsComponent petalsComponent;  //  @jve:decl-index=0:

	private static NotificationsView instance;
	
	private ClearNotificationsAction clearNotificationsAction;
	private SubscribeNotificationsAction subscribeNotificationsAction;

	private TableViewer tableViewer = null;
	
	public static NotificationsView getInstance() {
		return instance;
	}
	
	public NotificationsView() {
		instance = this;
	}
	
	@Override
	public void createPartControl(Composite parent) {
        GridLayout gridLayout = new GridLayout();
        gridLayout.horizontalSpacing = 0;
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        top = new Composite(parent, SWT.NONE);
        top.setLayout(gridLayout);
        
        tableViewer = createTable(top);
		notificationsTable = tableViewer.getTable();
        
        makeActions();
		fillLocalToolBar(getViewSite().getActionBars().getToolBarManager());
	}

	/**
	 * Cr\u00e9e les actions de nettoyage et de souscription aux notifications
	 */
	private void makeActions() {
		clearNotificationsAction = new ClearNotificationsAction(petalsComponent);
		subscribeNotificationsAction = new SubscribeNotificationsAction(petalsComponent);
	}
	
	/**
	 * Remplit la barre d'outils locale \u00e0 la vue
	 * @param toolbar
	 */
	private void fillLocalToolBar(IToolBarManager toolbar) {
		toolbar.add(subscribeNotificationsAction);
		toolbar.add(new Separator());
		toolbar.add(clearNotificationsAction);
	}
	
	@Override
	public void setFocus() {
		notificationsTable.setFocus();
	}

	/**
	 * D\u00e9finit le composant dont il faut observer les notifications
	 * @param petalsComponent
	 */
	public void setPetalsComponent(PetalsComponent petalsComponent) {
		this.petalsComponent = petalsComponent;
		subscribeNotificationsAction.setEnabled(petalsComponent != null);
		clearNotificationsAction.setEnabled(petalsComponent != null);
		
		if (petalsComponent == null) {
			tableViewer.setInput(null);
		} else {
			subscribeNotificationsAction.setChecked(petalsComponent.isSubscribed());
			subscribeNotificationsAction.setPetalsComponent(petalsComponent);
			clearNotificationsAction.setPetalsComponent(petalsComponent);
		}
	}

	/* (non-Javadoc)
	 * @see org.objectweb.petals.tools.plugin.monitor.views.NotificationChangedInterface#changedNotifications()
	 */
	public void changedNotifications() {
		if (petalsComponent != null) {
            if (petalsComponent.getNotification().size() > 0) {
                tableViewer.setInput(petalsComponent.getNotification());
            }
		}
	}
			
	class NotificationsContentProvider implements IStructuredContentProvider {

		private Object[] notifications;
		
		public Object[] getElements(Object inputElement) {
			return notifications;
		}

		public void dispose() {
			notifications = null;
		}

		@SuppressWarnings("unchecked")
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			List<Notification> _notifications = (List<Notification>)newInput;
			if (_notifications == null) {
				notifications = new Object[0];
				return;
			}
			notifications = _notifications.toArray(new Notification[_notifications.size()]);
		}
		
	}
	
	class NotificationsLabelProvider extends LabelProvider implements ITableLabelProvider {

		final Image CHANGED_STATE_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/refresh.gif").createImage();
		final Image UNKNOWN_IMAGE = AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/properties.gif").createImage();
		
		public Image getColumnImage(Object element, int columnIndex) {
			Notification notification = (Notification)element;
			switch(columnIndex) {
				case 0: 
					if (notification.getType().equalsIgnoreCase("jmx.attribute.change")) {
						return CHANGED_STATE_IMAGE;
					}
					return UNKNOWN_IMAGE;
				default: return null; 
			}
		}

		public String getColumnText(Object element, int columnIndex) {
			Notification notification = (Notification)element;
			switch(columnIndex) {
				case 0: return null;
				case 1: return notification.getMessage();
				case 2: return Long.toString(notification.getSequenceNumber());
				case 3: return new Timestamp(notification.getTimeStamp()).toString();
				default: return null;
			}
		}

		public void addListener(ILabelProviderListener listener) {
		}

		public void dispose() {
		}

		public boolean isLabelProperty(Object element, String property) {
			return false;
		}

		public void removeListener(ILabelProviderListener listener) {
		}
		
	}
	
	public TableViewer createTable(Composite client) {
		TableViewer tableViewer = new TableViewer(client, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
        tableViewer.setContentProvider(new NotificationsContentProvider());
		tableViewer.setLabelProvider(new NotificationsLabelProvider());
		
        GridData gridData = new GridData();
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.grabExcessVerticalSpace = true;
        gridData.verticalAlignment = GridData.FILL;
		
        Table notificationsTable = tableViewer.getTable();
        notificationsTable.setHeaderVisible(true);
        notificationsTable.setLayoutData(gridData);
        notificationsTable.setLinesVisible(true);
        notificationsTable.setEnabled(true);
        
        TableColumn hourColumn = new TableColumn(notificationsTable, SWT.LEFT, 0);
        hourColumn.setText("Date");
        hourColumn.setWidth(190);

        TableColumn numberColumn = new TableColumn(notificationsTable, SWT.CENTER, 0);
        numberColumn.setText("N");
        numberColumn.setWidth(40);
        
        TableColumn messageColumn = new TableColumn(notificationsTable, SWT.LEFT, 0);
        messageColumn.setText("Message");
        messageColumn.setWidth(800);
                
        TableColumn typeColumn = new TableColumn(notificationsTable, SWT.CENTER, 0);
        typeColumn.setWidth(20);
        
        return tableViewer;
	}
	
}

