/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.forms.widgets.FormToolkit;

/**
 * @author LAM
 */
public class PerformanceNotifierComposite extends Composite {

    private FormToolkit formToolkit = null;   //  @jve:decl-index=0:visual-constraint=""
    private Table perfNotifTable = null;
    private List<String[]> tableDatas;

    public PerformanceNotifierComposite(Composite parent, int style) {
        super(parent, style);
        initialize();
    }

    private void initialize() {
        initTable();
        setLayout(new GridLayout());
        tableDatas = new ArrayList<String[]>();
    }

    /**
     * This method initializes formToolkit	
     * 	
     * @return org.eclipse.ui.forms.widgets.FormToolkit	
     */
    private FormToolkit getFormToolkit() {
        if (formToolkit == null) {
            formToolkit = new FormToolkit(Display.getCurrent());
        }
        return formToolkit;
    }
    
    /**
     * Initialize
     */
    private void initTable() {
        GridData gridData = new GridData();
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = GridData.FILL;
        gridData.horizontalAlignment = GridData.FILL;
        gridData.grabExcessVerticalSpace = true;
        perfNotifTable = getFormToolkit().createTable(this, SWT.NONE);
        perfNotifTable.setHeaderVisible(true);
        perfNotifTable.setLayoutData(gridData);
        perfNotifTable.setLinesVisible(true);
        
        TableColumn columnId = new TableColumn (perfNotifTable, SWT.NONE);
        columnId.setText("Id");
        columnId.setWidth(150);
        
        TableColumn columnTS = new TableColumn (perfNotifTable, SWT.NONE);
        columnTS.setText("Timestamp");
        columnTS.setWidth(150);
        
        TableColumn columnOwner = new TableColumn (perfNotifTable, SWT.NONE);
        columnOwner.setText("Composant");
        columnOwner.setWidth(200);
        
        TableColumn columnDuration = new TableColumn (perfNotifTable, SWT.NONE);
        columnDuration.setText("Type");
        columnDuration.setWidth(150);
        
        TableColumn columnType = new TableColumn (perfNotifTable, SWT.NONE);
        columnType.setText("Dur\u00e9e (ms)");
        columnType.setWidth(65);
        
        TableColumn columnUserData = new TableColumn (perfNotifTable, SWT.NONE);
        columnUserData.setText("Donn\u00e9es utilisateur");
        columnUserData.setWidth(300);
        
        startFeedingTableThread();
    }
    
    /**
     * Adds a notification in the Table
     * 
     * @param lineArray The String Array that represents the notification
     */
    private void addTableLine(String[] lineArray) {
        TableItem newTableItem = new TableItem(perfNotifTable, SWT.NONE);
        newTableItem.setText(lineArray);
        tableDatas.add(lineArray);
    }
    
    /**
     * Launch a Thread that will update the table every 100ms if a new Notification is detected
     */
    private void startFeedingTableThread() {
        new Thread(new Runnable() {
            public void run() {
                while (true) {
                    try {
                        Thread.sleep(100);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    Display.getDefault().asyncExec(new Runnable() {
                        public void run() {
                            int maxIndex = PerformanceNotifierView.getInstance().getUserData().size() - 1;
                            int currentIndex = PerformanceNotifierView.getInstance().getUserDataPrintedIndex();
                            
                            if (currentIndex < maxIndex) {
                                // First index case
                                if (currentIndex == 0) {
                                    addTableLine(PerformanceNotifierView.getInstance().getUserData().get(currentIndex));
                                }
                                
                                while (currentIndex < maxIndex) {
                                    currentIndex++;
                                    addTableLine(PerformanceNotifierView.getInstance().getUserData().get(currentIndex));
                                    PerformanceNotifierView.getInstance().setUserDataPrintedIndex(currentIndex);
                                }
                            }
                        }
                    });
                }
            }
        }).start();
    }

    /**
     * @return the perfNotifTable
     */
    public Table getPerfNotifTable() {
        return perfNotifTable;
    }

    /**
     * @return the tableDatas
     */
    public List<String[]> getTableDatas() {
        return tableDatas;
    }
    


}
