/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.plugin.monitor.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.part.ViewPart;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearPerfNotifAction;
import org.objectweb.petals.tools.plugin.monitor.actions.PerfNotifExportCsvAction;

/**
 * @author LAM
 */
public class PerformanceNotifierView extends ViewPart {
    
    public static final String ID = "org.objectweb.petals.tools.plugin.monitor.perfNotifView";
    private static PerformanceNotifierView instance;
    private PerformanceNotifierComposite composite;
    private List<String[]> userData;
    // used to know the last rows printed at the time the thread has checked the list
    private int userDataPrintedIndex;
    private Action csvExportAction;
    private Action clearTableAction;
    
    public static PerformanceNotifierView getInstance() {
        return instance;
    }
    
    public PerformanceNotifierView() {
        instance = this;
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void createPartControl(Composite parent) {
        userData = new ArrayList<String[]>();
        GridLayout gridLayout = new GridLayout();
        gridLayout.horizontalSpacing = 0;
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        composite = new PerformanceNotifierComposite(parent, SWT.NONE);
        composite.setLayout(gridLayout);
        
        makeActions();
        hookContextMenu();
        hookToolbarMenu();
        contributeToActionBars();
    }
    
    /**
     * 
     */
    private void hookContextMenu() {
        MenuManager menuMgr = new MenuManager("#PopupMenu");
        menuMgr.setRemoveAllWhenShown(true);
        menuMgr.addMenuListener(new IMenuListener() {
            public void menuAboutToShow(IMenuManager manager) {
                PerformanceNotifierView.this.fillContextMenu(manager);
            }
        });
        Menu menu = menuMgr.createContextMenu(composite.getPerfNotifTable());
        composite.getPerfNotifTable().setMenu(menu);
    }
    
    /**
     * 
     */
    private void hookToolbarMenu() {
        IActionBars actionBars = getViewSite().getActionBars();
        IToolBarManager toolBar = actionBars.getToolBarManager();
        toolBar.add(clearTableAction);
    }
    
    /**
     * Remplit dynamiquement la popupmenu selon l'objet cliqu\u00e9
     * @param manager Popupmenu
     */
    private void fillContextMenu(IMenuManager manager) {
        manager.add(csvExportAction);
    }
    
    /**
     * Instanciates all actions for this view
     */
    private void makeActions() {
        clearTableAction = new ClearPerfNotifAction();
        csvExportAction = new PerfNotifExportCsvAction(getViewSite());
    }
    
    /**
     * 
     */
    private void contributeToActionBars() {
        IActionBars bars = getViewSite().getActionBars();
        fillLocalPullDown(bars.getMenuManager());
    }
    
    /**
     * Fill the local menubar
     * @param manager Menubar
     */
    private void fillLocalPullDown(IMenuManager manager) {
        manager.add(csvExportAction);
    }

    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
     */
    @Override
    public void setFocus() { }
    
    /**
     * Clear the table
     */
    public void clearTableList() {
        if (composite != null) {
            userData = new ArrayList<String[]>();
            userDataPrintedIndex = 0;
            composite.getPerfNotifTable().removeAll();
        }
    }

    /**
     * @return the userDataPrintedIndex
     */
    public int getUserDataPrintedIndex() {
        return userDataPrintedIndex;
    }

    /**
     * @param userDataPrintedIndex the userDataPrintedIndex to set
     */
    public void setUserDataPrintedIndex(int userDataPrintedIndex) {
        this.userDataPrintedIndex = userDataPrintedIndex;
    }

    /**
     * @param userData the userData to set
     */
    public void setUserData(List<String[]> userData) {
        this.userData = userData;
    }

    /**
     * @return the userData
     */
    public List<String[]> getUserData() {
        return userData;
    }
    
}
