package org.objectweb.petals.tools.plugin.monitor.views;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DecoratingLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.actions.AddPetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearAllComponentsAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ClearPetalsServersAction;
import org.objectweb.petals.tools.plugin.monitor.actions.CollapseTreeAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ConfigureComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ConnectPetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.DeployComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.DisconnectPetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ExpandTreeAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ExportUrlsAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ImportUrlsAction;
import org.objectweb.petals.tools.plugin.monitor.actions.InstallComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.InstallComponentByMenuAction;
import org.objectweb.petals.tools.plugin.monitor.actions.MonitorPetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.NMRofServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.NewComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.OpenJmxBrowserAction;
import org.objectweb.petals.tools.plugin.monitor.actions.PreferencesAction;
import org.objectweb.petals.tools.plugin.monitor.actions.PropertiesAction;
import org.objectweb.petals.tools.plugin.monitor.actions.QuickDiscoverPetalsServersAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ReachSUContainerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.RedeployComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.RefreshAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ReinstallComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.RemovePetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.RestartComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SearchPetalsServersAction;
import org.objectweb.petals.tools.plugin.monitor.actions.ShutdownComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.StartComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.StopComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.StopContainerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SubscribeCompPerfNotifAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SubscribeComponentNotificationsAction;
import org.objectweb.petals.tools.plugin.monitor.actions.SynchronizePetalsServerAction;
import org.objectweb.petals.tools.plugin.monitor.actions.UndeployComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.UninstallComponentAction;
import org.objectweb.petals.tools.plugin.monitor.actions.UnloadLibraryAction;
import org.objectweb.petals.tools.plugin.monitor.actions.UpdateAllComponentsAction;
import org.objectweb.petals.tools.plugin.monitor.configure.PetalsComponentInput;
import org.objectweb.petals.tools.plugin.monitor.dnd.ComponentDragSource;
import org.objectweb.petals.tools.plugin.monitor.dnd.ComponentDropTarget;
import org.objectweb.petals.tools.plugin.monitor.model.BindingComponent;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentType;
import org.objectweb.petals.tools.plugin.monitor.model.FolderComponent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsComponent;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceAssembly;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceEngine;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceInterface;
import org.objectweb.petals.tools.plugin.monitor.model.ServiceUnit;
import org.objectweb.petals.tools.plugin.monitor.model.SharedLibrary;
import org.objectweb.petals.tools.plugin.monitor.preferences.PreferenceConstants;
import org.objectweb.petals.tools.plugin.monitor.properties.PetalsServerComposite;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeObject;
import org.objectweb.petals.tools.plugin.monitor.providers.TreeParent;
import org.objectweb.petals.tools.plugin.monitor.providers.ViewContentProvider;
import org.objectweb.petals.tools.plugin.monitor.providers.ViewLabelProvider;
import org.objectweb.petals.tools.plugin.monitor.utils.DomUtils;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.utils.XmlUtils;
import org.w3c.dom.Document;


public class PetalsServersView extends ViewPart implements ITabbedPropertySheetPageContributor {
	
	public final static String ID = "org.objectweb.petals.tools.plugin.monitor.views.ESBView";
	
	private static TreeViewer viewer;
	private IStatusLineManager statusBar;
	private DrillDownAdapter drillDownAdapter;
	
	private Action addPetalsServerAction;
	private Action removePetalsServerAction;
	private Action clearPetalsServersAction;
	private Action connectPetalsServerAction;
	private Action disconnectPetalsServerAction;
	private Action propertiesAction;
	private Action preferencesAction;
	private Action collapseTreeAction;
	private Action expandTreeAction;
	private Action refreshTreeAction;
	private Action installComponentAction;
	private Action uninstallComponentAction;
	private Action reinstallComponentAction;
	private Action startComponentAction;
	private Action stopComponentAction;
	private Action restartComponentAction;
	private Action deployComponentAction;
	private Action undeployComponentAction;
	private Action redeployComponentAction;
	private Action shutdownComponentAction;
	private Action stopContainerAction;
	private Action quickDiscoverPetalsServerAction;
	private Action searchPetalsServersAction;
	private Action monitorPetalsServerAction;
	private Action openJmxBrowserAction;
	private Action unloadSharedLibrary;
	private Action newComponentAction;
	private Action configureComponentAction;
	private Action clearAllComponentsAction;
	private Action updateAllComponentsAction;
	private Action synchronizeServerAction;
	private Action nmrOfServerAction;
	
	private Action exportUrlsAction;
	private Action importUrlsAction;
	
	private Action subscribeComponentNotificationsAction;
    private Action subscribeCompPerfNotifAction;
	private Action reachSuContainerAction;
	
	/*TODO: avec les pr\u00e9f\u00e9rences > affichage > filtres
	private boolean showConnectedServers;
    private boolean showDisconnectedServers;
    private boolean showAllServers;
      */
	private static PetalsServersView instance;
	private ViewContentProvider provider;
	private ViewContentProvider history;
	private static ViewerFilter viewerFilter;
	private PetalsServer currentPetalsServer;
	private Object currentNode;
	
	private static List<PetalsServer> registeredServers = new ArrayList<PetalsServer>();
	
	public static ViewerFilter getFilter() {
		//TODO
		return viewerFilter;
	}
		
	public static void refresh() {
		if (getInstance() != null) {
			getInstance().refreshViewer();
		}
	}
	
	public static TreeViewer getViewer() {
		return viewer;
	}
		
	private IMemento memento;
	public static String jmxAddress = "service:jmx:rmi:///jndi/rmi://localhost:7700/management/rmi-jmx-connector";
	transient private Preferences prefs;
	
	/**
	 * Ajoute un serveur \u00e0 la racine de l'arbre
	 * @param ps Serveur Petals \u00e0 ajouter
	 */
	public void addPetalsServer(final PetalsServer ps) {
		if (ps !=null) {
			if (!provider.contains(ps)) {
				ps.setShell((getViewSite().getShell()));
				provider.add(ps);
				if (prefs.getBoolean(PreferenceConstants.AUTO_CONNECT_ADDED_SERVERS)) {
		            Display.getDefault().syncExec(new Runnable() {
		               public void run() {
		            	   ps.connect();
						   connect(ps);
		               };
		            });  
				}
				refreshViewer();	
			} else {
				System.err.println("Petals Server with JMX URL " + ps.getJmxUrl() + " has already been added");
			}
		}
	}
	
	/**
	 * MAJ du GUI lors de la connexion du serveur donn\u00e9
	 * @param petalsServer Serveur connect\u00e9
	 */
	public void disconnect(PetalsServer petalsServer) {
		if (petalsServer != null) {
			((TreeParent)petalsServer).clearChildren();
			refreshViewer(petalsServer);
			if (PetalsServerComposite.getInstance() != null) {
				PetalsServerComposite.getInstance().refresh();	
			}			
		}
	}
	
	/**
	 * MAJ du GUI lors de la d\u00e9connexion du serveur donn\u00e9
	 * @param petalsServer Serveur d\u00e9connect\u00e9
	 */
	public void connect(PetalsServer petalsServer) {
		if (petalsServer != null) {
			if (petalsServer.isConnected()) {
				((TreeParent)petalsServer).addChild(new TreeParent());
				refreshViewer(petalsServer);
				if (PetalsServerComposite.getInstance() != null) {
					PetalsServerComposite.getInstance().refresh();
				}		
				if (prefs.getBoolean(PreferenceConstants.START_ALL_EXPANDED_NODES)) {
					new Thread(new Runnable() {
						public void run() {
							Display.getDefault().asyncExec(new Runnable() {
								public void run() {
									viewer.expandAll();
								}
							});
						}
					}).start();	
				}
			}	
		}
	}
	
	/**
	 * V\u00e9rifie si il existe des composants installables 
	 * dans le r\u00e9pertoire des composants
	 * @return
	 */
	boolean hasInstallableComponents() {
		String componentsDirectory = prefs.getString(PreferenceConstants.COMPONENTS_DIRECTORY);
		if (componentsDirectory != null && !componentsDirectory.equals("")) {
			File dir = new File(componentsDirectory);
			if (dir.exists() && dir.canRead() && dir.listFiles(new FilenameFilter() {
				public boolean accept(File dir, String name) {
					return name.endsWith(".zip");
				}
			}).length > 0) {
				return true;
			}
		}
		return false;
	}
	
	/**
	 * V\u00e9rifie si l'URL pass\u00e9e en param\u00e8tre a d\u00e9j\u00e0 \u00e9t\u00e9 
	 * enregistr\u00e9e dans cette vue
	 * @param url URL \u00e0 tester l'existence
	 * @return vrai si l'URL a d\u00e9j\u00e0 \u00e9t\u00e9 ajout\u00e9e
	 */
	public boolean jmxUrlAlreadyAdded(String url) {
		return provider.contains(new PetalsServer(url));
	}
	
	public PetalsServersView() {
		prefs = Activator.getDefault().getPluginPreferences();
		provider = new ViewContentProvider();
		instance = this;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		drillDownAdapter = new DrillDownAdapter(viewer); 
		viewer.setContentProvider(provider);
		ViewLabelProvider viewLabelProvider = new ViewLabelProvider();
		viewer.setLabelProvider(new DecoratingLabelProvider(viewLabelProvider, viewLabelProvider));
		viewer.setSorter(new ViewerSorter());
		viewer.setInput(getViewSite());
		/* TODO
		 * viewerFilter = new ViewerFilter() {

			@Override
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				if (element instanceof BindingComponent) {
					return true;
				}
				return true;
			}
			
		};
		viewer.addFilter(viewerFilter);*/
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			public void selectionChanged(SelectionChangedEvent event) {
				for (Object o: ((IStructuredSelection)event.getSelection()).toList()) {
					System.out.println(o.toString());
					showObjectInStatusbar(o);
					if (NotificationsView.getInstance() == null) {
						try {
							PetalsServersView.this.getViewSite().getPage().showView(NotificationsView.ID);
						} catch (PartInitException e) {
							MessageDialog.openError(getSite().getShell(), "Ouverture vue notifications", "Impossible d'abonnement le composant car la vue notifications est inaccessible");
						}
					}
					NotificationsView nview = NotificationsView.getInstance();
					if (nview != null) {
						if (o instanceof PetalsComponent) {
							PetalsComponent pc = (PetalsComponent)o;
							nview.setPetalsComponent(pc);
							pc.addNotificationListener(nview);
							pc.processListeners();
						} else {
							nview.setPetalsComponent(null);
						}
					}
				}
			}
		});
		
		if (memento != null) {
			String value = memento.getString("TREE.ID");
			if (value != null) {
				viewer.setSelection(new StructuredSelection(value.split(",")));
			}
		}
		viewer.setInput(this);
		getSite().setSelectionProvider(viewer);
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		hookDragAndDrop();
		contributeToActionBars();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				PetalsServersView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		statusBar = bars.getStatusLineManager();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private URL[] getInstallableComponentsURL(PetalsServer petalsServer) {
		List<URL> urls = new ArrayList<URL>();
		String componentsDirectory = prefs.getString(PreferenceConstants.COMPONENTS_DIRECTORY);
		File dir = new File(componentsDirectory);
		URL url;
		for (File file: dir.listFiles(new FilenameFilter() {
			public boolean accept(File dir, String name) {
				return name.endsWith(".zip");
			}
		})) {
			try {
				url = file.toURL();
				if (!petalsServer.isComponentUrlAlreadyInstalled(url)) {
					urls.add(url);	
				}
			} catch (MalformedURLException e) { }
		}
		return urls.toArray(new URL[urls.size()]);
	}
	
	/**
	 * G&eacute;n&egrave;re le menu dynamique des composants installables depuis un serveurs
	 * Petals ou depuis un dossier composante
	 * @param obj Serveur petals ou dossier composante
	 * @return Menu dynamique des composants installables
	 */
	private MenuManager getInstallableComponentsSubmenu(Object obj) {
		MenuManager submenu = new MenuManager("Installer");
		submenu.add(newComponentAction);
		submenu.setRemoveAllWhenShown(true);
		currentNode = obj;
		if (obj instanceof FolderComponent) {
			currentPetalsServer = ((FolderComponent)obj).getServer();
		} else if (obj instanceof PetalsServer) {
			currentPetalsServer = (PetalsServer)obj;
		}
		submenu.addMenuListener(new IMenuListener() {
			
			boolean isRightNode(String nodeName, Object node) {
				if (nodeName != null && node != null && node instanceof FolderComponent) {
					return nodeName.equals(((FolderComponent)node).getName());
				}
				return false;
			}
			
			public void menuAboutToShow(IMenuManager m) {
				InstallComponentByMenuAction installAction;
				List<InstallComponentByMenuAction> slActions = new ArrayList<InstallComponentByMenuAction>();
				List<InstallComponentByMenuAction> seActions = new ArrayList<InstallComponentByMenuAction>();
				List<InstallComponentByMenuAction> bcActions = new ArrayList<InstallComponentByMenuAction>();
				List<InstallComponentByMenuAction> saActions = new ArrayList<InstallComponentByMenuAction>();
				for (URL url: getInstallableComponentsURL(currentPetalsServer)) {
					installAction = new InstallComponentByMenuAction(viewer);
					installAction.setUrl(url);
					if (!currentPetalsServer.isComponentAlreadyInstalled(installAction.getName()) && installAction.isActionable()) {
						installAction.setPetalsServer(currentPetalsServer);
						switch (installAction.getType()) {
							case bindingComponent: bcActions.add(installAction); break;
							case serviceEngine: seActions.add(installAction); break;
							case sharedLibrary: slActions.add(installAction); break;
							case serviceAssembly: saActions.add(installAction); break;
						}
					}
				}
				
				// ordonne les entr\u00e9es BC, SE puis SL				
				if (currentNode instanceof PetalsServer || isRightNode(PetalsServer.BC_TEXT, currentNode)) {
					for (InstallComponentByMenuAction action: bcActions) {
						m.add(action);
					}	
				}
				m.add(new Separator());
				if (currentNode instanceof PetalsServer || isRightNode(PetalsServer.SE_TEXT, currentNode)) {
					for (InstallComponentByMenuAction action: seActions) {
						m.add(action);
					}
				}
				m.add(new Separator());
				if (currentNode instanceof PetalsServer || isRightNode(PetalsServer.SA_TEXT, currentNode)) {
					for (InstallComponentByMenuAction action: saActions) {
						m.add(action);
					}
				}
				m.add(new Separator());
				if (currentNode instanceof PetalsServer || isRightNode(PetalsServer.SL_TEXT, currentNode)) {
					for (InstallComponentByMenuAction action: slActions) {
						m.add(action);
					}
				}
			}
		});
		return submenu;
	}
	
	
	
	/**
	 * Remplit dynamiquement la popupmenu selon l'objet cliqu\u00e9
	 * @param manager Popupmenu
	 */
	private void fillContextMenu(IMenuManager manager) {
		Object selectedObject  = ((IStructuredSelection)viewer.getSelection()).getFirstElement();
		Object obj = (TreeObject)selectedObject;
		
		if (obj instanceof ComponentParent) {
			if (((ComponentParent)obj).isRemovedTagged()) {
				return;
			}
		}
		
		if (obj instanceof FolderComponent || obj instanceof PetalsServer && ((PetalsServer)obj).isConnected()) {
			manager.add(newComponentAction);
			if (obj instanceof PetalsServer && ((PetalsServer)obj).hasComponents()) {
				manager.add(updateAllComponentsAction);
			}
			if (hasInstallableComponents()) {
				manager.add(getInstallableComponentsSubmenu(obj));
			}
			if (obj instanceof PetalsServer && ((PetalsServer)obj).hasComponents()) {
				manager.add(clearAllComponentsAction);
			}
			manager.add(new Separator());
		}
		
		if (obj instanceof SharedLibrary) {
			manager.add(unloadSharedLibrary);
		}
		
		if (obj instanceof PetalsServer) {
			PetalsServer ps = (PetalsServer)obj;
			if (ps.isConnected()) {
                manager.add(disconnectPetalsServerAction);
				manager.add(stopContainerAction);
				manager.add(synchronizeServerAction);
				manager.add(new Separator());
				manager.add(searchPetalsServersAction);
				manager.add(nmrOfServerAction);
				if (ps.getHttpUrl() != null) {
					manager.add(openJmxBrowserAction);
				}
				manager.add(new Separator());
				monitorPetalsServerAction.setChecked(ps.isMonitored());
				manager.add(monitorPetalsServerAction);
			} else {
				manager.add(connectPetalsServerAction);
			}
			manager.add(new Separator());
		}
		if (obj == null || obj instanceof PetalsServer) {
			manager.add(addPetalsServerAction);
		}
		
		if (obj instanceof PetalsServer) {
			manager.add(removePetalsServerAction);
			manager.add(clearPetalsServersAction);
			manager.add(new Separator());
		}
		
		if (obj instanceof BindingComponent || obj instanceof ServiceEngine) {
			PetalsComponent pc = (PetalsComponent)obj;
			if (pc.isInstalled()) {
				if (!pc.isUnknown()) {
					if (pc.isStarted()) {
						manager.add(stopComponentAction);
						manager.add(restartComponentAction);
					} else {
						manager.add(startComponentAction);
						if (!pc.isShutdown()) {
							manager.add(shutdownComponentAction);
						}
					}	
				}
				manager.add(new Separator());
				manager.add(uninstallComponentAction);
				if (pc.isUpdate()) {
					manager.add(reinstallComponentAction);	
				}
				manager.add(new Separator());
				if (pc.isConfigurable()) {
					manager.add(configureComponentAction);
					manager.add(new Separator());	
				}
			} else {
				manager.add(installComponentAction);	
			}
			manager.add(new Separator());
			subscribeComponentNotificationsAction.setChecked(((PetalsComponent)obj).isSubscribed());
			manager.add(subscribeComponentNotificationsAction);
            subscribeCompPerfNotifAction.setChecked(((PetalsComponent)obj).isPrefNotifSuscribed());
            manager.add(subscribeCompPerfNotifAction);
		} 
		
		if (obj instanceof ServiceInterface) {
			ServiceInterface si = (ServiceInterface)obj;
			if (si.isDeployed()) {
				if (si.isStarted()) {
					manager.add(stopComponentAction);
					manager.add(restartComponentAction);
				} else {
					manager.add(startComponentAction);
					if (!si.isShutdown()) {
						manager.add(shutdownComponentAction);	
					}
				}	
				manager.add(new Separator());
				manager.add(undeployComponentAction);
				if (si.isDeployable()) {
					manager.add(redeployComponentAction);
				}
				
				manager.add(new Separator());
			} else {
				if (si.isDeployable()) {
					manager.add(deployComponentAction);	
				}
			}
			if (obj instanceof ServiceUnit) {
				manager.add(reachSuContainerAction);
			}
		}
		
		
		manager.add(new Separator());
		if (obj instanceof PetalsServer) {
			manager.add(exportUrlsAction);
		}
		if (obj instanceof PetalsServer || obj == null) {
			manager.add(importUrlsAction);	
		}
				
		if (obj instanceof TreeObject) {
			manager.add(new Separator());
			manager.add(refreshTreeAction);
			manager.add(new Separator());
			manager.add(collapseTreeAction);	
			manager.add(expandTreeAction);
			manager.add(new Separator());
			manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
			manager.add(new Separator());
			drillDownAdapter.addNavigationActions(manager);
		}
		
		manager.add(new Separator());
		manager.add(preferencesAction);

		if (obj instanceof PetalsServer || obj instanceof PetalsComponent || obj instanceof ServiceEngine || obj instanceof ServiceAssembly || obj instanceof ServiceUnit || obj instanceof SharedLibrary || obj instanceof FolderComponent) {
			manager.add(new Separator());
			manager.add(propertiesAction);
		}		
	}
	
	/**
	 * Remplit d'actions la barre d'outils locale
	 * @param manager Barre d'outils
	 */
	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(addPetalsServerAction);
		manager.add(newComponentAction);
		manager.add(quickDiscoverPetalsServerAction);
		manager.add(new Separator());
		manager.add(refreshTreeAction);		
	}

	/**
	 * Remplit d'actions le menu d\u00e9roulant local
	 * @param manager Menu d\u00e9roulant
	 */
	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(clearPetalsServersAction);
		manager.add(new Separator());
		manager.add(expandTreeAction);
		manager.add(collapseTreeAction);
		manager.add(new Separator());
		drillDownAdapter.addNavigationActions(manager);
		manager.add(new Separator());
		manager.add(preferencesAction);
		manager.add(new Separator());		
		manager.add(propertiesAction);
	}
		
	/**
	 * Instancie toutes les actions utilis\u00e9es dans les menus et barre
	 * d'outils de la vue serveurs Petals
	 */
	private void makeActions() {
		addPetalsServerAction = new AddPetalsServerAction(viewer, getViewSite()); 
		removePetalsServerAction = new RemovePetalsServerAction(viewer, provider);
		clearPetalsServersAction = new ClearPetalsServersAction(viewer, provider);
		connectPetalsServerAction = new ConnectPetalsServerAction(viewer);
		disconnectPetalsServerAction = new DisconnectPetalsServerAction(viewer);
		propertiesAction = new PropertiesAction(viewer, getViewSite());
		preferencesAction = new PreferencesAction(viewer, getViewSite().getShell().getShell());
		refreshTreeAction = new RefreshAction(viewer);
		installComponentAction = new InstallComponentAction(viewer);
		uninstallComponentAction = new UninstallComponentAction(viewer);
		//updateComponentAction = new UpdateComponentAction(viewer);
		expandTreeAction = new ExpandTreeAction(viewer);
		collapseTreeAction = new CollapseTreeAction(viewer);
		startComponentAction = new StartComponentAction(viewer);
		stopComponentAction = new StopComponentAction(viewer);
		restartComponentAction = new RestartComponentAction(viewer, startComponentAction, stopComponentAction);
		deployComponentAction = new DeployComponentAction(viewer);
		undeployComponentAction = new UndeployComponentAction(viewer);
		redeployComponentAction = new RedeployComponentAction(viewer, deployComponentAction, undeployComponentAction);
		stopContainerAction = new StopContainerAction(viewer);
		quickDiscoverPetalsServerAction = new QuickDiscoverPetalsServersAction(viewer);
		searchPetalsServersAction = new SearchPetalsServersAction(viewer, getViewSite().getShell(), provider);
		openJmxBrowserAction = new OpenJmxBrowserAction(viewer, getViewSite());
		shutdownComponentAction = new ShutdownComponentAction(viewer);
		reinstallComponentAction = new ReinstallComponentAction(viewer, installComponentAction, uninstallComponentAction);
		newComponentAction = new NewComponentAction(viewer, getViewSite());
		unloadSharedLibrary = new UnloadLibraryAction(viewer);
		monitorPetalsServerAction = new MonitorPetalsServerAction(viewer);
		exportUrlsAction = new ExportUrlsAction(viewer, getViewSite(), provider);
		importUrlsAction = new ImportUrlsAction(viewer, getViewSite(), provider);
		subscribeComponentNotificationsAction = new SubscribeComponentNotificationsAction(viewer);
        subscribeCompPerfNotifAction = new SubscribeCompPerfNotifAction(viewer);
		reachSuContainerAction = new ReachSUContainerAction(viewer, provider);
		configureComponentAction = new ConfigureComponentAction(viewer, getViewSite());
		clearAllComponentsAction = new ClearAllComponentsAction(viewer);
		updateAllComponentsAction = new UpdateAllComponentsAction(viewer);
		synchronizeServerAction = new SynchronizePetalsServerAction(viewer);
		nmrOfServerAction = new NMRofServerAction(viewer, getViewSite());
	}

	/**
	 * Associe le handler au double clic sur une feuille 
	 * de l'arbre de la vue
	 */
	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				ISelection selection = viewer.getSelection();
				for (Object obj : ((IStructuredSelection) selection).toList()) {
					TreeObject too = (TreeObject) obj;
					
					if (!(too instanceof FolderComponent)) {
						
						if (too instanceof PetalsComponent) {
							try {
								PetalsComponent component = (PetalsComponent)too;
								if (component.isConfigurable()) {
									component.setEditor(getViewSite().getPage().openEditor(new PetalsComponentInput(component), "org.objectweb.petals.tools.plugin.monitor.componentConfigurationEditor", false));	
								}
							} catch (PartInitException e) {
								MessageDialog.openError(getViewSite().getShell(), "Configuration composant", "Impossible d'ouvrir la vue de configuration du composant");
							}
						} 
						
						// ouverture des propri\u00e9t\u00e9s
						if (getViewSite() != null) {
							try {
								if (!((ComponentParent)too).isRemovedTagged()) {
									getViewSite().getPage().showView(IPageLayout.ID_PROP_SHEET);	
								}
							} catch (PartInitException e) {
							}	
						}
						
					}
					
					// deploie/replie le noeud
					if (too instanceof TreeParent) {
						TreeParent tp = (TreeParent)too;
						if (tp.hasChildren()) {
							if (viewer.getExpandedState(tp)) {
								viewer.collapseToLevel(tp, TreeViewer.ALL_LEVELS);
							} else {
								viewer.expandToLevel(tp, TreeViewer.ALL_LEVELS);
							}
						}
					}
										
				}
			}
		});
	}
	
	private void hookDragAndDrop() {
		new ComponentDragSource(viewer);
		new ComponentDropTarget(viewer);
	}
	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
		
	/**
	 * Renvoie l'instance de la vue Serveurs Petals
	 * @return instance de la vue
	 */
	public static PetalsServersView getInstance() {
		return instance;
	}

	@Override
	public void init(IViewSite site, IMemento memento) throws PartInitException {		
		super.init(site, memento);
		this.memento = memento;
		restoreHistory();
		if (!registeredServers.isEmpty()) {
			for (PetalsServer registeredServer: registeredServers) {
				if (!registeredServer.equals("")) {
					provider.add(registeredServer);
				}
			}
			registeredServers.clear();
			return;
		}
		
		if (memento == null) {
			return;
		}
		
		// Chargement de la liste des urls sauvegard\u00e9e dans les pr\u00e9f\u00e9rences
		if (StringUtils.isNotEmpty(prefs.getString(PreferenceConstants.DEFAULT_SERVERS_URL))) {
			// R\u00e9cup\u00e9ration du nom de fichier ou url saisi
			String filePath = prefs.getString(PreferenceConstants.DEFAULT_SERVERS_URL);
			// Mise en forme url/fichier local
			filePath = StringUtils.miseEnFormeUrl(filePath);
			
			if (StringUtils.isNotEmpty(filePath)) {
				Document docFichierAnnuaire = XmlUtils.xmlParsing(filePath);
				if (docFichierAnnuaire!=null) {
					addServersFromDom(docFichierAnnuaire);
				}
			}
		}
		// Sinon on restaure la liste des URLS JMX pr\u00e9c\u00e9dentes
		else {
			String mementoString = memento.getString(MementoConstants.URLS_JMX);
			if (mementoString != null) {
				for (String url: mementoString.split(",")) {
					if (!url.equals("")) {
						addPetalsServer(new PetalsServer(url));
					}
				}
			}
		}
	}
	
	/**
	 * Parcourt du Document et ajout des serveurs trouv\u00e9s
	 * 
	 * @param annuaire Le Document contenant la liste des serveurs Petals
	 */
	private void addServersFromDom(Document annuaire) {
		List<PetalsServer> serverList = DomUtils.populateServersListFromDom(annuaire);

		// Parcourt des \u00e9l\u00e9ments PetalsServer
		for (PetalsServer currentServer : serverList) {
			addPetalsServer(currentServer);
		}
	}

	@Override
	public void saveState(IMemento memento) {
		// sauve la liste des URLS JMX
		StringBuffer sb = new StringBuffer();
		for (TreeObject to: provider.getChildren(null)) {
			sb.append(((PetalsServer)to).getJmxUrl());
			sb.append(',');
		}
		memento.putString(MementoConstants.URLS_JMX, sb.toString());
		saveHistory();
		super.saveState(memento);
		disconnectPetalsServers();
	}
	
	/**
	 * Sauvegarde ts les serveurs ainsi que leurs componsants 
	 * en m\u00e9moire de masse, dans le fichier history.ser du
	 * r\u00e9pertoire par d\u00e9faut de stockage du plugin
	 */
	public void saveHistory() {
		try {
			for (TreeObject to: provider.getChildren(null)) {
				if (to instanceof PetalsServer) {
					((PetalsServer)to).refreshTags();
				}
			}			
			FileOutputStream fos = new FileOutputStream(new File(Activator.getDefault().getStateLocation().toOSString() + File.separator + "history.ser"));
			ObjectOutputStream oos = new ObjectOutputStream(fos);
			oos.writeObject(provider);
			oos.close();
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	

	/**
	 * R\u00e9cup\u00e8re l'historique des serveurs et composants 
	 * de la pr\u00e9c\u00e9dente connexion
	 */
	public void restoreHistory() {
		try {
			/* check that history file exists */
			File serialization = new File(Activator.getDefault().getStateLocation().toOSString() + File.separator + "history.ser");
			if( !serialization.exists()) 
			{
				history = new ViewContentProvider();	// ensure "history" is not null
				return;
			}
			
			/* deserialize the history */
			FileInputStream fis = new FileInputStream( serialization );
			ObjectInputStream ois = new ObjectInputStream(fis);
			history = (ViewContentProvider)ois.readObject();
			ois.close();
			fis.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public void dispose() {
		super.dispose();		 
		registerPetalsServer();
	}

	void disconnectPetalsServers() {
		for (TreeObject o: provider.getChildren(null)) {
			if (o instanceof PetalsServer) {
				((PetalsServer)o).disconnect();
			}
		}
	}
	
	void registerPetalsServer() {
		for (TreeObject o: provider.getChildren(null)) {
			if (o instanceof PetalsServer) {
				registeredServers.add((PetalsServer)o);
			}
		}
	}
	
	/**
	 * Rafraichit le contenu de l'arbre des serveurs
	 */
	void refreshViewer() {
		new Thread(new Runnable() {
		      public void run() {
	            Display.getDefault().asyncExec(new Runnable() {
	               public void run() {
	            	   viewer.refresh();
	               }
	            });
		      }
		   }).start();
	}
	
 	/**
	 * Rafraichit une entit\u00e9 en particulier dans l'arbre des serveurs
	 * 
	 * @param objectToRefresh Entit\u00e9 objet \u00e0 rafraichir
	 */
	void refreshViewer(final Object objectToRefresh) {
		new Thread(new Runnable() {
			public void run() {
				Display.getDefault().asyncExec(new Runnable() {
					public void run() {
						Object[] expandedElements = viewer
								.getExpandedElements();
						viewer.refresh(objectToRefresh);
						viewer.setExpandedElements(expandedElements);
					}
				});
			}
		}).start();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(Class adapter) {
			if (adapter == IPropertySheetPage.class) {
				//return new PropertySheetPage();
				return new TabbedPropertySheetPage(this);
			}
			return super.getAdapter(adapter);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor#getContributorId()
	 */
	public String getContributorId() {
		return getSite().getId();
	}

	public IStatusLineManager getStatusBar() {
		return statusBar;
	}
	
	void showObjectInStatusbar(Object o) {
		if (o instanceof ServiceEngine) {
			statusBar.setMessage(ComponentType.serviceEngine.getImage(), "Service engine: " + o.toString());	
		} else if (o instanceof BindingComponent) {
			statusBar.setMessage(ComponentType.bindingComponent.getImage(), "Binding component: " + o.toString());
		} else if (o instanceof SharedLibrary) {
			statusBar.setMessage(ComponentType.sharedLibrary.getImage(), "Biblioth\u00e8que partag\u00e9e: " + o.toString());
		} else if (o instanceof ServiceAssembly) {
			statusBar.setMessage(ViewLabelProvider.SA_IMAGE, "Service Assembly: " + o.toString());
		} else if (o instanceof ServiceUnit) {
			statusBar.setMessage(ViewLabelProvider.SU_IMAGE, "Service Unit: " + o.toString());
		} else if (o instanceof FolderComponent) {
			statusBar.setMessage(ViewLabelProvider.FOLDER_IMAGE, "Dossier: " + o.toString());
		} else if (o instanceof PetalsServer) {
			statusBar.setMessage(((PetalsServer)o).isConnected() ? ViewLabelProvider.CONNECTED_PETALS_SERVER_IMAGE : ViewLabelProvider.DISCONNECTED_PETALS_SERVER_IMAGE, "Serveur: " + o.toString());
		} else {
			statusBar.setMessage(o.toString());
		}
	}

	public ViewContentProvider getProvider() {
		return provider;
	}
	
	public PetalsServer getHistory(String serverUrl){
		if (serverUrl != null) {
			for(Object o: history.getElements(null)) {
				if (o instanceof PetalsServer) {
					if (serverUrl.equals(((PetalsServer)o).getJmxUrl())) {
						return (PetalsServer)o;
					}
				}
			}
		}
		return null;
	}

}