package org.objectweb.petals.tools.plugin.monitor.views.widgets;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.wizards.ChangedWizardInterface;


/**
 * Widget utilis\u00e9 pour la cr\u00e9ation d'une table dans laquelle 
 * chaque ligne est pr\u00e9c\u00e9d\u00e9e d'une case \u00e0 cocher.
 * 
 * @author LAM
 *
 */
public class PetalsServersCheckTable {
	
	/**
	 * La Table \u00e0 cocher contenant le liste des Beans PetalsServer
	 */
	private Table petalsServersCheckTable;
	/**
	 * Liste des PetalsServers associ\u00e9s \u00e0 cette table
	 */
	private List<PetalsServer> petalsServerBeanList;
	
	private List<ChangedWizardInterface> listeners;
	
	/**
	 * Constructeur
	 * 
	 * @param parent Le conteneur
	 */
	public PetalsServersCheckTable(Composite parent) {
		listeners = new ArrayList<ChangedWizardInterface>();
		petalsServersCheckTable = new Table(parent, SWT.BORDER | SWT.CHECK);
		petalsServersCheckTable.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
			}
			public void widgetSelected(SelectionEvent e) {
				changedWizard();
			}
		});
		petalsServerBeanList = new ArrayList<PetalsServer>();
		
	}

	/**
	 * Getter
	 * 
	 * @return La liste des Beans PetalsServer
	 */
	public List<PetalsServer> getPetalsServerBeanList() {
		return petalsServerBeanList;
	}

	/**
	 * Setter
	 * 
	 * @param list La liste des Beans PetalsServer
	 */
	public void setPetalsServerBeanList(List<PetalsServer> list) {
		this.petalsServerBeanList = list;
		
		// Parcourt des \u00e9l\u00e9ments PetalsServer
		for (PetalsServer currentServer : petalsServerBeanList) {
			TableItem line = new TableItem(petalsServersCheckTable, SWT.NONE);
			// Coch\u00e9 par d\u00e9faut
			line.setChecked(true);
			
			String lineText = currentServer.getJmxUrl();
			
			if (StringUtils.isNotEmpty(currentServer.getDescription())) {
				lineText = lineText + " (" + currentServer.getDescription() + ")";
			}
			
			line.setText(lineText);
		}
	}
	
	/**
	 * Retourne la liste des Bean PetalsServer coch\u00e9s dans le tableau
	 * 
	 * @return La List des Beans PetalsServers
	 */
	public List<PetalsServer> getCheckedServers () { 
		List<PetalsServer> checkedList = new ArrayList<PetalsServer>();

		for (int i=0; i<petalsServersCheckTable.getItemCount(); i++) {
			TableItem tableItem = petalsServersCheckTable.getItem(i);
			if (tableItem.getChecked()) {
				checkedList.add(petalsServerBeanList.get(i));
			}
		}
		
		return checkedList;
	}

	/**
	 * Getter
	 * 
	 * @return La Table
	 */
	public Table getPetalsServersCheckTable() {
		return petalsServersCheckTable;
	}
	
	/**
	 * Enl\u00e8ve tous les \u00e9l\u00e9ments de la table \u00e0 cocher, et de la liste
	 */
	public void clearPetalsServerTable() {
		petalsServersCheckTable.removeAll();
		petalsServerBeanList.clear();
	}
		
	public void changedWizard() {
		for (ChangedWizardInterface listener: listeners) {
			listener.dialogChanged();
		}
	}
	
	public void addChangedWizardListener(ChangedWizardInterface listener) {
		if (listener != null) {
			listeners.add(listener);
		}
	}
	
	public void removeChangedWizardListener(ChangedWizardInterface listener) {
		if (listener != null) {
			listeners.remove(listener);
		}
	}
	
	public void clearChangedWizardListeners() {
		listeners.clear();
	}
	
}
