package org.objectweb.petals.tools.plugin.monitor.wizards;

import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.model.XmlExportFileConstants;
import org.objectweb.petals.tools.plugin.monitor.providers.ViewContentProvider;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import fr.acoss.dataacquisition.securite.DaeBlowFishCrypting;

public class ExportWizard extends Wizard implements IExportWizard {

	ExportWizardPage mainPage;
	ViewContentProvider provider;
	
	public void setProvider(ViewContentProvider provider) {
		this.provider = provider;
	}

	@Override
	public boolean performFinish() {		
		String pathFichierExport = mainPage.getExportWizardComposite().getEditor().getStringValue();
		Document docFichierExport = buildExportFile();
		saveXmlToFile(StringUtils.checkXmlExtension(pathFichierExport), docFichierExport);
		
        return true;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		setWindowTitle("Exportation des serveurs Petals"); //NON-NLS-1
		setNeedsProgressMonitor(true);
		mainPage = new ExportWizardPage("Exporter une liste de serveurs Petals"); //NON-NLS-1
	}

    public void addPages() {
        super.addPages();
        addPage(mainPage);
    }

    /**
     * Affectation de la liste des serveurs \u00e0 la table de cases \u00e0 cocher
     * 
     * @param petalsServerList La liste des Beans PetalsServer
     */
    public void setPetalsServerList(List<PetalsServer> petalsServerList) {
    	mainPage.setPetalsServerList(petalsServerList);
    }

	/**
	 * Construction du Document repr\u00e9sentant les donn\u00e9es du fichier XML d'export.
	 * Le Document est constitu\u00e9 \u00e0 partir de la liste des beans serveur du wizard.
	 * 
	 * @return Le Document contruit
	 */
	private Document buildExportFile() {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance ();
        dbf.setIgnoringElementContentWhitespace(false);
        DocumentBuilder db = null;
        DaeBlowFishCrypting dbfc = DaeBlowFishCrypting.getInstance();
        
        try {
            db = dbf.newDocumentBuilder();
            
        } catch (ParserConfigurationException e) {
        	throw new IllegalStateException("Erreur d'initialisation de DOM"); //$NON-NLS-1$
        }
		
        Document doc = db.newDocument ();
        Element root = doc.createElement (XmlExportFileConstants.TAG_ROOT); //$NON-NLS-1$
        doc.appendChild (root);
        
        for(PetalsServer petServ : mainPage.getExportWizardComposite().getTable().getCheckedServers()) {
        	// Balise serveur
            Element serverNode = doc.createElement(XmlExportFileConstants.TAG_SERVER);
            
            // Sous-balise Url JMX
            if (StringUtils.isNotEmpty(petServ.getJmxUrl())) {
            	Element serverNodeJmxUrl = doc.createElement(XmlExportFileConstants.TAG_SERVER_JMXURL);
            	serverNodeJmxUrl.setTextContent(petServ.getJmxUrl());
            	serverNode.appendChild(serverNodeJmxUrl);
            }
            
            // Sous-balise Description
            if (StringUtils.isNotEmpty(petServ.getDescription())) {
            	Element serverNodeDescription = doc.createElement(XmlExportFileConstants.TAG_SERVER_DESCRIPTION);
            	serverNodeDescription.appendChild(doc.createTextNode(petServ.getDescription()));
            	serverNode.appendChild(serverNodeDescription);
            }
            
            // Sous-balise jmxLogin
            if (StringUtils.isNotEmpty(petServ.getJmxLogin())) {
            	Element serverNodeJmxLogin = doc.createElement(XmlExportFileConstants.TAG_SERVER_JMXLOGIN);
            	serverNodeJmxLogin.appendChild(doc.createTextNode(petServ.getJmxLogin()));
            	serverNode.appendChild(serverNodeJmxLogin);
            }
            
            // Sous-balise jmxPassord
            if (StringUtils.isNotEmpty(petServ.getJmxPassword())) {
            	Element serverNodeJmxPassword = doc.createElement(XmlExportFileConstants.TAG_SERVER_JMXPASSWORD);
            	serverNodeJmxPassword.appendChild(doc.createTextNode(dbfc.crypt(petServ.getJmxPassword())));
            	serverNode.appendChild(serverNodeJmxPassword);
            }
            
            root.appendChild(serverNode);
        }
		
		return doc;
	}
	
	/**
	 * Enregistrement du Document DOM dans un fichier XML.
	 * Le Document DOM represente le contenu du fichier XML d'export.
	 * 
	 * @param filePath Le path complet du fichier a sauvegarder
	 * @param doc Le Document
	 */
	private void saveXmlToFile(String filePath, Document doc) {
		Element racine = doc.getDocumentElement();
		DOMSource domSource = new DOMSource(racine);
		StreamResult fluxDestination = new StreamResult(filePath);
        TransformerFactory fabrique = TransformerFactory.newInstance();
        Transformer transformationIdentite = null;
        try {
            transformationIdentite = fabrique.newTransformer();
            transformationIdentite.setOutputProperty(OutputKeys.INDENT, "yes");
            transformationIdentite.setOutputProperty(OutputKeys.ENCODING, "ISO-8859-1");
        }
        catch (TransformerConfigurationException e) {
            e.printStackTrace();
        }
        try {
            transformationIdentite.transform(domSource, fluxDestination);
        }
        catch (TransformerException e) {
            e.printStackTrace();
        }
	}
    
}
