package org.objectweb.petals.tools.plugin.monitor.wizards;

import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.widgets.Composite;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.utils.XmlUtils;
import org.w3c.dom.Document;



/**
 * Page principale du Wizard d'import pour la liste des serveurs Petals.
 * 
 * @author LA
 *
 */
public class ImportWizardPage extends WizardPage {
	/**
	 * Container des \u00e9l\u00e9ments graphiques de la page
	 */
	private ImportWizardComposite importWizardComposite;
	
	/**
	 * Construteur
	 * 
	 * @param pageName Le titre de la page
	 */
	public ImportWizardPage(String pageName) {
		super(pageName);
		setTitle(pageName);
		setDescription("Importer une liste de serveurs Petals monitor\u00e9s \u00e0 partir d'une URL ou d'un fichier XML local");
		setPageComplete(false);
	}

	/** (non-Javadoc)
	 * @see org.eclipse.ui.dialogs.WizardNewFileCreationPage#createAdvancedControls(org.eclipse.swt.widgets.Composite)
	 */	
	public void createControl(Composite parent) {
		importWizardComposite = new ImportWizardComposite(parent, SWT.NONE);
		
		importWizardComposite.getEditor().getTextControl(importWizardComposite).addModifyListener(new ModifyListener(){
			public void modifyText(ModifyEvent e) {
				// Grisage du bouton finish par d\u00e9faut
				setPageComplete(false);
				// Vidage du tableau des serveurs
				importWizardComposite.getTable().clearPetalsServerTable();
				// R\u00e9cup\u00e9ration du nom de fichier ou url saisi
				String filePath = importWizardComposite.getEditor().getStringValue();
				// Mise en forme url/fichier local
				filePath = StringUtils.miseEnFormeUrl(filePath);
				
				if (StringUtils.isNotEmpty(filePath)) {
					Document docFichierAnnuaire = XmlUtils.xmlParsing(importWizardComposite.getEditor().getStringValue());
					if (docFichierAnnuaire!=null) {
						importWizardComposite.addServerLines(docFichierAnnuaire);
						setPageComplete(true);
					}
				}
			}
		});
		setControl(importWizardComposite);
	}

	/**
	 * Getter
	 * 
	 * @return Le FileFieldEditor editor
	 */
	public FileFieldEditor getEditor() {
		return importWizardComposite.getEditor();
	}

	/**
	 * Getter
	 * 
	 * @return Le compiste de la page
	 */
	public ImportWizardComposite getImportWizardComposite() {
		return importWizardComposite;
	}
}
