package org.objectweb.petals.tools.plugin.monitor.wizards;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.runtime.Preferences;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentAnalyzer;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentType;
import org.objectweb.petals.tools.plugin.monitor.preferences.PreferenceConstants;


public class NewComponentComposite extends CommonComposite {

	private Label componentTypeLabel = null;
	private Label urlLabel = null;
	private Button browseButton = null;

	private Label detectedComponentTypeImage = null;
	private Combo comboUrl = null;
	private Label detectedComponentTypeLabel = null;
	
	transient private Preferences prefs; 
	
	public NewComponentComposite(Composite parent, int style) {
		super(parent, style);
		prefs = Activator.getDefault().getPluginPreferences();
		initialize();
	}

	private void initialize() {
		GridData gridData44 = new GridData();
		gridData44.horizontalAlignment = GridData.FILL;
		gridData44.verticalAlignment = GridData.CENTER;
		GridData gridData43 = new GridData();
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.CENTER;
		gridData.heightHint = 16;
		gridData.widthHint = 16;
		gridData.verticalAlignment = GridData.CENTER;
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		gridLayout.horizontalSpacing = 7;
		gridLayout.marginWidth = 4;
		gridLayout.marginHeight = 3;
		gridLayout.verticalSpacing = 3;
		this.setLayout(gridLayout);
		setSize(new Point(505, 98));
		urlLabel = new Label(this, SWT.NONE);
		urlLabel.setText("URL :");
		createComboUrl();
		browseButton = new Button(this, SWT.NONE);
		browseButton.setText("Parcourir...");

		browseButton.setLayoutData(gridData43);
				browseButton.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
					@Override
					public void mouseDown(MouseEvent e) {
						FileDialog fd = new FileDialog(getShell(), SWT.NONE);
						fd.setText("S\u00e9lectionner le ZIP du composant \u00e0 installer, d\u00e9ployer ou charger");
						fd.setFilterExtensions(new String[] {"*.zip"});
						
						String componentsDirectory = prefs.getString(PreferenceConstants.COMPONENTS_DIRECTORY);
						if (componentsDirectory == null || componentsDirectory.equals("")) {
							componentsDirectory = "%PETALS_HOME%\\installed";
						}
						fd.setFilterPath(componentsDirectory);
						String selectedFile = fd.open();
						if (selectedFile != null) {
							try {
								URL selectedFileURL = new URL("file:///" + selectedFile.replaceAll("\\\\", "/"));
								setUrl(selectedFileURL);
							} catch (MalformedURLException e1) {
								
							} finally {
								changedWizard();
							}
						}
					} 
				});
		componentTypeLabel = new Label(this, SWT.NONE);
		componentTypeLabel.setText("Type du composant :");
		detectedComponentTypeImage = new Label(this, SWT.RIGHT);
		detectedComponentTypeLabel = new Label(this, SWT.NONE);
		detectedComponentTypeLabel.setFont(new Font(Display.getDefault(), "Segoe UI", 8, SWT.BOLD));
		detectedComponentTypeLabel.setLayoutData(gridData44);
		detectedComponentTypeImage.setLayoutData(gridData);
		detectedComponentTypeImage.setAlignment(SWT.LEFT);		
	}

	public String getUrl() {
		return comboUrl.getText();
	}

	/**
	 * This method initializes comboUrl	
	 *
	 */
	private void createComboUrl() {
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.FILL;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.horizontalSpan = 2;
		gridData1.verticalAlignment = GridData.CENTER;
		comboUrl = new Combo(this, SWT.NONE);
		comboUrl.setLayoutData(gridData1);
		comboUrl.addModifyListener(new org.eclipse.swt.events.ModifyListener() {
			public void modifyText(org.eclipse.swt.events.ModifyEvent e) {
				if (comboUrl.getText().toLowerCase().endsWith(".zip")) {
					setComponentType(getType());
					changedWizard();	
				}
			}
		});
	}
	
	public void setComponentType(ComponentType type) {
		switch(type) {
			case bindingComponent: break;
			case serviceEngine: break;
			case sharedLibrary: break;
			case unknown: break;
		}
		detectedComponentTypeLabel.setText(type.toString());
		detectedComponentTypeImage.setImage(type.getImage());
	}
	
	public void setUrl(URL url) {
		if (url != null) {
			comboUrl.setText(url.toString());	
		}
	}
	
	public ComponentType getType() {
		String url = getUrl();
		if (url != null) {
			return new ComponentAnalyzer(url).getType();	
		}
		return ComponentType.unknown;
	}
	
	public void setComponentsFilenames(String[] componentsFilenames) {
		comboUrl.setItems(componentsFilenames);
	}
		
}  //  @jve:decl-index=0:visual-constraint="34,26"
