package org.objectweb.petals.tools.plugin.monitor.wizards;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentParent;
import org.objectweb.petals.tools.plugin.monitor.model.ComponentType;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


public class NewComponentWizard extends Wizard implements INewWizard {
	
	public final static String ID = "org.objectweb.petals.tools.plugin.monitor.wizards.newComponent";
	
	private NewComponentMainPage componentPage;
	private NewComponentServersPage serversPage;
	private ISelection selection;

	private String url;
	private ComponentType type;
	private List<PetalsServer> servers;
	
	/**
	 * Constructor for ESBNewWizard.
	 */
	public NewComponentWizard() {
		super();
		setNeedsProgressMonitor(true);
		setWindowTitle("Ajout d'un nouveau composant");
	}
	
	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		serversPage = new NewComponentServersPage(selection);
		addPage(serversPage);
		componentPage = new NewComponentMainPage(selection);
		addPage(componentPage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		this.url = componentPage.getUrl();
		this.type = componentPage.getType();
		this.servers = serversPage.getServers();
		for (PetalsServer ps: servers) {
			try {
				ComponentParent component = ps.install(new URL(url));
				if (component == null) {
					MessageDialog.openError(this.getShell(), "Installation composant", "Impossible d'installer le composant depuis l'URL '" + url + "' sur le serveur '" + ps + "'");	
					return false;
				}
				ps.add(component);
			} catch (MalformedURLException e) {
				MessageDialog.openError(this.getShell(), "Installation composant", "Impossible d'installer le composant depuis l'URL '" + url + "' sur le serveur '" + ps + "' (" + e.getMessage() + ")");
				return false;
			}
		}
		PetalsServersView.refresh();
		return true;
	}
		
	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}

	/**
	 * @return The url
	 */
	public String getUrl() {
		return url;
	}
	
	/**
	 * @return The component Type
	 */
	public ComponentType getType() {
		return type;
	}
	
	public void setComponentsFilenames(String[] componentsFilenames) {
		componentPage.setComponentsFilenames(componentsFilenames);
	}
	
	public void setServers(List<PetalsServer> servers) {
		serversPage.setServers(servers);
	}
	
	public List<PetalsServer> getServers() {
		return serversPage.getServers();
	}
	
}