package org.objectweb.petals.tools.plugin.monitor.wizards;

import java.net.MalformedURLException;

import javax.management.remote.JMXServiceURL;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.objectweb.petals.tools.plugin.monitor.Activator;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


public class NewPetalsServerMainPage extends WizardPage {
	
	private NewPetalsServerComposite container;

	public NewPetalsServerMainPage(ISelection selection) {
		super("wizardPage");
		setTitle("Nouveau serveur Petals");
		setDescription("Cet assistant permet d'ajouter un nouveau serveur Petals au monitoring \u00e0 partir de son URL JMX");
		setImageDescriptor(AbstractUIPlugin.imageDescriptorFromPlugin(Activator.PLUGIN_ID, "icons/newserver_wiz.gif"));
	}

	public void createControl(Composite parent) {
		container = new NewPetalsServerComposite(parent, SWT.NONE);
		container.addChangedWizardListener(new ChangedWizardInterface() {
			public void dialogChanged() {
				dialogHasChanged();	
			}
		});
		dialogHasChanged();
		setControl(container);
	}



	private void dialogHasChanged() {
		String serviceUrl = getServiceURLText();
		if (serviceUrl.length() == 0) {
			updateStatus("L'URL du service JMX doit \u00eatre sp\u00e9cifi\u00e9");
			return;
		}
		try {
			new JMXServiceURL(serviceUrl);
		} catch (MalformedURLException e) {
			updateStatus("L'URL du service JMX est mal format\u00e9e");
			return;
		}
		if (PetalsServersView.getInstance().jmxUrlAlreadyAdded(serviceUrl)) {
			updateStatus("L'URL '" + serviceUrl  + "' a d\u00e9ja \u00e9t\u00e9 ajout\u00e9e");
			return;
		}
		updateStatus(null);
	}

	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	public String getServiceURLText() {
		return container.getUrl();
	}
	
	public String getJMXUser() {
		return container.getUser();
	}
	
	public String getJMXPassword() {
		return container.getPassword();
	}
	
	public String getDescription() {
		return container.getDescription();
	}
	
	public String getServerName() {
		return container.getServerName();
	}

}