package org.objectweb.petals.tools.plugin.monitor.wizards;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.objectweb.petals.tools.plugin.monitor.model.PetalsServer;
import org.objectweb.petals.tools.plugin.monitor.utils.StringUtils;
import org.objectweb.petals.tools.plugin.monitor.views.PetalsServersView;


public class NewPetalsServerWizard extends Wizard implements INewWizard {
	
	public final static String ID = "org.objectweb.petals.tools.plugin.monitor.wizards.newPetalsServer";
	
	private NewPetalsServerMainPage page;
	private ISelection selection;

	/**
	 * Constructor for ESBNewWizard.
	 */
	public NewPetalsServerWizard() {
		super();
		setNeedsProgressMonitor(true);
		setWindowTitle("Ajout d'un serveur Petals");
	}
	
	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		page = new NewPetalsServerMainPage(selection);
		addPage(page);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
		final String serviceUrl = page.getServiceURLText();
		final String jmxLogin = page.getJMXUser();
		final String jmxPassword = page.getJMXPassword();
		final String name = page.getServerName();
		final String description = page.getDescription();
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(serviceUrl, jmxLogin, jmxPassword, name, description, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Erreur", realException.getMessage());
			return false;
		}
		return true;
	}
	
	/**
	 * The worker method. It will find the container, create the
	 * file if missing or just replace its contents, and open
	 * the editor on the newly created file.
	 * 
	 * @param serviceurl L'url de connection
	 * @param jmxLogin Le login de connection
	 * @param jmxPassword Le password de connection
	 * @param monitor
	 * 
	 * @throws CoreException
	 */
	private void doFinish(String serviceurl, String jmxLogin, String jmxPassword, String name, String description, IProgressMonitor monitor) throws CoreException {
		
		// create a sample file
		monitor.beginTask("Connecting to URL " + serviceurl, 2);
		// TODO: try to connect
		monitor.worked(1);
		
		monitor.setTaskName("Adding petals server");
		// TODO: add petals server
		monitor.worked(1);
		
		monitor.done();
		
		PetalsServersView instance = PetalsServersView.getInstance();
		if (instance != null) {
			PetalsServer nouveauServeur = new PetalsServer(serviceurl);
			
			if (StringUtils.isNotEmpty(jmxLogin)) {
				nouveauServeur.setJmxLogin(jmxLogin);
			}
			if (StringUtils.isNotEmpty(jmxPassword)) {
				// TODO : Crypter le password
				nouveauServeur.setJmxPassword(jmxPassword);
			}
			nouveauServeur.setName(name);
			nouveauServeur.setDescription(description);
			instance.addPetalsServer(nouveauServeur);
		}
	}
	
	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}